

#include <RQ_OBJECT.h>

/*
Class to mark a click in a pad. A click could a key pressed or a double mouse click.
A 
*/
class ClickCollector 
{
RQ_OBJECT("ClickCollector")
private:
  Int_t fLastPx, fLastPy;
  
private:   
  TMarker    fMarker;		// Marker attribute
  
  Bool_t     fIsActive;		// true if collect mode is on
  EEventType fCollectMode;	// type of event needed to collect
  
public:
  ClickCollector();
  ~ClickCollector() {;}
  
  void Collect(Bool_t mode = kTRUE);
  void DoCollect(Int_t event, Int_t px, Int_t py, TObject *obj);
};

ClickCollector::ClickCollector(): fMarker(), fIsActive(kTRUE), fCollectMode(kKeyPress) 
{	
	// set marker style
	fMarker.SetMarkerSize(1); fMarker.SetMarkerStyle(20); fMarker.SetMarkerColor(2);
	
	// default is collect on
	Collect();
}

void ClickCollector::DoCollect(Int_t event, Int_t px, Int_t py, TObject *obj)
{
  // Collect event.
  // if event = fCollectMode ( = double click by default ) it adds a "click"
  // to the list of clicks and displays a TCross at that position.

	if ( event == kMouseMotion ) { fLastPx = px; fLastPy = py; return; } // keep memory of the last position

	if ( event == fCollectMode ) {
		if ( gROOT->GetSelectedPad() ) {
			fMarker.DrawMarker(gROOT->GetSelectedPad()->AbsPixeltoX(fLastPx),
					   gROOT->GetSelectedPad()->AbsPixeltoY(fLastPy));
			gROOT->GetSelectedPad()->Update();
		}
	}

}

// start collect clicks on current pad
void ClickCollector::Collect(Bool_t mode)
{
	if ( mode ) { // 
	  if ( TVirtualPad::Pad() ) {
		TCanvas *c = TVirtualPad::Pad()->GetCanvas();
		c->Connect("ProcessedEvent(Int_t,Int_t,Int_t,TObject*)","ClickCollector",
       				 this,
       				 "DoCollect(Int_t,Int_t,Int_t,TObject*)");
		cout << " - Start collecting clicks in " << c->GetName() << endl ;	
	  }
	  else 
	  	cout << " - Cannot collect clicks, there is no current !!! " << endl ;		  
	}
	else { 
	  if ( TVirtualPad::Pad() ) {	
		TCanvas *c = TVirtualPad::Pad()->GetCanvas();	  
      		c->Disconnect("ProcessedEvent(Int_t,Int_t,Int_t,TObject*)",
				     this,
				     "DoCollect(Int_t,Int_t,Int_t,TObject*)"); 
		cout << " - Stop collecting clicks in " << c->GetName() << endl ;	     
	  }
	} 
}