//*CMZ : 28-07-2000
//*-- Author : zouzou

#include <MultiCanvas.h>

#include <TApplication.h>
#include <TGFileDialog.h>
#include <TFile.h>
#include <TH2.h>
#include <TF1.h>
#include <TGraph.h>
#include <TMultiGraph.h>
#include <TMath.h>
#include <TSystem.h>

////////////////////////////////////////////////////////////////////////////////
//									      //
//  MultiCanvas							      //
//  A set of windows ( in a folder style fXFolder ) to analyse gamma spectrum //
//  fXFolder is a container :						      //
//  	1 - window to load/unload spectra (gates) from files to memory	      //
//      2 - window to analyse 1D spectra (find peak, show gates etc .. )      //
//									      //
//  zouzou - 30 january 2000						      //
//									      //
////////////////////////////////////////////////////////////////////////////////


ClassImp(MultiCanvas)


static const char *gOpenRootType[] = { "ROOT files",   "*.root",
                                    "All files",    "*.*",
                                    0,              0 };

static const char *gOpenGpsiType[] = { "Gpsi files",   "*.s",
                                    "All files",    "*.*",
                                    0,              0 };

static const char *gSaveAsTypes[] = { "PostScript",   "*.ps",
                                      "Encapsulated PostScript", "*.eps",
                                      "Gif files",    "*.gif",
                                      "Macro files",  "*.C",
                                      "ROOT files",   "*.root",
                                      "All files",    "*",
                                      0,              0 };

MultiCanvas::MultiCanvas(const TGWindow *p, UInt_t w, UInt_t h) 
{
// constructor

// main frame
 fMain = new TGMainFrame(p,w,h); fMain->Connect("CloseWindow()","MultiCanvas",this,"CloseWindow()");

 gClient->GetColorByName("red",fRed); fBackground = fMain->GetBackground();
 Style("Gamma");

 // A / menu bar

  // Built each menu of the menu bar.
  // ROOT MENU .. EVERYTHING RELATED TO ROOT.  
  fRootMenu = new TGPopupMenu(gClient->GetRoot());
  fRootMenu->AddEntry("&Open TFile",FILE_OPEN_TFILE);   // To open a ROOT File. 
  fRootMenu->AddSeparator();
  fRootMenu->AddEntry("&Canvas",FILE_ROOT_CANVAS);      // open a new root canvas 
  fRootMenu->AddEntry("&Browser",FILE_ROOT_BROWSER);    // open a new root browser 
  fRootMenu->AddSeparator();
  fRootMenu->AddEntry("&Quit this",FILE_AS_CLOSE);      // delete Analysis Studio
  fRootMenu->AddSeparator();
  fRootMenu->AddEntry("&Quit Root",FILE_ROOT_CLOSE);    // close root session

  fRootMenu->DrawBorder();
  fRootMenu->Connect("Activated(Int_t)","MultiCanvas",this,"HandleMenu(Int_t)");

  // VIEW MENU. ACTIONS ON DISPLAYED FIGURES 
  fViewMenu = new TGPopupMenu(gClient->GetRoot());
  fViewMenu->AddEntry("&New",VIEW_NEW);                 // to create a new Display
  fViewMenu->AddEntry("&Delete",VIEW_DELETE);           // to delete the current display
  fViewMenu->AddSeparator();
  fViewMenu->AddEntry("&Save Canvas",VIEW_SAVEAS);      // save the current canvas
  fViewMenu->AddSeparator();
  fViewMenu->AddEntry("&Editor",VIEW_EDITOR);           // pop uo editor menu   
  fViewMenu->AddSeparator();
  fViewMenu->AddEntry("&Refresh",VIEW_REFRESH);         // Refresh the current display
  fViewMenu->Connect("Activated(Int_t)","MultiCanvas",this,"HandleMenu(Int_t)");

 // GLOBAL MENU. To change ROOT globals  
  fGlobalMenu = new TGPopupMenu(gClient->GetRoot());
  fGlobalMenu->AddEntry("&Title",GLOBAL_TITLE);         // toggle title 
  fGlobalMenu->AddEntry("&Statistics",GLOBAL_STAT);     // toggle statistics
  fGlobalMenu->AddSeparator();
  fGlobalMenu->AddEntry("&Default",GLOBAL_STYLE_DEFAULT);     // swicth on Default style
  fGlobalMenu->AddEntry("&Pub",GLOBAL_STYLE_PUB);             // switch on Pub
  fGlobalMenu->AddEntry("&Bold",GLOBAL_STYLE_BOLD);           // switch on Bold
  fGlobalMenu->AddEntry("&Gamma",GLOBAL_STYLE_GAMMA);         // swicth on Gamma style    
  fGlobalMenu->AddEntry("&User",GLOBAL_STYLE_USER);           // swicth on User

  if ( gStyle->GetOptTitle() ) fGlobalMenu->CheckEntry(GLOBAL_TITLE);
  if ( gStyle->GetOptStat()  ) fGlobalMenu->CheckEntry(GLOBAL_STAT);
  fGlobalMenu->CheckEntry(GLOBAL_STYLE_DEFAULT);
  fGlobalMenu->Connect("Activated(Int_t)","MultiCanvas",this,"HandleMenu(Int_t)");

  // HELP MENU. 
  fHelpMenu = new TGPopupMenu(gClient->GetRoot());
  fHelpMenu->AddEntry("&About",-1);
  fHelpMenu->Connect("Activated(Int_t)","MultiCanvas",this,"HandleMenu(Int_t)");

  // now built the global menu bar itself.
  fMenuBar = new TGMenuBar(fMain,1,1,kHorizontalFrame); 

  fMenuBar->AddPopup("&ROOT",fRootMenu, 
			fRootMenuLayout = new TGLayoutHints(kLHintsTop | kLHintsLeft, 0, 10, 0, 0) );

  fMenuBar->AddPopup("&View",fViewMenu, 
			fViewMenuLayout = new TGLayoutHints(kLHintsTop | kLHintsLeft, 0, 10, 0, 0) );

  fMenuBar->AddPopup("&Global",fGlobalMenu, 
			fGlobalMenuLayout = new TGLayoutHints(kLHintsTop | kLHintsLeft, 0, 10, 0, 0) );

  fMenuBar->AddPopup("&Help",fHelpMenu,
			fHelpMenuLayout = new TGLayoutHints(kLHintsTop | kLHintsRight) );

 fMain->AddFrame(fMenuBar, fMenuBarLayout = new TGLayoutHints(kLHintsTop | kLHintsLeft | kLHintsExpandX,0,0, 1, 1) );

 // to separate global menu from display

 fSeparator1 = new TGHorizontal3DLine(fMain);
 fMain->AddFrame(fSeparator1, fSeparator1Layout = new TGLayoutHints(kLHintsTop | kLHintsExpandX));

 // B / everything that helps for displays. 

 fDispAction = new TGCompositeFrame(fMain,w,h/10,kHorizontalFrame);
 fMain->AddFrame(fDispAction, fDispActionLayout = new TGLayoutHints(kLHintsLeft | kLHintsTop | kLHintsExpandX ,0,0,0,0));

  // first group frame. everything related to analysis with cursor.
  fDispCursor = new TGGroupFrame(fDispAction,"Cursor",kHorizontalFrame);
   // - Info - TGMatrixLoyout adopted by fDispCursor and deleted by it.
  fDispCursor->SetLayoutManager( new TGMatrixLayout(fDispCursor,2,0,kHorizontalFrame) );  
  fDispAction->AddFrame(fDispCursor, fDispCursorLayout = new TGLayoutHints(kLHintsLeft | kLHintsTop, 10, 0, 1, 5) );
   // on/off drawing of the cursor
   fShowCursor = new TGTextButton(fDispCursor,"&Show",CURSOR_SHOW);
   fDispCursor->AddFrame(fShowCursor, fShowCursorLayout = new TGLayoutHints(kLHintsCenterX | kLHintsCenterY) );
   fShowCursor->SetToolTipText("Everything related to Y");
   fShowCursor->Resize(60,fShowCursor->GetDefaultHeight());
   fShowCursor->Connect("Clicked()","MultiCanvas",this,"DoCursor()");

   // on, to collect cursor informations 
   fCollectCursor = new TGTextButton(fDispCursor,"&Collect",CURSOR_COLLECT);
   fDispCursor->AddFrame(fCollectCursor, fCollectCursorLayout = new TGLayoutHints(kLHintsCenterX | kLHintsCenterY) );
   fCollectCursor->Resize(60,fShowCursor->GetDefaultHeight());
   fCollectCursor->Connect("Clicked()","MultiCanvas",this,"DoCursor()");

   // on, to collect cursor informations 
   fInfoCursor = new TGTextButton(fDispCursor,"&Info",CURSOR_INFO);
   fDispCursor->AddFrame(fInfoCursor, fInfoCursorLayout = new TGLayoutHints(kLHintsCenterX | kLHintsCenterY) );
   fInfoCursor->Resize(60,fShowCursor->GetDefaultHeight());
   fInfoCursor->Connect("Clicked()","MultiCanvas",this,"DoCursor()");

   // on, to collect cursor informations with keyboard
   fKeyCursor = new TGTextButton(fDispCursor,"&Key",CURSOR_KEY);
   fDispCursor->AddFrame(fKeyCursor, fKeyCursorLayout = new TGLayoutHints(kLHintsCenterX | kLHintsCenterY) );
   fKeyCursor->Resize(60,fShowCursor->GetDefaultHeight());
   fKeyCursor->Connect("Clicked()","MultiCanvas",this,"DoCursor()");

   // reset collection.
   fResetCursor = new TGTextButton(fDispCursor,"&Reset",CURSOR_RESET);
   fDispCursor->AddFrame(fResetCursor, fResetCursorLayout =  new TGLayoutHints(kLHintsCenterX | kLHintsCenterY)  );
   fResetCursor->Resize(60,fShowCursor->GetDefaultHeight());
   fResetCursor->Connect("Clicked()","MultiCanvas",this,"DoCursor()");
   
   // what is about to be reset.
   fWhichCursor = new TGComboBox(fDispCursor, 88);
   fDispCursor->AddFrame(fWhichCursor, fWhichCursorLayout = new TGLayoutHints(kLHintsCenterX | kLHintsCenterY) );
   fWhichCursor->AddEntry(" Last ",1);  fWhichCursor->Select(1);
   fWhichCursor->AddEntry(" Pad ",2);
   fWhichCursor->AddEntry(" All ",3);
   fWhichCursor->Resize(60,fShowCursor->GetDefaultHeight());

   fDispCursor->MapSubwindows(); fDispCursor->Layout(); 

  // second group frame. everything related to pads and canvas. fDispPads
  // clear, delete, logx and logy
  
  fDispPads = new TGGroupFrame(fDispAction,"Pads/canvas",kHorizontalFrame);
  // - Info - TGMatrixLoyout adopted by fDispPads and deleted by it.
  fDispPads->SetLayoutManager( new TGMatrixLayout(fDispPads,2,0,4) );
  fDispAction->AddFrame(fDispPads, 
			fDispPadsLayout = new TGLayoutHints(kLHintsLeft | kLHintsTop | kLHintsExpandX  , 10, 0, 1, 10));
 
   fLabelAction = new TGLabel(fDispPads, new TGHotString(" Actions ") );
   fDispPads->AddFrame(fLabelAction, fLabelActionLayout = new TGLayoutHints(kLHintsCenterX | kLHintsCenterY, 1, 1, 1, 1) );
   fLabelAction->Resize(60,fShowCursor->GetDefaultHeight()); 

   fClearPads = new TGTextButton(fDispPads,"   Clear   ",DISPLAY_CLEAR);
   fDispPads->AddFrame(fClearPads, fClearPadsLayout = new TGLayoutHints(kLHintsCenterX | kLHintsCenterY, 1, 1, 1, 1) );
   fClearPads->Connect("Pressed()","MultiCanvas",this,"DoDisplay()");
   fClearPads->Resize(60,fClearPads->GetDefaultHeight());

   fAction[0] = new TGCheckButton(fDispPads,"gpad",MODE_GPAD);
   fDispPads->AddFrame(fAction[0], fActionLayout[0] = new TGLayoutHints(kLHintsCenterX | kLHintsCenterY, 1, 1, 1, 1 ) );
   fAction[0]->SetState(kButtonDown);
   fAction[0]->Connect("Pressed()","MultiCanvas",this,"DoDisplay()");
   fAction[0]->Resize(60,fShowCursor->GetDefaultHeight());

   fDeletePads = new TGTextButton(fDispPads,"  Delete  ", DISPLAY_DELETE);
   fDeletePads->Resize(60,fClearPads->GetDefaultHeight());
   fDispPads->AddFrame(fDeletePads, fDeletePadsLayout = new TGLayoutHints(kLHintsCenterX | kLHintsCenterY, 1, 1, 1, 1)  );
   fDeletePads->Connect("Pressed()","MultiCanvas",this,"DoDisplay()");

   fAction[1] = new TGCheckButton(fDispPads," all",MODE_ALL);
   fDispPads->AddFrame(fAction[1], fActionLayout[1] = new TGLayoutHints(kLHintsCenterX | kLHintsCenterY, 1, 1, 1, 1) );
   fAction[1]->Connect("Pressed()","MultiCanvas",this,"DoDisplay()");
   fAction[1]->Resize(60,fShowCursor->GetDefaultHeight());

   fLogxPads = new TGTextButton(fDispPads,"   LogX   ", DISPLAY_LOGX );
   fLogxPads->Resize(60,fClearPads->GetDefaultHeight());
   fDispPads->AddFrame(fLogxPads, fLogxPadsLayout = new TGLayoutHints(kLHintsCenterX | kLHintsCenterY, 1, 1, 1, 1)  );
   fLogxPads->Connect("Pressed()","MultiCanvas",this,"DoDisplay()");

   fAction[2] = new TGCheckButton(fDispPads,"canv", MODE_CANVAS);
   fDispPads->AddFrame(fAction[2], fActionLayout[2] = new TGLayoutHints(kLHintsCenterX | kLHintsCenterY, 1, 1, 1, 1) );
   fAction[2]->Connect("Pressed()","MultiCanvas",this,"DoDisplay()");
   fAction[2]->Resize(60,fShowCursor->GetDefaultHeight());

   fLogyPads = new TGTextButton(fDispPads,"   LogY  ", DISPLAY_LOGY );
   fLogyPads->Resize(60,fClearPads->GetDefaultHeight());
   fDispPads->AddFrame(fLogyPads, fLogyPadsLayout = new TGLayoutHints(kLHintsCenterX | kLHintsCenterY, 1, 1, 1, 1)  );
   fLogyPads->Connect("Pressed()","MultiCanvas",this,"DoDisplay()");

   fUserX = new TGLabel(fDispPads, new TGHotString("InputX :") );
   fDispPads->AddFrame(fUserX, fUserXLayout = new TGLayoutHints(kLHintsCenterX | kLHintsCenterY, 1, 1, 1, 1) );
   fUserX->Resize(60,fShowCursor->GetDefaultHeight()); 

   fUserY = new TGLabel(fDispPads, new TGHotString("InputY :") );
   fDispPads->AddFrame(fUserY, fUserYLayout = new TGLayoutHints(kLHintsCenterX | kLHintsCenterY, 1, 1, 1, 1) );
   fUserY->Resize(60,fShowCursor->GetDefaultHeight()); 

   fEntryXPads = new TGTextEntry(fDispPads, new TGTextBuffer(40) );
   fEntryXPads->SetToolTipText("Everything related to X");
   fEntryXPads->Resize(60,fClearPads->GetDefaultHeight());
   fDispPads->AddFrame(fEntryXPads, fEntryXPadsLayout = new TGLayoutHints(kLHintsCenterX | kLHintsCenterY, 1, 1, 1, 1) );

   fEntryYPads = new TGTextEntry(fDispPads, new TGTextBuffer(40) );
   fEntryYPads->SetToolTipText("Everything related to Y");
   fEntryYPads->Resize(60,fClearPads->GetDefaultHeight());
   fDispPads->AddFrame(fEntryYPads, fEntryYPadsLayout = new TGLayoutHints(kLHintsCenterX | kLHintsCenterY, 1, 1, 1, 1) );

   fDvSym = new TGTextButton(fDispPads,"  DvSym  ", DISPLAY_DVSYM );
   fDvSym->Resize(60,fClearPads->GetDefaultHeight());
   fDispPads->AddFrame(fDvSym, fDvSymLayout = new TGLayoutHints(kLHintsCenterX | kLHintsCenterY, 1, 1, 1, 1)  );
   fDvSym->Connect("Pressed()","MultiCanvas",this,"DoDisplay()");

   fDvAsym = new TGTextButton(fDispPads,"  DvAsym ", DISPLAY_DVASYM );
   fDvAsym->Resize(60,fClearPads->GetDefaultHeight());
   fDispPads->AddFrame(fDvAsym, fDvAsymLayout = new TGLayoutHints(kLHintsCenterX | kLHintsCenterY, 1, 1, 1, 1)  );
   fDvAsym->Connect("Pressed()","MultiCanvas",this,"DoDisplay()");

   fExpandX = new TGTextButton(fDispPads," ExpandX ", DISPLAY_EXPANDX );
   fExpandX->Resize(60,fClearPads->GetDefaultHeight());
   fDispPads->AddFrame(fExpandX, fExpandXLayout = new TGLayoutHints(kLHintsCenterX | kLHintsCenterY, 1, 1, 1, 1)  );
   fExpandX->Connect("Pressed()","MultiCanvas",this,"DoDisplay()");

   fExpandY = new TGTextButton(fDispPads," ExpandY ", DISPLAY_EXPANDY );
   fExpandY->Resize(60,fClearPads->GetDefaultHeight());
   fDispPads->AddFrame(fExpandY, fExpandYLayout = new TGLayoutHints(kLHintsCenterX | kLHintsCenterY, 1, 1, 1, 1)  );
   fExpandY->Connect("Pressed()","MultiCanvas",this,"DoDisplay()");

   fDrawAxis = new TGTextButton(fDispPads," DoAxis ", DISPLAY_DRAWAXIS );
   fDrawAxis->Resize(60,fClearPads->GetDefaultHeight());  
   fDispPads->AddFrame(fDrawAxis, fDrawAxisLayout = new TGLayoutHints(kLHintsCenterX | kLHintsCenterY, 1, 1, 1, 1)  );
   fDrawAxis->Connect("Pressed()","MultiCanvas",this,"DoDisplay()");

//    fCursorFit = new TGTextButton(fDispPads,"  GFit   ", DISPLAY_CURSORFIT );
//    fCursorFit->Resize(60,fClearPads->GetDefaultHeight());
//    fDispPads->AddFrame(fCursorFit, fCursorFitLayout = new TGLayoutHints(kLHintsCenterX | kLHintsCenterY, 1, 1, 1, 1)  );
//    fCursorFit->Connect("Pressed()","MultiCanvas",this,"DoDisplay()");

   fDispPads->MapSubwindows(); fDispPads->Layout();  
   
 // Working window divided in two frames: fDispMain and fFrameHSlider
 //   fDispMain is itself divided in two frames: a vertical slider fFrameVSlider 
 //   and the frame to display histograms fDisplays

 fDispAction->MapSubwindows(); fDispAction->Layout();

 fWorking = new TGCompositeFrame(fMain,w,9*h/10,kVerticalFrame);
 fMain->AddFrame(fWorking, 
		 fWorkingLayout = new TGLayoutHints(kLHintsLeft | kLHintsTop | kLHintsExpandX | kLHintsExpandY));
  // fDispMain
  fDispMain = new TGCompositeFrame(fWorking,w,h,kHorizontalFrame);
  fWorking->AddFrame(fDispMain,
		     fDispMainLayout = new TGLayoutHints(kLHintsLeft | kLHintsTop | kLHintsExpandX | kLHintsExpandY));

   // vertical slider
   fFrameVSlider = new TGCompositeFrame(fDispMain,w,h,kVerticalFrame);
   fDispMain->AddFrame(fFrameVSlider,
		       fFrameVSliderLayout = new TGLayoutHints(kLHintsLeft | kLHintsTop | kLHintsExpandY,0, 0, 20, 0));

    fPicYUnzoom = gClient->GetPicture("unzoom_vertical.xpm");
    fYUnzoom = new TGPictureButton(fFrameVSlider, fPicYUnzoom, SLIDER_V_UNZOOM); fYUnzoom->SetToolTipText("Unzoom Y axis");
    fYUnzoom->Connect("Pressed()","MultiCanvas",this,"DoButtonScaling()");
    fFrameVSlider->AddFrame(fYUnzoom, fYUnzoomLayout = new TGLayoutHints(kLHintsCenterX ) );  

    fPicArrowUp = gClient->GetPicture("arrow_up.xpm");
    fArrowUp = new TGPictureButton(fFrameVSlider, fPicArrowUp, SLIDER_UP); fArrowUp->SetToolTipText("Move Up");
    fArrowUp->Connect("Pressed()","MultiCanvas",this,"DoButtonScaling()");
    fFrameVSlider->AddFrame(fArrowUp, fArrowUpLayout = new TGLayoutHints(kLHintsCenterX ) );

    fVslider = new TGDoubleVSlider(fFrameVSlider,h , kDoubleScaleNo, SLIDERVID, kVerticalFrame, fMain->GetDefaultFrameBackground(), kTRUE);
    fVslider->SetRange(MINSLIDER,MAXSLIDER); fVslider->SetPosition(MINSLIDER,MAXSLIDER);
    fVslider->Connect("Released()","MultiCanvas",this,"DoScaling()");
    fFrameVSlider->AddFrame(fVslider, fVsliderLayout = new TGLayoutHints(kLHintsLeft | kLHintsTop | kLHintsExpandY ));

    fPicArrowDown = gClient->GetPicture("arrow_down.xpm");
    fArrowDown = new TGPictureButton(fFrameVSlider, fPicArrowDown, SLIDER_DOWN); fArrowDown->SetToolTipText("Move Down");
    fArrowDown->Connect("Pressed()","MultiCanvas",this,"DoButtonScaling()");
    fFrameVSlider->AddFrame(fArrowDown, fArrowDownLayout = new TGLayoutHints(kLHintsCenterX ) );

   // TGTab = a stack of canvas
   fDisplays = new TGTab(fDispMain,w,h);
   fDispMain->AddFrame(fDisplays, fDisplaysLayout = new TGLayoutHints(kLHintsLeft | kLHintsTop | kLHintsExpandX | kLHintsExpandY));
   fDisplays->Connect("Selected(Int_t)","MultiCanvas",this,"CdCanvas(Int_t)");

   fEmbeddedCanvasLayout = new TGLayoutHints(kLHintsLeft | kLHintsTop| kLHintsExpandX | kLHintsExpandY);
   AddCanvas();  // start with one embedded canvas.

  // horizontal slider .. belongs to fWorking
  fFrameHSlider = new TGCompositeFrame(fWorking,w,h,kHorizontalFrame);
  fWorking->AddFrame(fFrameHSlider, fFrameHSliderLayout = new TGLayoutHints(kLHintsLeft | kLHintsTop | kLHintsExpandX, 20, 0, 0, 0));

   fPicXUnzoom = gClient->GetPicture("unzoom_horizontal.xpm");
   fXUnzoom = new TGPictureButton(fFrameHSlider, fPicXUnzoom, SLIDER_H_UNZOOM); fXUnzoom->SetToolTipText("Unzoom X axis");
   fXUnzoom->Connect("Pressed()","MultiCanvas",this,"DoButtonScaling()");
   fFrameHSlider->AddFrame(fXUnzoom, fXUnzoomLayout = new TGLayoutHints(kLHintsCenterY ) );  

   fPicArrowLeft = gClient->GetPicture("arrow_left.xpm");
   fArrowLeft = new TGPictureButton(fFrameHSlider, fPicArrowLeft, SLIDER_LEFT);
   fArrowLeft->Connect("Pressed()","MultiCanvas",this,"DoButtonScaling()");
   fFrameHSlider->AddFrame(fArrowLeft, fArrowLeftLayout = new TGLayoutHints(kLHintsCenterY) );

   fHslider = new TGDoubleHSlider(fFrameHSlider,w, kDoubleScaleNo , SLIDERHID);
   fHslider->SetRange(MINSLIDER,MAXSLIDER); fHslider->SetPosition(MINSLIDER,MAXSLIDER);
   fHslider->Connect("Released()","MultiCanvas",this,"DoScaling()");
   // fHslider->Connect("PositionChanged()","MultiCanvas",this,"DoScaling()");
   fFrameHSlider->AddFrame(fHslider, fHsliderLayout = new TGLayoutHints(kLHintsLeft | kLHintsTop | kLHintsExpandX ));

   fPicArrowRight = gClient->GetPicture("arrow_right.xpm");
   fArrowRight = new TGPictureButton(fFrameHSlider, fPicArrowRight, SLIDER_RIGHT);
   fArrowRight->Connect("Pressed()","MultiCanvas",this,"DoButtonScaling()");
   fFrameHSlider->AddFrame(fArrowRight, fArrowRightLayout = new TGLayoutHints(kLHintsCenterY) );

 // fSeparator2 .
 fSeparator2 = new TGHorizontal3DLine(fMain); fSeparator2->DrawBorder();
 fMain->AddFrame(fSeparator2, fSeparator2Layout = new TGLayoutHints(kLHintsTop | kLHintsExpandX));

 // to display useful informations.
 fInfoBar = new TGCompositeFrame(fMain,w,h,kHorizontalFrame | kSunkenFrame);  

  fStatusBar = new TGStatusBar(fInfoBar,w,h);
  Int_t parts[6] = {18, 18, 24, 13, 13, 13}; fStatusBar->SetParts(parts,6); 
  fStatusBar->SetText(" Current Canvas",0);
  fStatusBar->SetText(" Current Pad: ",1);
  fStatusBar->SetText(" Selected Object: ",2);
  fStatusBar->SetText(" X : ",3);
  fStatusBar->SetText(" Y : ",4);
  fStatusBar->SetText(" Z : ",5);
  fStatusBar->Draw3DCorner(kTRUE);
  fInfoBar->AddFrame(fStatusBar, fStatusBarLayout = new TGLayoutHints(kLHintsRight | kLHintsTop | kLHintsExpandX  ) );
  
 fMain->AddFrame(fInfoBar, fInfoBarLayout = new TGLayoutHints(kLHintsLeft | kLHintsTop | kLHintsExpandX ));     

 ///////////////

 fMain->MapSubwindows();
 fMain->SetWindowName("Gammabase Analysis Studio");
 fMain->Resize(fMain->GetDefaultSize());

 fMain->MapWindow();

}

MultiCanvas::~MultiCanvas() 
{
// destructor .. 

    delete fStatusBarLayout;
    delete fStatusBar;
    delete fInfoBarLayout;
    delete fInfoBar;
    delete fSeparator2Layout;
    delete fSeparator2;
    delete fArrowRightLayout;
    delete fArrowRight;
    delete fHsliderLayout;
    delete fHslider;
    delete fArrowLeftLayout;
    delete fArrowLeft;
    delete fXUnzoomLayout;
    delete fXUnzoom;
    delete fFrameHSliderLayout;
    delete fFrameHSlider;
    delete fEmbeddedCanvasLayout;
//     fCursor.Delete(); 
    fEmbeddedCanvas.Delete();
    delete fDisplaysLayout;
    delete fDisplays;
    delete fArrowDownLayout;
    delete fArrowDown;
    delete fVsliderLayout;
    delete fVslider;
    delete fArrowUpLayout;
    delete fArrowUp;
    delete fYUnzoomLayout;
    delete fYUnzoom;
    delete fFrameVSliderLayout;
    delete fFrameVSlider;
    delete fDispMainLayout;
    delete fDispMain;
    delete fWorkingLayout;
    delete fWorking;
    delete fCursorFitLayout;
    delete fCursorFit;
    delete fDrawAxisLayout;
    delete fDrawAxis;
    delete fExpandYLayout;
    delete fExpandY;
    delete fExpandXLayout;
    delete fExpandX;
    delete fDvAsymLayout;
    delete fDvAsym;
    delete fDvSymLayout;
    delete fDvSym;
    delete fEntryYPadsLayout;
    delete fEntryYPads;
    delete fEntryXPadsLayout;
    delete fEntryXPads;
    delete fUserYLayout;
    delete fUserY;
    delete fUserXLayout;
    delete fUserX;
    delete fLogyPadsLayout;
    delete fLogyPads;
    delete fActionLayout[0], fActionLayout[1], fActionLayout[2];
    delete fAction[0], fAction[1], fAction[2];
    delete fLogxPadsLayout;
    delete fLogxPads;
    delete fDeletePadsLayout;
    delete fDeletePads;
    delete fClearPadsLayout;
    delete fClearPads;
    delete fLabelActionLayout;
    delete fLabelAction;
    delete fDispPadsLayout;
    delete fDispPads;
    delete fWhichCursorLayout;
    delete fWhichCursor;
    delete fResetCursorLayout;
    delete fResetCursor;
    delete fKeyCursorLayout;
    delete fKeyCursor;
    delete fInfoCursorLayout;
    delete fInfoCursor;
    delete fCollectCursorLayout;
    delete fCollectCursor;
    delete fShowCursorLayout;
    delete fShowCursor;
    delete fDispCursorLayout;
    delete fDispCursor;
    delete fDispActionLayout;
    delete fDispAction;
    delete fSeparator1Layout;
    delete fSeparator1;
    delete fMenuBarLayout;
    delete fMenuBar;
    delete fHelpMenuLayout;
    delete fGlobalMenuLayout;
    delete fViewMenuLayout;
    delete fRootMenuLayout;
    delete fHelpMenu;
    delete fGlobalMenu;
    delete fViewMenu;
    delete fRootMenu;

    delete fMain;
}

void MultiCanvas::CloseWindow()
{
    // to change it 
    delete this;
}

void MultiCanvas::CdCanvas(Int_t which)
{
    // Embedded canvas number which becomes the current pad

    if ( fDisplays->GetTabTab(which) != NULL ) {

	// change current canvas, cursor and update pads.
	TRootEmbeddedCanvas *canvas = (TRootEmbeddedCanvas *)fEmbeddedCanvas[which];
	AnalysisCursor *cr = (AnalysisCursor *)fCursor[which];

	if ( canvas->GetCanvas() != NULL ) gPad = canvas->GetCanvas();	
	gPad->Modified(); canvas->GetCanvas()->Update();

	// now check if colors are consistents for fShowCursor, fCollectCursor and fKey
	// show or hide
	if ( fShowCursor->GetBackground() == fBackground ) {
	    if ( cr->IsVisible() ) { cr->Hide(); DoCursor(CURSOR_SHOW); }
	}
	else {
	    if ( !cr->IsVisible() ) { cr->Show(); DoCursor(CURSOR_SHOW); }
	}

	// collect or not
	if ( fCollectCursor->GetBackground() == fBackground ) {
	    if ( cr->IsCollecting() ) { cr->Collect(); DoCursor(CURSOR_COLLECT); }
	}
	else {
	    if ( !cr->IsCollecting() ) { cr->UnCollect(); DoCursor(CURSOR_COLLECT); }
	}

	// kKeyPress or kButton1Double
	if ( fKeyCursor->GetBackground() == fBackground ) {
	    if ( cr->GetCollectMode() == kKeyPress ) { cr->SetCollectMode(); DoCursor(CURSOR_KEY); }
	}
	else {
	    if ( cr->GetCollectMode() != kKeyPress ) { cr->SetCollectMode(kKeyPress); DoCursor(CURSOR_KEY); }
	}
    }
}

void MultiCanvas::AddCanvas()
{
  // create an new enbedded canvas in TGtab. All canvas will have the following
  // generic name : Display1, Display2, .. etc and the embedded canvas the name Canv1, Canv2 etc

  TGCompositeFrame *window; TRootEmbeddedCanvas *canvas; // to creat a new embedded canvas

  Int_t i; Bool_t exist = kFALSE;
  Char_t genericname[10];

  if ( fEmbeddedCanvas.GetEntries() == MAXDISPLAY  ) 
      { cout << " Analysis Studio :  Sorry, I cannot add one more display ... " << endl; return; }

   // creat a new tab
   sprintf(genericname,"Display%d",fEmbeddedCanvas.GetEntries()); window = fDisplays->AddTab(genericname);  
   cout << "  Analysis Studio : A new window has been created with name " << genericname;

   // creat a new embeeded canvas
   sprintf(genericname,"Canv%d",fEmbeddedCanvas.GetEntries());
   canvas = new TRootEmbeddedCanvas(genericname,window,500,300);
   window->AddFrame(canvas, fEmbeddedCanvasLayout );
   cout << " (Canvas : " << genericname << ")" << endl;

   fEmbeddedCanvas.AddLast(canvas); 
   canvas->GetCanvas()->Connect("ProcessedEvent(Int_t,Int_t,Int_t,TObject*)",
				 "MultiCanvas",
				 this,
				 "HandleEmbeddedCanvas(Int_t,Int_t,Int_t,TObject*)");

   // create a cursor for this canvas,
   sprintf(genericname,"Cursor%d",fEmbeddedCanvas.GetEntries()-1);   
//    fCursor.AddLast( new AnalysisCursor(genericname,canvas->GetCanvas()) );

   fDisplays->SetTab( fDisplays->GetNumberOfTabs()-1 ); // CdCanvas(fEmbeddedCanvas.GetEntries());

   fDisplays->MapSubwindows(); fDisplays->Layout();
}

void MultiCanvas::RemoveCanvas()
{
  // remove current tab from the fDisplays folder

  if ( fDisplays->GetCurrent() == 0 ) {
    cout << " - ! - Analysis Studio : Cannot delete the first display " << endl; return;
  }

  TRootEmbeddedCanvas *canvas = (TRootEmbeddedCanvas *)fEmbeddedCanvas.RemoveAt(fDisplays->GetCurrent());
//  AnalysisCursor *cr = (AnalysisCursor *)fCursor.RemoveAt(fDisplays->GetCurrent()); 
//  if ( cr != NULL ) delete cr;

  SafeDelete(canvas); 

  // compress all collections.
  fDisplays->RemoveTab( fDisplays->GetCurrent() ); fEmbeddedCanvas.Compress(); // fCursor.Compress();

  if (   fDisplays->GetNumberOfTabs() != fEmbeddedCanvas.GetEntries()  )
      { cout << " - ! - Analysis Studio : Number of displays is different that number of embedded canvas " << endl; return; }

  // now renamed all canvas and displays
  Char_t genericname[10]; Int_t i;
  for ( i = 0; i < fEmbeddedCanvas.GetEntries(); i++ ) {
      sprintf(genericname,"Display%d",i); fDisplays->GetTabTab(i)->SetText( new TGString(genericname) );
      sprintf(genericname,"Canv%d",i); 
      canvas = (TRootEmbeddedCanvas *)fEmbeddedCanvas[i]; 
      if ( canvas != NULL ) canvas->GetCanvas()->SetName(genericname);
//      cr = (AnalysisCursor *)fCursor[i]; 
//      sprintf(genericname,"Cursor%d",i); cr->SetName(genericname);
  }
  
  CdCanvas(0); // current display is 0 so current canvas is also 0

  fDisplays->MapSubwindows(); fDisplays->Layout(); 
}

void MultiCanvas::ShowBrowser()
{
  // creat a new browser. check first if the browser 
  // with title ROOT Object Browser doesn't exist. if it exists
  // delete it first before creating a new one. 

  TBrowser *b; 

  if ( gROOT->GetListOfBrowsers()->GetSize() == 0 ) {
    b = new TBrowser("Browser","ROOT Object Browser"); return;
  }

  TIter next(gROOT->GetListOfBrowsers()); 

  while ( (b = (TBrowser *)next()) != NULL ) {
    if ( strcmp("ROOT Object Browser",b->GetTitle()) == 0 ) {
      gROOT->GetListOfBrowsers()->Remove(b); delete b;
      b = new TBrowser("Browser","ROOT Object Browser");
    }
  }
}

TCanvas *MultiCanvas::ExtraCanvas()
{
  // creat a new canvas. Extra canvas will have the same generic name :
  // extra_c1, extra_c2 .. etc. This member first loops over canvas to find the 
  // first free name. it returns the created canvas.

  TCanvas *can = NULL; Int_t i = 0; Char_t name[10];

  do {
    i++; sprintf(name,"extra_c%d",i);
  } while ( ( can = (TCanvas *)gROOT->GetListOfCanvases()->FindObject(name) ) );

  cout << " Analysis Studio : A New Extra canvas has been built " << name << endl;
  return new TCanvas(name,"Extra Analysis Studio",0,0,600,400); 

}


Bool_t MultiCanvas::Style(const Char_t *whichstyle)
{
  // Change current style. creat Gamma style it if it doesn't exist.

  TStyle *style = NULL;

  if ( gROOT->GetListOfStyles()->FindObject("Gamma") == NULL ) {

    style = new TStyle("Gamma","GammaRay");
    // style->SetOptStat(0000001);
    style->SetOptStat(1);

    // no place for axis attributes
    style->SetPadLeftMargin(0.0);
    style->SetPadRightMargin(0.0);
    style->SetPadBottomMargin(0.0);
    style->SetPadTopMargin(0.0);

    style->SetHistLineColor(4);

    style->BuildStyles();
  }

  style = (TStyle *)gROOT->GetListOfStyles()->FindObject(whichstyle);
  if ( style == 0 ) {
      gStyle = (TStyle *)gROOT->GetListOfStyles()->FindObject("Default");
      cout << " Analysis Studio : Current style is now " << gStyle->GetName() << endl; return kFALSE;
  }
  else { 
      gStyle = style; 
      cout << " Analysis Studio : Current style is now " << gStyle->GetName() << endl; return kTRUE;
  }
}

void MultiCanvas::DoDisplay(Int_t id) 
{
    // handle cursor related actions
    TString inputext; 
    Double_t min, max, xlow, xup, ylow, yup, tmp;
    AnalysisCursor *cr; TObjArray *xpos, *ypos; MouseClick *low, *up; 

    if ( id == -1 ) {
	TGButton *b; 
	b = (TGButton *)gTQSender; id = b->WidgetId();
    }

    Int_t dvx, dvy;
    TList *pads = new TList();
    Int_t i;  Bool_t ok; TVirtualPad *pad;

    switch ( id ) {
	case MODE_GPAD :
	case MODE_ALL :
	case MODE_CANVAS :
	    for ( i = 0; i < 3; i ++ ) {
		if ( fAction[i]->WidgetId() != id  ) fAction[i]->SetState(kButtonUp);
	    }
	    break;

	case DISPLAY_CLEAR :
	    if ( fAction[0]->GetState() == kButtonDown ) { // only in gPad
		gPad->Clear(); gPad->Modified(); gPad->Update();
	    }
	    if ( fAction[1]->GetState() == kButtonDown ) { // in all pads.

		AddPadsPrimitives(pads,GetCurrentCanvas(),"TVirtualPad"); 

		pad = (TVirtualPad *)pads->First();	       
		do {
		    pad->Clear(); pad->Modified(); pad->Update();		    
		} while ( ( pad = (TVirtualPad *)pads->After(pad) ) != NULL  );
	    }
	    if ( fAction[2]->GetState() == kButtonDown ) { // in  canvas.
		GetCurrentCanvas()->Clear(); GetCurrentCanvas()->Modified(); GetCurrentCanvas()->Update();
	    }
	    break;

	case DISPLAY_DELETE :
	    if ( fAction[0]->GetState() == kButtonDown ) { // only in gPad
		if ( gPad != GetCurrentCanvas() ) delete gPad;
	    }
	    if ( fAction[1]->GetState() == kButtonDown ) { // in all pads.

		AddPadsPrimitives(pads,GetCurrentCanvas(),"TVirtualPad"); 
		pads->Delete("slow");
	    }
	    if ( fAction[2]->GetState() == kButtonDown ) { // in  canvas.
		RemoveCanvas();
	    }
	    break;

	case DISPLAY_LOGX :
	    fHslider->SetPosition(MINSLIDER,MAXSLIDER); 
	    ZoomX(MINSLIDER,MAXSLIDER);
	    if ( fAction[0]->GetState() == kButtonDown ) { // only in gPad

		if ( gPad->GetLogx() ) gPad->SetLogx(0);
		else gPad->SetLogx(1);
		gPad->Modified();

	    }
	    else { // in all pads.

		AddPadsPrimitives(pads,GetCurrentCanvas(),"TVirtualPad"); 

		pad = (TVirtualPad *)pads->First();	       
		do {
		    if ( pad->GetLogx() ) pad->SetLogx(0);
		    else pad->SetLogx();
		    pad->Modified();		    
		} while ( ( pad = (TVirtualPad *)pads->After(pad) ) != NULL  );
	    }
	    break;

	case DISPLAY_LOGY :
	    fVslider->SetPosition(MINSLIDER,MAXSLIDER);
	    ZoomY(MINSLIDER,MAXSLIDER);
	    if ( fAction[0]->GetState() == kButtonDown ) { // only in gPad

		if ( gPad->GetLogy() ) gPad->SetLogy(0);
		else gPad->SetLogy(1);
		gPad->Modified();

	    }
	    else { // in all pads.

		AddPadsPrimitives(pads,GetCurrentCanvas(),"TVirtualPad"); 

		pad = (TVirtualPad *)pads->First();	       
		do {
		    if ( pad->GetLogy() ) pad->SetLogy(0);
		    else pad->SetLogy();
		    pad->Modified();		    
		} while ( ( pad = (TVirtualPad *)pads->After(pad) ) != NULL  );
	    }
	    break;

        case DISPLAY_DVSYM :
	    {
	    // get information from input strings
	    istrstream inputdvx( fEntryXPads->GetText() );  inputdvx >> dvx;
	    istrstream inputdvy( fEntryYPads->GetText() );  inputdvy >> dvy; 

	    if ( inputdvx.fail() ) dvx = 1; if ( inputdvy.fail() ) dvy = 1;

	    // now do the job depending on fAction states.
	    if ( fAction[0]->GetState() == kButtonDown ) { // only in gPad
		gPad->Divide(dvx,dvy,0,0);
	    }
	    if ( fAction[1]->GetState() == kButtonDown ) { // in all pads.

		AddPadsPrimitives(pads,GetCurrentCanvas(),"TVirtualPad"); 

		pad = (TVirtualPad *)pads->First();	       
		do {
		    pad->Divide(dvx,dvy,0,0); 		    
		} while ( ( pad = (TVirtualPad *)pads->After(pad) ) != NULL  );
	    }
	    if ( fAction[2]->GetState() == kButtonDown ) { // in  canvas.
		GetCurrentCanvas()->Divide(dvx,dvy,0,0);	
	    }	    
	    }
	    break ;

        case DISPLAY_DVASYM :
	    {
	    // get information from input strings
	    istrstream inputdvx( fEntryXPads->GetText() );  inputdvx >> dvx;
	    istrstream inputdvy( fEntryYPads->GetText() );  inputdvy >> dvy; 

	    if ( inputdvx.fail() ) dvx = 1; if ( inputdvy.fail() ) dvy = 1;

	    // now do the job depending on fAction states.
	    if ( fAction[0]->GetState() == kButtonDown ) { // only in gPad
		gPad->Divide(dvx,dvy,0,0);
	    }
	    if ( fAction[1]->GetState() == kButtonDown ) { // in all pads.

		AddPadsPrimitives(pads,GetCurrentCanvas(),"TVirtualPad"); 

		pad = (TVirtualPad *)pads->First();	       
		do {
		    pad->Divide(dvx,dvy,0,0); 		    
		} while ( ( pad = (TVirtualPad *)pads->After(pad) ) != NULL  );
	    }
	    if ( fAction[2]->GetState() == kButtonDown ) { // in  canvas.
		GetCurrentCanvas()->Divide(dvx,dvy,0,0);	
	    }	    
	    }
	    break ;

	case DISPLAY_EXPANDX :	
	    // Expand with cursor by using the last two clicks belonging to the same pad 
	    // It first calculates the new values for fHSlider than Emit the Released signal.
	    // Do nothing in case of problems. 
	    ok = kFALSE; cr = GetCurrentCursor(); xpos = cr->GetArray();
	    if ( xpos->GetEntries() > 1 ) {

		Int_t last = xpos->GetLast(); 
		low = (MouseClick *)xpos->At(last-1); up = (MouseClick *)xpos->At(last);

		xlow = low->GetPad()->GetUxmin(); xup  = up->GetPad()->GetUxmax();
		min = low->GetX(); max = up->GetX();
                if ( min > max ) { tmp = min; min = max; max = tmp; }

		min = MINSLIDER + (MAXSLIDER - MINSLIDER) * ( min - xlow ) / ( xup - xlow );
		max = MINSLIDER + (MAXSLIDER - MINSLIDER) * ( max - xlow ) / ( xup - xlow );

		ok = kTRUE;
	    }
	    else { 
		istrstream localbuffer( fEntryXPads->GetText() ); localbuffer >> min >> max ;
		if ( !localbuffer.fail()  ) {

		    cout << " from input " << fEntryXPads->GetText() << min << " " << max << endl;
		    if ( min > max ) { tmp = min; min = max; max = tmp; }

		    xlow = gPad->GetUxmin(); xup  = gPad->GetUxmax();

		    min = MINSLIDER + (MAXSLIDER - MINSLIDER) * ( min - xlow ) / ( xup - xlow );
		    max = MINSLIDER + (MAXSLIDER - MINSLIDER) * ( max - xlow ) / ( xup - xlow );

		    ok = kTRUE;		    
		}
		else localbuffer.clear();
	    } 
	    if ( ok ) { ZoomX(MINSLIDER,MAXSLIDER); fHslider->SetPosition(min,max); fHslider->Emit("Released()"); }
	    else {
		cout << " - Info - Select two values with the cursor (or Input text) to expand " << endl;
	    }
	    break;

	case DISPLAY_EXPANDY :	 
	    // Expand with cursor by using the last two clicks belonging to the same pad 
	    // It first calculates the new values for fVSlider than Emit the Released signal.
	    // Do nothing in case of problems. 
	    ok = kFALSE; cr = GetCurrentCursor(); ypos = cr->GetArray();
	    if ( ypos->GetEntries() > 1 ) {

		Int_t last = ypos->GetLast(); 
		low = (MouseClick *)ypos->At(last-1); up = (MouseClick *)ypos->At(last);

		ylow = low->GetPad()->GetUymin(); yup  = up->GetPad()->GetUymax();
		min = low->GetY(); max = up->GetY();
                if ( min > max ) { tmp = min; min = max; max = tmp; }

		min = MINSLIDER + (MAXSLIDER - MINSLIDER) * ( min - ylow ) / ( yup - ylow );
		max = MINSLIDER + (MAXSLIDER - MINSLIDER) * ( max - ylow ) / ( yup - ylow );

		ok = kTRUE;
	    }
	    else { 
		istrstream localbuffer(fEntryYPads->GetText()); localbuffer >> min >> max ;
		if ( !localbuffer.fail()  ) {

		    if ( min > max ) { tmp = min; min = max; max = tmp; }

		    ylow = gPad->GetUymin(); yup  = gPad->GetUymax();

		    min = MINSLIDER + (MAXSLIDER - MINSLIDER) * ( min - ylow ) / ( yup - ylow );
		    max = MINSLIDER + (MAXSLIDER - MINSLIDER) * ( max - ylow ) / ( yup - ylow );

		    ok = kTRUE;		    
		}
	    } 
	    //	    cout << " from input after " << min << " " << max << endl;
	    if ( ok ) { ZoomY(MINSLIDER,MAXSLIDER); fVslider->SetPosition(min,max); fVslider->Emit("Released()"); }
	    else {
		cout << " - Info - Select two values with the cursor (or Input text) to expand " << endl;
	    }   
	    break;

        default :
            printf(" - ! - SORRY ! NOT YET IMPLEMENTED \n ");
	    break;
    }

    GetCurrentCanvas()->Update();
    pads->Clear("nodelete"); delete pads;
}

void MultiCanvas::DoCursor(Int_t id) 
{
    // handle cursor related actions

    if ( id == -1 ) {
	TGButton *b; 
	b = (TGButton *)gTQSender; id = b->WidgetId();
    }

    switch ( id ) {
	case CURSOR_SHOW :
	    if ( fShowCursor->GetBackground() != fRed ) { // red means cursor is shown
		fShowCursor->ChangeBackground(fRed); 
//		((AnalysisCursor *)(fCursor[fDisplays->GetCurrent()]))->Show(); 
		gClient->NeedRedraw(fShowCursor); 
	    }
	    else {
		fShowCursor->ChangeBackground( fBackground ); 
//		((AnalysisCursor *)(fCursor[fDisplays->GetCurrent()]))->Hide(); 
		gClient->NeedRedraw(fShowCursor); 
		GetCurrentCanvas()->Modified(); GetCurrentCanvas()->Update();
	    }
	    break;
        case CURSOR_COLLECT :
	    if ( fCollectCursor->GetBackground() != fRed ) { // red means cursor is shown
		fCollectCursor->ChangeBackground(fRed); 
//		((AnalysisCursor *)(fCursor[fDisplays->GetCurrent()]))->Collect(); 
		gClient->NeedRedraw(fCollectCursor); 
	    }
	    else {
		fCollectCursor->ChangeBackground( fBackground ); 
//		((AnalysisCursor *)(fCursor[fDisplays->GetCurrent()]))->UnCollect(); 
		gClient->NeedRedraw(fCollectCursor); 
	    }
	    break;
        case CURSOR_INFO :
// 	    ((AnalysisCursor *)(fCursor[fDisplays->GetCurrent()]))->Info(); 
	    break;
        case CURSOR_KEY :
	    if ( fKeyCursor->GetBackground() != fRed ) { // red means cursor is shown
		fKeyCursor->ChangeBackground(fRed); 
// 		((AnalysisCursor *)(fCursor[fDisplays->GetCurrent()]))->SetCollectMode(kKeyPress); 
		gClient->NeedRedraw(fKeyCursor); 
	    }
	    else {
		fKeyCursor->ChangeBackground( fBackground ); 
// 		((AnalysisCursor *)(fCursor[fDisplays->GetCurrent()]))->SetCollectMode(kButton1Double); 
		gClient->NeedRedraw(fKeyCursor); 
	    }	    
	    break;
        case CURSOR_RESET :
// 	    ((AnalysisCursor *)(fCursor[fDisplays->GetCurrent()]))
// 		->Reset(((TGTextLBEntry *)fWhichCursor->GetSelectedEntry())->GetText()->GetString());
	    break;
        default :
            printf(" - ! - SORRY NOT YET IMPLEMENTED \n ");
	    break;
    }
}

void MultiCanvas::ZoomX(Float_t inputmin, Float_t inputmax) 
{
// zoom  along X axis between min and max (slider position). if option is gpad do it only for the current
// pad. if first find all histograms, graphs, function .. and change the axis range of which does't have 
// the "same" option. if gpad is not selected, ZoomX is applied to all pads of the current canvas. 
 
   Bool_t ismodified; TString option;
   TVirtualPad *pad, *currentpad, *keeppad; 
   TObject *obj; TH1 *hist; TObjOptLink *link;

   TList *histos = new TList(); TList *pads = new TList();

   ismodified = kFALSE; keeppad = gPad;

   Int_t binmin, binmax; Float_t min, max;
   min = inputmin; max = inputmax;

   ismodified = kFALSE;

   if ( fAction[0]->GetState() == kButtonDown ) { currentpad = gPad; }
   else { currentpad = GetCurrentCanvas(); }

   pads->Add(currentpad); AddPadsPrimitives(pads,currentpad,"TVirtualPad"); 

   if ( pads->GetSize() != 0 ) {
		
       pad = (TVirtualPad *)pads->First(); 

       do {

	   pad->cd();

	   AddPadsPrimitivesOpt(histos,pad,"TH1"); AddPadsPrimitivesOpt(histos,pad,"TGraph");  
	   AddPadsPrimitivesOpt(histos,pad,"TF1"); AddPadsPrimitivesOpt(histos,pad,"TMultiGraph"); 

	   if ( histos->GetSize() != 0 ) {  

	       link = (TObjOptLink *)histos->FirstLink(); obj = link->GetObject();	       
	       while ( obj != NULL ) {

	           if ( obj->IsA()->InheritsFrom("TH1") )    hist = (TH1 *)obj;
		   if ( obj->IsA()->InheritsFrom("TGraph") ) hist = ((TGraph *)obj)->GetHistogram();
		   if ( obj->IsA()->InheritsFrom("TF1") )    hist = ((TF1 *)obj)->GetHistogram();
		   if ( obj->IsA()->InheritsFrom("TMultiGraph") ) hist = ((TMultiGraph *)obj)->GetHistogram();

		   option = link->GetOption();

		   if ( !option.Contains("same") ) {
		     min = hist->GetXaxis()->GetXmin() + 
		         ( hist->GetXaxis()->GetXmax()- hist->GetXaxis()->GetXmin() )*(inputmin/(MAXSLIDER-MINSLIDER));
		     max = hist->GetXaxis()->GetXmin() + 
		         ( hist->GetXaxis()->GetXmax()- hist->GetXaxis()->GetXmin() )*(inputmax/(MAXSLIDER-MINSLIDER));

		     binmin = hist->GetXaxis()->FindBin(min); binmax = hist->GetXaxis()->FindBin(max) - 1; 
		     if ( binmin < 1 )  binmin = 1; 
		     if ( binmax > hist->GetXaxis()->GetNbins() ) binmax = hist->GetXaxis()->GetNbins();
		     hist->GetXaxis()->SetRange( binmin , binmax );  
		   }

		   if ( (link = (TObjOptLink *)link->Next()) == NULL ) obj = NULL;
		   else  obj = link->GetObject();

	       } //  while ( obj != NULL )

	       ismodified = kTRUE; pad->Modified();  
	   }

	   histos->Clear("nodelete");

       } while ( ( pad = (TVirtualPad *)pads->After(pad) ) != NULL  ) ;
   }

   keeppad->cd(); if ( ismodified ) GetCurrentCanvas()->Update();
   histos->Clear("nodelete"); delete histos; pads->Clear("nodelete"); delete pads;  

}


void MultiCanvas::ZoomY(Float_t inputmin, Float_t inputmax) 
{
// zoom  along Y axis between min and max (slider position). if option is gpad do it only for the current
// pad. if first find all histograms, graphs, function .. and change the axis range of which does't have 
// the "same" option. if gpad is not selected, ZoomY is applied to all pads of the current canvas. 
 
   Bool_t ismodified; TString option;
   TVirtualPad *pad, *currentpad, *keeppad; 
   TObject *obj; TH1 *hist; TObjOptLink *link;

   TList *histos = new TList(); TList *pads = new TList();

   ismodified = kFALSE; keeppad = gPad;

   Int_t binmin, binmax; Float_t min, max;
   min = inputmin; max = inputmax;

   ismodified = kFALSE;

   if ( fAction[0]->GetState() == kButtonDown ) { currentpad = gPad; }
   else { currentpad = GetCurrentCanvas(); }

   pads->Add(currentpad); AddPadsPrimitives(pads,currentpad,"TVirtualPad"); 

   if ( pads->GetSize() != 0 ) {
		
       pad = (TVirtualPad *)pads->First(); 

       do {

	   pad->cd();

	   AddPadsPrimitivesOpt(histos,pad,"TH1"); AddPadsPrimitivesOpt(histos,pad,"TGraph");  
	   AddPadsPrimitivesOpt(histos,pad,"TF1"); AddPadsPrimitivesOpt(histos,pad,"TMultiGraph"); 

	   if ( histos->GetSize() != 0 ) {  

	       link = (TObjOptLink *)histos->FirstLink(); obj = link->GetObject();	       
	       while ( obj != NULL ) {

	           if ( obj->IsA()->InheritsFrom("TH1") )    hist = (TH1 *)obj;
		   if ( obj->IsA()->InheritsFrom("TGraph") ) hist = ((TGraph *)obj)->GetHistogram();
		   if ( obj->IsA()->InheritsFrom("TF1") )    hist = ((TF1 *)obj)->GetHistogram();
		   if ( obj->IsA()->InheritsFrom("TMultiGraph") ) hist = ((TMultiGraph *)obj)->GetHistogram();

		   option = link->GetOption();

		   if ( !option.Contains("same") ) {

		       if ( hist->GetDimension() == 2 ) {

			   min = hist->GetYaxis()->GetXmin() + 
			       ( hist->GetYaxis()->GetXmax()- hist->GetYaxis()->GetXmin() )*(inputmin/(MAXSLIDER-MINSLIDER));
			   max = hist->GetYaxis()->GetXmin() + 
			       ( hist->GetYaxis()->GetXmax()- hist->GetYaxis()->GetXmin() )*(inputmax/(MAXSLIDER-MINSLIDER));

			   binmin = hist->GetYaxis()->FindBin(min); binmax = hist->GetYaxis()->FindBin(max) - 1; 
			   if ( binmin < 1 )  binmin = 1; 
			   if ( binmax > hist->GetYaxis()->GetNbins() ) binmax = hist->GetYaxis()->GetNbins();
			   hist->GetYaxis()->SetRange( binmin , binmax );  
		       }
		       else {
			   
			   hist->SetMinimum(); hist->SetMaximum(); 
			   Double_t realmin =  hist->GetMinimum(); Double_t realmax = hist->GetMaximum();

			   min = realmin  + (realmax-realmin)*(inputmin/(MAXSLIDER-MINSLIDER));
			   max = 1.1 * ( realmin + (realmax-realmin)*(inputmax/(MAXSLIDER-MINSLIDER)) );
			   if ( min <= realmin ) min = realmin;
			   if ( max >= realmax ) max = realmax;

			   hist->SetMinimum( min  ); hist->SetMaximum( max );

			   if ( inputmin <= MINSLIDER ) hist->SetMinimum();
			   if ( inputmax >= MAXSLIDER ) hist->SetMaximum();

		       }
		   }

		   if ( (link = (TObjOptLink *)link->Next()) == NULL ) obj = NULL;
		   else  obj = link->GetObject();

	       } //  while ( obj != NULL )

	       ismodified = kTRUE; pad->Modified();  
	   }

	   histos->Clear("nodelete");

       } while ( ( pad = (TVirtualPad *)pads->After(pad) ) != NULL  ) ;
   }

   keeppad->cd(); if ( ismodified ) GetCurrentCanvas()->Update();
   histos->Clear("nodelete"); delete histos; pads->Clear("nodelete"); delete pads;
}


void MultiCanvas::DoScaling() 
{
   // Handle message generated by the canvas scrollbars.

   Float_t min,max;
   TGDoubleSlider *whichslider = (TGDoubleSlider *) gTQSender; Int_t id = whichslider->WidgetId();

   switch ( id ) {
	case SLIDERHID :

	    min = fHslider->GetMinPosition(); max = fHslider->GetMaxPosition();
	    ZoomX(min,max);

	    break;
	    
	case SLIDERVID :

	    min = fVslider->GetMinPosition(); max = fVslider->GetMaxPosition();
	    ZoomY(min,max);

	    break;

        default :
            printf(" Default \n ");
	    break;
   }

}

void MultiCanvas::DoButtonScaling(Int_t id) 
{
    // handle unzoom on X and Y, move up, down, left, right.
    if ( id == -1 ) {
	TGButton *b; 
	b = (TGButton *)gTQSender; id = b->WidgetId();
    }
    Float_t min, max, step = 0.1;

    switch ( id ) {
      case SLIDER_H_UNZOOM :
	fHslider->SetPosition(MINSLIDER,MAXSLIDER); ZoomX(MINSLIDER,MAXSLIDER);
	break;
	  
      case SLIDER_LEFT :
	fHslider->GetPosition(min,max);
	min = TMath::Max(MINSLIDER,min-step*(max-min));
	max = TMath::Min(MAXSLIDER,max-step*(max-min));
	if ( min <= MINSLIDER ) min = MINSLIDER;
	if ( max >= MAXSLIDER ) max = MAXSLIDER;
	fHslider->SetPosition(min,max); ZoomX(min,max);
	break;

      case SLIDER_RIGHT :
	fHslider->GetPosition(min,max);
	min = TMath::Max(MINSLIDER,min+step*(max-min));
	max = TMath::Min(MAXSLIDER,max+step*(max-min));
	if ( min <= MINSLIDER ) min = MINSLIDER;
	if ( max >= MAXSLIDER ) max = MAXSLIDER;
	fHslider->SetPosition(min,max); ZoomX(min,max);
	break;

      case SLIDER_V_UNZOOM :
	fVslider->SetPosition(MINSLIDER,MAXSLIDER); ZoomY(MINSLIDER,MAXSLIDER);
	break;

      case SLIDER_UP :
	fVslider->GetPosition(min,max);
	min = TMath::Max(MINSLIDER,min+step*(max-min));
	max = TMath::Min(MAXSLIDER,max+step*(max-min));
	if ( min <= MINSLIDER ) min = MINSLIDER;
	if ( max >= MAXSLIDER ) max = MAXSLIDER;
	fVslider->SetPosition(min,max); ZoomY(min,max);
	break;

      case SLIDER_DOWN :
	fVslider->GetPosition(min,max);
	min = TMath::Max(MINSLIDER,min-step*(max-min));
	max = TMath::Min(MAXSLIDER,max-step*(max-min));
	if ( min <= MINSLIDER ) min = MINSLIDER;
	if ( max >= MAXSLIDER ) max = MAXSLIDER;
	fVslider->SetPosition(min,max); ZoomY(min,max);
	break;

      default :
	printf(" Default \n ");
	break;
    }

}

void MultiCanvas::HandleMenu(Int_t id) 
{
// process events generated by the buttons in the graphical frame

  TCanvas *canv;

  switch(id) {

     case FILE_OPEN_TFILE:
        {
	  static TString dir("."); TGFileInfo finfo;
	  finfo.fFileTypes = gOpenRootType; finfo.fIniDir = StrDup(dir);
	  new TGFileDialog(gClient->GetRoot(), fMain, kFDOpen, &finfo);
	  if (!finfo.fFilename) return; // cancel has been pushed down
	  dir = finfo.fIniDir; new TFile(finfo.fFilename,"UPDATE");
        }
        break;

     case FILE_OPEN_GPSI:
        {
	  static TString dir("."); TGFileInfo finfo;
	  finfo.fFileTypes = gOpenGpsiType; finfo.fIniDir = StrDup(dir);
	  new TGFileDialog(gClient->GetRoot(), fMain, kFDOpen, &finfo);
	  if (!finfo.fFilename) return; // cancel has been pushed down
	  dir = finfo.fIniDir;
	  // now open a root file name GPSI.root and move all spectra of
	  // the gpsi file to the root file in the directory that has
	  // the name of the gpsi file
	  // new TFile(finfo.fFilename,"UPDATE");
	  TDirectory *old = gDirectory, *sub;
	  GpsiFile *gpsi = new GpsiFile(); TFile *froot; 

	  if ( (froot = (TFile *)gROOT->GetListOfFiles()->FindObject("GPSI.root")) == NULL )
	      froot = new TFile("GPSI.root","update"); 

	  if ( gpsi->Open(finfo.fFilename,BaseStream::kIn) && froot->IsOpen() ) { 
	      TString st = dir; st.ReplaceAll("/",":");
	      if ( !froot->cd( gSystem->BaseName(finfo.fFilename) ) ){ 
		  sub = froot->mkdir(gSystem->BaseName(finfo.fFilename),st.Data()); sub->cd();
	      } 
	      TObjArray spectra; gpsi->Add(&spectra); 
	      spectra.Write(); spectra.Compress();
	  }
	  else { delete froot; }
	  delete gpsi; gDirectory = old; 
	      //	  TObjArray spectra;
	  //	  cout << " Directory " << finfo.fIniDir << endl;
	  //	  cout << " Gpsi name " << finfo.fFilename << endl;
        }
        break;

     case FILE_ROOT_BROWSER:
	  ShowBrowser();
	  break;

     case FILE_ROOT_CANVAS:
	  ExtraCanvas();
	  break;

     case FILE_AS_CLOSE:
	  cout << " Analysis Studio : Quit ... bye !! " << endl;
	  CloseWindow();
	  break;

     case FILE_ROOT_CLOSE:
	  cout << " Analysis Studio :  Quit ROOT ... bye !! " << endl;
	  gApplication->Terminate(0);
	  break;

     case VIEW_NEW:
	  AddCanvas(); 
	  break;

     case VIEW_DELETE:
	  RemoveCanvas();
	  break;

     case VIEW_SAVEAS:
	  canv = ((TRootEmbeddedCanvas *)fEmbeddedCanvas.At(fDisplays->GetCurrent()))->GetCanvas();
	  {
              static TString dir(".");
              TGFileInfo fi;
              fi.fFileTypes = gSaveAsTypes; fi.fIniDir    = StrDup(dir);
              new TGFileDialog(gClient->GetRoot(), fMain, kFDSave,&fi);
              if (!fi.fFilename) return;
              dir = fi.fIniDir;
              if ( strstr(fi.fFilename, ".root") ||
                   strstr(fi.fFilename, ".ps")   ||
                   strstr(fi.fFilename, ".eps")  ||
                   strstr(fi.fFilename, ".gif") )
                       canv->SaveAs(fi.fFilename);
                   else if (strstr(fi.fFilename, ".C"))
                       canv->SaveSource(fi.fFilename);
                   else
		       cout << " - ! - in HandleMenu, file cannot be save with this extension " << fi.fFilename << endl;
	    }
	    break;

      case VIEW_REFRESH:
		GetCurrentCanvas()->Paint(); GetCurrentCanvas()->Update();
	    break;

      case GLOBAL_TITLE:
	    if ( gStyle->GetOptTitle() ) {
		cout << " Analysis Studio : Default is now no title " << endl;
		gStyle->SetOptTitle(0); fGlobalMenu->UnCheckEntry(GLOBAL_TITLE);
	    }
	    else {
		cout << " Analysis Studio :  Default is now title " << endl;
		gStyle->SetOptTitle(1); fGlobalMenu->CheckEntry(GLOBAL_TITLE);
	    }
	    break;

       case GLOBAL_STAT:
	    if ( gStyle->GetOptStat() ) {
		cout << " Analysis Studio :  Default is now no stat " << endl;
		gStyle->SetOptStat(0); fGlobalMenu->UnCheckEntry(GLOBAL_STAT);
	    }
	    else {
		cout << " Analysis Studio :  Default is now stat " << endl;
		gStyle->SetOptStat(1); fGlobalMenu->CheckEntry(GLOBAL_STAT);
	    }
	    break;

	case GLOBAL_STYLE_DEFAULT:
	    fGlobalMenu->UnCheckEntry(GLOBAL_STYLE_DEFAULT);
	    fGlobalMenu->UnCheckEntry(GLOBAL_STYLE_PUB);
	    fGlobalMenu->UnCheckEntry(GLOBAL_STYLE_BOLD);
	    fGlobalMenu->UnCheckEntry(GLOBAL_STYLE_GAMMA);
	    fGlobalMenu->UnCheckEntry(GLOBAL_STYLE_USER);

	    fGlobalMenu->CheckEntry(GLOBAL_STYLE_DEFAULT);
	    Style("Default");
	    break;

	case GLOBAL_STYLE_PUB:
	    fGlobalMenu->UnCheckEntry(GLOBAL_STYLE_DEFAULT);
	    fGlobalMenu->UnCheckEntry(GLOBAL_STYLE_PUB);
	    fGlobalMenu->UnCheckEntry(GLOBAL_STYLE_BOLD);
	    fGlobalMenu->UnCheckEntry(GLOBAL_STYLE_GAMMA);
	    fGlobalMenu->UnCheckEntry(GLOBAL_STYLE_USER);

	    if ( Style("Pub") ) fGlobalMenu->CheckEntry(GLOBAL_STYLE_PUB);
	    else fGlobalMenu->CheckEntry(GLOBAL_STYLE_DEFAULT);
	    break;

	case GLOBAL_STYLE_BOLD:
	    fGlobalMenu->UnCheckEntry(GLOBAL_STYLE_DEFAULT);
	    fGlobalMenu->UnCheckEntry(GLOBAL_STYLE_PUB);
	    fGlobalMenu->UnCheckEntry(GLOBAL_STYLE_BOLD);
	    fGlobalMenu->UnCheckEntry(GLOBAL_STYLE_GAMMA);
	    fGlobalMenu->UnCheckEntry(GLOBAL_STYLE_USER);

	    if ( Style("Bold") ) fGlobalMenu->CheckEntry(GLOBAL_STYLE_BOLD);
	    else fGlobalMenu->CheckEntry(GLOBAL_STYLE_DEFAULT);
	    break;

	case GLOBAL_STYLE_GAMMA:
	    fGlobalMenu->UnCheckEntry(GLOBAL_STYLE_DEFAULT);
	    fGlobalMenu->UnCheckEntry(GLOBAL_STYLE_PUB);
	    fGlobalMenu->UnCheckEntry(GLOBAL_STYLE_BOLD);
	    fGlobalMenu->UnCheckEntry(GLOBAL_STYLE_GAMMA);
	    fGlobalMenu->UnCheckEntry(GLOBAL_STYLE_USER);

	    if ( Style("Gamma") ) fGlobalMenu->CheckEntry(GLOBAL_STYLE_GAMMA);
	    else fGlobalMenu->CheckEntry(GLOBAL_STYLE_DEFAULT);
	    break;

	case GLOBAL_STYLE_USER:
	    fGlobalMenu->UnCheckEntry(GLOBAL_STYLE_DEFAULT);
	    fGlobalMenu->UnCheckEntry(GLOBAL_STYLE_PUB);
	    fGlobalMenu->UnCheckEntry(GLOBAL_STYLE_BOLD);
	    fGlobalMenu->UnCheckEntry(GLOBAL_STYLE_GAMMA);
	    fGlobalMenu->UnCheckEntry(GLOBAL_STYLE_USER);

	    if ( Style("User") ) fGlobalMenu->CheckEntry(GLOBAL_STYLE_USER);
	    else fGlobalMenu->CheckEntry(GLOBAL_STYLE_DEFAULT);
	    break;
	    
        default:
	    break;

  } // switch(id)

}

void MultiCanvas::HandleEmbeddedCanvas(Int_t event, Int_t x, Int_t y, TObject *sel) 
{
    // used to display informations in the status bar.

    static TVirtualPad *lastpad = NULL; // last "visited" pad - keep memory of the last pad the mouse has passed through
                                        // different that current pad 

    Char_t text[150];
    if ( sel->InheritsFrom("TVirtualPad") ) lastpad = (TVirtualPad *)sel;

    TVirtualPad *pad = gROOT->GetSelectedPad(); if ( pad == NULL || lastpad == NULL ) return;

    sprintf(text," Current Canvas : %s ",pad->GetCanvas()->GetName() ); fStatusBar->SetText(text,0);
    sprintf(text," Current Pad : %s ", pad->GetName() ); fStatusBar->SetText(text,1);
    sprintf(text," Selected : name %s - type %s",sel->GetName(),sel->ClassName() ); fStatusBar->SetText(text,2);

    if ( sel->InheritsFrom("TH1") ) {

       if ( sel->InheritsFrom("TH2") ) {
         TH2 *hist2d = (TH2 *)sel; 
	 //         Int_t binx = hist2d->GetXaxis()->FindBin(pad->PadtoX(pad->AbsPixeltoX(x)));
	 //         Int_t biny = hist2d->GetYaxis()->FindBin(pad->PadtoY(pad->AbsPixeltoY(y)));
	 //         sprintf(text," X : %.3f - bin %d ",pad->AbsPixeltoX(x), binx ); fStatusBar->SetText(text,3);
	 //         sprintf(text," Y : %.3f - bin %d ",pad->AbsPixeltoY(y), biny ); fStatusBar->SetText(text,4);
	 //         sprintf(text," Z : %.f ",hist2d->GetCellContent(binx,biny)); fStatusBar->SetText(text,5);

         Int_t binx = hist2d->GetXaxis()->FindBin(lastpad->PadtoX(lastpad->AbsPixeltoX(x)));
         Int_t biny = hist2d->GetYaxis()->FindBin(lastpad->PadtoY(lastpad->AbsPixeltoY(y)));
	 if ( pad == lastpad ) {
           sprintf(text," X : %.3f - bin %d ",lastpad->AbsPixeltoX(x), binx ); fStatusBar->SetText(text,3);
           sprintf(text," Y : %.3f - bin %d ",lastpad->AbsPixeltoY(y), biny ); fStatusBar->SetText(text,4);
           sprintf(text," Z : %.f ",hist2d->GetCellContent(binx,biny)); fStatusBar->SetText(text,5);
	 }
	 else {
           sprintf(text," ! X : %.3f - bin %d !",lastpad->AbsPixeltoX(x), binx ); fStatusBar->SetText(text,3);
           sprintf(text," ! Y : %.3f - bin %d !",lastpad->AbsPixeltoY(y), biny ); fStatusBar->SetText(text,4);
           sprintf(text," ! Z : %.f !",hist2d->GetCellContent(binx,biny)); fStatusBar->SetText(text,5);
	 }
       }
       else {
         TH1 *hist1d = (TH1 *)sel; 
	 //         Int_t bin = hist1d->GetXaxis()->FindBin(pad->PadtoX(pad->AbsPixeltoX(x)));
	 //         sprintf(text," X : %.3f - bin %d ",pad->AbsPixeltoX(x), bin ); fStatusBar->SetText(text,3);
	 //        sprintf(text," Y : %.3f - binc %d ",pad->AbsPixeltoY(y), hist1d->GetBinContent(bin) ); fStatusBar->SetText(text,4);
	 //        sprintf(text," Z : 0 "); fStatusBar->SetText(text,5);

         Int_t bin = hist1d->GetXaxis()->FindBin(lastpad->PadtoX(lastpad->AbsPixeltoX(x)));
	 if ( pad == lastpad ) {
           sprintf(text," X : %.3f - bin %d ",lastpad->AbsPixeltoX(x), bin ); fStatusBar->SetText(text,3);
           sprintf(text," Y : %.3f - binc %d ",lastpad->AbsPixeltoY(y), Int_t(hist1d->GetBinContent(bin)) ); fStatusBar->SetText(text,4);
           sprintf(text," Z : 0 "); fStatusBar->SetText(text,5);
	 }
	 else {
           sprintf(text," ! X : %.3f - bin %d !",lastpad->AbsPixeltoX(x), bin ); fStatusBar->SetText(text,3);
           sprintf(text," ! Y : %.3f ! binc %d !",lastpad->AbsPixeltoY(y), Int_t(hist1d->GetBinContent(bin)) ); fStatusBar->SetText(text,4);
           sprintf(text," ! Z : 0 !"); fStatusBar->SetText(text,5);
	 }
       }
    }
    else {
       sprintf(text," X : %.3f ",pad->AbsPixeltoX(x) ); fStatusBar->SetText(text,3);
       sprintf(text," Y : %.3f ",pad->AbsPixeltoY(y) ); fStatusBar->SetText(text,4);
       sprintf(text," Event : %d ",event ); fStatusBar->SetText(text,5);
    }
}



#ifdef STANDALONE
TROOT root("GUI", "GUI test environement");


int main(int argc, char **argv)
{
   TApplication theApp("App", &argc, argv);

   if (gROOT->IsBatch()) {
      fprintf(stderr, "%s: cannot run in batch mode\n", argv[0]);
      return 1;
   }

   MultiCanvas();

   theApp.Run();

   return 0;
}
#endif












