#ifndef GW_MULTICANVAS_H
#define GW_MULTICANVAS_H

#include <Rtypes.h>
#include <TGMenu.h>
#include <TGLabel.h>
#include <TGSplitter.h>
#include <TBrowser.h>

#include <TGFrame.h>
#include <TGTab.h>
#include <TRootEmbeddedCanvas.h>
#include <TGButton.h>
#include <TGListBox.h>
#include <TGComboBox.h>
#include <TGTextEntry.h>
#include <TGTextBuffer.h>
#include <TObjArray.h>
#include <TStyle.h>
#include <TROOT.h>
#include <TG3DLine.h>
#include <TGSlider.h>
#include <TGDoubleSlider.h>
#include <TGTableLayout.h>

#include <TCanvas.h>
#include <RQ_OBJECT.h>
#include <TGStatusBar.h>
#include <TGScrollBar.h>


#include <iostream>
using namespace std;


#define MAXDISPLAY 20 

 
#define MINSLIDER  0
#define MAXSLIDER  100000 

// AS for MultiCanvas.

enum MultiCanvasCommandsId {
        FILE_OPEN_TFILE = 100,
        FILE_OPEN_GPSI,
	FILE_ROOT_BROWSER,
	FILE_ROOT_CANVAS,
	FILE_AS_CLOSE,
	FILE_ROOT_CLOSE,

	VIEW_NEW,
	VIEW_DELETE,
	VIEW_SAVEAS,
	VIEW_EDITOR,
	VIEW_REFRESH,

	GLOBAL_TITLE,
	GLOBAL_STAT,
	GLOBAL_STYLE_DEFAULT,
	GLOBAL_STYLE_PUB,
	GLOBAL_STYLE_BOLD,
	GLOBAL_STYLE_GAMMA,
	GLOBAL_STYLE_USER,

	EMBEDDED_NEW,
	EMBEDDED_RESET,
	EMBEDDED_DELETE,
	EMBEDDED_NEWPAD,

	SLIDERVID,
	SLIDERHID,

	CURSOR_SHOW,
	CURSOR_COLLECT,
	CURSOR_INFO,
	CURSOR_KEY,
	CURSOR_RESET,

	MODE_GPAD,
	MODE_ALL,
	MODE_CANVAS,

	DISPLAY_CLEAR,
	DISPLAY_DELETE,
	DISPLAY_LOGX,
	DISPLAY_LOGY,
	DISPLAY_DVSYM,
	DISPLAY_DVASYM,
	DISPLAY_EXPANDX,
	DISPLAY_EXPANDY,
	DISPLAY_SETRANGE,
	DISPLAY_DRAWAXIS,
	DISPLAY_CURSORFIT,
	
	SLIDER_H_UNZOOM,
	SLIDER_LEFT,
	SLIDER_RIGHT,
	SLIDER_V_UNZOOM,
	SLIDER_UP,
	SLIDER_DOWN

};


class MultiCanvas : public TGMainFrame
{


private:

 // level 0
 TGMainFrame   *fMain;        // main frame

  // level 1
  TGMenuBar     *fMenuBar;        // menu bar for global actions
  TGLayoutHints *fMenuBarLayout;  // layout for menu bar 
   // level 2
   TGPopupMenu   *fRootMenu;      // Root related facilities (browser, canvas ..)
   TGLayoutHints *fRootMenuLayout;     
   TGPopupMenu   *fViewMenu;      // Action on ROOTCANVAS
   TGLayoutHints *fViewMenuLayout;
   TGPopupMenu   *fGlobalMenu;    // Global Actions
   TGLayoutHints *fGlobalMenuLayout;
   TGPopupMenu   *fHelpMenu;      // help menu 
   TGLayoutHints *fHelpMenuLayout;

  // level 1
  TGCompositeFrame *fDispAction;  // panel of commands
  TGLayoutHints    *fDispActionLayout;
   // level 2
   TGGroupFrame     *fDispCursor;       // commands specific to cursor
   TGLayoutHints    *fDispCursorLayout;
    // level 3
    TGTextButton  *fShowCursor;        // draw cursor if selected 
    TGLayoutHints *fShowCursorLayout;
    TGTextButton  *fCollectCursor;     // keep memory of 'clicks' if selected 
    TGLayoutHints *fCollectCursorLayout;
    TGTextButton  *fInfoCursor;        // cursor status 
    TGLayoutHints *fInfoCursorLayout;
    TGTextButton  *fKeyCursor;         // keep memory when a key is hitten, otherwise with mouse 
    TGLayoutHints *fKeyCursorLayout;
    TGTextButton  *fResetCursor;       // reset cursor memory 
    TGLayoutHints *fResetCursorLayout;
    TGComboBox    *fWhichCursor;       // select a mode of resetting
    TGLayoutHints *fWhichCursorLayout;
   // level 2
   TGGroupFrame  *fDispPads;         // commands specific to pads/canvases
   TGLayoutHints *fDispPadsLayout;
     TGLabel       *fLabelAction,       *fUserX,       *fUserY; 
     TGLayoutHints *fLabelActionLayout, *fUserXLayout, *fUserYLayout;          
     TGCheckButton   *fAction[3];      // to select what kind of action: current pad, all pad, canvas 
     TGLayoutHints   *fActionLayout[3];
     TGTextButton  *fClearPads;        // clear pads - selection is done via fAction
     TGLayoutHints *fClearPadsLayout;
     TGTextButton  *fDeletePads;       // delete pads - selection is done via fAction 
     TGLayoutHints *fDeletePadsLayout;
     TGTextButton  *fLogxPads;         // set logx in pads - selection is done via fAction 
     TGLayoutHints *fLogxPadsLayout;
     TGTextButton  *fLogyPads;         // set logy in pads - selection is done via fAction
     TGLayoutHints *fLogyPadsLayout;
     TGTextEntry   *fEntryXPads;       // to entry values for X .. see Expand and divide
     TGLayoutHints *fEntryXPadsLayout;
     TGTextEntry   *fEntryYPads;       // to entry values for Y .. see Expand and divide
     TGLayoutHints *fEntryYPadsLayout;
     TGTextButton  *fDvSym;            // divide pad - selection is done via fAction, number in EntryX and EntryY
     TGLayoutHints *fDvSymLayout;
     TGTextButton  *fDvAsym;           // same as before .. however a margin is kept at the left and  at the bottom
     TGLayoutHints *fDvAsymLayout;
     TGTextButton  *fExpandX;          // expand using the last two values stored in cursor
     TGLayoutHints *fExpandXLayout;
     TGTextButton  *fExpandY;          // expand using the last two values stored in cursor
     TGLayoutHints *fExpandYLayout;
     TGTextButton  *fDrawAxis;         // draw axis of pads in the mother pad.
     TGLayoutHints *fDrawAxisLayout;
     TGTextButton  *fSetRange;         // set range in pad(s). use input to determine the ranges
     TGLayoutHints *fSetRangeLayout;
     TGTextButton  *fCursorFit;        // fit using values stored in cursor. 
     TGLayoutHints *fCursorFitLayout;

  // level 1
  TGCompositeFrame *fWorking;	   // to display histograms and sliders 
  TGLayoutHints    *fWorkingLayout;
   // level 2
   TGCompositeFrame *fDispMain;	     // vertical slider and TGTab to display histograms 
   TGLayoutHints    *fDispMainLayout;
    // level 3
    TGCompositeFrame *fFrameVSlider;   // frame for vertical slider 
    TGLayoutHints    *fFrameVSliderLayout;
     // level 4
     TGDoubleVSlider  *fVslider;
     TGLayoutHints    *fVsliderLayout;
     const TGPicture *fPicArrowUp, *fPicArrowDown, *fPicYUnzoom;
     TGPictureButton  *fArrowUp,       *fArrowDown,       *fYUnzoom;
     TGLayoutHints    *fArrowUpLayout, *fArrowDownLayout, *fYUnzoomLayout;
    // level 3
    TGTab         *fDisplays; 		    // a set of windows to display spectra
    TGLayoutHints *fDisplaysLayout;
     TObjArray      fEmbeddedCanvas;        // Embbeded canvas of each TabElement of fDisplays
     TGLayoutHints *fEmbeddedCanvasLayout;
   // level 2
   TGCompositeFrame *fFrameHSlider;   // frame for horizontal slider
   TGLayoutHints    *fFrameHSliderLayout; 
    // level 3
    TGDoubleHSlider *fHslider;
    TGLayoutHints   *fHsliderLayout;
    const TGPicture *fPicArrowLeft, *fPicArrowRight, *fPicXUnzoom;
    TGPictureButton *fArrowLeft,       *fArrowRight,       *fXUnzoom;
    TGLayoutHints   *fArrowLeftLayout, *fArrowRightLayout, *fXUnzoomLayout;
  // level 1
  TGCompositeFrame *fInfoBar ;    // to display usefull informations
  TGLayoutHints    *fInfoBarLayout;
   TGStatusBar   *fStatusBar;         
   TGLayoutHints *fStatusBarLayout;

  TGHorizontal3DLine *fSeparator1;     // to separate global menu from working
  TGLayoutHints      *fSeparator1Layout; 
  TGHorizontal3DLine *fSeparator2;     // to separate working from statusbar
  TGLayoutHints      *fSeparator2Layout; 

  ULong_t fRed, fBackground ;

  void ZoomX(Float_t, Float_t); void ZoomY(Float_t, Float_t);


public:
  MultiCanvas(const TGWindow *win, UInt_t w = 900, UInt_t h = 300);
	//	MultiCanvas(const TGWindow *win, UInt_t w = 900, UInt_t h = 500);
	//MultiCanvas(const TGWindow *, UInt_t, UInt_t);

  ~MultiCanvas();
        
  void CdCanvas(Int_t);
  void AddCanvas();
  void RemoveCanvas();
  TCanvas *GetCurrentCanvas();

  void ShowBrowser();
  Bool_t Style(const Char_t *);

  TCanvas *ExtraCanvas();

  // Slots
  void HandleMenu(Int_t);
  void HandleEmbeddedCanvas(Int_t,Int_t,Int_t,TObject *);
  void CloseWindow();
  void DoButtonScaling(Int_t id = -1);
  void DoScaling();
  void DoCursor(Int_t id = -1);
  void DoDisplay(Int_t id = -1);

  ClassDef(MultiCanvas,0) // a graphical utility to analyse gamma spectra
};
// MultiCanvas inline members ///////////////////////////////////////////////
inline TCanvas *MultiCanvas::GetCurrentCanvas() 
{ return ((TRootEmbeddedCanvas *)fEmbeddedCanvas.At(fDisplays->GetCurrent()))->GetCanvas(); }
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
#endif






