
#ifndef __CINT__

#include <stdio.h> 
#include <unistd.h> 
#include <stdlib.h>

#include <TStopwatch.h>
#include <TFile.h>
#include <TTree.h>
#include <Riostream.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#endif

#include <Buffer.h>

using namespace Gws; // Gammaware System needed

//const Int_t size = 16*KBYTE; // size of each array
const Int_t size = 32*KBYTE; // size of each array

void Endian()
{
	if ( Memory::IsBytes(Memory::kLittle) )  { cout << " endian type for this machine is little " << endl; }
	else  { cout << " endian type for this machine is big " << endl; }
}

void Swap()
{
	// buffers with the same endian type that your machine
	Buffer  bufferS(sizeof(Short_t)*size), bufferI(sizeof(Int_t)*size), bufferF(sizeof(Float_t)*size),bufferD(sizeof(Double_t)*size); 
	
	// buffers with a different endian type that your machine		
	EndianBuffer bufferS_swap(sizeof(Short_t)*size), bufferI_swap(sizeof(Int_t)*size), bufferF_swap(sizeof(Float_t)*size), bufferD_swap(sizeof(Double_t)*size); 
	
	TStopwatch watch;
	for (Int_t ii = 0; ii < size; ii++) {
		Short_t s = 1; Float_t f = 1; Int_t i = 1; Double_t d = 1;
		
		// write them without swapping
		bufferS << s; 
		bufferI << i; 
		bufferF << f; 
		bufferD << d; 	
	}
	watch.Stop(); // watch.Print("u");	
	printf("  --> Writing rate without swapping %f MB/s (%f/%f) \n",
		Float_t(bufferS.Size()+bufferI.Size()+bufferF.Size()+bufferD.Size())/(MBYTE*watch.RealTime()),	
		Float_t(bufferS.Size()+bufferI.Size()+bufferF.Size()+bufferD.Size())/MBYTE,watch.RealTime());
		
	watch.Start(kTRUE);
	for (Int_t ii = 0; ii < size; ii++) {
		Short_t s = 1; Float_t f = 1; Int_t i = 1; Double_t d = 1;
			
		// write them with swapping
		bufferS_swap << s; 
		bufferI_swap << i; 
		bufferF_swap << f; 
		bufferD_swap << d; 	
	}
	watch.Stop();// watch.Print("u");
	
	printf("  --> Writing rate with swapping %f MB/s (%f/%f) \n",
		Float_t(bufferS.Size()+bufferI.Size()+bufferF.Size()+bufferD.Size())/(MBYTE*watch.RealTime()),	
		Float_t(bufferS.Size()+bufferI.Size()+bufferF.Size()+bufferD.Size())/MBYTE,watch.RealTime());
		
	bufferS.SetOffset(0);bufferI.SetOffset(0);bufferF.SetOffset(); bufferD.SetOffset(0);
	watch.Start(kTRUE); 
	for (Int_t ii = 0; ii < size; ii++) {
		Short_t s; Float_t f; Int_t i; Double_t d;
		
		// read them without swapping
		bufferS >> s; 
		bufferI >> i; 
		bufferF >> f; 
		bufferD >> d; 	
	}
	watch.Stop(); // watch.Print("u");	
	printf("  --> Reading rate without swapping %f MB/s (%f/%f) \n",
		Float_t(bufferS.Size()+bufferI.Size()+bufferF.Size()+bufferD.Size())/(MBYTE*watch.RealTime()),	
		Float_t(bufferS.Size()+bufferI.Size()+bufferF.Size()+bufferD.Size())/MBYTE,watch.RealTime());
		
	bufferS_swap.SetOffset(0);bufferI_swap.SetOffset(0);bufferF_swap.SetOffset(); bufferD_swap.SetOffset(0);
	watch.Start(kTRUE);
	for (Int_t ii = 0; ii < size; ii++) {
		Short_t s; Float_t f; Int_t i; Double_t d;
		
		// read them with swapping
		bufferS_swap >> s; 
		bufferI_swap >> i; 
		bufferF_swap >> f; 
		bufferD_swap >> d; 	
	}
	watch.Stop(); // watch.Print("u");
	printf("  --> Reading rate with swapping %f MB/s (%f/%f) \n",
		Float_t(bufferS.Size()+bufferI.Size()+bufferF.Size()+bufferD.Size())/(MBYTE*watch.RealTime()),	
		Float_t(bufferS.Size()+bufferI.Size()+bufferF.Size()+bufferD.Size())/MBYTE,watch.RealTime());	
}


void WriteIO_SYS()
{
	Int_t id; Memory::EEndian inv_endian; TString noswap, swap; TStopwatch watch;

	// to know the endian type of your machine
	if ( Memory::IsBytes(Memory::kLittle) )  { noswap = "little_sys"; swap = "big_sys"; inv_endian = Memory::kBig; }
	else  { noswap = "big_sys"; swap = "little_sys"; inv_endian = Memory::kLittle; }
	
	cout << noswap << " endian type for this machine " << endl;

	// buffers with the same endian type that your machine
	Buffer  bufferS(sizeof(Short_t)*size), 
		bufferI(sizeof(Int_t)*size), 
		bufferF(sizeof(Float_t)*size), 
		bufferD(sizeof(Double_t)*size); 
	
	// fill the buffers and write them on disk
	watch.Start(kTRUE);	
	id = open(noswap.Data(),O_WRONLY | O_CREAT,S_IRWXU);
	if ( id == -1 ) {
		printf("Cannot open file %s\n",noswap.Data()); return ;
	}
	
	// write blocks and compute the needed time
	for (Int_t ii = 0; ii < size; ii++) {
		Short_t s; Float_t f; Int_t i; Double_t d;
		
		s = ii-size/2;
		if ( ii-size/2 < kMinShort ) s = kMinShort;
		if ( ii-size/2 > kMaxShort ) s = kMaxShort;
		i = ii-size/2;
		f = ii-size/2;
		d = ii-size/2;
		
		// write them without swapping
		bufferS << s; if ( bufferS.IsStatus(Buffer::kFail) ) cout << " PB in buffer " << s << " " << ii << endl;
		bufferI << i; if ( bufferI.IsStatus(Buffer::kFail) ) cout << " PB in buffer " << i << " " << ii << endl;
		bufferF << f; if ( bufferF.IsStatus(Buffer::kFail) ) cout << " PB in buffer " << f << " " << ii << endl;
		bufferD << d; if ( bufferD.IsStatus(Buffer::kFail) ) cout << " PB in buffer " << d << " " << ii << endl;
	}
	write(id,bufferS.GetBuffer(),bufferS.Size());	
	write(id,bufferI.GetBuffer(),bufferI.Size());	
	write(id,bufferF.GetBuffer(),bufferF.Size());	
	write(id,bufferD.GetBuffer(),bufferD.Size());	
	
	close(id);
	
	watch.Stop();	
	printf("  --> Writing rate without swapping %f MB/s (%f/%f) \n",
		Float_t(bufferS.Size()+bufferI.Size()+bufferF.Size()+bufferD.Size())/(MBYTE*watch.RealTime()),	
		Float_t(bufferS.Size()+bufferI.Size()+bufferF.Size()+bufferD.Size())/MBYTE,watch.RealTime());
	
	// fill the buffers and write them on disk
	id = open(swap.Data(),O_WRONLY | O_CREAT, S_IRWXU);
	if ( id == -1 ) {
		printf("Cannot open file %s\n",swap.Data()); return ;
	}
	
	// Best way to allocate buffers since it gives a buffer that is aware of byte swapping
	Buffer *bufferS_swap = Buffer::New(inv_endian,sizeof(Short_t)*size);
	Buffer *bufferI_swap = Buffer::New(inv_endian,sizeof(Int_t)*size);
	Buffer *bufferF_swap = Buffer::New(inv_endian,sizeof(Float_t)*size);
	Buffer *bufferD_swap = Buffer::New(inv_endian,sizeof(Double_t)*size);	

	// write blocks and compute the needed time
	watch.Start(kTRUE);
	for (Int_t ii = 0; ii < size; ii++) {
		Short_t s; Float_t f; Int_t i; Double_t d;
		s = ii-size/2;
		if ( ii-size/2 < kMinShort ) s = kMinShort;
		if ( ii-size/2 > kMaxShort ) s = kMaxShort;		
		i = ii-size/2;
		f = ii-size/2;
		d = ii-size/2;

		(*bufferS_swap) << s; if ( bufferS_swap->IsStatus(Buffer::kFail) ) cout << " PB in swap " << s << " " << ii << endl;
		(*bufferI_swap) << i; if ( bufferI_swap->IsStatus(Buffer::kFail) ) cout << " PB in swap " << i << " " << ii << endl;
		(*bufferF_swap) << f; if ( bufferF_swap->IsStatus(Buffer::kFail) ) cout << " PB in swap " << f << " " << ii << endl;
		(*bufferD_swap) << d; if ( bufferD_swap->IsStatus(Buffer::kFail) ) cout << " PB in swap " << d << " " << ii << endl;
		// write them with swapping
	}
	write(id,bufferS_swap->GetBuffer(),bufferS_swap->Size());	
	write(id,bufferI_swap->GetBuffer(),bufferI_swap->Size());	
	write(id,bufferF_swap->GetBuffer(),bufferF_swap->Size());	
	write(id,bufferD_swap->GetBuffer(),bufferD_swap->Size());	
	
	close(id);
	
	watch.Stop();	
	printf("  --> Writing rate with swapping %f MB/s (%f/%f) \n",
		Float_t(bufferS.Size()+bufferI.Size()+bufferF.Size()+bufferD.Size())/(MBYTE*watch.RealTime()),	
		Float_t(bufferS.Size()+bufferI.Size()+bufferF.Size()+bufferD.Size())/MBYTE,watch.RealTime());
	
}

void ReadIO_SYS()
{
	Int_t id; Memory::EEndian inv_endian; TString noswap, swap; TStopwatch watch;

	// to know the endian type of your machine
	if ( Memory::IsBytes(Memory::kLittle) )  { noswap = "little_sys"; swap = "big_sys"; inv_endian = Memory::kBig; }
	else  { noswap = "big_sys"; swap = "little_sys"; inv_endian = Memory::kLittle; }
	
	cout << noswap << " endian type for this machine " << endl;

	// 32 KBYTES buffers with the same endian type that your machine
	Buffer  bufferS(sizeof(Short_t)*size), 
		bufferI(sizeof(Int_t)*size), 
		bufferF(sizeof(Float_t)*size), 
		bufferD(sizeof(Double_t)*size); 
	
	// read the buffers from disk
	id = open(noswap.Data(),O_RDONLY);
	if ( id == -1 ) {
		printf("Cannot open file %s\n",noswap.Data()); return ;
	}
	
	// read blocks and compute the needed time
	watch.Start(kTRUE);
	
	read(id,bufferS.GetBuffer(),bufferS.Size());	
	read(id,bufferI.GetBuffer(),bufferI.Size());	
	read(id,bufferF.GetBuffer(),bufferF.Size());	
	read(id,bufferD.GetBuffer(),bufferD.Size());	
	
		
	for (Int_t ii = 0; ii < size; ii++) {
		Short_t s; Float_t f; Int_t i; Double_t d;
		bufferS >> s; 
		bufferI >> i; 
		bufferF >> f; 
		bufferD >> d; // read them without swapping
		
		Short_t stmp = ii-size/2;
		if ( ii-size/2 < kMinShort ) stmp = kMinShort;
		if ( ii-size/2 > kMaxShort ) stmp = kMaxShort;		
				
		if ( s != stmp ) cout << " ERROR [short]" << ii-size/2 << " expected " << s << " read ! " << endl;
		if ( i != ii-size/2 ) cout << " ERROR [int]" << ii-size/2 << " expected " << i << " read ! " << endl;
		if ( f != ii-size/2 ) cout << " ERROR [float]" << ii-size/2 << " expected " << f << " read ! " << endl;
		if ( d != ii-size/2 ) cout << " ERROR [double]" << ii-size/2 << " expected " << d << " read ! " << endl;
	}	
	close(id);
		
	watch.Stop();	
	printf("  --> Reading rate without swapping %f MB/s (%f/%f) \n",
		Float_t(bufferS.Size()+bufferI.Size()+bufferF.Size()+bufferD.Size())/(MBYTE*watch.RealTime()),	
		Float_t(bufferS.Size()+bufferI.Size()+bufferF.Size()+bufferD.Size())/MBYTE,watch.RealTime());
	
	// fill the buffers and write them on disk
	id = open(swap.Data(),O_RDONLY);
	if ( id == -1 ) {
		printf("Cannot open file %s\n",swap.Data()); return ;
	}
	
	// Best way to allocate buffers since it gives a buffer that is aware of byte swapping
	Buffer *bufferS_swap = Buffer::New(inv_endian,sizeof(Short_t)*size);
	Buffer *bufferI_swap = Buffer::New(inv_endian,sizeof(Int_t)*size);
	Buffer *bufferF_swap = Buffer::New(inv_endian,sizeof(Float_t)*size);
	Buffer *bufferD_swap = Buffer::New(inv_endian,sizeof(Double_t)*size);	

	// write blocks and compute the needed time
	watch.Start(kTRUE);
		
	read(id,bufferS_swap->GetBuffer(),bufferS_swap->Size());	
	read(id,bufferI_swap->GetBuffer(),bufferI_swap->Size());	
	read(id,bufferF_swap->GetBuffer(),bufferF_swap->Size());	
	read(id,bufferD_swap->GetBuffer(),bufferD_swap->Size());
	
	for (Int_t ii = 0; ii < size; ii++) {
		Short_t s; Float_t f; Int_t i; Double_t d;
		
		(*bufferS_swap) >> s; (*bufferI_swap) >> i; (*bufferF_swap) >> f; (*bufferD_swap) >> d;
		
		Short_t stmp = ii-size/2;
		if ( ii-size/2 < kMinShort ) stmp = kMinShort;
		if ( ii-size/2 > kMaxShort ) stmp = kMaxShort;			
			
		if ( s != stmp ) cout << " ERROR [short swap]" << ii-size/2 << " expected " << s << " read ! " << endl;
		if ( i != ii-size/2 ) cout << " ERROR [int swap]" << ii-size/2 << " expected " << i << " read ! " << endl;
		if ( f != ii-size/2 ) cout << " ERROR [float swap]" << ii-size/2 << " expected " << f << " read ! " << endl;
		if ( d != ii-size/2 ) cout << " ERROR [double swap]" << ii-size/2 << " expected " << d << " read ! " << endl;
		// read them with swapping
	}
	close(id);
	
	watch.Stop();	
	printf("  --> Reading rate with swapping %f MB/s (%f/%f) \n",
		Float_t(bufferS.Size()+bufferI.Size()+bufferF.Size()+bufferD.Size())/(MBYTE*watch.RealTime()),	
		Float_t(bufferS.Size()+bufferI.Size()+bufferF.Size()+bufferD.Size())/MBYTE,watch.RealTime());
	
}

void WriteIO_C()
{
	FILE *id; Memory::EEndian inv_endian; TString noswap, swap; TStopwatch watch;

	// to know the endian type of your machine
	if ( Memory::IsBytes(Memory::kLittle) )  { noswap = "little_c"; swap = "big_c"; inv_endian = Memory::kBig; }
	else  { noswap = "big_c"; swap = "little_c"; inv_endian = Memory::kLittle; }
	
	cout << noswap << " endian type for this machine " << endl;

	// 32 KBYTES buffers with the same endian type that your machine
	Buffer  bufferS(sizeof(Short_t)*size), 
		bufferI(sizeof(Int_t)*size), 
		bufferF(sizeof(Float_t)*size), 
		bufferD(sizeof(Double_t)*size); 
	
	// cout << bufferS.Size() << " " << bufferI.Size() << " " << bufferF.Size() << " " << bufferD.Size() << endl;
	// fill the buffers and write them on disk
	id = fopen(noswap.Data(),"w");
	if ( id == NULL ) {
		printf("Cannot open file %s\n",noswap.Data()); return ;
	}
	
	// write blocks and compute the needed time
	watch.Start(kTRUE);
	for (Int_t ii = 0; ii < size; ii++) {
		Short_t s; Float_t f; Int_t i; Double_t d;
		
		s = ii-size/2;
		if ( ii-size/2 < kMinShort ) s = kMinShort;
		if ( ii-size/2 > kMaxShort ) s = kMaxShort;
		i = ii-size/2;
		f = ii-size/2;
		d = ii-size/2;
		
		// write them without swapping
		bufferS << s; if ( bufferS.IsStatus(Buffer::kFail) ) cout << " PB in buffer " << s << " " << ii << endl;
		bufferI << i; if ( bufferI.IsStatus(Buffer::kFail) ) cout << " PB in buffer " << i << " " << ii << endl;
		bufferF << f; if ( bufferF.IsStatus(Buffer::kFail) ) cout << " PB in buffer " << f << " " << ii << endl;
		bufferD << d; if ( bufferD.IsStatus(Buffer::kFail) ) cout << " PB in buffer " << d << " " << ii << endl;
	}
	fwrite(bufferS.GetBuffer(),bufferS.Size(),1,id);	
	fwrite(bufferI.GetBuffer(),bufferI.Size(),1,id);	
	fwrite(bufferF.GetBuffer(),bufferF.Size(),1,id);	
	fwrite(bufferD.GetBuffer(),bufferD.Size(),1,id);	
	
	fclose(id);
	
	watch.Stop();	
	printf("  --> Writing rate without swapping %f MB/s (%f/%f) \n",
		Float_t(bufferS.Size()+bufferI.Size()+bufferF.Size()+bufferD.Size())/(MBYTE*watch.RealTime()),	
		Float_t(bufferS.Size()+bufferI.Size()+bufferF.Size()+bufferD.Size())/MBYTE,watch.RealTime());
	
	// fill the buffers and write them on disk
	id = fopen(swap.Data(),"w");
	if ( id == NULL ) {
		printf("Cannot open file %s\n",swap.Data()); return ;
	}
	
	// Best way to allocate buffers since it gives a buffer that is aware of byte swapping
	Buffer *bufferS_swap = Buffer::New(inv_endian,sizeof(Short_t)*size);
	Buffer *bufferI_swap = Buffer::New(inv_endian,sizeof(Int_t)*size);
	Buffer *bufferF_swap = Buffer::New(inv_endian,sizeof(Float_t)*size);
	Buffer *bufferD_swap = Buffer::New(inv_endian,sizeof(Double_t)*size);	

	// write blocks and compute the needed time
	watch.Start(kTRUE);
	for (Int_t ii = 0; ii < size; ii++) {
		Short_t s; Float_t f; Int_t i; Double_t d;
		s = ii-size/2;
		if ( ii-size/2 < kMinShort ) s = kMinShort;
		if ( ii-size/2 > kMaxShort ) s = kMaxShort;		
		i = ii-size/2;
		f = ii-size/2;
		d = ii-size/2;

		(*bufferS_swap) << s; if ( bufferS_swap->IsStatus(Buffer::kFail) ) cout << " PB in swap " << s << " " << ii << endl;
		(*bufferI_swap) << i; if ( bufferI_swap->IsStatus(Buffer::kFail) ) cout << " PB in swap " << i << " " << ii << endl;
		(*bufferF_swap) << f; if ( bufferF_swap->IsStatus(Buffer::kFail) ) cout << " PB in swap " << f << " " << ii << endl;
		(*bufferD_swap) << d; if ( bufferD_swap->IsStatus(Buffer::kFail) ) cout << " PB in swap " << d << " " << ii << endl;
		// write them with swapping
	}	
	fwrite(bufferS_swap->GetBuffer(),bufferS.Size(),1,id);	
	fwrite(bufferI_swap->GetBuffer(),bufferI.Size(),1,id);	
	fwrite(bufferF_swap->GetBuffer(),bufferF.Size(),1,id);	
	fwrite(bufferD_swap->GetBuffer(),bufferD.Size(),1,id);
			
	fclose(id);
	
	watch.Stop();	
	printf("  --> Writing rate with swapping %f MB/s (%f/%f) \n",
		Float_t(bufferS.Size()+bufferI.Size()+bufferF.Size()+bufferD.Size())/(MBYTE*watch.RealTime()),	
		Float_t(bufferS.Size()+bufferI.Size()+bufferF.Size()+bufferD.Size())/MBYTE,watch.RealTime());
	
}

void ReadIO_C()
{
	FILE *id; Memory::EEndian inv_endian; TString noswap, swap; TStopwatch watch;

	// to know the endian type of your machine
	if ( Memory::IsBytes(Memory::kLittle) )  { noswap = "little_c"; swap = "big_c"; inv_endian = Memory::kBig; }
	else  { noswap = "big_c"; swap = "little_c"; inv_endian = Memory::kLittle; }
	
	cout << noswap << " endian type for this machine " << endl;

	// 32 KBYTES buffers with the same endian type that your machine
	Buffer  bufferS(sizeof(Short_t)*size), 
		bufferI(sizeof(Int_t)*size), 
		bufferF(sizeof(Float_t)*size), 
		bufferD(sizeof(Double_t)*size); 
	
	// read the buffers from disk
	id = fopen(noswap.Data(),"r");
	if ( id == NULL ) {
		printf("Cannot open file %s\n",noswap.Data()); return ;
	}
	
	// read blocks and compute the needed time
	watch.Start(kTRUE);
	
	fread(bufferS.GetBuffer(),bufferS.Size(),1,id);	
	fread(bufferI.GetBuffer(),bufferI.Size(),1,id);	
	fread(bufferF.GetBuffer(),bufferF.Size(),1,id);	
	fread(bufferD.GetBuffer(),bufferD.Size(),1,id);	
		
	for (Int_t ii = 0; ii < size; ii++) {
		Short_t s; Float_t f; Int_t i; Double_t d;
		bufferS >> s; 
		bufferI >> i; 
		bufferF >> f; 
		bufferD >> d; // read them without swapping
		
		Short_t stmp = ii-size/2;
		if ( ii-size/2 < kMinShort ) stmp = kMinShort;
		if ( ii-size/2 > kMaxShort ) stmp = kMaxShort;		
				
		if ( s != stmp ) cout << " ERROR [short]" << ii-size/2 << " expected " << s << " read ! " << endl;
		if ( i != ii-size/2 ) cout << " ERROR [int]" << ii-size/2 << " expected " << i << " read ! " << endl;
		if ( f != ii-size/2 ) cout << " ERROR [float]" << ii-size/2 << " expected " << f << " read ! " << endl;
		if ( d != ii-size/2 ) cout << " ERROR [double]" << ii-size/2 << " expected " << d << " read ! " << endl;
	}	
	fclose(id);
		
	watch.Stop();	
	printf("  --> Reading rate without swapping %f MB/s (%f/%f) \n",
		Float_t(bufferS.Size()+bufferI.Size()+bufferF.Size()+bufferD.Size())/(MBYTE*watch.RealTime()),	
		Float_t(bufferS.Size()+bufferI.Size()+bufferF.Size()+bufferD.Size())/MBYTE,watch.RealTime());
	
	// fill the buffers and write them on disk
	id = fopen(swap.Data(),"r");
	if ( id == NULL ) {
		printf("Cannot open file %s\n",swap.Data()); return ;
	}
	
	// Best way to allocate buffers since it gives a buffer that is aware of byte swapping
	Buffer *bufferS_swap = Buffer::New(inv_endian,sizeof(Short_t)*size);
	Buffer *bufferI_swap = Buffer::New(inv_endian,sizeof(Int_t)*size);
	Buffer *bufferF_swap = Buffer::New(inv_endian,sizeof(Float_t)*size);
	Buffer *bufferD_swap = Buffer::New(inv_endian,sizeof(Double_t)*size);	

	// write blocks and compute the needed time
	watch.Start(kTRUE);
		
	fread(bufferS_swap->GetBuffer(),bufferS.Size(),1,id);	
	fread(bufferI_swap->GetBuffer(),bufferI.Size(),1,id);	
	fread(bufferF_swap->GetBuffer(),bufferF.Size(),1,id);	
	fread(bufferD_swap->GetBuffer(),bufferD.Size(),1,id);
		
	for (Int_t ii = 0; ii < size; ii++) {

	}
	fclose(id);
	
	watch.Stop();	
	printf("  --> Reading rate with swapping %f MB/s (%f/%f) \n",
		Float_t(bufferS.Size()+bufferI.Size()+bufferF.Size()+bufferD.Size())/(MBYTE*watch.RealTime()),	
		Float_t(bufferS.Size()+bufferI.Size()+bufferF.Size()+bufferD.Size())/MBYTE,watch.RealTime());
	
}


void WriteIO_CPP()
{
	ofstream id; Memory::EEndian inv_endian; TString noswap, swap; TStopwatch watch;

	// to know the endian type of your machine
	if ( Memory::IsBytes(Memory::kLittle) )  { noswap = "little_cpp"; swap = "big_cpp"; inv_endian = Memory::kBig; }
	else  { noswap = "big_cpp"; swap = "little_cpp"; inv_endian = Memory::kLittle; }
	
	cout << noswap << " endian type for this machine " << endl;

	// 32 KBYTES buffers with the same endian type that your machine
	Buffer  bufferS(sizeof(Short_t)*size), 
		bufferI(sizeof(Int_t)*size), 
		bufferF(sizeof(Float_t)*size), 
		bufferD(sizeof(Double_t)*size); 
	
	// cout << bufferS.Size() << " " << bufferI.Size() << " " << bufferF.Size() << " " << bufferD.Size() << endl;
	// fill the buffers and write them on disk
	id.open(noswap.Data());
	if ( !id.is_open() ) {
		printf("Cannot open file %s\n",noswap.Data()); return ;
	}
	
	// write blocks and compute the needed time
	watch.Start(kTRUE);
	for (Int_t ii = 0; ii < size; ii++) {
		Short_t s; Float_t f; Int_t i; Double_t d;
		
		s = ii-size/2;
		if ( ii-size/2 < kMinShort ) s = kMinShort;
		if ( ii-size/2 > kMaxShort ) s = kMaxShort;
		i = ii-size/2;
		f = ii-size/2;
		d = ii-size/2;
		
		// write them without swapping
		bufferS << s; if ( bufferS.IsStatus(Buffer::kFail) ) cout << " PB in buffer " << s << " " << ii << endl;
		bufferI << i; if ( bufferI.IsStatus(Buffer::kFail) ) cout << " PB in buffer " << i << " " << ii << endl;
		bufferF << f; if ( bufferF.IsStatus(Buffer::kFail) ) cout << " PB in buffer " << f << " " << ii << endl;
		bufferD << d; if ( bufferD.IsStatus(Buffer::kFail) ) cout << " PB in buffer " << d << " " << ii << endl;
	}
	id.write(bufferS.GetBuffer(),bufferS.Size());
	id.write(bufferI.GetBuffer(),bufferI.Size());
	id.write(bufferF.GetBuffer(),bufferF.Size());
	id.write(bufferD.GetBuffer(),bufferD.Size());
				
	id.close();
	
	watch.Stop();	
	printf("  --> Writing rate without swapping %f MB/s (%f/%f) \n",
		Float_t(bufferS.Size()+bufferI.Size()+bufferF.Size()+bufferD.Size())/(MBYTE*watch.RealTime()),	
		Float_t(bufferS.Size()+bufferI.Size()+bufferF.Size()+bufferD.Size())/MBYTE,watch.RealTime());
	
	// fill the buffers and write them on disk
	id.open(swap.Data());
	if ( !id.is_open() ) {
		printf("Cannot open file %s\n",swap.Data()); return ;
	}
	
	// Best way to allocate buffers since it gives a buffer that is aware of byte swapping
	Buffer *bufferS_swap = Buffer::New(inv_endian,sizeof(Short_t)*size);
	Buffer *bufferI_swap = Buffer::New(inv_endian,sizeof(Int_t)*size);
	Buffer *bufferF_swap = Buffer::New(inv_endian,sizeof(Float_t)*size);
	Buffer *bufferD_swap = Buffer::New(inv_endian,sizeof(Double_t)*size);	

	// write blocks and compute the needed time
	watch.Start(kTRUE);
	for (Int_t ii = 0; ii < size; ii++) {
		Short_t s; Float_t f; Int_t i; Double_t d;
		s = ii-size/2;
		if ( ii-size/2 < kMinShort ) s = kMinShort;
		if ( ii-size/2 > kMaxShort ) s = kMaxShort;		
		i = ii-size/2;
		f = ii-size/2;
		d = ii-size/2;

		(*bufferS_swap) << s; if ( bufferS_swap->IsStatus(Buffer::kFail) ) cout << " PB in swap " << s << " " << ii << endl;
		(*bufferI_swap) << i; if ( bufferI_swap->IsStatus(Buffer::kFail) ) cout << " PB in swap " << i << " " << ii << endl;
		(*bufferF_swap) << f; if ( bufferF_swap->IsStatus(Buffer::kFail) ) cout << " PB in swap " << f << " " << ii << endl;
		(*bufferD_swap) << d; if ( bufferD_swap->IsStatus(Buffer::kFail) ) cout << " PB in swap " << d << " " << ii << endl;
		// write them with swapping
	}	
	id.write(bufferS_swap->GetBuffer(),bufferS.Size());
	id.write(bufferI_swap->GetBuffer(),bufferI.Size());
	id.write(bufferF_swap->GetBuffer(),bufferF.Size());
	id.write(bufferD_swap->GetBuffer(),bufferD.Size());	
			
	id.close();	
	
	watch.Stop();	
	printf("  --> Writing rate with swapping %f MB/s (%f/%f) \n",
		Float_t(bufferS.Size()+bufferI.Size()+bufferF.Size()+bufferD.Size())/(MBYTE*watch.RealTime()),	
		Float_t(bufferS.Size()+bufferI.Size()+bufferF.Size()+bufferD.Size())/MBYTE,watch.RealTime());
	
}

void ReadIO_CPP()
{
	ifstream id; Memory::EEndian inv_endian; TString noswap, swap; TStopwatch watch;

	// to know the endian type of your machine
	if ( Memory::IsBytes(Memory::kLittle) )  { noswap = "little_cpp"; swap = "big_cpp"; inv_endian = Memory::kBig; }
	else  { noswap = "big_cpp"; swap = "little_cpp"; inv_endian = Memory::kLittle; }
	
	cout << noswap << " endian type for this machine " << endl;

	// 32 KBYTES buffers with the same endian type that your machine
	Buffer  bufferS(sizeof(Short_t)*size), 
		bufferI(sizeof(Int_t)*size), 
		bufferF(sizeof(Float_t)*size), 
		bufferD(sizeof(Double_t)*size); 
	
	// read the buffers from disk
	id.open(noswap.Data());
	if ( !id.is_open() ) {
		printf("Cannot open file %s\n",noswap.Data()); return ;
	}
	
	// read blocks and compute the needed time
	watch.Start(kTRUE);
	
	id.read(bufferS.GetBuffer(),bufferS.Size());
	id.read(bufferI.GetBuffer(),bufferI.Size());
	id.read(bufferF.GetBuffer(),bufferF.Size());
	id.read(bufferD.GetBuffer(),bufferD.Size());
		
	for (Int_t ii = 0; ii < size; ii++) {
		Short_t s; Float_t f; Int_t i; Double_t d;
		bufferS >> s; 
		bufferI >> i; 
		bufferF >> f; 
		bufferD >> d; // read them without swapping
		
		Short_t stmp = ii-size/2;
		if ( ii-size/2 < kMinShort ) stmp = kMinShort;
		if ( ii-size/2 > kMaxShort ) stmp = kMaxShort;		
				
		if ( s != stmp ) cout << " ERROR [short]" << ii-size/2 << " expected " << s << " read ! " << endl;
		if ( i != ii-size/2 ) cout << " ERROR [int]" << ii-size/2 << " expected " << i << " read ! " << endl;
		if ( f != ii-size/2 ) cout << " ERROR [float]" << ii-size/2 << " expected " << f << " read ! " << endl;
		if ( d != ii-size/2 ) cout << " ERROR [double]" << ii-size/2 << " expected " << d << " read ! " << endl;
	}	

	id.close();
	
	watch.Stop();	
	printf("  --> Reading rate without swapping %f MB/s (%f/%f) \n",
		Float_t(bufferS.Size()+bufferI.Size()+bufferF.Size()+bufferD.Size())/(MBYTE*watch.RealTime()),	
		Float_t(bufferS.Size()+bufferI.Size()+bufferF.Size()+bufferD.Size())/MBYTE,watch.RealTime());
	
	// fill the buffers and write them on disk
	id.open(swap.Data());
	if ( !id.is_open() ) {
		printf("Cannot open file %s\n",noswap.Data()); return ;
	}
		
	// Best way to allocate buffers since it gives a buffer that is aware of byte swapping
	Buffer *bufferS_swap = Buffer::New(inv_endian,sizeof(Short_t)*size);
	Buffer *bufferI_swap = Buffer::New(inv_endian,sizeof(Int_t)*size);
	Buffer *bufferF_swap = Buffer::New(inv_endian,sizeof(Float_t)*size);
	Buffer *bufferD_swap = Buffer::New(inv_endian,sizeof(Double_t)*size);	

	// write blocks and compute the needed time
	watch.Start(kTRUE);
	id.read(bufferS_swap->GetBuffer(),bufferS.Size());
	id.read(bufferI_swap->GetBuffer(),bufferI.Size());
	id.read(bufferF_swap->GetBuffer(),bufferF.Size());
	id.read(bufferD_swap->GetBuffer(),bufferD.Size());
		
		
	for (Int_t ii = 0; ii < size; ii++) {
		Short_t s; Float_t f; Int_t i; Double_t d;
		
		(*bufferS_swap) >> s; (*bufferI_swap) >> i; (*bufferF_swap) >> f; (*bufferD_swap) >> d;
		
		Short_t stmp = ii-size/2;
		if ( ii-size/2 < kMinShort ) stmp = kMinShort;
		if ( ii-size/2 > kMaxShort ) stmp = kMaxShort;			
			
		if ( s != stmp ) cout << " ERROR [short swap]" << ii-size/2 << " expected " << s << " read ! " << endl;
		if ( i != ii-size/2 ) cout << " ERROR [int swap]" << ii-size/2 << " expected " << i << " read ! " << endl;
		if ( f != ii-size/2 ) cout << " ERROR [float swap]" << ii-size/2 << " expected " << f << " read ! " << endl;
		if ( d != ii-size/2 ) cout << " ERROR [double swap]" << ii-size/2 << " expected " << d << " read ! " << endl;
		// read them with swapping
	}

	id.close();	
	
	watch.Stop();	
	printf("  --> Reading rate with swapping %f MB/s (%f/%f) \n",
		Float_t(bufferS.Size()+bufferI.Size()+bufferF.Size()+bufferD.Size())/(MBYTE*watch.RealTime()),	
		Float_t(bufferS.Size()+bufferI.Size()+bufferF.Size()+bufferD.Size())/MBYTE,watch.RealTime());
	
}



