/** \file BrowserDB.C ROOT macro to display (radware, midas, gpsi, etc) spectra from the ROOT TBrowser

	This macro is an illustration on how HistoDB could be used to enhance the 
	ROOT TBrowser object to be able to display spectra from other histo DB systems
	just by clicking on icons. 
	
	First, you must install properly the GammaWare on you system [http://agata.in2p3.fr].
	
	Open your $(HOME)/.root.mines file. If you don't have this file, copy one from the ROOT
	distribution (in the etc directory). This is the file that is read by ROOT to configure
	the ROOT TBrowser icons and on-click actions. In this file adds the following lines: 
	
	[root/file] \n
	pattern = *.s *.spe *.eg \n
	icon = hdb_s.xpm hdb_t.xpm \n
	action = .x BrowserDB.C("%s") \n

	Go to the GammaWare demos/tools directory, run root and open a TBrowser:
	
	root \n
	root [0] TBrowser b \n
	
	In the window corresponding to the current system directory, you should see something like:
	
	\image html RadinROOT.gif 
	
	Double clicked on the spectrum (radware or midas) you want to see and it will be displayed.
	The macro just list the content of the gpsi .. but it could be easily modified 
	to load all the spectra for instance in a THStack collection which is a ROOT
	dedicated object for collections of histograms.

 \author Olivier Stezowski
 
 \date 14/10/2005	
*/
void BrowserDB(const char *hname)
{
	TH1 *histo;
	
	if ( gSystem->Load("libGWTOOLS.so") < 0 ) 
		{ cout << "libGWTOOLS.so is needed to display " << hname << endl; return; }
	
	Gw::HistoDB hdb; TString t = hname;
	
	if (t.EndsWith(".s"))   { // here the action for gpsi .. could be changed to THstack
		t.Prepend("gpsi:"); hdb.Open("tmp",t.Data()); hdb.ls();
	}
	if (t.EndsWith(".spe")) { // here the action for radware spectra
		// open a HistoDB system
		t = "rad:"; t += gSystem->WorkingDirectory(); hdb.Open("tmp",t.Data());
		
		// read the spectrum and display it
		TH1 *h=new TH1F(); h->SetName(hname); hdb >> (*h); if (h) { h->Draw(); gPad->Update(); }
	}
	if (t.EndsWith(".eg")) { // here the action for midas spectra
		// open a HistoDB system
		t = "eg:"; t += gSystem->WorkingDirectory(); hdb.Open("tmp",t.Data());

		// read the spectrum and display it		
		histo = hdb.Get(hname); if (histo) { histo->Draw(); gPad->Update(); }
	}	
}
