/** \file DemoCalibrator.C to illustrate how to use the Calibrator tools.
	
 \author Bertrand Rosse
*/

using namespace std;

using namespace Gw;

#define MAX_STRING_LENGTH 50


/** Function to calibrate a Co60 spectrum with the CalibCo60 service class

  Usage: 
	- root
	- root [0] .L DemoCalibrators.C
	- root [1] DemoCo60() // default values (search peaks between 0 and 4096, with graphical result)
	- or
	- root [1] DemoCo60(xmin,xmax,draw) //


\author Bertrand Rosse - 09/2007
*/
void DemoCo60(Double_t xmin=0, Double_t xmax=4096)
{
	// construct a function with 2 peaks to simulate the 2 Co60 peaks
	TString str;
	str = "gaus(0) + gaus(3) + [6]*x + [7]";
	TF1 f("Co60raw",str.Data(),0,10000);
	f.SetParameters(1000,2256.23,1.,800,2596.17,1.5,0.01,10);
	TH1D *hraw = new TH1D("RAWhisto","Raw Data",4096,0,4096);
	hraw->FillRandom("Co60raw",100000);

	// Calibrator Co60
	CalibCo60 *calibrator = new CalibCo60();
	hraw->GetXaxis()->SetRange(xmin,xmax);
	TF1 *calfunction = new TF1 ("CalibrationFunction","pol1(0)");
	
	calibrator->Calibrate(hraw,calfunction);

	H1Calibrator *cal = new H1Calibrator();
	cal->Calibrate(hraw,calfunction);

}

/** Function to calibrate a Co60 spectrum with the CalibCo60 service class

  Usage: 
	- root
	- root [0] .L DemoCalibrators.C
	- root [1] DemoCo60ByHand(channel1, channel2) // peak1 on channel1 and peak2 on channel2


\author Bertrand Rosse - 09/2007
 */
void DemoCo60ByHand(Double_t channel1=2256, Double_t channel2=2596)
{
	// construct a function with 2 peaks to simulate the 2 Co60 peaks
	TString str;
	str = "gaus(0) + gaus(3) + [6]*x + [7]";
	TF1 f("Co60raw",str.Data(),0,10000);
	f.SetParameters(1000,2256.23,1.,800,2596.17,1.5,0.01,10);
	TH1D *hraw = new TH1D("RAWhisto","Raw Data",4096,0,4096);
	hraw->FillRandom("Co60raw",100000);

	// Calibrator Co60
	CalibCo60 *calibrator = new CalibCo60();

	TF1 *calfunction = new TF1 ("CalibrationFunction","pol1(0)");
	
	calibrator->Calibrate(hraw,calfunction,channel1,channel2);
	
	H1Calibrator *cal = new H1Calibrator();
	cal->Calibrate(hraw,calfunction);
}

