/** \file ToGeant.C to illustrate how to use the GammaWare to produce input for Enrico's program */

using namespace Gw;

/** To produce sequences of gamma-ray energies and format them to Enrico's program

Function to generate gammas from on a level Scheme and store them in a file formatted for Enrico's programs

- nameGLS : name of the GLS file
- nbcas:  corresponds to the number of cascades to be simulated
- nameOUT: file to store the simulated events \warning any existing file with the same name will be lost

A ROOT file is open with the name toGEANT101.root in order to store the simulated spectrum and the multiplicity distribution

 \date 07/2004 \author Olivier Stezowski

PS: libGWPHYSICS and libGWGEM must be loaded before running this macro (see rootlogon.C)
*/
void toGEANT1(const Char_t *nameAGS, Int_t nbcas = 100, const Char_t *nameOUT = "toGEANT101.event")
{

	// load AGS file
	BaseGEM random;
	if ( random.InitAGS(nameAGS) == 0 ) {
		cout << " GLS level Scheme not correctly loaded " << endl; return;
	}

	// open a file to store the simulated events.
	FILE *fileOUT = fopen(nameOUT,"w");
	if ( fileOUT == NULL ) {
		printf(" Cannot open file %s \n",nameOUT); return;
	}
	fprintf(fileOUT,"FORMAT 4 4 \n");
	fprintf(fileOUT,"# \n");
	fprintf(fileOUT,"# GAMMAS GENERATED FROM THE %s LEVEL SCHEME \n",nameAGS);
	fprintf(fileOUT,"# \n");
	fprintf(fileOUT,"REACTION %d %d %d %d %.1f\n", 1, 1, 6, 12, 0.0);
	fprintf(fileOUT,"# \n");
	fprintf(fileOUT,"EMITTED 1 1 \n");
	fprintf(fileOUT,"# \n");

	// Open the ROOT file and creat the spectrum to be filled
	TFile *froot = (TFile*)gROOT->GetListOfFiles()->FindObject("toGEANT101.root");
	if ( froot == NULL ) { froot = new TFile("toGEANT101.root","UPDATE"); }

	TH1F *K = new TH1F("K","Multiplicity Distribution",100,0,100);
	TH1F *h = new TH1F(nameAGS,"Gamma-ray spectrum",8192,0,4096);

	// start the Monte-Carlo
	Cascade cas; GammaLink *gam;

	for (Int_t i = 0; i < nbcas; i++ ) {

		random.DoCascade(&cas); // next cascade

		K->Fill(cas.GetSize()) ;

		fprintf(fileOUT,"$ \n -101 \n ");

		for (Int_t j = 0; j < cas.GetSize(); j++ ) {

			gam = (GammaLink *)cas.At(j); h->Fill(gam->GetEnergy().Get());

			fprintf(fileOUT,"1 %.1f 0. %.1f \n ",gam->GetEnergy().Get(),8);
		}
	}

	// close the ROOT file
	K->Write(); h->Write(); froot->Close();

	// close the output file
	fprintf(fileOUT,"# \n"); fclose(fileOUT);
}
