/**
*
* \file UtilLevelScheme.C Some useful functions to deal with level schemes
*
*/

using namespace Gw;

/** 

 \date 02/2005 \author Olivier Stezowski

PS: libGWPHYSICS and libGWGEM must be loaded before running this macro (see rootlogon.C)
*/
void ShowGamma(TH1 *histo, LevelScheme *lev, Color_t col = 2, Style_t st = 23)
//void ShowGamma(TAttMarker mark = TAttMarker(1,23,2))
{
	TObject *obj; GammaLink *gamma;
	
	// marker use to display peaks position
	TMarker mark; 
	//mark->SetMarkerSize(attmark->GetMarkerSize());		
	//mark->SetMarkerColor(attmark->GetMarkerColor());
	mark.SetMarkerColor(col); mark.SetMarkerStyle(st);

	// check out few things
	if ( histo == NULL || lev == NULL ) return;
	
	// now loop on gammas to display them on the spectrum
	TIter links(&lev->GetLinks()); 
	while ( (obj = links()) != NULL ) {
		if ( obj->InheritsFrom("GammaLink") ) {
			gamma = (GammaLink *)obj;
			Float_t e = gamma->GetEnergy().GetValue();
			mark.DrawMarker(e,histo->GetBinContent(histo->FindBin(e)));
		}
	}
}

void ShowGamma(LevelScheme *lev)
//void ShowGamma(TAttMarker mark = TAttMarker(1,23,2))
{
	TObject *obj; GammaLink *gamma; Level *ini, *fin;
	
	// marker use to display peaks position
	TMarker mark; TArrow ar;
	ar.SetFillColor(15); ar.SetLineColor(15); ar.SetAngle(30);
	// ar.SetOption("->-");
	//mark->SetMarkerSize(attmark->GetMarkerSize());		
	//mark->SetMarkerColor(attmark->GetMarkerColor());
	mark.SetMarkerColor(1); mark.SetMarkerStyle(8);

	// check out few things
	if ( lev == NULL ) return;
	
	// now loop on gammas to display them on the spectrum
	TIter links(&lev->GetLinks()); 
	while ( (obj = links()) != NULL ) {
		if ( obj->InheritsFrom("GammaLink") ) {
			gamma = (GammaLink *)obj;
			ini = gamma->GetIL(); fin = gamma->GetFL();
			
			// mark.DrawMarker((ini->GetX1()+ini->GetX2())/2,ini->GetY1());
			ar.DrawArrow((ini->GetX1()+ini->GetX2())/2,ini->GetY1(),
				     (fin->GetX1()+fin->GetX2())/2,fin->GetY1(),0.01);
		}
	}
}

