
/** \file MacroToRoot.C to create a root file of TMacros 

It is an slightly modified version of the importCode.C macro provided in the root/tutorial
directory. From a system directory, it creates a root file with the same 
tree structure that the system directory. All root macros found (*.C) are
written in the root file as TMacros so that a TMacro could be executed from the
ROOT TBrowser just by clicking 

*/

#include "TFile.h"
#include "TSystem.h"
#include "TMacro.h"

Bool_t istopdir = false;
      
void importdir(const char *dirname) 
{
  //Example of script showing how to create a ROOT file with subdirectories.
  //The script scans a given directory tree and recreates the
  //same structure in the ROOT file.
  //All source files of type .h,cxx,c,dat,py are imported as TMacro objects
  //see also other tutorial readCode.C
  
   char *slash = (char*)strrchr(dirname,'/');
   char *locdir;
   if (slash) locdir = slash+1;
   else       locdir = (char*)dirname;
   printf("processing dir %s\n",dirname);
   
   TDirectory *savdir = gDirectory; // to avoid having top directory inside the root file
   if ( istopdir ) 
   	{ adir = gFile; istopdir = false; }
   else 
   	adir = savdir->mkdir(locdir);
	
   adir->cd();
   void *dirp = gSystem->OpenDirectory(dirname);
   if (!dirp) return;
   char *direntry;
   Long_t id, size,flags,modtime;
   //loop on all entries of this directory
   while ((direntry=(char*)gSystem->GetDirEntry(dirp))) {
      TString afile = Form("%s/%s",dirname,direntry);
      gSystem->GetPathInfo(afile,&id,&size,&flags,&modtime);
      if (strstr(direntry,"G__")) continue;
      if (strstr(direntry,"~"))   continue;  
      if (strstr(direntry,"%"))   continue;           
      if (strstr(direntry,".c")    ||
          strstr(direntry,".h")    ||
          strstr(direntry,".m")    ||
          strstr(direntry,".dat")  ||
          strstr(direntry,".py")   ||
          strstr(direntry,".C")) {
         TMacro *m = new TMacro(afile);
         m->Write(direntry);
         delete m;
      } else {
         if (flags != 3)                     continue; //must be a directory
         if (direntry[0] == '.')             continue; //forget the "." and ".." special cases
         if (!strcmp(direntry,"CVS"))        continue; //forget some special directories
         if (!strcmp(direntry,"htmldoc"))    continue;
         if (strstr(dirname,"root/include")) continue;
         //we have found a valid sub-directory. Process it
         importdir(afile);
     }
  }
  gSystem->FreeDirectory(dirp);
  savdir->cd();
}

void MacroToRoot(const char *topdir, const char *rootname) 
{
   TFile *f = new TFile(rootname,"recreate"); istopdir = kTRUE; importdir(topdir); f->Close(); delete f;
}
