

void CollectOn()
{
	if ( TVirtualPad::Pad() ) {
		TCanvas *c = TVirtualPad::Pad()->GetCanvas(); ClickCollector *thecollector = ClickCollector::TheCollector();
		c->Connect("ProcessedEvent(Int_t,Int_t,Int_t,TObject*)","ClickCollector",
				thecollector,
				"DoCollect(Int_t,Int_t,Int_t,TObject*)");
		cout << " - Start collecting clicks in " << c->GetName() << endl ;	
		
		thecollector->SetMode();
	}
	else 
		cout << " - Cannot collect clicks, there is no current pad !!! " << endl ;	 

}

#include <RQ_OBJECT.h>

class ClickCollector 
{
RQ_OBJECT("ClickCollector")

private:
	static ClickCollector *gfCollector;	// the click collector
	
private:    
	Int_t fLastPx, fLastPy;		// to keep track of the last postion of the mouse in the pad
	EEventType fCollectMode;	// type of event needed to collect
	
	TMarker    fMarker;		// Marker attribute used to draw markers

	Bool_t	fIsActive;		// true if collect mode is on
		
private:
	ClickCollector();
  
public:
	~ClickCollector() {;}
  
	void SetMode(Bool_t mode = true) { fIsActive = mode; }
	void SetMarker(TMarker m) { fMarker = m; }

	// the unique collector
	static ClickCollector * TheCollector(); 
	
	// action to be performed for any events
	void DoCollect(Int_t event, Int_t px, Int_t py, TObject *obj);
};

// set statics
ClickCollector *ClickCollector::gfCollector = 0;

ClickCollector::ClickCollector() : fLastPx(0), fLastPy(0), fCollectMode(kKeyPress), fMarker()
{	
	// set marker style for peaks
	fMarker.SetMarkerStyle(20); fMarker.SetMarkerColor(2); gfCollector = this;
}


ClickCollector * ClickCollector::TheCollector()
{
	if ( gfCollector == NULL ) new ClickCollector(); return gfCollector;
}


void ClickCollector::DoCollect(Int_t event, Int_t px, Int_t py, TObject *obj)
{
  	// Collect event.
  	// if event = fCollectMode ( = double click by default ) it adds a "click"
  	// to the list of clicks and displays a TCross at that position.

	if ( event == kMouseMotion ) { fLastPx = px; fLastPy = py; return; } // keep memory of the last position
	
	if ( event == fCollectMode && fIsActive ) {
		if ( gROOT->GetSelectedPad() ) {
			fMarker.DrawMarker(gROOT->GetSelectedPad()->AbsPixeltoX(fLastPx),
			gROOT->GetSelectedPad()->AbsPixeltoY(fLastPy));
			gROOT->GetSelectedPad()->Update();
		}
	}
}





