#
# Script to download, compile and install some AGATA Software
# 	the Agata Processing Package, 
#   the dependencies, 
#   other AddOns (gw,root) and specifics to some centers (Legnaro, GSI, GANIL ...)
#
# Author O. Stezowski 
#
# in case of questions/problems, agata{_{at}_}ipnl.in2p3.fr
#

#
# list of programs needed to load/compile/install
#
list_of_prog_needed_load    = ['svn','wget','gunzip','tar']
list_of_prog_needed_compile = ['gcc','make']
list_of_prog_needed_install = ['make','tar']

#
# list of svn server, web site where the different packages could be found
#
entry_point = {}
# ADF
# svn address 08/03/2011
entry_point['adf'] = 'svn://gamma01.lnl.infn.it/agata/trunk/narval_emulator/' 

# GW
# svn address 08/03/2011
entry_point['gw'] = 'svn+ssh://anonsvn@anonsvn.in2p3.fr/agata/gammaware/'
# skstream skstream / tarball from web
entry_point['skstream']	= 'http://prdownloads.sourceforge.net/worldforge/'
# boost
entry_point['boost'] = 'http://svn.boost.org/svn/boost/'
# adpp Legnaro
entry_point['adpp']	= 'svn://gamma01.lnl.infn.it/agata/'
# oft
entry_point['oft'] = 'http://csngwinfo.in2p3.fr:2401/oft/'
# root
entry_point['root'] = 'http://root.cern.ch/svn/root/'

#
ARCH = 'linux'
FORCE_ARCH = False # True if --with-arch is called i.e. the user is sure about it
#

help = """
 Usage : python AgataSoftware.py -h --options=opt actions

	keywords for the list of actions :
		* load
		* compile
		* install
		* all = load + compile + install
		
	possible options are [be careful, the order does matter !] :
	
		* -h  : print help
		* --arch  : set architecture (for compilation) as given by root-config. Default is test with root-config if found, linux otherwise

		* --adf : follow by = (default, means trunk) or by a valid path to a tag version in the repository
		* --gw  : follow by = (default, means trunk) or by a valid path to a tag version in the repository
		* --skstream : default (0.3.7) or valid version number in the file skstream-V.tar.gz on the http server
		* --boost : follow by = (default, means trunk) or by a valid path to a tag version in the repository
		* --adpp : follow by = (default, means trunk) or by a valid path to a tag version in the repository

		* --root : follow by = (default, means trunk) or by a valid path to a tag version in the repository
		
	ex : 
		python AgataSoftware.py -h
		python AgataSoftware.py --arch macosx64 --adf= --skstream= --boost=tags/release/Boost_1_46_0/ --adpp= load
		python AgataSoftware.py --root=tags/v5-26-00b all
"""

import os
import shutil
import getopt, sys

# print help and current conditions
def Usage() :
	print	
	print ' ***********************************************************************'
	print help	
	print ' List of entry points for the known modules :' 
	for index in range(len(entry_point.keys())) : 
		print '  + ', entry_point.keys()[index], ' ==> ', entry_point.values()[index]	
	print
	print ""
	print "Guessed Architecture, change it with --arch if not correct : ", ARCH
	print ""
	print ' ****** in case of questions/problems, agata{_{at}_}ipnl.in2p3.fr ******'
	print	

# check whether or not a program exists
def check_one_program(name, verbose = 0):
	for dir in os.environ['PATH'].split(':'):
		prog = os.path.join(dir, name)
		if os.path.exists(prog) : 
			return prog
	if ( verbose > 0 ) :
	    print name + ' : **** command does not exist on the system **** !!! '
	return 0

# check all the programs required to install the Agata Processing Package are there
def check_all_program(list_of):
	for i, name in enumerate(list_of):
		if ( check_one_program(name,1) == 0 ): 
			return 0
	print ' All programs found ', list_of
	return 1
	
# check a dir exist and creats it if not
def check_directory(dir) :
	# check dir exists, creat it if it does not exist
	if ( os.path.exists(dir) == 0 ) :
		print dir , ' does not exist == > CREATE it'
		os.mkdir(dir)
		if ( os.path.exists(dir) == 0 ) :
			print '**** Cannot create ', dir ,' ****'
			return 0	
	return 1		
	
# check the main directories required to install the Agata Processing Package are there
def check_main_directories() :
	# check ADPP exists, creat it if it does not exist
	if ( check_directory('ADPP') == 0 ) :
		return 0		
	# check directory for dependencies exists, creat it if it does not exist
	if ( check_directory('ADPD') == 0 ) :
		return 0			
	# check main directory to install exists, creat it if it does not exist
	if ( check_directory('ADPL') == 0 ) :
		return 0		
	# check destination exists, creat it if it does not exist
	if ( check_directory('AddOns') == 0 ) :
		return 0		
	# check dir for Legnaro packages exists, creat it if it does not exist
	if ( check_directory('Legnaro') == 0 ) :
		return 0		
	# check dir for GSI packages exists, creat it if it does not exist
	if ( check_directory('GSI') == 0 ) :
		return 0	
	# check dir for Ganil packages exists, creat it if it does not exist
	if ( check_directory('Ganil') == 0 ) :
		return 0	
		
	return 1
	
# check a load from svn is ok 
def is_load_from_svn(command) :
	#
	string_pipe = os.popen(command)
	#
	# check the return value of the command
	#
	check_with = """doesn't exist"""
	if ( check_with in string_pipe ) :
		return 0 
	return 1	
	
# check a load using wget is ok 
def is_load_from_wget(command,output) :
	# remove previous if aLready there 
	if ( os.path.exists(output) ) :
		os.remove(output)
	#
	os.system(command + ' -O ' + output)
	print command + ' -O ' + output
	#
	# check the return value of the command
	#
	if ( os.path.isfile(output) == False ) :
		return 0 
	return 1	
	
# check a load using wget is ok 
def set_ARCH(arg_arch = '') :
	global ARCH
	global FORCE_ARCH
	# remove previous if aLready there 
	if ( arg_arch == '' ) :
		# try and see if root-config is installed
		if ( check_one_program("root-config") ) :
			# open a pipe to the root-config command a take care of removing \n
			f = os.popen("root-config --arch")
			t = f.read()
			ARCH = t[:len(t)-1]
			f.close()
		print "==> Guessed architecture is " , ARCH
	else :
		ARCH = arg_arch
		FORCE_ARCH = True
		print "==> Force architecture to " , ARCH

	
# to get the core libraries (consumers, filters, producers) for AGATA
def get_ADPP(which_phase, release, svn_cmd = 'co'):
	path_in_svn = release
	if ( release == '' ) :
		path_in_svn = 'trunk/narval_emulator/'
	#	 
	command = ' svn ' + svn_cmd + ' ' + entry_point['adpp'] + path_in_svn + ' ADPP' 
	#
	print  command
	if ( is_load_from_svn(command) == 0 ) :
		return 0
	#
	return 1

# to compile the core libraries (consumers, filters, producers) for AGATA
def compile_ADPP():
	# use global ARCH variable
	global ARCH, FORCE_ARCH
	# in case something wrong happens
	result = 1
	# keep to directory 
	agata_path = os.getcwd()
	os.chdir('ADPP')

	# just check a typical file is there
	if ( os.path.isfile("configure.in") == False ) :
		print " *** Are you sure ADPP is properly installed ?! *** "
		return 0
#	else :
#		if ( os.path.isfile("configure") == False ) :
#			os.system("autoconf -f")
	
	# build command from the founded directories
	command  = "./configure "
	command += '--with-arch=' + ARCH
	command += ' --prefix=' + agata_path + '/ADPL'
	print command
	os.system(command)
	command = 'make -f Makefile library'
	print command
	os.system(command)
			
	os.chdir(agata_path)
	#
	return result

# to install the core libraries (consumers, filters, producers) for AGATA
def install_ADPP():
	# use global ARCH variable
	global ARCH, FORCE_ARCH
	# in case something wrong happens
	result = 1
	# keep to directory 
	agata_path = os.getcwd()
	os.chdir('ADPP')

	# just check a typical file is there
	if ( os.path.isfile("Makefile") == False ) :
		print " *** Are you sure ADPP is properly installed/compiled ?! *** "
		return 0
	
	command = 'make -f Makefile install_sd'
	print command
	os.system(command)
			
	os.chdir(agata_path)
	#
	return result
		
# to get the dependencies libraries 
def get_ADPD(which_dependency, release, svn_cmd = 'co'):
	result = 0
	# ADF 
	path_in_svn = release
	if ( which_dependency == '--adf' ) :
		result = 1
		if ( release == '' ) :
			path_in_svn = 'myADF0.2'
		#
		command = ' svn ' + svn_cmd + ' ' + entry_point['adf'] + path_in_svn + ' ADPD/ADF'
		#
		print command
		if ( is_load_from_svn(command) == 0 ) :
			return 0			
		set_mode = 'chmod u+x ADPD/ADF/configure'
		os.system(set_mode)
		set_mode = 'chmod u+x ADPD/ADF/mkinstalldirs'
		os.system(set_mode)			
	# OFT
	path_in_svn = release
	if ( which_dependency == '--oft' ) :
		result = 1
		if ( release == '' ) :
			path_in_svn = 'Trunk/'
		#
		command = ' svn ' + svn_cmd + ' --no-auth-cache --username tracking --password tracking ' + entry_point['oft'] + path_in_svn + ' ADPD/OFT'
		#
		print  command
		if ( is_load_from_svn(command) == 0 ) :
			return 0
		#
		#print ' apply patch to have the legnaro version '
		#command = ' svn ' + svn_cmd + ' svn://gamma01.lnl.infn.it/agata/trunk/narval_emulator/filters/Tracking/includeOFT/  ADPD/OFT/patch_Legnaro'
		#
		#print  command
		#if ( is_load_from_svn(command) == 0 ) :
		#	return 0	
		#os.system('cp ADPD/OFT/patch_Legnaro/*.h ADPD/OFT/patch_Legnaro/*.c ADPD/OFT')
	# skstream
	if ( which_dependency == '--skstream' ) :
		result = 1
		skstream_version = release
		if ( release == '' ) :
			skstream_version = '0.3.7'
		#
		command = ' wget ' + entry_point['skstream'] +'skstream-'+ skstream_version + '.tar.gz'
		# load the file into skstream.tar.gz 	
		print  command
		if ( is_load_from_wget(command,'ADPD/'+'skstream-'+skstream_version+'.tar.gz') == 0 ) :
			return 0
		else :
			# unzip
			cpath = os.getcwd()
			# go to ADPD to Unzip, untar and create a copy as 
			os.chdir('ADPD')
			# remove previous if already there 
			if ( os.path.exists('skstream-'+skstream_version) ) :
				shutil.rmtree('skstream-'+skstream_version)
			# unzip/untar 
			os.system('tar -zxvf ' +'skstream-'+ skstream_version + '.tar.gz')
			# rename as standard dir skstream
			if ( os.path.exists('skstream') ) :
				shutil.rmtree('skstream')	
			# and rename as current skstream	
			os.rename('skstream-'+skstream_version,'skstream')
			# back to AgataSotware directory
			os.chdir(cpath)
	# boost
	path_in_svn = release
	if ( which_dependency == '--boost' ) :
		result = 1
		if ( release == '' ) :
			path_in_svn = 'trunk/'
		#
		command = ' svn ' + svn_cmd + ' ' + entry_point['boost'] + path_in_svn + ' ' + 'ADPD/boost'
		#
		print  command
		print ' It may be long ... '
		if ( is_load_from_svn(command) == 0 ) :
			return 0					
	# result
	return result	

# to compile dependencies ... make sure it exist !
def compile_ADPD(which_dependency, release):
	global ARCH, FORCE_ARCH
	# to know if something has been done
	result = 0
	# keep to directory 
	agata_path = os.getcwd()
	# ADF 
	if ( which_dependency == '--adf' ) :
		 # just check a typical file is there 
		if ( os.path.isfile("ADPD/ADF/ADFConfig.h.in") == False ) :
			print " *** Are you sure ADF is properly installed ?! *** "
			return 0
		result = 1
		os.chdir('ADPD/ADF')
		# call configure script and set install directory as ADPL
		command = "./configure --with-arch=" + ARCH + " --prefix=" + agata_path + "/ADPL"
		print command
		os.system(command)
		command = "make"
		print command
		os.system(command)	
	# OFT 
	if ( which_dependency == '--oft' ) :
		if ( os.path.isfile("ADPD/OFT/configure") == False ) :
			print " *** Are you sure OFT is properly installed ?! *** "
			return 0
		result = 1
		os.chdir('ADPD/OFT')
		command = "./configure " + " --prefix=" + agata_path + "/ADPL"
		print command
		os.system(command)
		command = "make"
		print command
		os.system(command)
	# skstream 
	if ( which_dependency == '--skstream' ) :
		 # required by configure
		if ( os.path.isfile("ADPD/skstream/configure") == False ) :
			print " *** Are you sure skstream is properly installed ?! *** "
			return 0
		result = 1
		os.chdir('ADPD/skstream')
		command = "./configure --prefix=$PWD"
		print command
		os.system(command)
		command = "make"
		print command
		os.system(command)
	# boost 
	if ( which_dependency == '--boost' ) :
		 # required by configure
		if ( os.path.isfile("ADPD/boost/bootstrap.sh") == 0 ) :
			print " *** bootstrap.sh is missing ???!!! *** "
			return 0
		result = 1
		os.chdir('ADPD/boost')
		command = "./bootstrap.sh --libdir=../../ADPL/ --includedir=../../ADPL/"
		print command
		os.system(command)
		command = "./bjam"
		print command
		os.system(command)
			
	os.chdir(agata_path)
	#
	return result

# to install dependencies ... make sure it exist !
def install_ADPD(which_dependency, release):
	global ARCH, FORCE_ARCH
	# to know if something has been done
	result = 0
	# keep path to top directory 
	agata_path = os.getcwd()
	# ADF 
	if ( which_dependency == '--adf' ) :
		result = 1
		os.chdir('ADPD/ADF')
		command = "make install"
		print command
		os.system(command)
	# GW 
	if ( which_dependency == '--gw' ) :
		result = 1
		os.chdir('AddOns/gammaware')
		command = "make install"
		print command
		os.system(command)
	# OFT 
	if ( which_dependency == '--oft' ) :
		result = 1
		os.chdir('ADPD/OFT')
		command = "make install"
		print command
		os.system(command)
		command = "cp OFT/libtracking_OFT.so ../../ADPL/libtracking.so"
		print command
		os.system(command)
	# skstream 
	if ( which_dependency == '--skstream' ) :
		skstream_version = release
		if ( release == '' ) :
			skstream_version = '0.3.7'
		result = 1
		os.chdir('ADPD/skstream')
		command = "make install"
		print command
		os.system(command)
		# Set library name with a uniquename to simplify linking
		libname = ""
		size = 0
		# try and guess the name of the library
		for filename in os.listdir('lib'):
			if ( filename.find('libskstream') != -1 ) :
				if ( os.path.getsize('lib/'+filename) > size ) :
					size = os.path.getsize('lib/'+filename)
					libname = filename
		print 'The guessed skstream lib is :', libname
		
		if ( ARCH.find('macosx') == 0 ) :
			shutil.copy('lib/'+libname,'lib/libskstream.so')
			shutil.copy('lib/'+libname,'lib/libskstream.dylib')
		else :
			shutil.copy('lib/'+libname,'lib/libskstream.so')
	# boost 
	if ( which_dependency == '--boost' ) :
		result = 1
		os.chdir('ADPD/boost')
		command = "./bjam"
		print command
		os.system(command)
	os.chdir(agata_path)
			
	return result

#	
# The three following functions are for AddOns
# load
def get_AddOns(which_dependency, release, svn_cmd = 'co'):
	result = 0
	# root 
	path_in_svn = release
	if ( which_dependency == '--root' ) :
		result = 1
		if ( release == '' ) :
			path_in_svn = 'trunk/'
		#
		command = ' svn ' + svn_cmd + ' ' + entry_point['root'] + path_in_svn + ' AddOns/root'
		#
		print command
		if ( is_load_from_svn(command) == 0 ) :
			return 0
	# GW 
	path_in_svn = release
	if ( which_dependency == '--gw' ) :
		result = 1
		if ( release == '' ) :
			path_in_svn = 'trunk/'
		#
		command = ' svn ' + svn_cmd + ' ' + entry_point['gw'] + path_in_svn + ' AddOns/gammaware'
		#
		print command
		if ( is_load_from_svn(command) == 0 ) :
			return 0
	#
	return 1
# compile 
def compile_AddOns(which_dependency, release):
	global ARCH, FORCE_ARCH
	# to know if something has been done
	result = 0
	# keep to directory 
	agata_path = os.getcwd()
	# root 
	if ( which_dependency == '--root' ) :
		 # just check a typical file is there 
		if ( os.path.isfile("AddOns/root/configure") == False ) :
			print " *** Are you sure ROOT is properly installed ?! *** "
			return 0
		result = 1
		os.chdir('AddOns/root')
		# call configure script, keep install in root directory
		command = "./configure "
		if ( FORCE_ARCH == True ) :
			command += ARCH
		print command
		os.system(command)
		command = "make"
		print command
		os.system(command)	
	# GW 
	if ( which_dependency == '--gw' ) :
		 # required by configure
		if ( check_one_program('root-config') == 0 ) :
			print " *** root-config mandatory to install GW, please set ROOT properly ! *** "
			return 0  
		if ( os.path.isfile("AddOns/gammaware/configure") == False ) :
			print " *** Are you sure GW is properly installed ?! *** "
			return 0
		result = 1
		os.chdir('AddOns/gammaware')
		command = "./configure " + " --prefix=" + agata_path + "/ADPL"
		print command
		os.system(command)
		command = "make"
		print command
		os.system(command)
		
	os.chdir(agata_path)
	#
	return result
# to install
def install_AddOns(which_dependency, release):
	global ARCH, FORCE_ARCH
	# to know if something has been done
	result = 0
	# keep path to top directory 
	agata_path = os.getcwd()
	# ADF 
	if ( which_dependency == '--root' ) :
		result = 1
		print "Nothing to install"
	# GW 
	if ( which_dependency == '--gw' ) :
		result = 1
		os.chdir('AddOns/gammaware')
		command = "make install"
		print command
		os.system(command)
	os.chdir(agata_path)
	#
	return result

#	
# The three following functions are for Legnaro
# load
def get_Legnaro(which_dependency, release, svn_cmd = 'co'):
	result = 0
	# femul
	path_in_svn = release
	if ( which_dependency == '--femul' ) :
		result = 1
		if ( release == '' ) :
			path_in_svn = 'trunk/narval_emulator/WinCtest'
		#
		command = ' svn ' + svn_cmd + ' ' + entry_point['adpp'] + path_in_svn + ' ' + 'Legnaro/femul'
		#
		print  command
		if ( is_load_from_svn(command) == 0 ) :
			return 0
	# PRISMA
	path_in_svn = release
	if ( which_dependency == '--prisma' ) :
		result = 1
		if ( release == '' ) :
			path_in_svn = 'trunk/narval_emulator/PRISMA'
		#
		command = ' svn ' + svn_cmd + ' ' + entry_point['adpp'] + path_in_svn + ' ' + 'Legnaro/PRISMA'
		#
		print  command
		if ( is_load_from_svn(command) == 0 ) :
			return 0	
	#
	return 1
# compile 
def compile_Legnaro(which_dependency, release):
	global ARCH, FORCE_ARCH
	# to know if something has been done
	result = 0
	# keep to directory 
	agata_path = os.getcwd()
	# femul
	path_in_svn = release
	if ( which_dependency == '--femul' ) :
		result = 1
		os.chdir('Legnaro/femul')
		command = "make -f ADPPMakefile femul"
		print command
		os.system(command)
	# PRISMA
	path_in_svn = release
	if ( which_dependency == '--prisma' ) :
		result = 1
		print  " !!! NOT YET IMPLEMENTED !!! "
	os.chdir(agata_path)
	#
	return result
	
# to install
def install_Legnaro(which_dependency, release):
	global ARCH, FORCE_ARCH
	# to know if something has been done
	result = 0
	# keep path to top directory 
	agata_path = os.getcwd()	
	# femul
	path_in_svn = release
	if ( which_dependency == '--femul' ) :
		result = 1
		print  " !!! NOT YET IMPLEMENTED !!! "
	# PRISMA
	path_in_svn = release
	if ( which_dependency == '--prisma' ) :
		result = 1
		print  " !!! NOT YET IMPLEMENTED !!! "
	#
	os.chdir(agata_path)
	#
	return result				
	
# main 
def main():
	global ARCH, FORCE_ARCH
	if (sys.version_info < (2,3,4) ) :
		raise "Version python min required 2.3.4 !"

	# guess architecture, required for compilation, install
	set_ARCH()
	
	# check options
	try:
		opts, args = getopt.getopt(sys.argv[1:], "h", [
			"arch=", "all=", "adf=", "oft=", "skstream=", "boost=", "adpp=", "femul=", "prisma=", 
			"gw=", "root="])
		# architecture is given explicitely
		for index in range(len(opts)) :
			if ( opts[index][0] == '--arch' ) :
				set_ARCH(opts[index][1])
	except getopt.GetoptError, err:
		print ''
		print str(err)
		Usage()
		sys.exit(2)

	# display help
	for index in range(len(opts)) :
		if ( opts[index][0] == '-h' ) :
			Usage()
			return 0

	# check commands are there
	if ( 'load' in args or 'all' in args ) :
		print '[load] ==> Checking commands are there ', list_of_prog_needed_load
		if ( check_all_program(list_of_prog_needed_load) == 0 ) :
			return 0
	if ( 'compile' in args or 'all' in args ) :
		print '[compile] ==> Checking commands are there ', list_of_prog_needed_compile
		if ( check_all_program(list_of_prog_needed_compile) == 0 ) :
			return 0
	if ( 'install' in args or 'all' in args ) :
		print '[install] ==> Checking commands are there ', list_of_prog_needed_install
		if ( check_all_program(list_of_prog_needed_install) == 0 ) :
			return 0
							
	# structure of the directory
	if ( check_main_directories() == 0 ) :
		return 0

	for index in range(len(opts)) :
		# which method to be called 1 ADPD, 2 ADPP, 3 AddOns, 4 Legnaro
		which = 0
		ok = 0
		if ( opts[index][0] == '--adf' ) :
			which = 1		
		if ( opts[index][0] == '--gw' ) :
			which = 3
		if ( opts[index][0] == '--skstream' ) :
			which = 1
		if ( opts[index][0] == '--boost' ) :
			which = 1
		if ( opts[index][0] == '--oft' ) :
			which = 1
		if ( opts[index][0] == '--femul' ) :
			which = 4
		if ( opts[index][0] == '--prisma' ) :
			which = 4
		if ( opts[index][0] == '--adpp' ) :
			which = 2
		if ( opts[index][0] == '--root' ) :
			which = 3
						
		if ( 'load' in args or 'all' in args ) :
			print '[load] ==> Start loading the packages ', opts[index][0]
			ok = 1
			if ( which == 1 ) :
				ok = get_ADPD(opts[index][0],opts[index][1])
			if ( which == 2 ) :
				ok = get_ADPP(opts[index][0],opts[index][1])
			if ( which == 3 ) :
				ok = get_AddOns(opts[index][0],opts[index][1])
			if ( which == 4 ) :
				ok = get_Legnaro(opts[index][0],opts[index][1])
			if ( ok == 0 ) :
				print ' *** Something wrong @ loading *** ' + opts[index][0]
				return
			print '[load] DONE ' 
		if ( 'compile' in args or 'all' in args ) :
			print '[compile] ==> Start compiling the packages ', opts[index][0]
			ok = 1
			if ( which == 1 ) :
				ok = compile_ADPD(opts[index][0],opts[index][1])
			if ( which == 2 ) :
				ok = compile_ADPP()
			if ( which == 3 ) :
				ok = compile_AddOns(opts[index][0],opts[index][1])
			if ( which == 4 ) :
				ok = compile_Legnaro(opts[index][0],opts[index][1])
			if ( ok == 0 ) :
				print ' *** Something wrong @ compiling *** ' + opts[index][0]
				return
			print '[compile] DONE ' 
		if ( 'install' in args or 'all' in args ) :
			print '[install] ==> Start installing the packages ', opts[index][0]
			ok = 1
			if ( which == 1 ) :
				ok = install_ADPD(opts[index][0],opts[index][1])
			if ( which == 2 ) :
				ok = install_ADPP()
			if ( which == 3 ) :
				ok = install_AddOns(opts[index][0],opts[index][1])
			if ( which == 4 ) :
				ok = install_Legnaro(opts[index][0],opts[index][1])
			if ( ok == 0 ) :
				print ' *** Something wrong @ installing *** ' + opts[index][0]
				return
			print '[install] DONE '

# execute main 
if __name__ == "__main__":
	main()
	


