#!/usr/bin/python

import sys
import ftplib
import os

usage = """ 
----------------------------------------------------------------------------------------------
 It dowload a root version, install it and creates a script to set the environment for that 
 particular version.
 
 Usage: install_root.py release prefix system [debug]
 
	* release is an existing root relased version 
	  --> for root_v5.12.00b.source.tar.gz, release correspond to v5.12.00b
	
	* prefix is the directory where a v5.12.00b sub-directory is created to install root
	
	* system is the string describing your operating system and passed to configure 
	
	* debug is set to yes/no whether or not you would like to compile with debug flags
	
 Author: Olivier Stezowski
----------------------------------------------------------------------------------------------
"""

# print usage in case something is missing
if len(sys.argv) < 4 :
	print usage
	sys.exit(1)

# set variables from the command line. debug is optionnal .. nothing means no
extraconf = ""	
if len(sys.argv) == 4:
	extraconf = ""	
elif sys.argv[4] == "yes":
	extraconf = "--build=debug"
else:
	extraconf = ""	

release = sys.argv[1]
prefix  = sys.argv[2]
mysys   = sys.argv[3]

zipfile = "root_v" + release + ".source.tar.gz"
tarball = "root_v" + release + ".source.tar"

if prefix.endswith('/'):
	prefix = prefix[0:len(prefix)-1]

# it goes to the tmp directory to dowload the sources and build root
os.chdir(prefix + "/tmp")
if os.getcwd() != prefix + "/tmp":
	print "Cannot go to " + prefix + "/tmp"
	sys.exit(1)	


# to get the tarball from ftp server
myftp = ftplib.FTP("root.cern.ch")

# anonymous login
myftp.login()
myftp.cwd("/root")
print " --> Loading sources from ROOT ftp server ..."
myftp.retrbinary("RETR " + zipfile,open(zipfile,'wb').write)
print " --> ... DONE "
myftp.quit()

# unzip and untar the file
# it might be different in future version with specific pyton modules to make the task
if os.system("gunzip -f " + zipfile) != 0:
	print "Cannot unzip " + zipfile
	sys.exit(1)
	
if os.system("tar -xvf " + tarball) != 0:
	print "Cannot untar " + tarball
	sys.exit(1)

# to root directory in order to compile
os.chdir(prefix + "/tmp/root")
if os.getcwd() != prefix + "/tmp/root":
	print "Cannot go to " + prefix + "/tmp/root"
	sys.exit(1)	

command = "./configure " + mysys + " " + extraconf + "--prefix=" + prefix + "/root_v" + release + " --etcdir=" + prefix + "/root_v" + release +"/etc"
os.system(command)
os.system("make -j2")
os.system("make install")

# remove temp directory
os.system("rm -rf " + prefix + "/tmp/root")

# now it creates a python script in the prefix directory to set up that particular root version
os.chdir(prefix)
if os.getcwd() != prefix :
	print "Cannot go to " + prefix 
	sys.exit(1)	

# 
fcsh = open("root" + release + ".csh","w")

fcsh.write("setenv ROOTPREFIX " + prefix + "/root_v" + release)
fcsh.write("""
if ($?PATH) then
   setenv PATH ${ROOTPREFIX}/bin:${PATH}      # Linux, ELF HP-UX
else
   setenv PATH ${ROOTPREFIX}/bin
endif

if ($?LD_LIBRARY_PATH) then
   setenv LD_LIBRARY_PATH ${ROOTPREFIX}/lib/root:${LD_LIBRARY_PATH}      # Linux, ELF HP-UX
else
   setenv LD_LIBRARY_PATH ${ROOTPREFIX}/lib/root
endif

if ($?DYLD_LIBRARY_PATH) then
   setenv DYLD_LIBRARY_PATH ${ROOTPREFIX}/lib/root:${DYLD_LIBRARY_PATH}  # Mac OS X
else
   setenv DYLD_LIBRARY_PATH ${ROOTPREFIX}/lib/root
endif

if ($?SHLIB_PATH) then
   setenv SHLIB_PATH ${ROOTPREFIX}/lib/root:${SHLIB_PATH}                # legacy HP-UX
else
   setenv SHLIB_PATH ${ROOTPREFIX}/lib/root
endif

if ($?LIBPATH) then
   setenv LIBPATH ${ROOTPREFIX}/lib/root:${LIBPATH}                     # AIX
else
   setenv LIBPATH ${ROOTPREFIX}/lib/root
endif

if ($?MANPATH) then
   setenv MANPATH ${ROOTPREFIX}/man:${MANPATH}
else
   setenv MANPATH ${ROOTPREFIX}/man
endif

"""
)
fcsh.write("unsetenv ROOTPREFIX")
fcsh.close()

# update file in which are stored the versions installed on this machine
f = open("root.available","a")
f.write("root" + release + ".csh \n")
f.close()



