/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   agata(AT)ipnl.in2p3.fr                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file ADFConfig.h to set informations about the ADF configuration
*
*
*/ 
#ifndef ADF_ADFConfig
#define ADF_ADFConfig
#define ADF_CONFIG_H ADF_ADFConfig

#include <string>

/* Define to 1 if compiled as a standalone library */
#define ADF_STANDALONE @IS_STANDALONE@

/* Define to 1 if you have the Narval on your system </narval/xxx.h> header file. */
#define HAVE_NARVAL_H @IS_NARVAL@

/* Define to the version of this package. */
#define PACKAGE_VERSION @adf_VERSION@

/* Define to the full name of this package. */
#define PACKAGE_NAME @PROJECT_NAME@

// ROOT includes in case the library is embedded in GammaWare
#if ADF_STANDALONE == 0

#ifndef ROOT_Bytes
#include <Bytes.h>
#endif

#else

//---- types from ROOT -------------------------------------------------------------------

typedef char           Char_t;      //Signed Character 1 byte (char)
typedef unsigned char  UChar_t;     //Unsigned Character 1 byte (unsigned char)
typedef short          Short_t;     //Signed Short integer 2 bytes (short)
typedef unsigned short UShort_t;    //Unsigned Short integer 2 bytes (unsigned short)
typedef int            Int_t;       //Signed integer 4 bytes (int)
typedef unsigned int   UInt_t;      //Unsigned integer 4 bytes (unsigned int)

// Note: Long_t and ULong_t are currently not portable types
#ifdef R__B64    
typedef long           Long_t;      //Signed long integer 8 bytes (long)
typedef unsigned long  ULong_t;     //Unsigned long integer 8 bytes (unsigned long)
#else
typedef long           Long_t;      //Signed long integer 4 bytes (long)
typedef unsigned long  ULong_t;     //Unsigned long integer 4 bytes (unsigned long)
#endif

#if defined(R__WIN32) 
typedef __int64            Long64_t;  //Portable signed long integer 8 bytes
typedef unsigned __int64   ULong64_t; //Portable unsigned long integer 8 bytes
#else
typedef long long          Long64_t;  //Portable signed long integer 8 bytes
typedef unsigned long long ULong64_t; //Portable unsigned long integer 8 bytes
#endif

typedef float          Float_t;     //Float 4 bytes (float)
typedef double         Double_t;    //Double 8 bytes
typedef double         Double32_t;  //Double 8 bytes in memory, written as a 4 bytes float
typedef bool           Bool_t;      //Boolean (0=false, 1=true) (bool)

#ifndef NULL
#define NULL 0
#endif

#endif

namespace ADF {

const UInt_t    kMaxUChar_t    = UChar_t(~0);
const Int_t     kMaxChar_t     = kMaxUChar_t >> 1;
const Int_t     kMinChar_t     = -kMaxChar_t - 1;

const UInt_t    kMaxUShort_t   = UShort_t(~0);
const Int_t     kMaxShort_t    = kMaxUShort_t >> 1;
const Int_t     kMinShort_t    = -kMaxShort_t - 1;

const UInt_t    kMaxUInt_t     = UInt_t(~0);
const Int_t     kMaxInt_t      = Int_t(kMaxUInt_t >> 1);
const Int_t     kMinInt_t      = -kMaxInt_t - 1;

const char kBool_t[]	= "B";
const char kUShort_t[]	= "US";
const char kShort_t[]	= "S";
const char kUInt_t[]	= "UI";
const char kInt_t[]	= "I";
const char kULong64_t[]	= "UL";
const char kLong64_t[]	= "L";
const char kFloat_t[]	= "F";
const char kDouble_t[]	= "D";
const char kString_t[]  = "STR";

std::string GetPathToConfFile();

}

#endif
