/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** @file ADFLogCollector.h 


*/

#ifndef ADF_LogCollector
#define ADF_LogCollector
#define ADF_LOGCOLLECTOR_H ADF_LogCollector

#include <iostream>
#include <sstream>
#include <vector>

namespace ADF {

class LogMessage;

//! Base class for a Log collector
/*!

*/
class LogCollector 
{     
private:    
	//! name of the collector ... should be unique !
	std::string fName;
	
protected:    	
	//! to send to the log server only message having their level between min and max	
	unsigned short fMinLevel;
	unsigned short fMaxLevel;
		
private:    
	//! if true, the collector is deleted by the central log
	bool  fIsDeleteByCentralLog;      
	  
public: 
	//!   
	LogCollector(const char *, bool isdelbycental = false);  
	//!   
	virtual ~LogCollector();

	//! return the current logcollector
	static LogCollector *theCollector();
	
	//! LogMessages processed by the LogCollector if their level is between min and max
	void SetLevelFiltering(unsigned short min, unsigned short max)
		{ fMinLevel = min; fMaxLevel= max; }
		
	//! Default, collect nothing ! 
	/*!
		This is the method to overwritte to implement your own action.
		Default is clear the current LogMessage
	*/
	virtual void DoCollect(LogMessage &) 
		{;} 
		
	virtual std::string &GetName() 
		{return fName;}
		
	//! if true, the collector is deleted by the central log	
	virtual bool IsDeleteByCentralLog() const
		{ return fIsDeleteByCentralLog; }	
}; 

//! Base class for a Log collector
/*!

*/
class OStreamCollector : public LogCollector
{
private:
	std::ostream *fOutputStream;
         
public:  
	OStreamCollector(const char *, bool isdelbycental = false); 
	virtual ~OStreamCollector();
	
	//! write the message in a file 
	virtual void DoCollect(LogMessage &);
};
	
} // namespace

#endif





