

#include "ADFObjects.h"
#include "FrameFactory.h"
#include "PSAFrame.h"
#include "TrackedFrame.h"

#include <iostream>
#include <sstream>
#include <string>

/*
compilation (standalone) : 
	g++ -O -Wall -fPIC -I../ -L../ -lADF ADFObject.C -o ADFObject
	g++ -g -pg -I../ -L../ -lADF ADFObject.C -o ADFObject

In a root session (with include to gammaware's includes properly defined)
	root
	gSystem->Load("libGWCORE")
	gSystem->Load("libGWADF")
	.L ADFObject.C++
	
--> DEBUG 
Linux SL4

--> VALGRIND

Memory leak: ok
Profiling:
	
*/

using namespace ADF;
using namespace std;

//! To show how to add named items to a GObject and how work links and proxy 
/*!
	A link between a NamedItem and an external variable allows to read/write ... 
	It is unique !! 
	(otherwise would be a mess if several external variables could change the same variable)
	
	Proxy is used only to get the value (reading mode) of a NamedItem ... you 
	can have as many as you need
*/
int test1()
{
	GObject ob1;
	
// add items to ob1
	ob1.AddItem( new NamedItem<UShort_t>("Crys",0u) );
	ob1.AddItem( new NamedItem<Float_t>("E",0.0) );
	ob1.AddItem( new NamedItem<Int_t>("T0",0) );
	
	cout << "**** ob1 : definition " << endl;	
	ob1.Print();
	cout << "**** " << endl;	
		
// link methods	
	cout << "**** ob1 : modification with local variable and links " << endl;	
	UShort_t Crys_link; Float_t E_link; Int_t T0_link;
	ob1.LinkItem("Crys",&Crys_link);
	ob1.LinkItem("E",&E_link);
	
	// now change the value of Crys with yyour local variable and tell ob1 to synchronize the data
	Crys_link = 1u; 
	E_link    = 1.1;
	ob1.SetItems();
	cout << "**** " << endl;	
		
	ob1.Print();
	
// proxy methods
	cout << "**** ob1 : proxy, reading mode only " << endl;		
	const NamedItem<UShort_t> *Crys_prox = ob1.Get<UShort_t>("Crys");
	const NamedItem<Float_t>   *E_prox  = ob1.Get<Float_t>("E");
	
	ob1.Print();			
	cout << " PROXY " << Crys_prox->GetName() << " " << Crys_prox->Get() << endl;
	cout << " PROXY " << E_prox->GetName()    << " " <<    E_prox->Get() << endl;	
	cout << "**** " << endl;	
	
// check clone
/*	cout << "**** Clone method " << endl;		
	NamedItem<UShort_t> ori("Val",0);
	ori.Set(8u);
	NamedItem<UShort_t> *clone = dynamic_cast<NamedItem<UShort_t> *>(ori.MakeAClone());
	
	cout << " Clone " << clone->Get() << endl;	
	cout << " Origi " <<   ori.Get() << endl;

	cout << "**** " << endl;*/	
		
	return 0;
}
//! To show how to transfer links and proxy from one Object to anoter one
/*!

*/
int test2()
{
	GObject ob1, ob2, ob3;
	
// add items to ob1
	ob1.AddItem( new NamedItem<UShort_t>("Crys",0u) );
	ob1.AddItem( new NamedItem<Float_t>("E",0.0) );
	ob1.AddItem( new NamedItem<Int_t>("T0",0) );
		
	ob2.AddItem( new NamedItem<UShort_t>("Crys",0u) );
	ob2.AddItem( new NamedItem<Float_t>("E",0.0) );
	ob2.AddItem( new NamedItem<Double_t>("V",0) );

	// the order does not matter
	ob3.AddItem( new NamedItem<UShort_t>("Crys",0u) );
	ob3.AddItem( new NamedItem<Int_t>("T0",0) );
	ob3.AddItem( new NamedItem<Double_t>("V",0) );
	
// link methods	
	UShort_t Crys_link; Float_t E_link; Int_t T0_link ; Double_t V_link;
	ob1.LinkItem("Crys",&Crys_link);
	ob1.LinkItem("E",&E_link);
	ob1.LinkItem("T0",&T0_link);
	ob1.LinkItem("V",&V_link);
		
	// now change the value of Crys with yyour local variable and tell ob1 to synchronize the data
	cout << "---> Set all items to 1 " << endl;
	Crys_link = 1u; 
	E_link    = 1.1;
	T0_link = 1;
	V_link = 1.1;
	ob1.SetItems();

// proxy methods
	const NamedItem<UShort_t> *Crys_prox = ob1.Get<UShort_t>("Crys");
	const NamedItem<Float_t>   *E_prox   = ob1.Get<Float_t>("E");
	
	cout << "**** Before calling TransferLinkAndProxy o1 -> o2 " << endl;
	cout << "**** ob1 " << endl;	
	ob1.Print();
	cout << "**** ob2 " << endl;	
	ob2.Print();	
// transfer link and proxy for one to 2
	ob1.TransferLinkAndProxy(ob2);	
	cout << "**** After calling TransferLinkAndProxy o1 -> o2 " << endl;	
	cout << "**** ob1 " << endl;	
	ob1.Print();
	cout << "**** ob2 " << endl;	
	ob2.Print();
	cout << "**** ob1 cannot be changed anymore " << endl;	
	cout << "---> Set Crys and E items to 2 " << endl;
	Crys_link = 2u; 
	E_link    = 2.2;	
	ob1.SetItems();
	ob1.Print();
	cout << "**** ob2 link and proxy works " << endl;	
	ob2.SetItems();
	ob2.Print();
	cout << " PROXY " << Crys_prox->GetName() << " " << Crys_prox->Get() << endl;
	cout << " PROXY " << E_prox->GetName()    << " " <<    E_prox->Get() << endl;
	
// transfer link and proxy for 2 to 3
	cout << "**** After calling TransferLinkAndProxy o2 -> o3 " << endl;	
	ob2.TransferLinkAndProxy(ob3);	
	cout << "---> Set Crys, T0 and E items to 3 " << endl;
	Crys_link = 3u; 
	T0_link = 3;
	E_link    = 3.3;	
	ob3.SetItems();
	cout << "**** ob3 " << endl;	
	ob3.Print();
	cout << " PROXY " << Crys_prox->GetName() << " " << Crys_prox->Get() << endl;
	cout << " PROXY " << E_prox->GetName()    << " " <<    E_prox->Get() << endl;	
	
	cout << "**** " << endl;	
	
	
	return 0;
}

int test3()
{
	GObject ob1;
	Anonymous ano_link;

	// put whatever you want in the free space
	ano_link.Set<UShort_t>(3u,0u);
	ano_link.Set<Float_t> (0.1,2u);
	ano_link.fRealSize = sizeof(UShort_t) + sizeof(Float_t);

	// link the anonymous with a local variable to change the content of the one in ob1
	ob1.AddItem( new NamedItem<Anonymous>("MySpace",0) );
	ob1.Link<Anonymous>("MySpace",&ano_link);
	ob1.SetItems(); // copy the content of ano_link in the one in Global

	// get back using proxy
	const NamedItem<Anonymous> *Ano_prox = ob1.Get<Anonymous>("MySpace");
	UShort_t ano1;
	Float_t  ano2;

	ano1 = (Ano_prox->Get()).Get<UShort_t>(0u);
	ano2 = (Ano_prox->Get()).Get<Float_t> (2u);	

	cout << " PROXY " << Ano_prox->GetName() << " " << ano1 << endl;
	cout << " PROXY " << Ano_prox->GetName() << " " << ano2 << endl;	

	ob1.Print();
}

int test4(UInt_t number)
{
	// register factories
	KeyFactory::Register("Agata"); FrameFactory::Register("Agata");	
	
	// allocate the frame
	Frame *f = FrameFactory::New(FactoryItem("Agata","data:tracked",Version(1,0)),FactoryItem("Agata","data:tracked",Version(65000,1)) );
	// a pointer to the data interface
	GammaTrackedInterface *data = GetDataPointer<GammaTrackedInterface>(f);
	
	// open the file
	FILE *fp = fopen("FRAME.adf","rb");
	if ( fp == 0x0 )
		return 1;
	
	for (UInt_t i = 0u; i < number; i++ ) {
		if ( ! f->Load(fp) )
			break;
		f->Read();
	}

	fclose(fp);
}

#if ADF_STANDALONE == 1
int main(int argc, char **argv)
{
	const int nbtest = 1;
	if ( argc == 1 ) 
		std::cout << " Usage is : \n\t" 
			  << argv[0] << " i \n i being a integer from 1 to "<< nbtest << std::endl;
	
	int nwhat = 0; UInt_t number = 100;
	if ( argc > 1 ) {
		std::istringstream what(argv[1]); what >> nwhat;	
	}
	if ( argc > 2 ) {
		std::istringstream what(argv[2]); what >> number;
	}
		
	switch( nwhat ) {
		case 1:
			test1();
			break;
		case 2:
			test2();
			break;	
		case 3:
			test3();
			break;
		case 4:
			test4(number);
			break;		
		default:
			std::cout << " Unknown test number " << nwhat << std::endl;	
	}
	
	return 0;
}
#endif


