/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** @file AgataDataFrame.h header file for AgataDataFrame.cpp */

#ifndef ADF_AgataDataFrame
#define ADF_AgataDataFrame

#ifndef ADF_GenericFrame
#include "GenericFrame.h"
#endif

namespace ADF
{ 

class AgataFrameFactory;

//! General interface for a AgataDataFrame
/*!
*/
template <typename Data_T>
class AgataDataFrame : public ConcreteDataFrame< Data_T >
{
	friend class AgataFrameFactory;
	
protected: 
	AgataDataFrame(Key *key): 
		ConcreteDataFrame< Data_T >(key) {;}
		
	//! to get from the frame the data content
	virtual UInt_t ReadImp()  = 0;
	//! to set from the frame the data content	
	virtual UInt_t WriteImp() = 0;
	
public:		
	virtual ~AgataDataFrame() 
		{;}
	
	//! to reset the data content
	virtual  void DataReset() = 0;	

	virtual UInt_t Read() 
	{ 
		// to know the expected length of the frame 
		UInt_t length_from_key 
			= AgataDataFrame< Data_T >::GetKey()->GetDataLength();
			
		// test on 64 bits ??
		
		// the buffer is ready to be read
		AgataDataFrame< Data_T >::fBuffer->SetEffectiveSize(length_from_key);
		AgataDataFrame< Data_T >::fBuffer->SetStatus(BaseBuffer::kGood);
		AgataDataFrame< Data_T >::fBuffer->SetOffset();
		
		UInt_t size_read = ReadImp();
		
		// check if everything is all right and synchronize Global items
		if ( AgataDataFrame< Data_T >::fBuffer->IsStatus(BaseBuffer::kGood) 
			&& size_read <= length_from_key ) { 
			AgataDataFrame< Data_T >::Data()->Global()->GetItems(); 	
		} else { // something goes wrong
			size_read = 0u;	
		}
		return size_read;
	}
	virtual UInt_t Write() 
	{
		// first, synchronize external items into local ones
		AgataDataFrame< Data_T >::Data()->Global()->SetItems(); 
		
		// set current offset to 0
		AgataDataFrame< Data_T >::fBuffer->SetOffset(); 
		AgataDataFrame< Data_T >::fBuffer->SetStatus(BaseBuffer::kGood);
		
		UInt_t size_written = WriteImp();
		
		if ( size_written > 0u && AgataDataFrame< Data_T >::fBuffer->IsStatus(BaseBuffer::kGood) )
		{
			// to align an AgataFrame on 32 bits ... REQUIRED by ADA
			const UInt_t align = 4u;
			UInt_t offset 
			  = (size_written + AgataDataFrame< Data_T >::GetKey()->GetKeyLength()) % align;
			
			if ( offset > 0u ) {
				AgataDataFrame< Data_T >::fBuffer->Reserve(align);
				UInt_t toadd = align - offset; 	
				AgataDataFrame< Data_T >::fBuffer->FillWith(0,toadd);
			}
			// end to align
					
			// check if everything is all right and update the Key part
			Bool_t ok = AgataDataFrame< Data_T >::fBuffer->IsStatus(BaseBuffer::kGood);
			if ( ok ) { 
				AgataDataFrame< Data_T >::GetKey()->SetDataLength
					(AgataDataFrame< Data_T >::fBuffer->Offset());
			} else { // something goes wrong
				AgataDataFrame< Data_T >::GetKey()->SetDataLength(0u);
			}
		}
		return  AgataDataFrame< Data_T >::GetKey()->GetDataLength();
	}
	virtual void Reset() 
		{ ConcreteFrame::FastReset(); DataReset(); }
};


} // namespace ADF
#endif

