/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file AgataRawFrame.cpp compiled in libGWADF.so */

#ifndef ADF_AgataRawFrame
#include "AgataRawFrame.h"
#endif

using namespace ADF;

UInt_t AgataRawFrame::Write()
{ 	
	UInt_t size_written = RawFrame::Write();
	
	if ( size_written > 0u && GetRealBuffer()->IsStatus(BaseBuffer::kGood) )
	{
		// to align an AgataFrame on 32 bits ... REQUIRED by ADA
		const UInt_t align = 4u;
		UInt_t offset 
			= (size_written + GetKey()->GetKeyLength()) % align;
			
		if ( offset > 0u ) {
			RawBuffer().Reserve(align);
				
			// TODO to optimise
			UInt_t toadd = align - offset; 	
			RawBuffer().FillWith(0,toadd);
		}
		// end to align
					
		// check if everything is all right and update the Key part
		Bool_t ok = GetRealBuffer()->IsStatus(BaseBuffer::kGood);
		if ( ok ) { 
			GetKey()->SetDataLength (GetRealBuffer()->Offset());
		} else { // something goes wrong
			GetKey()->SetDataLength(0u);
		}
	}
	return  GetKey()->GetDataLength();
}


