/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** @file Algo.h header file for Algo.cpp */

#ifndef ADF_Algo
#define ADF_Algo
#define ADF_ALGO_H ADF_Algo

#ifndef ADF_ADFConfig
#include "ADFConfig.h"
#endif

namespace ADF
{ 

//! Base class for an algorithm
/*!

*/
class Algo
{   
public:   
    Algo();  
    virtual ~Algo() ;
    
    //! To get the name of that algorithm  
    virtual const Char_t *GetName() { return "An algorithm"; }
        
    //! To fill the local variables with input values
    /*!
    	return 0 if everything is OK
    */   
    virtual Int_t SetInput() = 0;
    
    //! To Process the algorithm on a single event (input)
    /*!
    	return 0 if everything is OK    
    */      
    virtual Int_t Process() = 0; 
    
    //! To fill the ouput variables with the result of the algorithm
    /*!
    	return 0 if everything is OK        
    */       
    virtual Int_t SetOutput() = 0;      
}; 

} // namespace ADF
#endif





