/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/** @file BasicAFC.h header file for BasicAFC.cpp */

#ifndef _BasicAFC
#define _BasicAFC

#ifndef ADF_NarvalInterface
#include "NarvalInterface.h"
#endif
#ifndef ADF_Counter
#include "Counter.h"
#endif

#include "Trigger.h"

//! basic consumer, it counts and dumps Frame in files
/*!
 */
class BasicAFC : public ADF::NarvalConsumer 
{   
private:
	//! destination of the frames
	ADF::FileBlock fEndOfFrames;
	
private:
	//! directory where to write files
	std::string fPath;
	//! base for all the files produced
	std::string fBaseForName;
	//! current file 
	FILE *fCurrentFile;
	//! current file number
	UInt_t fCurrentFileNumber;
	//! max size for a file (in bytes)
	UInt_t fMaxSize;
	//! id of the instance, used for printouts
	Int_t  fMyID;
	// current file name
	std::string fCurrentFileName;
	std::string fCurrentName;
	
private:
	ADF::AgataFrameTrigger *fVertex;
	
protected:
	//! open a new file
	Bool_t NewFile();
	
public:
	BasicAFC(Int_t id = -1);
 	virtual ~BasicAFC(); 
	
	void SetID(Int_t id) {fMyID = id;}

	//! to init globals (static) from a directory
	static void process_config (const Char_t *, UInt_t *) ;
	
	//! Constructor implementation 	
	virtual	void process_initialise (UInt_t *error_code);
	
	//! Destructor implementation  
	virtual	void process_reset (UInt_t *error_code);
	
	virtual UInt_t ProcessBlock (ADF::FrameBlock &) ;
	
};

#endif
