/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/** \file BlackHole.cpp compiled in libADF.so */

#include "BlackHole.h"

#include <iostream>
#include <iomanip>
#include <fstream>

using namespace ADF; 

BlackHole::BlackHole() : NarvalConsumer(), fConsumedBytes(0UL)
{
	// log system
	Log.GetProcessName() = "BlackHole"; 
	Log.SetPID(GetPID()); 
	
	// frameio related
	GetFrameIO().SetName("BlackHole");
	// to be sure unknown frames are written in the ouput
	GetFrameIO().SetModel(ConfAgent::kSafe); 	
}

BlackHole::~BlackHole() 
{  
	// in principle not needed ... just in case reset it has not been called by narval
	UInt_t error = 0u; process_reset(&error) ; 
} 

void BlackHole::process_config (const Char_t *directory_path, UInt_t *error_code)
{
	// first init narval stuff
	NarvalInterface::process_config(directory_path,error_code);
	
	if ( (*error_code) == 0u ) {
		// now init your stuff if required
	}
}

UInt_t BlackHole::ProcessBlock (ADF::FrameBlock &in)
{	
	Log.ClearMessage(); Log.SetProcessMethod("ProcessBlock"); 
	
	// just switch to kRead to tell that the block has been consumed
	in.SetModeIO(ConfAgent::kRead);
//	printf(" Black::ProcessBlock %d \n ",in.GetSize());
	fConsumedBytes += in.GetSize();
	
	return 0u;
}


void BlackHole::process_initialise (UInt_t *error_code)
{
	*error_code = 0;
	Log.ClearMessage(); Log.SetProcessMethod("process_initialise"); 
	
	Log << dolog;		
}

void BlackHole::process_reset (UInt_t *error_code)
{
	*error_code = 0;
	Log.ClearMessage(); Log.SetProcessMethod("process_reset");
	
	Log << info << fConsumedBytes << " bytes have been consumed " << dolog;
}

