# CMake for ADF/Standalone
# O. Stezowski
#
project (adf)
#
#
MESSAGE("[ADF] ${PROJECT_NAME}")
#
set(LIB_NAME ADF)
set(adf_VERSION "0.9.0")
#
# define format for libraries (force .so for macos)
if(WIN32)
  set(_libprefix lib)
  set(_libsuffix .dll)
elseif(APPLE)
  set(_libprefix lib)
  set(_libsuffix .so)
else()
  set(_libprefix lib)
  set(_libsuffix .so) 
endif()
set(ADF_LIBRARY_PROPERTIES 
      SUFFIX ${_libsuffix}
      PREFIX ${_libprefix} )
#
# internal and external dependencies ... NONE !!!
#
set( EXTRA_INTERNAL_LIBRARIES )
set( EXTRA_EXTERNAL_LIBRARIES )
#
# CMake directories
#
set(CMAKE_LIBRARY_OUTPUT_DIRECTORY ${CMAKE_BINARY_DIR}/lib)
set(CMAKE_ARCHIVE_OUTPUT_DIRECTORY ${CMAKE_BINARY_DIR}/lib)
set(CMAKE_RUNTIME_OUTPUT_DIRECTORY ${CMAKE_BINARY_DIR}/bin)
#
# install dir. if not given, set to CMAKE_BINARY_DIR. Allows to have the same procedure to install :
#	cmake, make, make install 
# whatever the situation 
if ( CMAKE_INSTALL_PREFIX_INITIALIZED_TO_DEFAULT )
	set ( CMAKE_INSTALL_PREFIX ${CMAKE_BINARY_DIR} CACHE PATH "Install in building directory" FORCE )
endif()
MESSAGE("[ADF] Install is done in ${CMAKE_INSTALL_PREFIX} ")
#
# configure file
#
set ( IS_STANDALONE 1 )
set ( IS_NARVAL 0 )
if (EXISTS ${WITH_NARVAL}/narval_install)
	MESSAGE("[ADF] Narval found, additional modules	added")
	set( IS_NARVAL 1 )	
else()
	MESSAGE("[ADF] Narval has not been found on this machine")	
endif()
set( ADF_LIB_DIR ${CMAKE_INSTALL_PREFIX}/lib )
#
#
# headers / sources 
#
if ( EXISTS ${CMAKE_CURRENT_SOURCE_DIR}/ADFConfig.h ) # in case build with configure has been called previously
	file(REMOVE ${CMAKE_CURRENT_SOURCE_DIR}/ADFConfig.h)	
endif()
configure_file (
  "${CMAKE_CURRENT_SOURCE_DIR}/ADFConfig.h.cmake"
  "${CMAKE_BINARY_DIR}/include/ADFConfig.h"
)
configure_file (
  "${CMAKE_CURRENT_SOURCE_DIR}/standalone/Env.h.cmake"
  "${CMAKE_BINARY_DIR}/include/standalone/Env.h"
)
file(GLOB headers ${CMAKE_CURRENT_SOURCE_DIR}/*.h ${CMAKE_BINARY_DIR}/include/ADFConfig.h)
if ( IS_NARVAL )
	file(GLOB sources ${CMAKE_CURRENT_SOURCE_DIR}/*.cpp ${CMAKE_CURRENT_SOURCE_DIR}/narval/*.cpp)	
else ()
	file(GLOB sources ${CMAKE_CURRENT_SOURCE_DIR}/*.cpp)	
endif()
#
include_directories(${CMAKE_BINARY_DIR}/include)
include_directories(BEFORE ${CMAKE_CURRENT_SOURCE_DIR})
#
#----------------------------------------------------------------------------
#
add_library(${LIB_NAME} SHARED ${sources})
#
target_link_libraries(${LIB_NAME} ${EXTRA_EXTERNAL_LIBRARIES} ${EXTRA_INTERNAL_LIBRARIES})
#		
set_target_properties(${LIB_NAME} PROPERTIES ${ADF_LIBRARY_PROPERTIES})
#

# copy headers file in a single directory			 	
# because cmake2.6 does not provide COPY in file, do it another way
if( ${CMAKE_VERSION} VERSION_GREATER "2.8.0" )
  file(COPY ${headers} DESTINATION ${CMAKE_BINARY_DIR}/include) 
  file(COPY ADF.conf DESTINATION ${CMAKE_BINARY_DIR}/lib)
else()
  if ( EXISTS ${CMAKE_BINARY_DIR}/include )
  else()
    execute_process(COMMAND ${CMAKE_COMMAND} -E make_directory ${CMAKE_BINARY_DIR}/include/ )
  endif()
  foreach ( f ${headers} )
    execute_process(COMMAND ${CMAKE_COMMAND} -E copy ${f} ${CMAKE_BINARY_DIR}/include/ )
  endforeach()
  execute_process(COMMAND ${CMAKE_COMMAND} -E copy  ADF.conf  ${CMAKE_BINARY_DIR}/lib )
endif()					
#										 						 
install(TARGETS ${LIB_NAME} RUNTIME DESTINATION bin
								LIBRARY DESTINATION lib
								ARCHIVE DESTINATION lib
								COMPONENT libraries)
#
install(FILES ${headers} DESTINATION include COMPONENT headers)	
install(FILES ${CMAKE_BINARY_DIR}/include/standalone/Env.h DESTINATION include/standalone COMPONENT headers)
if ( IS_NARVAL )
	install(DIRECTORY narval DESTINATION include PATTERN ".svn" EXCLUDE)
endif()									 
install(FILES ADF.conf DESTINATION lib COMPONENT libraries)
