# CMake for GammaWare/ADF
# O. Stezowski
#
# To add a file, add it in the with_dictionnary list or in the without_dictionnary list depending
#  whether or not the ROOT dictionnary is to be generated.
#  !! Do not forget to fill the LinkDef file !!
#  !! Do not forget to add EXTRA_INTERNAL_LIBRARIES and EXTRA_EXTERNAL_LIBRARIES with the names of the libraries this one relies on !!
#
project (adf)
#
MESSAGE("[GW]  + ${PROJECT_NAME}")
#
#
set(GWLIBNAME GWADF)
set(adf_VERSION ${GammaWare_VERSION}) # to snchronize adf version and gw version
#
# headers / dictionnary / sources 
# 
# so far no automatic methods for computing which class has a dictionnary or not
# manual method : base file name separated in two lists with different treatment.
#
if ( EXISTS ${CMAKE_CURRENT_SOURCE_DIR}/ADFConfig.h ) # in case build with configure has been called previously
	file(REMOVE ${CMAKE_CURRENT_SOURCE_DIR}/ADFConfig.h)
endif()
#
set( with_dictionnaries
)
set( without_dictionnaries
	ADFConfig
	ADFObjects
	AgataCompositeFrame
	AgataConfigurationFrame
	AgataDataFrame
	AgataFrameFactory
	AgataKeyFactory
	AgataRawFrame
	Algo
	BaseBuffer
	BasicAFC
	BasicAFP
        ReadMezzAFP
	BufferIO
	ADFCentralLog 
	CompositeFrame
	ConfAgent
	ConfigurationFrame
	Counter
	CrystalFrame
	DFAgent
	DataHandling
	DefaultKeyFactory
	DotConf
	FactoryItem
	Frame
	FrameBlock
	FrameFactory
	FrameIO
	GObject
	GenericFrame
	Hits
	Key
	KeyFactory
	ADFLogCollector  
	ADFLogMessage
	MetaFrame
	NarvalInterface
	PSAFrame
	PtrStack
	RunAgent
	Signals
	StackOfObjects
	TestFIO
	TrackedFrame
	Trigger
	Version
	DefaultFrameFactory
	BlackHole
)	
# add includes : root + current (before to have the latest) + all gw includes copied in CMAKE_BINARY_DIR/include 
#
include_directories(${ROOT_INCLUDES})
include_directories(${CMAKE_BINARY_DIR}/include)
include_directories(BEFORE ${CMAKE_CURRENT_SOURCE_DIR})
#
set (ALL_GW_INCLUDES ${CMAKE_CURRENT_SOURCE_DIR} ${CMAKE_BINARY_DIR}/include ${ROOT_INCLUDES}) 
#
link_directories(${ROOT_LIBRARY_DIR})
link_directories(${CMAKE_LIBRARY_OUTPUT_DIRECTORY})
#
# configure file
#
set ( IS_STANDALONE 0 )
set ( IS_NARVAL 0 )
if (EXISTS ${WITH_NARVAL}/narval_install)
	MESSAGE("[ADF] Narval found, additional modules	added")
	set( IS_NARVAL 1 )	
else()
	MESSAGE("[ADF] Narval has not been found on this machine")	
endif()
#
configure_file (
  "${CMAKE_CURRENT_SOURCE_DIR}/ADFConfig.h.cmake"
  "${CMAKE_BINARY_DIR}/include/ADFConfig.h"
)
#
# headers / linkdef etc ... 
#
set(headers ${CMAKE_BINARY_DIR}/include/ADFConfig.h)
set(all_headers "")
set(sources "")
#
# set headers/ etc and generate dictionnary for the files which require it 
#
foreach( f ${with_dictionnaries} )
	set(sources ${sources} ${CMAKE_CURRENT_SOURCE_DIR}/${f}.cpp)
	set(headers ${headers} ${CMAKE_CURRENT_SOURCE_DIR}/${f}.h)
endforeach()
#
# Dictionnary from one file only 'a la ROOT' + internal and external dependencies
#
if ( RDICTIONNARY EQUAL 1 )
#ROOT_GENERATE_DICTIONARY(G__${GWLIBNAME} ${headers} MODULE ${GWLIBNAME} LINKDEF LinkDef.h OPTIONS "-writeEmptyRootPCM")
#    set( EXTRA_EXTERNAL_LIBRARIES Core Graf MathCore XMLParser Hist Gui )
    # MESSAGE("[GW-CLING]")
else ()
#ROOT_GENERATE_DICTIONARY(G__${GWLIBNAME} ${headers} LINKDEF ${CMAKE_CURRENT_SOURCE_DIR}/LinkDef.h)
#    set( EXTRA_EXTERNAL_LIBRARIES Core Cint Graf MathCore XMLParser Hist Gui )
    # MESSAGE("[GW-CINT]")
endif()
set(EXTRA_INTERNAL_LIBRARIES GWCORE)
#
foreach( f ${without_dictionnaries} )
	set(sources ${sources} ${CMAKE_CURRENT_SOURCE_DIR}/${f}.cpp)
	if ( EXISTS ${CMAKE_CURRENT_SOURCE_DIR}/${f}.h )
		set(headers ${headers} ${CMAKE_CURRENT_SOURCE_DIR}/${f}.h)	
	endif()
endforeach()

#
set(headers ${headers} ${CMAKE_CURRENT_SOURCE_DIR}/LogMessage.h
                       ${CMAKE_CURRENT_SOURCE_DIR}/LogCollector.h ${CMAKE_CURRENT_SOURCE_DIR}/CentralLog.h)
#

#----------------------------------------------------------------------------
#
# Use root functions to generate the library and the root map file 
#
ROOT_LINKER_LIBRARY(${GWLIBNAME} ${sources} DEPENDENCIES ${EXTRA_INTERNAL_LIBRARIES} ${EXTRA_EXTERNAL_LIBRARIES})
#ROOT_GENERATE_ROOTMAP(${GWLIBNAME} LINKDEF ${CMAKE_CURRENT_SOURCE_DIR}/LinkDef.h DEPENDENCIES ${EXTRA_INTERNAL_LIBRARIES} ${EXTRA_EXTERNAL_LIBRARIES})
#
# copy headers file in a single directory	so that is becomes available for ohers sub-projects
# because cmake2.6 does not provide COPY in file, do it another way
if( ${CMAKE_VERSION} VERSION_GREATER "2.8.0" )
  file(COPY ${headers} DESTINATION ${CMAKE_BINARY_DIR}/include) 
else()
  if ( EXISTS ${CMAKE_BINARY_DIR}/include )
  else()
    execute_process(COMMAND ${CMAKE_COMMAND} -E make_directory ${CMAKE_BINARY_DIR}/include/ )
  endif()
  foreach ( f ${headers} )
    execute_process(COMMAND ${CMAKE_COMMAND} -E copy ${f} ${CMAKE_BINARY_DIR}/include/ )
  endforeach()
endif()
	
#		
# Install lib and headers 
#
install(TARGETS ${GWLIBNAME} RUNTIME DESTINATION bin
								LIBRARY DESTINATION lib
								ARCHIVE DESTINATION lib
								COMPONENT libraries)
#
install(FILES ${headers} DESTINATION ${GW_INCLUDE_DIR} COMPONENT headers )	
install(FILES ADF.conf DESTINATION lib COMPONENT libraries )										 
								 
																 						 
																						 
																						 
																						 
																						 

									 									
