/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   aLong_t with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file Counter.cpp compiled in libGWADF.so */

#ifndef ADF_Counter_H
#include "Counter.h"
#endif

using namespace ADF;

Counter::Counter(Double_t min, Double_t max, const Char_t *unit): 
	fRangeMin(min),fRangeMax(max),fUnits(unit),fN(0uL),fValue(0.0),fValueMin(max),fValueMax(min){;}  
	
void Counter::operator +=(Double_t value) 
{ 
	fValue += value;
	fN++;  
	
	if ( value < fValueMin )  fValueMin = value;
	if ( value > fValueMax )  fValueMax = value;	
}
	
void Counter::operator =(const Counter &c)
{	
	fRangeMin = c.fRangeMin;
	fRangeMax = c.fRangeMax;
	fUnits    = c.fUnits;
	fN        = c.fN;
	fValue    = c.fValue;
	fValueMin = c.fValueMin;
	fValueMax = c.fValueMax;

}	

void Counter::Print(std::ostream &out)  const
{
	out << " " << GetValue() << " " << fUnits ;
	if ( fN ) 
	    out << " [N,Mean,Min,Max] : [" << fN << ","
	        << GetMean() 
	        << "," << GetMin() << "," << GetMax() << "]";
		
	out << std::endl;
}	
	
/*
Counter::Counter(Double_t xmin, Double_t xmax, UInt_t bins) 
{	

}

void Counter::Init(Double_t xmin, Double_t xmax, UInt_t bins)
{

}
*/



	

