/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   aLong_t with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** @file Counter.h header file for Counter.cpp */

#ifndef ADF_Counter
#define ADF_Counter
#define ADF_COUNTER_H ADF_Counter

#ifndef ADF_ADFConfig
#include "ADFConfig.h"
#endif

#include <iostream>
#include <string>

namespace ADF
{ 

//! A Counter
/*!	
	This is a light object to be used for global statistics:
	
*/   
class Counter 
{	
private:
	Double_t fRangeMin, fRangeMax;
private:
	std::string fUnits;  
private:
	ULong64_t fN;
	Double_t fValue, fValueMin, fValueMax;
	
public:
	Counter(Double_t, Double_t, const Char_t *unit = "");  
	virtual ~Counter() {;}
	
	Double_t GetValue() const { return fValue; }
		
	Double_t GetMean() const { if ( fN == 0uL ) return 0.0; return fValue/fN; }
	Double_t GetMin()  const { return fValueMin; };
	Double_t GetMax()  const { return fValueMax; };
	
	void operator +=(Double_t value);
	void operator ++() { fN++; }
	
	void operator =(const Counter &);	
	
	void Print(std::ostream &out = std::cout) const;	
};

} // namespace ADF
#endif

