/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file DefaultFrameFactory.cpp compiled in libGWADF.so */


#include "DefaultFrameFactory.h"
#include "KeyFactory.h"

using namespace ADF;

DefaultFrameFactory::DefaultFrameFactory(): AFrameFactory<DefaultFrameFactory>("Default") 
{
}

SharedFP *DefaultFrameFactory::DoNew(Key *key, const FactoryItem & which_frame)
{
	// cannot deal with request asking for a version greater that this version
	if ( which_frame.GetFactoryVersion() > GetMaxVersion() ) 
		return 0x0;
	
	// allocation is done in two steps. First Frame, then the SharedFP. Specific ones (inheriting for SharedFP) could be implemented. So far only SharedFP
	SharedFP *sfp = 0x0; Frame *frame = 0x0; 

// just a raw Frame
	if ( which_frame.GetItemName().find("data") == 0 ) {
		frame = new RawFrame(key); 
	}
// any conf frames
	if ( which_frame.GetItemName().find("conf") == 0 ) {
		frame = new ConfigurationFrame(key); 
	}

// composite frames --------------------------------------------------------------------------- //
		
// composite frames --------------------------------------------------------------------------- //
	
	// now return SharedFP. If not specific, and frame is allocated, create a base one and add Frame to it
	if ( sfp == 0x0 && frame ) 
		sfp = new SharedFP(frame);
	
	// WARNING : could be 0x0 ! 
	
	return sfp;
}
    
    




  
    
