/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file DefaultKeyFactory.cpp compiled in libGWADF.so */

#ifndef ADF_DefaultKeyFactory
#include "DefaultKeyFactory.h"
#endif

using namespace ADF; 

Key *DefaultKeyFactory::DoNew(const FactoryItem &item)
{
	// cannot deal with request asking for a version greater that this version
	if ( item.GetFactoryVersion() > GetMaxVersion() ) 
		return 0x0;
	
	Key *key = 0x0; 
	
	if( item.GetItemName().find("FS") == 0 ) { 
		if (  item.GetVersion() == Version(0,0)  ) { 
			key = new FSKey_0_0(GetEndian()); // Frame length coded with 2 bytes
		}
		if (  item.GetVersion() == Version(0,1)  ) { 
			key = new FSKey_0_1(GetEndian()); // Frame length coded with 4 bytes
		}		
	}
	if( item.GetItemName().find("UID") == 0 ) { 
		if (  item.GetVersion() == Version(0,0)  ) { 
			key = new UIDKey_0_0(GetEndian()); // frame length is cst, not coded, id on 4 bytes
		}
		if (  item.GetVersion() == Version(0,1)  ) { 
			key = new UIDKey_0_1(GetEndian()); // frame length is cst, not coded, id on 8 bytes
		}		
		if (  item.GetVersion() == Version(1,0)  ) { 
			key = new UIDKey_1_0(GetEndian()); // Frame length coded with 2 bytes, id on 4 bytes 
		}
		if (  item.GetVersion() == Version(1,1)  ) { 
			key = new UIDKey_1_1(GetEndian()); // Frame length coded with 2 bytes, id on 8 bytes
		}				
	}
	return key;
}



