/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file FactoryItem.cpp compiled in libGWADF.so */

#ifndef ADF_FactoryItem
#include "FactoryItem.h"
#endif

#include <stdio.h> 
#include <iomanip>


/*

*/

using namespace ADF; 

FactoryItem::FactoryItem() : 
	fWhichFactory(""), 
	fWhichFactoryVersion(-1), 
	fWhichItem(""), 
	fWhichVersion(0u,0u),
	fPtrFactory(0x0)
{

}
FactoryItem::FactoryItem(const Char_t *factname, const Char_t *whichitem, Version version) : 
	fWhichFactory(factname), 
	fWhichFactoryVersion(-1), 
	fWhichItem(whichitem), 
	fWhichVersion(version),
	fPtrFactory(0x0)
{

}	
FactoryItem::FactoryItem(const Char_t *factname, Short_t factversion, const Char_t *whichitem, Version version) :
	fWhichFactory(factname), 
	fWhichFactoryVersion(factversion), 
	fWhichItem(whichitem), 
	fWhichVersion(version),
	fPtrFactory(0x0)
{
	
}	

FactoryItem::FactoryItem(const FactoryItem &item)	
{
	fWhichFactory			= item.fWhichFactory.data();
	fWhichFactoryVersion	= item.fWhichFactoryVersion;	
	fWhichItem				= item.fWhichItem.data();	
	fWhichVersion			= item.fWhichVersion;
	fPtrFactory				= item.fPtrFactory;
}

void FactoryItem::SetItem(const Char_t *whichfactory, Short_t factversion, const Char_t *whichitem, Version itemversion, void *ptr)
{
	fWhichFactory			= whichfactory;
	fWhichFactoryVersion	= factversion;
	fWhichItem				= whichitem;	
	fWhichVersion			= itemversion;
	
	fPtrFactory				= ptr;	
}

void FactoryItem::SetItem(const Char_t *whichfactory, const Char_t *whichitem, Version itemversion, Short_t factversion, void *ptr)
{
	SetItem(whichfactory,factversion,whichitem,itemversion,ptr);
}

Bool_t FactoryItem::operator == (const FactoryItem &item) const
{
	Bool_t ok = true;
	
	ok = ok && ( fWhichFactory == item.fWhichFactory);
	ok = ok && ( fWhichItem    == item.fWhichItem);	
	ok = ok && ( fWhichVersion == item.fWhichVersion);
	
	return ok;	
}

Bool_t FactoryItem::IsAVersion(const FactoryItem &item) const
{
	Bool_t ok = true;
	
	ok = ok && ( fWhichFactory == item.fWhichFactory);
	ok = ok && ( fWhichItem    == item.fWhichItem);	
	
	return ok;
}

void FactoryItem::Print() const
{
	printf("Factory %s \n",fWhichFactory.data());
	printf("Frame %s \n",fWhichItem.data());	
}



std::ostream & std::operator << (std::ostream &  os, const ADF::FactoryItem &item)
{
	os << " " << item.GetFactoryName()
		<< " " << item.GetFactoryVersion()
		<< " " << std::setw(20) << std::left << item.GetItemName() << std::right
		<< " " << item.GetVersion().GetMajor()
		<< " " << item.GetVersion().GetMinor();
	return os;
}

std::istream & std::operator >> (std::istream &is, ADF::FactoryItem &item)
{
	std::string tmp1, tmp2; UInt_t ui1, ui2; Short_t s;
	
	is >> tmp1 >> s >> tmp2 >> ui1 >> ui2;
	if ( is.good() ) 
		item.SetItem(tmp1.data(),tmp2.data(),Version(ui1,ui2),s);
	
	return is;
} 
    
