
#ifndef ADF_ProxyFrame
#define ADF_ProxyFrame

#ifndef ADF_Frame
#include "Frame.h"
#endif
#ifndef ADF_ConfigurationFrame
#include "ConfigurationFrame.h"
#endif
#ifndef ADF_FrameFactory
#include "FrameFactory.h"
#endif

namespace ADF
{ 

//! A DataFrame has a Data Interace that helps streaming ADF objects from/to the underlying Frame
/*!
	 \warning It is the responsability of the user to Read/Write correctly from/to the buffer
*/
template <typename Data_T>
class DataFrame
{   
public:
	DataFrame()
		{;}
	virtual ~DataFrame()
		{;}
		
	virtual const Data_T *GetData() const = 0; 
	virtual Data_T *Data() = 0; 
};

//! ConcreteDataFrame : a RawFrame with a data interface
/*!
*/
template <typename Data_T>
class ConcreteDataFrame :  public RawFrame, public DataFrame<Data_T>, protected Data_T
//class ConcreteDataFrame :  public ConcreteFrame, public DataFrame<Data_T>, public Data_T
{   
public:
	ConcreteDataFrame(Key *key):
		RawFrame(key), DataFrame<Data_T>(), Data_T() {;}
	virtual ~ConcreteDataFrame()
		{;}
		
	virtual const Data_T *GetData() const
		{ return this; }
		
	virtual Data_T *Data()
		{ return this; }
};

//! to get the data part of a DataFrame
template <class Data_T>
Data_T *GetDataPointer(Frame *f)
{
	Data_T *data = 0x0;
	
	if ( f == 0x0 )
		return data;
		
	DataFrame<Data_T> *dataframe = dynamic_cast< DataFrame<Data_T> * >(f) ;
	if ( dataframe ) 
		data = dataframe->Data();

	return data;
}
template <class Data_T>
const Data_T *GetCstDataPointer(Frame *f)
{
	const Data_T *data = 0x0;
	
	if ( f == 0x0 )
		return data;
			
	DataFrame<Data_T> *dataframe = dynamic_cast< DataFrame<Data_T> * >(f) ;
	if ( dataframe ) 
		data = dataframe->GetData();

	return data;
}
template <class Data_T>
Data_T *GetDataPointer(SharedFP *fp)
{
	Data_T *data = 0x0;
	
	if ( fp == 0x0 )
		return data;
	else
		if ( fp->GetFrame() == 0x0 )
			return data;
		
	DataFrame<Data_T> *dataframe = dynamic_cast< DataFrame<Data_T> * >(fp->GetFrame()) ;
	if ( dataframe ) 
		data = dataframe->Data();

	return data;
}
template <class Data_T>
const Data_T *GetCstDataPointer(SharedFP *fp)
{
	const Data_T *data = 0x0;
	
	if ( fp == 0x0 )
		return data;
	else
		if ( fp->GetFrame() == 0x0 )
			return data;
		
	DataFrame<Data_T> *dataframe = dynamic_cast< DataFrame<Data_T> * >(fp->GetFrame()) ;
	if ( dataframe ) 
		data = dataframe->Data();

	return data;
}

} // namespace ADF
#endif

