/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file Key.cpp compiled in libGWADF.so */

#ifndef ADF_Key
#include "Key.h"
#endif

using namespace ADF; 

/*
Key::KeySignature::KeySignature(std::string keytype): 
	FactoryItem("Default","",Version(0,0)), 
	fType(keytype), 
	fUniqueID(0) 
{
	;
}
 */
//Key::Key(): fSignature("Key"), fBuffer(0)
Key::Key(): fSignature("Default","",Version(0,0)), fBuffer(0)
{
	fBuffer = new BufferIO(); // default is length 1 byte, endian type of the system
}

Key::Key(BaseBuffer::EEndian e, UInt_t s): fSignature("Default","",Version(0,0)), fBuffer(0)
{
	fBuffer = BufferIO::New(e,s); // return a buffer adapted for the endian of the system.
}   

void Key::Print(std::ostream &out) const
{
	out << "[" << typeid((*this)).name() 
	    << " (" << GetVersion().GetMajor() << "," << GetVersion().GetMinor() << ") ";	
} 
    
/*
void Key::ls() 
{ 
	fBuffer->SetOffset(); 
	for( UInt_t i = 0; i < fBuffer->Size() ; i++ ) { 
		Char_t c; 
		(*fBuffer) >> c; 
		
		std::cout << c << std::endl;;
		// printf("%s \n",c);
	} 
} */

Key::~Key()
{
// destructor
	delete fBuffer; 
}

Bool_t Key::Link(const Char_t *buf, UInt_t size)
{
	if ( size >= GetKeyLength() ) 
		return fBuffer->Link(buf,GetKeyLength());
	else 
		return false;
}  

Bool_t Key::Link(const BufferIO &buf, UInt_t pos, UInt_t size) 
{ 
	if ( size >= GetKeyLength() ) 
		return fBuffer->Link(buf,pos,GetKeyLength());
	else 
		return false;
}  

Bool_t Key::Copy(const Char_t *buf, UInt_t size)
{
	if ( size >= GetKeyLength() ) 
		return fBuffer->Copy(buf,GetKeyLength());
	else 
		return false;
}  

Bool_t Key::Copy(const Key *akey)
{
/*	if ( akey == NULL || fSignature.fUniqueID != akey->GetUniqueID() 
			  || akey->GetKeyLength() != GetKeyLength() ) return false; 
 
 Move to exactly signature : see if it works
 */
	if ( !Key::IsAKey(akey) )
		return false;
		
	return fBuffer->Copy(akey->GetRealBuffer()->GetAddress(),akey->GetKeyLength());
}

    
    
