/***************************************************************************
 *   Copyright (C) 2010 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file RunAgent.cpp compiled in libGWADF.so */

#ifndef ADF_RunAgent
#include "RunAgent.h"
#endif

using namespace ADF;
using namespace std;

RunAgent::RunAgent(const Char_t *rid, const Char_t *srid, Bool_t is_record) : 
	DotConf(rid,srid,is_record),
	fExperiment(""),
	fDate(""),
	fRunNumber(-1),
	fSubRunNumber(-1),
	fGlobalVertex(7)
{ 
	fLog.GetProcessName() = "ADF::RunAgent"; 
	Reset();
} 

RunAgent::~RunAgent() 
{
}

void RunAgent::Reset(const Char_t *opt)
{
	fExperiment = "An experiment"; fRunNumber = fSubRunNumber = 0;
	// x,y,z
	fGlobalVertex[0] = 0;
	fGlobalVertex[1] = 0;
	fGlobalVertex[2] = 0;
	// dx,dy,dz
	fGlobalVertex[3] = 0;
	fGlobalVertex[4] = 0;
	fGlobalVertex[5] = 0;
	// beta
	fGlobalVertex[6] = 0;
}

Bool_t RunAgent::ProcessLine(const string &aline)
{
	string tmp1, tmp2; Bool_t ok_decode = true;
	
	if ( aline.find("Experiment:") == 0 ) { 
		fExperiment = aline.substr(11);
	}	
	// get run number
	if ( aline.find("RunNumber:") == 0 ) { 
		Int_t run, subrun; 

		istringstream decode(aline);
		// just to be sure ...
		decode.clear(); 
		decode >> tmp1 >> run >> subrun ;
		if ( decode.good() ) { // expected format
			fRunNumber = run; fSubRunNumber = subrun;
		}
		else ok_decode = false;
	}
	// get run number
	if ( aline.find("GlobalVertex:") == 0 ) { 
		Double_t d0, d1, d2, d3, d4, d5, d6; 
		
		istringstream decode(aline);
		// just to be sure ...
		decode.clear(); 
		decode >> tmp1 >> d0 >> d1 >> d2 >> d3 >> d4 >> d5 >> d6 ;
		if ( decode.good() ) { // expected format
			// x,y,z
			fGlobalVertex[0] = d0;
			fGlobalVertex[1] = d1;
			fGlobalVertex[2] = d2;
			// dx,dy,dz
			fGlobalVertex[3] = d3;
			fGlobalVertex[4] = d4;
			fGlobalVertex[5] = d5;
			// beta
			fGlobalVertex[6] = d6;			
		}
		else ok_decode = false;
	}	
	return ok_decode;
}

Bool_t RunAgent::DoConfigure (ostream &out)
{
	// add a start record
	AddRecord(out,'b');

	out << "# Experiment Name \n";
	out << "Experiment: " << fExperiment << "\n";
	out << "# Run, Subrun \n";
	out << "RunNumber: " << fRunNumber << " " << fSubRunNumber << "\n";
	out << "# Global vertex x,y,z,dx,dy,dz,beta \n";
	out << "GlobalVertex: " 
		<< fGlobalVertex[0] 
		<< " " 
		<< fGlobalVertex[1] 
		<< " " 
		<< fGlobalVertex[2] 
		<< " " 
		<< fGlobalVertex[3] 
		<< " " 
		<< fGlobalVertex[4] 
		<< " " 
		<< fGlobalVertex[5] 
		<< " " 
		<< fGlobalVertex[6]
		<< "\n";
	
	// add an end of record
	AddRecord(out,'e');
	
	return out.good();
}
 


