/***************************************************************************
 *   Copyright (C) 2010 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** @file RunAgent.h header file for RunAgent.cpp */

#ifndef ADF_RunAgent
#define ADF_RunAgent

#ifndef ADF_DotConf
#include "DotConf.h"
#endif

#include <vector>

namespace ADF
{

//! RunAgent
/*!
	The RunAgent contains all the information related to the current conditions.
 
 The run number, the sub run number (or files number in case a run is composed of several files) 
 The experiment name ...
 
*/
class RunAgent : public DotConf
{	
private:  
	//! Name of the current experiment   
	std::string fExperiment;
	//! Name of the experiment     
	std::string fDate;	
	//! current run number       
	Int_t fRunNumber;     
	//! current sub run number in case a run is itself divided       
	Int_t fSubRunNumber;   
	//! vector to set a global vertex frame
	std::vector <Double_t> fGlobalVertex;

protected:
	virtual Bool_t ProcessLine(const std::string &);
	
public:
	RunAgent(const Char_t *rid, const Char_t *srid, Bool_t is_record = true); 
	virtual ~RunAgent(); 
	
	//! to get run and subrun number
	Int_t GetRun() const
		{ return fRunNumber; }
	Int_t GetSubRun() const
		{ return fSubRunNumber; }
	
	//! change the run and subrun number
	void SetRun(Int_t run, Int_t subrun)
		{ fRunNumber = run;  fSubRunNumber = subrun; }
	//! change the run and subrun number
	void SetSubRun(Int_t subrun)
		{ fSubRunNumber = subrun; }
	
	//! source position
	Double_t GetX()
		{ return fGlobalVertex[0]; }
	Double_t GetY()
		{ return fGlobalVertex[1]; }
	Double_t GetZ()
		{ return fGlobalVertex[2]; }
	Double_t GetDX()
		{ return fGlobalVertex[3]; }
	Double_t GetDY()
		{ return fGlobalVertex[4]; }
	Double_t GetDZ()
		{ return fGlobalVertex[5]; }
	Double_t GetBeta()
		{ return fGlobalVertex[6]; }
	
	//! Reset this object 
	virtual void Reset(const Char_t *opt = "");
	
	//! configure this from an input stream
	virtual Bool_t DoConfigure(std::istream &in, Bool_t allow_init)
		{ return DotConf::DoConfigure(in, allow_init);}
	//! write the configuration in an output stream	
	virtual Bool_t DoConfigure(std::ostream &);
}; 

} // namespace ADF

#endif








