/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   aLong_t with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** @file TestFIO.h header file for TestFIO.cpp */

#ifndef _TestFIO
#define _TestFIO

#ifndef ADF_NarvalInterface
#include "NarvalInterface.h"
#endif
#ifndef ADF_Trigger
#include "Trigger.h"
#endif

//! TestFIO algorithm embedded in Narval
/*!
	It just triggs on a Frame (embedded or not) and produce a output Frame 
	All Frames are filled with Stallion.
*/
class TestFIO : public ADF::NarvalFilter
{   
private:
	//! if true, Frame->Read is called	
	Bool_t fDoRead;
	//! if true, SetOuput is called, false to test only reading part
	Bool_t fDoOutput;
	//! if true, Write is called in the SetOupt method
	Bool_t fDoWrite;	
	//! if true, Record is called 	
	Bool_t fDoRecord;
		
protected:	
	//! The Trigger
	ADF::AgataFrameTrigger fTrigger;	
	
protected:
	virtual UInt_t SetInput();
	virtual UInt_t Process();	
	virtual UInt_t SetOutput();
	
public:
 	TestFIO();
 	virtual ~TestFIO(); 
			
	//! to init globals (static) from a directory
	static void process_config (const Char_t *, UInt_t *) ;
	
	//! Constructor implementation 	
	virtual	void process_initialise (UInt_t *error_code);
	
	//! Destructor implementation  
	virtual	void process_reset (UInt_t *error_code);
		
	//! To process just a block of data
	/*!
		
	*/	
	virtual UInt_t ProcessBlock (ADF::FrameBlock &, ADF::FrameBlock &);	      

	virtual	void process_start (UInt_t *error_code);
	virtual	void process_stop (UInt_t *error_code);
	virtual	void process_pause (UInt_t *error_code);
	virtual	void process_resume (UInt_t *error_code);
	
	
//	virtual	void process_unload (UInt_t *error_code);
};

#if HAVE_NARVAL_H == 1 && DUMMY_FILTER == 1
extern "C" {
/* you must have the following symbols */
/* see John Cresswell document for details : */
/* AGATA PSA and Tracking Algorithm Integration*/
//!
void process_config (const Char_t *directory_path, UInt_t *error_code);
//!
TestFIO *process_register (UInt_t *error_code);
//!
void process_unload (TestFIO *algo_data,UInt_t *error_code);

//!
void process_block (TestFIO *algo,
		    void *input_buffer,
                    UInt_t size_of_input_buffer,
		    void *output_buffer,
		    UInt_t size_of_output_buffer,
		    UInt_t *used_size_of_output_buffer,
		    UInt_t *error_code);
/* optionnal symbols */
/* this symbol isn't documented for the moment, it should contain 0 or the */
/* maximum size of one event, value 0 is safe */
int max_size = 0;
void process_initialise (TestFIO *algo_data, UInt_t *error_code);
void process_reset (TestFIO *algo_data,UInt_t *error_code);
void process_start (TestFIO *algo_data,UInt_t *error_code);
void process_stop (TestFIO *algo_data,UInt_t *error_code);
void process_pause (TestFIO *algo_data,UInt_t *error_code);
void process_resume (TestFIO *algo_data, UInt_t *error_code);
} // extern C
#endif
#endif
