/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** @file TrackedFrame.h header file for TrackedFrame.cpp */

#ifndef ADF_TrackedFrame
#define ADF_TrackedFrame

#ifndef ADF_AgataDataFrame
#include "AgataDataFrame.h"
#endif
#ifndef ADF_ADFObjects
#include "ADFObjects.h"
#endif

namespace ADF
{ 

class AgataFrameFactory;

//! The tracking algorithm produces a stack of TrackedHits
/*!
*/
class GammaTrackedInterface : public DataInterface
{
public:
	GammaTrackedInterface(): 
		DataInterface() {;} 
	virtual ~GammaTrackedInterface() 
		{;} 
	
	//! To set the current number of gammas. Returns false if not possible
	/*!
		0 reset the current stack. if do_reset is set to false, the objects added
		to the stack are not reset (means the users has to fully change the content to be safe)
	*/	
	virtual Bool_t SetNbGamma(UShort_t how_many, Bool_t do_reset = true) = 0; 
	
	//! Add a gamma on the top of the current stack
	virtual TrackedHit *NewGamma() = 0;      
	
	//! To get the current number of gammas in the stack
	virtual UShort_t  GetNbGamma() const = 0;       
	
 	//! To get the current number of gammas in the stack     
	virtual TrackedHit *GetGamma(UShort_t) const = 0; 
};

//! General interface for a Tracked Frame
/*!
*/
class ATrackedFrame : public AgataDataFrame< GammaTrackedInterface > 
{
	friend class AgataFrameFactory;
protected:
	ATrackedFrame(Key *key): 
		AgataDataFrame< GammaTrackedInterface >(key) {;}  
public:
	virtual ~ATrackedFrame()
		{;}   
};

//! Proxy for a TrackedFrame ... help to have the same 
// typedef ProxyDataFrame<ATrackedFrame,GammaTrackedInterface> TrackedFrame;

} // namespace ADF
#endif



