/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** @file Version.h header file for Version.cpp */

#ifndef ADF_Version
#define ADF_Version
#define ADF_VERSION_H ADF_Version

#ifndef ADF_ADFConfig
#include "ADFConfig.h"
#endif

namespace ADF
{ 

//! Base class for version numbers 
/*!
	A version is a major and a minor number.
*/
class Version
{
private:
	UShort_t fMajor;	// major number of the revision
	UShort_t fMinor;	// minor number of the revision
		
public:
	Version();
	Version(UShort_t,UShort_t);  
	~Version(); 
	
	std::string GetString() const;

	//! copy constructor       
	Version(const Version &);  
	
	//! to compare two bersions, must have the same minor and major    
	Bool_t operator==(const Version &) const;
	
	void SetMajor(UShort_t maj) { fMajor = maj; }
	void SetMinor(UShort_t min) { fMinor = min; }
	
	UShort_t GetMajor() const { return fMajor; }
	UShort_t GetMinor() const { return fMinor; }
	
	//! to change both values
	/*!
	\code
		Version v; // minor = major = 0
		v(0,1);    // major = 0, minor = 1; 
	\endcode
	*/   
	void operator () (UShort_t maj, UShort_t min) 
		{ fMajor = maj; fMinor = min; }
};

} // namespace ADF
#endif





