/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   aLong_t with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file DummyFilter.cpp compiled in libADF.so !! only in standalone mode !! */

#ifndef ADF_DUMMYFILTER_H
#include "DummyFilter.h"
#endif

#ifndef ADF_AgataKeyFactory
#include "AgataKeyFactory.h"
#endif
#ifndef ADF_AgataFrameFactory
#include "AgataFrameFactory.h"
#endif

#include <iostream>

using namespace ADF; 

// Bool_t ADF::AgataKeyWithEvent(const FactoryItem &item_from, const FactoryItem &item_to)
// {
// 	Bool_t ok = item_from.IsAVersion(item_to)
// 	if ( item_from.IsAVersion(item_to) )
// 		;
// }

DummyFilter::DummyFilter() : 
	NarvalFilter(), 
	AlgoDummy(), 
	fTrigger("PSA"), 
	fTriggerEvent("Embedded PSA")
{	
	Log.GetProcessName() = "DummyFilter";
	fFrameIO.SetName("DummyFilter");
}

DummyFilter::~DummyFilter() 
{  
	// in principle not needed ... just in case reset has not been called by narval
	UInt_t error = 0u; process_reset(&error) ; 
} 

void DummyFilter::process_config (const Char_t *directory_path, UInt_t *error_code)
{
	// In case your algorithm needs to initiate globals, it should be done here
	// For instance, if you need an proper FrameFactory:
	//	FrameFactory &main_frame_factory = FrameFactory::theMainFactory(); 
	//	main_frame_factory.Register( new MyFrameFactory() );
	
	// Globals for ADF as read from ADF.conf (should be in your algo path !!)
	NarvalInterface::process_config(directory_path,error_code);
  
	if ( (*error_code) == 0u ) { // now init your stuff
	
  		// here could come the reading of a file in which are stored the 
		// units (position,energy) you are working with i.e. modifications
		// of UnitPosition UnitEnergy defined in the base class AlgoDummy
		// + ....
		// NarvalInterface (through process_config(), so don't forget it) 
		// keeps the name of the directory_path. To get it back: GetConfPath()
	}
}

Int_t DummyFilter::SetInput()
{	
	EvoPSAFrame *psaframe; PSAInterface *data;
	if ( fTrigger.IsFired() ) {
		psaframe = dynamic_cast< EvoPSAFrame * > (fTrigger.GetInputFrame(0u));
	}
	if ( fTriggerEvent.IsFired() ) {
		psaframe = dynamic_cast< EvoPSAFrame * > (fTriggerEvent.GetInputFrame(1u));
	}
	data = psaframe->Data();
	
	// loads the values from the frame into the ADFObject attached to the frame
	psaframe->Read();
	
	if ( data->GetNbHits() >  MAXSIZE ) {  // cannot be treated
	
		Log.SetProcessMethod("SetInput()");
		Log << error << " Too many hits to be treated " <<  data->GetNbHits() << dolog;
		 		
		Mult = 0u; return 1; 
	} 
	
	// now copy in your local variables	
	Mult = data->GetNbHits();
	for (UInt_t nbhit = 0u; nbhit < Mult; nbhit++ ) {

		const Hit *ahit = data->GetHit(nbhit);	 
		
		X[nbhit] = UnitPosition*ahit->GetX();
		Y[nbhit] = UnitPosition*ahit->GetY();
		Z[nbhit] = UnitPosition*ahit->GetZ();	
		
		E[nbhit] = UnitEnergy*ahit->GetE();	
	}
	return 0;
}

Int_t DummyFilter::SetOutput()
{
	TrackedFrame *trackframe; GammaTrackedInterface *data;
	if ( fTrigger.IsFired() ) {
		trackframe = dynamic_cast< TrackedFrame * > (fTrigger.GetOutputFrame());
	}
	if ( fTriggerEvent.IsFired() ) {
		trackframe = dynamic_cast< TrackedFrame * > (fTriggerEvent.GetOutputFrame());
	}	
	data = trackframe->Data();
	
	Int_t result = 0;
	trackframe->Reset(); // MANDATORY to start filling a new Frame to set # of gammas to 0
		
	// hits	
//	for (UInt_t i = 0u; i < 3; i++ ) {

		TrackedHit *ahit = data->NewGamma();
			
		ahit->SetE(EM);
		ahit->SetXYZ(XM,YM,ZM); ahit->SetStatus(ID);
		
	std::cout << ahit->GetStatus() << std::endl;	
//	}

	// write everything in the frame
	if ( trackframe->Write() == 0u ) 
		result = 1; 
	
	return result;
}
	
UInt_t DummyFilter::ProcessBlock (ADF::FrameBlock &in, ADF::FrameBlock &out)
{
	Log.ClearMessage(); Log.GetProcessMethod("DummyFilter::process_block");

  	// attach the input/output buffer to the FrameIO system
 	fFrameIO.Attach(&in,&out);
		
	// start the processing
 	UInt_t error_code = 0u;
	while ( fFrameIO.Notify() ) { // process the input 
		
		// fill local variables with data from the input 
		// process the input buffer
		// fill the output buffer
  		if ( ! (SetInput() == 0u && Process() == 0u && SetOutput() == 0u) )  
			{ error_code =  1u; break; } 
		else {
			// ok, so send the produced frame to the ouput
			if ( !fFrameIO.Record() )
				{ error_code =  2u; break; }
		}
		
		
	}
	if ( error_code == 1u ) {
		Log << error 
		    << " During Algo processing : SetInput -> Process -> SetOuput() " << nline;
	}
	if ( error_code == 2u ) {
		Log.GetProcessMethod() = "DummyFilter::process_block";
		Log << error 
		    << " During Record() " << nline;
	}	
	fFrameIO.Detach(&in,&out);
	
	Lof << dolog;
	return error_code;
}	      
	
// void DummyFilter::process_block (void *input_buffer,
// 			      UInt_t size_of_input_buffer,
// 			      void *output_buffer,
// 			      UInt_t size_of_output_buffer,
// 			      UInt_t *used_size_of_output_buffer,
// 			      UInt_t *error_code)
// {  
// 
// }

void DummyFilter::process_initialise (UInt_t *error_code)
{
	// log related
	Log.ClearMessage(); Log.SetProcessMethod("process_initialise");
	(*error_code) = 0;
	
// definition of the frames needed by this
	EvoPSAFrame *psaframe; Frame *frame; Version vkey(1,0), vframe(65000,0);
	
// Trigger 1, trigger on evolutive psa frames
	frame = fTrigger.Add("data:psa",vkey,vframe,true,true);
	if ( frame ) {
		// to link some named item with the local variables.
		psaframe = dynamic_cast< EvoPSAFrame * > (frame);
		GObject *glob
			= psaframe->Data()->Global();
		glob->LinkItem("CrystalID","US",&ID); 
	}
	fTrigger.SetOutputFrame("data:tracked",vkey,vframe);
	
	// the trigger is registered
	fFrameIO.Register(&fTrigger);
	
// Trigger 2, trigger on psa event frame
	fTriggerEvent.Add("data:event",vkey,vkey,true);
	
	frame = fTriggerEvent.Add("data:psa",vkey,vframe,true,true);	
	if ( frame )  {
		// to link some named item with the local variables.
		psaframe = dynamic_cast< EvoPSAFrame * > (frame);
		GObject *glob
			= psaframe->Data()->Global();
		glob->LinkItem("CrystalID","US",&ID);
	}
	fTriggerEvent.SetOutputFrame("data:tracked",vkey,vframe);
	
	// the trigger is registered
	fFrameIO.Register(&fTriggerEvent);
	
// ready to run ..
	fFrameIO.SetStatus(BaseFrameIO::kIdle);
	
	Log << dolog; 
		
	(*error_code) = 0;
}


void DummyFilter::process_reset (UInt_t *error_code)
{
	Log.ClearMessage(); Log.SetProcessMethod("process_reset"); 
	
	NarvalFilter::process_reset(error_code);
		
	Log << dolog;

}
void DummyFilter::process_start (UInt_t *error_code)
{
	
	Log.SetProcessMethod("process_block"); 
	(*error_code) = 0;
}

void DummyFilter::process_stop (UInt_t *error_code)
{
	std::cout << "process_stop called with GetPID()" << GetPID() << std::endl;
	*error_code = 0;
}
void DummyFilter::process_pause (UInt_t *error_code)
{
	std::cout << "process_pause called with GetPID()" << GetPID() << std::endl;
	*error_code = 0;
}
void DummyFilter::process_resume (UInt_t *error_code)
{
	std::cout << "process_resume called with GetPID()" << GetPID() << std::endl;
	*error_code = 0;
}
/*
void DummyFilter::process_unload (UInt_terror_code)
{
  std::cout << "process_unload called with GetPID()" << GetPID() << std::endl;
  *error_code = 0;
} */

#if HAVE_NARVAL_H == 1 && DUMMY_FILTER == 1
void process_config (const char *directory_path, UInt_t *error_code)
{
  DummyFilter::process_config (directory_path, error_code);
}

DummyFilter *process_register (UInt_t *error_code) { return new DummyFilter(); }

/*void process_unload (DummyFilter *algo_data, UInt_t *error_code)
{
  algo_data->process_unload (error_code);
}*/

void process_block (DummyFilter *algo_data,
		    void *input_buffer,
                    UInt_t size_of_input_buffer,
		    void *output_buffer,
		    UInt_t size_of_output_buffer,
		    UInt_t *used_size_of_output_buffer,
		    UInt_t *error_code)
{
  algo_data->process_block (input_buffer, size_of_input_buffer,
			    output_buffer, size_of_output_buffer,
			    used_size_of_output_buffer, error_code);
}

void process_initialise (DummyFilter *algo_data, UInt_t *error_code)
{
  algo_data->process_initialise (error_code);
}
void process_reset (DummyFilter *algo_data,UInt_t *error_code)
{
  algo_data->process_reset (error_code);
}
void process_start (DummyFilter *algo_data, UInt_t *error_code)
{
  algo_data->process_start (error_code);
}
void process_stop (DummyFilter *algo_data,UInt_t *error_code)
{
  algo_data->process_stop (error_code);
}
void process_pause (DummyFilter *algo_data,UInt_t *error_code)
{
  algo_data->process_pause (error_code);
}
void process_resume (DummyFilter *algo_data, UInt_t *error_code)
{
  algo_data->process_resume (error_code);
}
#endif




