/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   aLong_t with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** @file DummyFilter.h header file for DummyFilter.cpp */

#ifndef _DummyFilter
#define _DummyFilter


#ifndef _AlgoDummy
#include "AlgoDummy.h"
#endif
#ifndef ADF_NarvalInterface
#include "NarvalInterface.h"
#endif
#ifndef ADF_PSAFrame
#include "PSAFrame.h"
#endif
#ifndef ADF_TrackedFrame
#include "TrackedFrame.h"
#endif
#ifndef ADF_AgataCompositeFrame
#include "AgataCompositeFrame.h"
#endif
#ifndef ADF_Trigger
#include "Trigger.h"
#endif

//! dummy algorithm embedded in Narval
/*!
	It reads PSAFrame and calculates the means values x,y,z and e to produce
	tracked frames with thus just one gamma.
*/
class DummyFilter : public ADF::NarvalFilter, public AlgoDummy 
{   
private:	
	//! Trigger for the input frame
	//! in case psa frames are at the first level in the dataflow
	ADF::AgataFrameTrigger fTrigger;
	//! in case psa frames are embedded in an agataeventframe
	ADF::AgataFrameTrigger fTriggerEvent;
	
public:
 	DummyFilter();
 	virtual ~DummyFilter(); 
	
	//! to init your local variables with the ones from the buffers
	virtual Int_t SetInput();
	//! to copy the result of the algrithm into the frame through ADFObjects
	virtual Int_t SetOutput();
		
	//! to init globals (static) from a directory
	static void process_config (const Char_t *, UInt_t *) ;
	
	//! Constructor implementation 	
	virtual	void process_initialise (UInt_t *error_code);
	
	//! Destructor implementation  
	virtual	void process_reset (UInt_t *error_code);
		
	//! To process just a block of data
	/*!
		
	*/	
// 	virtual	void process_block (void *input_buffer,
// 		      UInt_t size_of_input_buffer,
// 		      void *output_buffer,
// 		      UInt_t size_of_output_buffer,
// 		      UInt_t *used_size_of_output_buffer,
// 		      UInt_t *error_code);
		      
	virtual UInt_t ProcessBlock (ADF::FrameBlock &, ADF::FrameBlock &);	      

	virtual	void process_start (UInt_t *error_code);
	virtual	void process_stop (UInt_t *error_code);
	virtual	void process_pause (UInt_t *error_code);
	virtual	void process_resume (UInt_t *error_code);
	
	
//	virtual	void process_unload (UInt_t *error_code);
};

#if HAVE_NARVAL_H == 1 && DUMMY_FILTER == 1
extern "C" {
/* you must have the following symbols */
/* see John Cresswell document for details : */
/* AGATA PSA and Tracking Algorithm Integration*/
//!
void process_config (const Char_t *directory_path, UInt_t *error_code);
//!
DummyFilter *process_register (UInt_t *error_code);
//!
void process_unload (DummyFilter *algo_data,UInt_t *error_code);

//!
void process_block (DummyFilter *algo,
		    void *input_buffer,
                    UInt_t size_of_input_buffer,
		    void *output_buffer,
		    UInt_t size_of_output_buffer,
		    UInt_t *used_size_of_output_buffer,
		    UInt_t *error_code);
/* optionnal symbols */
/* this symbol isn't documented for the moment, it should contain 0 or the */
/* maximum size of one event, value 0 is safe */
int max_size = 0;
void process_initialise (DummyFilter *algo_data, UInt_t *error_code);
void process_reset (DummyFilter *algo_data,UInt_t *error_code);
void process_start (DummyFilter *algo_data,UInt_t *error_code);
void process_stop (DummyFilter *algo_data,UInt_t *error_code);
void process_pause (DummyFilter *algo_data,UInt_t *error_code);
void process_resume (DummyFilter *algo_data, UInt_t *error_code);
} // extern C
#endif
#endif
