#include "ADFLogCollector.h"

#include "dev/DummyFilter.h"
#include "BasicAFP.h"
#include "BasicAFC.h"

#include <iostream>
#include <sys/times.h>
/*
compilation: 
	g++ -O -Wall -fPIC -I../ -L../ -lADF -lDUMMY dummy0.1.c -o dummy0.1
	g++ -g -pg -I../ -L../ -lADF -lDUMMY dummy0.1.c -o dummy0.1 */

using namespace ADF;

int main(int argc, char **argv)
{	
	// A log collector is created to collector logs in a file
	// it becomes automatically the current collector
	// WARNING !! should be pointer since all collectors are collected
	// and destroyed at the end by the global collector
 	OStreamCollector *dummy_collector = new OStreamCollector("TEST.log");
	
	UInt_t error = 0u;
	
	// buffers to pass data from one actor to another one
	const UInt_t size_buf_in  = 400*aKByte;
	const UInt_t size_buf_out = aMByte;	
	Char_t *buf1 = NULL;
	Char_t *buf2 = NULL;	
	buf1 = BaseBuffer::New(buf1,size_buf_in);
	buf2 = BaseBuffer::New(buf2,size_buf_out); 
	
	// it creates and inits the different actors
	BasicAFP::process_config("./",&error); 
	if ( error > 0u ) 
		printf("Error in BasicAFP::process_config %d \n ",error);
	BasicAFP *pro = new BasicAFP(); 
	pro->process_initialise(&error);
	if ( error > 0u ) 
		printf("Error in BasicAFP::process_config %d \n ",error);
	DummyFilter::process_config("./",&error);
	if ( error > 0u ) 
		printf("Error in BasicAFP::process_config %d \n ",error);	
	
	DummyFilter *filter = new DummyFilter() ; 
	filter->process_initialise(&error);	
	if ( error > 0u ) 
		printf("Error in BasicAFP::process_config %d \n ",error);
	BasicAFC::process_config("./",&error);
	if ( error > 0u ) 
		printf("Error in BasicAFP::process_config %d \n ",error);
	BasicAFC *cons = new BasicAFC() ; 
	cons->process_initialise(&error); 
	if ( error > 0u ) 
		printf("Error in BasicAFP::process_config %d \n ",error);
		
	// loop on events
	UInt_t realsize1 = 1u, realsize2, nb_loop = 0u;
	struct tms t0, t; times(&t0);	
	while ( error == 0u ) {
		// printf("nb_loop_in %d \n",nb_loop);
		// to get an input block
		pro->process_block(buf1,size_buf_in,&realsize1,&error); if ( realsize1 == 0u ) break;
		printf("realsize1 %d \n",realsize1);
		//
		filter->process_block(buf1,realsize1,buf2,size_buf_out,&realsize2,&error);
		//
		printf("realsize2 %d \n",realsize2);
		cons->process_block(buf2,realsize2,&error); nb_loop++;
	}
	printf("nb_loop %d %d\n",nb_loop,error);	
	// 
	printf("\n");
	times(&t);
	std::cout << "User " << Float_t(t.tms_utime-t0.tms_utime)/sysconf(_SC_CLK_TCK)
	     << " second, System " << Float_t(t.tms_stime-t0.tms_stime)/sysconf(_SC_CLK_TCK) 
	     << " seconds" << std::endl; 
	     
	delete cons;
	delete filter; 
	delete pro; 

	BaseBuffer::Delete(buf2);BaseBuffer::Delete(buf1);
	return 0;
}
