

#include "ADFLogCollector.h"

#ifndef _BasicAFP
#include "BasicAFP.h"
#endif
#ifndef _BasicAFC
#include "BasicAFC.h"
#endif

#ifndef _DummyFilter
#include "DummyFilter.h"
#endif

#include <sys/times.h>

#include <iostream>
#include <sstream>
#include <string>

/*
compilation: 
	g++ -O -Wall -fPIC -I../ -L../ -lADF -lDUMMY dummy0.2.c -o dummy0.2
	g++ -g -pg -I../ -L../ -lADF -lDUMMY dummy0.2.c -o dummy0.2 
	
--> DEBUG 
Linux SL4

--> VALGRIND

Memory leak
Profiling
	
*/

using namespace ADF;

int main(int argc, char **argv)
{		
	const char *MainPATH = "./"; 	
	std::string tmp_path;
	
	// to-pre initiate the ADF system (not mandatory)
	UInt_t error_adf = 0u;	
	NarvalInterface::process_config(MainPATH,&error_adf);
	if ( error_adf > 0u ) 
		{ printf("Error in NarvalInterface::process_config %d \n ",error_adf); return 1; }
			
	// A log collector is created to collector logs in a file
	// it becomes automatically the current collector
	// WARNING !! should be pointer since all collectors are collected
	// and destroyed at the end by the global collector
 	OStreamCollector *dummy_collector = new OStreamCollector("TEST.log");
	
	// block going from producer to filter and from filter to consumer
	OneBlock FB0_1(aMByte);  
	OneBlock FB1_2(aMByte);  
		
	// it creates and inits the different actors
	// producer
	UInt_t error_pro = 0u;
	tmp_path = MainPATH;
	BasicAFP::process_config(tmp_path.data(),&error_pro); 
	if ( error_pro > 0u ) 
		printf("Error in BasicAFP::process_config %d \n ",error_pro);
	BasicAFP *pro = new BasicAFP(); 
	pro->process_initialise(&error_pro);
	if ( error_pro > 0u ) 
		printf("Error in BasicAFP::process_config %d \n ",error_pro);
		
	// filter
	UInt_t error_filter = 0u;
	tmp_path = MainPATH;
	DummyFilter::process_config(tmp_path.data(),&error_filter);
	if ( error_filter > 0u ) 
		printf("Error in DummyProducer::process_config %d \n ",error_filter);
	DummyFilter *filter = new DummyFilter() ; 
	filter->process_initialise(&error_filter);	
	if ( error_filter > 0u ) 
		printf("Error in DummyProducer::process_config %d \n ",error_filter);

	// consumer
	UInt_t error_cons = 0u;
	tmp_path = MainPATH;	
	BasicAFC::process_config(tmp_path.data(),&error_cons); 
	if ( error_cons > 0u ) 
		printf("Error in BasicAFC::process_config %d \n ",error_cons);
	BasicAFC *cons = new BasicAFC(); 
	cons->process_initialise(&error_cons);
	if ( error_cons > 0u ) 
		printf("Error in BasicAFC::process_config %d \n ",error_cons);
		
	// loop on events
	UInt_t nb_loop = 0u, error = 0u; struct tms t0, t; times(&t0);	
	while ( error == 0u ) {
		// to get an input block
		printf("Call producer \n");		
		pro->ProcessBlock(FB0_1); 
		if ( FB0_1.GetSize() == 0u )
			break;
		else
			printf("Produced block with size %d \n",FB0_1.GetSize());	
		// to apply the algorithm
		printf("Call filter \n");		
		filter->ProcessBlock(FB0_1,FB1_2);
		if ( FB1_2.GetSize() == 0u )
			break;
		else
			printf("Output filter size %d \n",FB1_2.GetSize());
		// to save the outpout of the algorithm
		printf("Call consumer \n");
		cons->ProcessBlock(FB1_2); 
// 		cons->ProcessBlock(FB0_1); 	
		nb_loop++;
// 		if ( nb_loop == 1u )
// 			break;
	}
	// 
	printf("\n");
	printf("[[ dummy0.2 ]] : BENCHMARKS \n[...\n");
	printf("nb_loop %d %d\n",nb_loop,error);
	times(&t);
	std::cout << "User " << Float_t(t.tms_utime-t0.tms_utime)/sysconf(_SC_CLK_TCK)
	     << " second, System " << Float_t(t.tms_stime-t0.tms_stime)/sysconf(_SC_CLK_TCK) 
	     << " seconds" << std::endl; 
	printf("...]\n\n");
	  
	if ( pro ) 
		delete pro; 
	if ( filter )
		delete filter;
	if ( cons )
		delete cons;
		
	return 0;
}

