

#include "ConfAgent.h"
#include "AgataKeyFactory.h"
#include "AgataFrameFactory.h"
#include "BufferIO.h"
#include "ConfigurationFrame.h"
 
#ifndef ADF_PSAFrame
#include "PSAFrame.h"
#endif
#ifndef ADF_TrackedFrame
#include "TrackedFrame.h"
#endif
#ifndef ADF_AgataCompositeFrame
#include "AgataCompositeFrame.h"
#endif

#include <iostream>
#include <sstream>
#include <string>

/*
compilation: 
	g++ -O -Wall -fPIC -I../ -L../ -lADF  producer.c -o producer
	g++ -g -pg -I../ -L../ -lADF  producer.c -o producer 
	
--> DEBUG 
Linux SL4

--> VALGRIND

Memory leak
Profiling
	
*/

using namespace ADF;

//! To produce quickly two ADF files filled with just one type of Agata frame (stallion)
/*!
	The two files start with a global configuration frame and are named AFP_0.adf and AFP_0.adf 
	so that they could be processed by the TestFIO filter (test program).
*/
int test1(UInt_t nb = 5000)
{
	// To customise
	// the factory for everything	
	const char factory[] = "Agata";
	// the type of the produced frame	
	const char frame_type[] = "data:psa";
	// the version of the primary key and the frame produced
	Version key_v(2,0), frame_v(65000,1); 	
	// end to customize ... works without any changes after	
	
	// FactoryItem 
	FactoryItem prikeydef(factory,frame_type,key_v);	
	FactoryItem aframedef(factory,frame_type,frame_v);
	
	// Check factories are loaded (in principle done at least for the factories defined in ADF)
	if ( !FrameFactory::IsKnown(aframedef.GetFactoryName().data()) )
		{ std::cout << " Agata Factory not properly loaded " << std::endl; return 1; }
		
	// Ask global agent to fill a Configuration frame which is dumped
	Frame *frame = NULL;
	frame = FrameFactory::New(prikeydef,aframedef);
	
	// change the agent to take into account this configuration
	ConfAgent::theGlobalAgent()->SetComment("from producer.c");
	ConfAgent::theGlobalAgent()->AddKnownFrame(frame,'r');	
	
	Frame *cframe = NULL;
	cframe = FrameFactory::New(FactoryItem(factory,"conf:global",key_v),
				   FactoryItem(factory,"conf:global",frame_v));
		
	ConfigurationFrame *gconfframe = NULL;	
	if ( cframe ) 
		gconfframe = dynamic_cast<ConfigurationFrame *>(cframe);
		
	ConfAgent::theGlobalAgent()->Configure(gconfframe,"out");
	gconfframe->Dump("AFP_0.adf",true);
	gconfframe->Dump("AFP_1.adf",true);
		
	for( UInt_t i = 0u; i < nb; i++ ){
		frame->Stallion();
		if ( i < nb / 2 ) 
			frame->Dump("AFP_0.adf");
		else 
			frame->Dump("AFP_1.adf");	
	}

	if ( frame ) 
		delete frame;
	if ( gconfframe ) 
		delete gconfframe;	
	
	return 0;
}

//! To produce quickly two ADF files filled with two types of Agata frame (stallion)
/*!
	The two files start with a global configuration frame and are named AFP_0.adf and AFP_0.adf 
	so that they could be processed by the TestFIO filter (test program).
	
	It shows how Frames without triggers are just passed without treatment
*/
int test2(UInt_t nb = 5000)
{
	// To customise
	// the factory for everything 
	const char factory[] = "Agata"; 
	// the two types of embedded frames
	const char frame_type1[] = "data:psa"; 
	const char frame_type2[] = "data:crystal";
	// the version of the primary key and the frames to be embedded	
	Version key_v(1,0), frame_v1(65000,0),frame_v2(65000,0); 	
	// end to customize ... works without any changes after	
	
	FactoryItem prikeydef1(factory,frame_type1,key_v);
	FactoryItem aframedef1(factory,frame_type1,frame_v1);
	FactoryItem prikeydef2(factory,frame_type2,key_v);	
	FactoryItem aframedef2(factory,frame_type2,frame_v2);
// 	FactoryItem prikeydef(factory,"agata",key_v);

	// Check factories are loaded (in principle done at least for the factories defined in ADF)
	if ( !FrameFactory::IsKnown(aframedef1.GetFactoryName().data()) )
		{ std::cout << " Agata Factory not properly loaded " << std::endl; return 1; }
		
	// Ask global agent to fill a Configuration frame which is dumped
	Frame *frame1 = NULL;
	frame1 = FrameFactory::New(prikeydef1,aframedef1);
	Frame *frame2 = NULL;
	frame2 = FrameFactory::New(prikeydef2,aframedef2);
	
	// change the agent to take into account this configuration	
	ConfAgent::theGlobalAgent()->SetComment("from producer.c");
	
	ConfAgent::theGlobalAgent()->AddKnownFrame(frame1,'r');	
	ConfAgent::theGlobalAgent()->AddKnownFrame(frame2,'+');	
				
	Frame *cframe = NULL;
	cframe = FrameFactory::New(FactoryItem(factory,"conf:global",key_v),
				   FactoryItem(factory,"conf:global",key_v));
		
	ConfigurationFrame *gconfframe = NULL;	
	if ( cframe ) 
		gconfframe = dynamic_cast<ConfigurationFrame *>(cframe);
		
	ConfAgent::theGlobalAgent()->Configure(gconfframe,"out");
	gconfframe->Dump("AFP_0.adf",true);
	gconfframe->Dump("AFP_1.adf",true);
		
	for( UInt_t i = 0u; i < nb; i++ ){
		frame1->Stallion();
		frame2->Stallion();
		if ( i < nb / 2 ) {
			frame1->Dump("AFP_0.adf");
			frame2->Dump("AFP_0.adf");
		}
		else {
			frame1->Dump("AFP_1.adf");
			frame2->Dump("AFP_1.adf");
		}
	}

	if ( frame1 ) 
		delete frame1;
	if ( frame2 ) 
		delete frame2;	
	if ( gconfframe ) 
		delete gconfframe;	
	
	return 0;
}

//! To produce quickly two ADF files filled with two types of Agata frame (stallion) emmbedded in an data:event frame
/*!
	The two files start with a global configuration frame and are named AFP_0.adf and AFP_0.adf 
	so that they could be processed by the TestFIO filter (test program).
	
	It shows how to produce CompositeFrame, how the trigger is able to find embedded frame
	and how these Frame are extracted ready to be treated by an algorithm.To produce quickly two ADF files filled with just one type of Agata frame (stallion)
*/
int test3(UInt_t nb = 5000)
{
	// To customise
	// the factory for everything 
	const char factory[] = "Agata"; 
	// the two types of embedded frames
	const char frame_type1[] = "data:psa"; 
	const char frame_type2[] = "data:crystal";
	// the version of the primary key and the frames to be embedded
	Version key_v(2,0), frame_v1(65000,1),frame_v2(65000,0); 	
	// end to customize ... works without any changes after	
	
	FactoryItem mainkeydef(factory,"event:data",key_v);
	FactoryItem mainframedef(factory,"event:data",key_v);	
	
	FactoryItem prikeydef1(factory,frame_type1,key_v);	
	FactoryItem aframedef1(factory,frame_type1,frame_v1);
	FactoryItem prikeydef2(factory,frame_type2,key_v);
	FactoryItem aframedef2(factory,frame_type2,frame_v2);
			
	// Check factories are loaded (in principle done at least for the factories defined in ADF)
	if ( !FrameFactory::IsKnown(aframedef1.GetFactoryName().data()) )
		{ std::cout << " Agata Factory not properly loaded " << std::endl; return 1; }
		
	// Ask global agent to fill a Configuration frame which is dumped
	Frame *mainframe  = NULL;
	mainframe  = FrameFactory::New(mainkeydef,mainframedef);
	
	Frame *frame1 = NULL;
	frame1 = FrameFactory::New(prikeydef1,aframedef1); 
	Frame *frame2 = NULL;
	frame2 = FrameFactory::New(prikeydef2,aframedef2); 
		
	Frame *cframe = NULL;
	cframe = FrameFactory::New(FactoryItem("Agata","conf:global",key_v),
				   FactoryItem("Agata","conf:global",frame_v1));

	// change the agent to take into account this configuration	
	ConfAgent::theGlobalAgent()->SetComment("from producer.c");
	ConfAgent::theGlobalAgent()->AddKnownFrame(mainframe,'r');
	ConfAgent::theGlobalAgent()->AddKnownFrame(frame1,'+');	
	ConfAgent::theGlobalAgent()->AddKnownFrame(frame2,'+');	
			
	ConfigurationFrame *gconfframe = NULL;	
	if ( cframe ) 
		gconfframe = dynamic_cast<ConfigurationFrame *>(cframe);
		
	ConfAgent::theGlobalAgent()->Configure(gconfframe,"out");
	gconfframe->Dump("AFP_0.adf",true);
	gconfframe->Dump("AFP_1.adf",true);
		
	for( UInt_t i = 0u; i < nb; i++ ){
	
		mainframe->Reset();
		frame1->Reset(); frame2->Reset();
		
		frame1->Stallion();
		frame2->Stallion();

		mainframe->AddSubFrame((*frame2));
		mainframe->AddSubFrame((*frame1));
		
		if ( i < nb / 2 ) 
			mainframe->Dump("AFP_0.adf");
		else 
			mainframe->Dump("AFP_1.adf");
	}

	if ( mainframe ) 
		delete mainframe;
	if ( frame1 ) 
		delete frame1;
	if ( frame2 ) 
		delete frame2;	
	if ( gconfframe ) 
		delete gconfframe;	
		
	return 0;
}

UShort_t crystal_id = 0u, crystal_status = 0u, PSAStatus = 0; // to test Global
Short_t T0;
Float_t CoreE1, CoreE2;
	
void PSAStallion(Frame *frame)
{
	PSAFrame *psai = dynamic_cast<PSAFrame *>(frame);
	
	frame->Reset();
	
	psai->Data()->Global()->LinkItem("CrystalID",&crystal_id); 
	psai->Data()->Global()->LinkItem("CrystalStatus",&crystal_status); 	
	psai->Data()->Global()->LinkItem("PSAStatus",&PSAStatus); 
	psai->Data()->Global()->LinkItem("T0",&T0); 
	psai->Data()->Global()->LinkItem("CoreE1",&CoreE1); 	
	psai->Data()->Global()->LinkItem("CoreE2",&CoreE2); 
	
	crystal_id++; crystal_status++; PSAStatus--; 
	T0 = 69;
	CoreE1 = 1332.516;
	CoreE2 = 1332.;

	// fill the hits
	for (UInt_t h = 0u; h < 3u; h++){
		
		Hit *ahit = psai->Data()->NewHit();
		
		if ( h == 0u ) { ahit->SetE(1000.); ahit->SetDE(2.5); } 
		if ( h == 1u ) { ahit->SetE(300.);  ahit->SetDE(1.2); } 
		if ( h == 2u ) { ahit->SetE(32.516);ahit->SetDE(0.5); } 
		
		ahit->SetT(1.2*h);  
		ahit->SetDT(1.2*h);
			
		ahit->SetXYZ(5.,10.,15.); 
		ahit->SetDXYZ(1.,2.,3.);
			
		ahit->SetStatus(h);
	} 
	frame->Write();
}

//! To produce quickly two ADF files filled with just one type of Agata frame (PSA with data)
/*!
	The two files start with a global configuration frame and are named AFP_0.adf and AFP_0.adf 
	so that they could be processed by the DummyFilter filter (dummmy0.2 program).
*/
int dummy1(UInt_t nb = 5000)
{
	// To customise
	// the factory for everything	
	const char factory[] = "Agata";
	// the type of the produced frame	
	const char frame_type[] = "data:psa";
	// the version of the primary key and the frame produced
	Version key_v(2,0), frame_v(65000,1); 	
	// end to customize ... works without any changes after	
	
	// FactoryItem 
	FactoryItem prikeydef(factory,frame_type,key_v);	
	FactoryItem aframedef(factory,frame_type,frame_v);
			
	// Check factories are loaded (in principle done at least for the factories defined in ADF)
	if ( !FrameFactory::IsKnown(aframedef.GetFactoryName().data()) )
		{ std::cout << " Agata Factory not properly loaded " << std::endl; return 1; }
		
	// Ask global agent to fill a Configuration frame which is dumped
	Frame *frame = NULL;
	frame = FrameFactory::New(prikeydef,aframedef);
	
	Frame *cframe = NULL;
	cframe = FrameFactory::New(FactoryItem(factory,"conf:global",key_v),
				   FactoryItem("Agata","conf:global",frame_v));
		
	ConfigurationFrame *gconfframe = NULL;	
	if ( cframe ) 
		gconfframe = dynamic_cast<ConfigurationFrame *>(cframe);
		
	// change the agent to take into account this configuration
	ConfAgent::theGlobalAgent()->SetComment("from producer.c");
	ConfAgent::theGlobalAgent()->AddKnownFrame(frame,'r');	
		
	ConfAgent::theGlobalAgent()->Configure(gconfframe,"out");
	gconfframe->Dump("AFP_0.adf",true);
	gconfframe->Dump("AFP_1.adf",true);
		
	for( UInt_t i = 0u; i < nb; i++ ){
		PSAStallion(frame);
		if ( i < nb / 2 ) 
			frame->Dump("AFP_0.adf");
		else 
			frame->Dump("AFP_1.adf");
	}

	if ( frame ) 
		delete frame;
	if ( gconfframe ) 
		delete gconfframe;	

	return 0;
}

//! To produce quickly two files with two types of Agata frame (PSA with data and crystal stallion) emmbedded in an data:event frame
/*!

*/
int dummy2(UInt_t nb = 5000)
{
	// To customise
	// the factory for everything 
	const char factory[] = "Agata"; 
	// the two types of embedded frames, the first one should stay data:psa
	const char frame_type1[] = "data:psa"; 
	const char frame_type2[] = "data:crystal";
	// the version of the primary key and the frames to be embedded
	Version key_v(2,0), frame_v1(65000,1),frame_v2(65000,0); 	
	// end to customize ... works without any changes after	
	
	FactoryItem mainkeydef(factory,"event:data",key_v);
	FactoryItem mainframedef(factory,"event:data",key_v);	
	
	FactoryItem prikeydef1(factory,frame_type1,key_v);	
	FactoryItem aframedef1(factory,frame_type1,frame_v1);
	FactoryItem prikeydef2(factory,frame_type2,key_v);
	FactoryItem aframedef2(factory,frame_type2,frame_v2);
			
	// Check factories are loaded (in principle done at least for the factories defined in ADF)
	if ( !FrameFactory::IsKnown(aframedef1.GetFactoryName().data()) )
		{ std::cout << " Agata Factory not properly loaded " << std::endl; return 1; }
		
	// Ask global agent to fill a Configuration frame which is dumped
	Frame *mainframe  = NULL;
	mainframe  = FrameFactory::New(mainkeydef,mainframedef);
	
	Frame *frame1 = NULL;
	frame1 = FrameFactory::New(prikeydef1,aframedef1); 
	Frame *frame2 = NULL;
	frame2 = FrameFactory::New(prikeydef2,aframedef2); 
		
	Frame *cframe = NULL;
	cframe = FrameFactory::New(FactoryItem("Agata","conf:global",key_v),
				   FactoryItem("Agata","conf:global",frame_v1));
		
	ConfigurationFrame *gconfframe = NULL;	
	if ( cframe ) 
		gconfframe = dynamic_cast<ConfigurationFrame *>(cframe);
		
	// change the agent to take into account this configuration	
	ConfAgent::theGlobalAgent()->SetComment("from producer.c");
	ConfAgent::theGlobalAgent()->AddKnownFrame(mainframe,'r');
	ConfAgent::theGlobalAgent()->AddKnownFrame(frame1,'+');	
	ConfAgent::theGlobalAgent()->AddKnownFrame(frame2,'+');	
		
	ConfAgent::theGlobalAgent()->Configure(gconfframe,"out");
	gconfframe->Dump("AFP_0.adf",true);
	gconfframe->Dump("AFP_1.adf",true);
		
	for( UInt_t i = 0u; i < nb; i++ ){
	
		mainframe->Reset(); frame1->Reset(); frame2->Reset();
		
		PSAStallion(frame1); 
		frame2->Stallion();
		
		mainframe->AddSubFrame((*frame1));
		mainframe->AddSubFrame((*frame2));
		
		if ( i < nb / 2 ) 
			mainframe->Dump("AFP_0.adf");
		else 
			mainframe->Dump("AFP_1.adf");
	}

	if ( mainframe ) 
		delete mainframe;
	if ( frame1 ) 
		delete frame1;
	if ( frame2 ) 
		delete frame2;	
	if ( gconfframe ) 
		delete gconfframe;	
		
	return 0;
}


//! To produce quickly two ADF files filled with just one type of Agata frame (PSA stallion) and a configuration frame
/*!
	The file starts with a global configuration frame
*/
int test4(UInt_t nb = 5000)
{
	// To customise
	// the factory for everything	
	const char factory[] = "Agata";
	// the type of the produced frame	
	const char frame_type[] = "data:psa";
	const char frame_conf[] = "conf:psa";	
	// the version of the primary key and the frame produced
	Version key_v(2,0), frame_v(65000,0); 	
	// end to customize ... works without any changes after	
	
	// FactoryItem 
	FactoryItem prikeydef(factory,frame_type,key_v);	
	FactoryItem aframedef(factory,frame_type,frame_v);

	// Check factories are loaded (in principle done at least for the factories defined in ADF)
	if ( !FrameFactory::IsKnown(aframedef.GetFactoryName().data()) )
		{ std::cout << " Agata Factory not properly loaded " << std::endl; return 1; }
		
	// Ask global agent to fill a Configuration frame which is dumped
	Frame *frame = NULL;
	frame = FrameFactory::New(prikeydef,aframedef);
	
	Frame *cframe = NULL;
	cframe = FrameFactory::New(FactoryItem(factory,"conf:global",key_v),
				   FactoryItem("Agata","conf:global",frame_v));
		
	ConfigurationFrame *gconfframe = NULL;	
	if ( cframe ) 
		gconfframe = dynamic_cast<ConfigurationFrame *>(cframe);
		
	ConfigurationFrame *psaconfframe = NULL;
	cframe = FrameFactory::New(FactoryItem(factory,frame_conf,key_v),
				   FactoryItem(factory,frame_conf,Version()));
	if ( cframe ) 
		psaconfframe = dynamic_cast<ConfigurationFrame *>(cframe);

	// change the agent to take into account this configuration
	ConfAgent::theGlobalAgent()->SetComment("from producer.c");
	ConfAgent::theGlobalAgent()->AddKnownFrame(frame,'r');	
	ConfAgent::theGlobalAgent()->AddKnownFrame(psaconfframe,'+');
	
	ConfAgent::theGlobalAgent()->Configure(gconfframe,"out");
	gconfframe->Dump("AFP_0.adf",true);
	gconfframe->Dump("AFP_1.adf",true);
						
	for( UInt_t i = 0u; i < nb; i++ ){
		frame->Stallion();
		if ( i == nb / 4 || i == nb * 3 / 4) {
			
			// in principle the way it should be ... but for the moment, nothing declared
			// so this method does nothing and return false ... thus the filling 
			// of the pasconfframe is emulated here 
			frame->Configure(psaconfframe,"out");
			// just to emulate 
			if ( i == nb / 4 ) {
				psaconfframe->String() = 
					"[Some variables of the PSAFrame Interface are changed File 0]";
				psaconfframe->Write();
				psaconfframe->Dump("AFP_0.adf");	
			}
			else {
				psaconfframe->String() = 
					"[Some variables of the PSAFrame Interface are changed File 1]";
				psaconfframe->Write();
				psaconfframe->Dump("AFP_1.adf");	
			}
			// end just to emulate
		}
		if ( i < nb / 2 ) 
			frame->Dump("AFP_0.adf");
		else 
			frame->Dump("AFP_1.adf");
	}

	if ( frame ) 
		delete frame;
	if ( gconfframe ) 
		delete gconfframe;	
	if ( psaconfframe ) 
		delete psaconfframe;
	return 0;
}

//! To produce quickly two ADF files filled with just one type of Agata frame (PSA stallion) and a configuration frame
/*!
	The file starts with a global configuration frame
*/
int EB1(UInt_t nb = 5000)
{
	// To customise
	// the factory for everything	
	const char factory[] = "Agata";
	// the type of the produced frame	
	const char frame_type[] = "data:psa";
	const char frame_conf[] = "conf:psa";	
	// the version of the primary key and the frame produced
	Version key_v(4,0), frame_v(65000,0); 	
	// end to customize ... works without any changes after	
	
	// FactoryItem 
	FactoryItem prikeydef(factory,frame_type,key_v);	
	FactoryItem aframedef(factory,frame_type,frame_v);
		
	// Check factories are loaded (in principle done at least for the factories defined in ADF)
	if ( !FrameFactory::IsKnown(aframedef.GetFactoryName().data()) )
		{ std::cout << " Agata Factory not properly loaded " << std::endl; return 1; }
		
	// Ask global agent to fill a Configuration frame which is dumped
	Frame *frame = NULL;
	frame = FrameFactory::New(prikeydef,aframedef);
	
	Frame *cframe = NULL;
	cframe = FrameFactory::New(FactoryItem(factory,"conf:global",key_v),
				   FactoryItem(factory,"conf:global",Version()));
				   		
	ConfigurationFrame *gconfframe = NULL;	
	if ( cframe ) 
		gconfframe = dynamic_cast<ConfigurationFrame *>(cframe);	
		
	ConfigurationFrame *psaconfframe = NULL;
	cframe = FrameFactory::New(FactoryItem(factory,"conf:psa",key_v),
				   FactoryItem("Agata","conf:psa",Version()));
	if ( cframe ) 
		psaconfframe = dynamic_cast<ConfigurationFrame *>(cframe);
		
	// change the agent to take into account this configuration
	ConfAgent::theGlobalAgent()->SetComment("from producer.c");
	ConfAgent::theGlobalAgent()->AddKnownFrame(frame,'r');	
	ConfAgent::theGlobalAgent()->AddKnownFrame(psaconfframe,'+');
				
	ConfAgent::theGlobalAgent()->Configure(gconfframe,"out");
	gconfframe->Dump("AFP_0.adf",true);
	gconfframe->Dump("AFP_1.adf",true);
	
	AgataKey *fkey = 
		dynamic_cast<AgataKey *>(frame->GetKey());
	AgataKey *fckey = 
		dynamic_cast<AgataKey *>(cframe->GetKey());
		
	for( UInt_t i = 0u; i < nb; i++ ){
		frame->Stallion();
		fkey->SetEventNumber(i); fkey->SetTimeStamp(i);
			
		if ( i == nb / 4 || i == nb * 3 / 4) {
		
			fckey->SetEventNumber(i); fckey->SetTimeStamp(i);
			
			// in principle the way it should be ... but for the moment, nothing declared
			// so this method does nothing and return false ... thus the filling 
			// of the pasconfframe is emulated here 
			frame->Configure(psaconfframe,"out");
			// just to emulate 
			if ( i == nb / 4 ) {
				psaconfframe->String() = 
					"[Some variables of the PSAFrame Interface are changed File 0]";
				psaconfframe->Write();
				psaconfframe->Dump("AFP_0.adf");	
			}
			else {
				psaconfframe->String() = 
					"[Some variables of the PSAFrame Interface are changed File 1]";
				psaconfframe->Write();
				psaconfframe->Dump("AFP_1.adf");	
			}
			// end just to emulate
		}
		if ( i < nb / 2 ) 
			frame->Dump("AFP_0.adf");
		else 
			frame->Dump("AFP_1.adf");
	}

	if ( frame ) 
		delete frame;
	if ( gconfframe ) 
		delete gconfframe;	
	if ( psaconfframe ) 
		delete psaconfframe;
	return 0;
}

int main(int argc, char **argv)
{
	ConfAgent::theGlobalAgent();

	const int nbtest = 3;
	if ( argc == 1 ) 
		std::cout << " Usage is : \n\t" 
			  << argv[0] << " i \n i being a integer from 0 to "<< nbtest << std::endl;
	
	int nwhat; UInt_t number = 100;
	if ( argc > 1 ) {
		std::istringstream what(argv[1]); what >> nwhat;	
	}
	if ( argc > 2 ) {
		std::istringstream what(argv[2]); what >> number;
	}
		
	switch( nwhat ) {
		case 1:
			test1(number);
			break;
		case 2:
			test2(number);
			break;
		case 3:
			test3(number);
			break;	
		case 4:
			test4(number);
			break;	
		case 5:
			dummy1(number);
			break;
		case 6:
			dummy2(number);
			break;
		case 7:
 			EB1(number);
			break;	
		default:
			std::cout << " Unknown test number " << nwhat << std::endl;	
	}
	ConfAgent::KilltheGlobalAgent();	
	
	return 0;
}
