/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   aLong_t with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file DummyFilter.cpp compiled in libADF.so !! only in standalone mode !! */

#ifndef ADF_DUMMYFILTER_H
#include "DummyFilter.h"
#endif

#ifndef ADF_AgataKeyFactory
#include "AgataKeyFactory.h"
#endif
#ifndef ADF_AgataFrameFactory
#include "AgataFrameFactory.h"
#endif
#ifndef ADF_NarvalSF
#include "NarvalSF.h"
#endif

#include <iostream>

using namespace ADF; 

DummyFilter::DummyFilter() 
 : NarvalFilter(), AlgoDummy(), fFrameEvent(NULL), fNarvalIO(NULL), 
   fFramePSA(NULL), fFrameTRACK(NULL), fTrigger(NULL), fTriggerEvent(NULL)
{	

}

DummyFilter::~DummyFilter() 
{  
	// in principle not needed ... just in case reset has not been called by narval
	UInt_t *error = 0u; process_reset(error) ; 
} 

void DummyFilter::process_config (const Char_t *directory_path, UInt_t *error_code)
{
	// In case your algorithm needs to initiate globals, it should be done here
	// For instance, if you need an proper FrameFactory:
	//	FrameFactory &main_frame_factory = FrameFactory::theMainFactory(); 
	//	main_frame_factory.Register( new MyFrameFactory() );
	
	// Globals for ADF as read from ADF.conf (should be in your algo path !!)
	NarvalInterface::process_config(directory_path,error_code);
  
	if ( (*error_code) == 0u ) { // now init your stuff
	
  		// here could come the reading of a file in which are stored the 
		// units (position,energy) you are working with i.e. modifications
		// of UnitPosition UnitEnergy defined in the base class AlgoDummy
		// + ....
		// NarvalInterface (through process_config(), so don't forget it) 
		// keeps the name of the directory_path. To get it back: GetConfPath()
	}
}

Int_t DummyFilter::SetInput()
{
	// loads the values from the frame into the ADFObject attached to the frame
	fFramePSA->Read();
	if ( fFramePSA->GetNbHits() >  MAXSIZE ) {  // cannot be treated
	
		Log.GetProcessMethod() = "SetInput()";
		Log << error << " Too many hits to be treated " <<  fFramePSA->GetNbHits() << dolog;
		 		
		Mult = 0u; return 1; 
	} 
	
	// now copy in your local variables	
	Mult = fFramePSA->GetNbHits();
	for (UInt_t nbhit = 0u; nbhit < Mult; nbhit++ ) {

		const Hit *ahit = fFramePSA->GetHit(nbhit);	 
		
		X[nbhit] = UnitPosition*ahit->GetX();
		Y[nbhit] = UnitPosition*ahit->GetY();
		Z[nbhit] = UnitPosition*ahit->GetZ();	
		
		E[nbhit] = UnitEnergy*ahit->GetE();	
	}
	return 0;
}

Int_t DummyFilter::SetOutput()
{
	Int_t result = 0;
	
	fFrameTRACK->Reset(); // MANDATORY to start filling a new Frame to set # of gammas to 0
		
	// hits	
//	for (UInt_t i = 0u; i < 3; i++ ) {
		TrackedHit *ahit = fFrameTRACK->NewGamma();
			
		ahit->SetE(EM);
		ahit->SetXYZ(XM,YM,ZM); ahit->SetStatus(ID);
//	}

	if ( fFrameTRACK->Write() == 0u ) result = 1; // write everything in the frame
	
	return result;
}
	
void DummyFilter::process_block (void *input_buffer,
			      UInt_t size_of_input_buffer,
			      void *output_buffer,
			      UInt_t size_of_output_buffer,
			      UInt_t *used_size_of_output_buffer,
			      UInt_t *error_code)
{  
  	// attach the input/output buffer to the NarvalIO system
 	fNarvalIO->Attach(input_buffer,size_of_input_buffer,output_buffer,size_of_output_buffer);
	
	// start the processing
 	(*error_code) = 0u;
	while ( fNarvalIO->Notify() ) { // process the input 
		
		// fill local variables with data from the input 
		// process the input buffer
		// fill the output buffer
		if ( ! (SetInput() == 0 && Process() == 0 && SetOutput() == 0) )  { (*error_code) =  1u; break; } 
		else {
			// ok, so send the produced frame to the ouput
			if (fTrigger->IsFired()) 
				fNarvalIO->Record(fTrigger); 
			if (fTriggerEvent->IsFired()) 
				fNarvalIO->Record(fTriggerEvent);
		}
		
	}
	if ( (*error_code) > 0u ) {
		Log.GetProcessMethod() = "DummyFilter::process_block";
		Log << error 
		    << " During SetInput -> Process -> SetOuput() " << dolog;
	}
	// to get back the real size of the output buffer 
	(*used_size_of_output_buffer) = fNarvalIO->GetOutput().Offset();
}

void DummyFilter::process_initialise (UInt_t *error_code)
{
// log related
	// init log with the proper description 
	Log.ClearMessage();
	Log.GetProcessName() = "DummyFilter"; Log.GetProcessMethod() = "process_initialise"; Log.SetPID(GetPID()); 
	
// In case everything is all right, this is not needed because the constructor should give things 
// properly initialised to 0 or because this method follows a process_reset call.
	// if ( fNarvalIO ) process_reset(error_code);
	
// to allocate the needed objects:
//   input and oupt frame, trigger 
		
	// load the main frame/key factories
 	KeyFactory &main_key_factory 
		= KeyFactory::theMainFactory(); 
	FrameFactory &main_frame_factory 
		= FrameFactory::theMainFactory(); 
	
	if ( KeyFactory::Register("Agata") == false || FrameFactory::Register("Agata") == false ) {
		Log << error << "Agata factories cannot be loaded " << dolog; 
		
		*error_code = 1u; return;
	}
		
	// to get the current version of psa frames
	Version v(65000u,0u);
	
	if ( GetCurrentFrameVersion("Agata","data:psa",v) == false ) { 
		Log << error << "Agata data:psa Version " 
		    << v.GetMajor() << " " << v.GetMinor() << " unknown " << dolog; 
			
		*error_code = 2u; return; 
	}
	// here should come a test of compatibility between algo version and psa frame version	
	
	// now, init a proper key and the input frame
	AgataKey *psakey 
		= dynamic_cast<AgataKey *>(main_key_factory.New("Agata","data:psa",GetCurrentKeyVersion()));
			
	fFramePSA = dynamic_cast<PSAFrame *>(main_frame_factory.New(psakey,v));	
	
	if (  fFramePSA == NULL ) {
		Log << error << " Cannot allocate PSA Frame " << " Agata, data:psa " 
		    << v.GetMajor() << " " << v.GetMinor() << dolog; 
		
		*error_code = 3u; return; 	
	}
	delete psakey; // not needed anymore, the key attached to the frame is a clone 
	
	// the frame is allocated, here you should link your globals.
	fFramePSA->Global().LinkItem("CrystalID","US",&ID); 
//	fFramePSA->Global().LinkItem("CrystalID","US",&ID); 
	
	// to get the current version of tracked frames	
	if ( GetCurrentFrameVersion("Agata","data:tracked",v) == false ) { 
		Log << error << "Agata data:tracked Version " 
		    << v.GetMajor() << " " << v.GetMinor() << " unknown " << dolog; 
			
		*error_code = 4u; return; 
	}	
	// here should come a test of compatibility between algo version and psa frame version	
		
	// now, init a proper key and the input frame
	AgataKey *trackkey 
		= dynamic_cast<AgataKey *>(main_key_factory.New("Agata","data:tracked",GetCurrentKeyVersion()));
		
	trackkey->SetDataLength(32*aKByte);
	fFrameTRACK = dynamic_cast<TrackedFrame *>(main_frame_factory.New(trackkey,v));	
	
	if (  fFrameTRACK == NULL ) {
		Log << error << " Cannot allocate Tracked Frame " << dolog; 
		*error_code = 5u; 
		return; 	
	}
	delete trackkey; // not needed anymore, the key attached to the fram is a clone 
	
	// to get the current version of tracked frames	
	if ( GetCurrentFrameVersion("Agata","data:event",v) == false ) { 
		Log << error << "Agata data:event Version " 
		    << v.GetMajor() << " " << v.GetMinor() << " unknown " << dolog; 
			
		*error_code = 6u; return; 
	}	
	// here should come a test of compatibility between algo version and psa frame version	
		
	// now, init a proper key and the input frame
	AgataKey *agataeventkey 
		= dynamic_cast<AgataKey *>(main_key_factory.New("Agata","data:event",GetCurrentKeyVersion()));
		
	fFrameEvent = dynamic_cast<AgataEventFrame *>(main_frame_factory.New(agataeventkey,Version(1,0)));	
	
	if (  fFrameEvent == NULL ) {
		Log << error << " Cannot allocate Event Frame " << dolog; 
		*error_code = 6u; 
		return; 	
	}
	delete agataeventkey; // not needed anymore, the key attached to the fram is a clone 
	// the frame is allocated, here you should link your globals.
	
	// 
//	fNarvalIO = new NarvalSF("Agata","agata",GetCurrentKeyVersion());
	fNarvalIO = new NarvalIO("Agata","data:event",GetCurrentKeyVersion());
		
	fNarvalIO->Configure("./","gg");
//	fNarvalIO->SetModel(FrameIO::kStrict);	
//	fNarvalIO->SetModel(FrameIO::kSafe);
	fNarvalIO->SetModel(FrameIO::kGrowing);	
//	fNarvalIO->Configure("./","NarvalSF.conf");	
	// now creates the trigger 
	Bool_t cons_main = false, cons_psa = true;
		
	fTrigger = new FrameTrigger("Trigger on psa frame",fFramePSA,cons_psa); 
	fTrigger->SetOuputFrame(fFrameTRACK); 
	fNarvalIO->Register(fTrigger);
	
	fTriggerEvent = new FrameTrigger("Trigger on event frame with PSA",fFrameEvent,cons_main);
	fTriggerEvent->Add(fFramePSA,cons_psa);
	
	fTriggerEvent->SetOuputFrame(fFrameTRACK);
	fNarvalIO->Register(fTriggerEvent); 
		
  	(*error_code) = 0;
}


void DummyFilter::process_reset (UInt_t *error_code)
{
	Log.ClearMessage(); Log.GetProcessMethod() = "process_reset"; fNarvalIO->Print(Log()); 
	
	if ( fFrameEvent ) 
		{ delete fFrameEvent; fFrameEvent = NULL; } 
		
	// connect to the frameIO
	if ( fNarvalIO ) 
		{ delete fNarvalIO; fNarvalIO = NULL; }

	// to get the input/output frameIO
	if ( fFramePSA	) 
		{ delete fFramePSA; fFramePSA = NULL; }
	
	if ( fFrameTRACK ) 
		{ delete fFrameTRACK; fFrameTRACK = NULL; }
	
	if ( fTrigger ) 
		{ delete fTrigger; fTrigger = NULL; }	
		
	if ( fTriggerEvent )
		{ delete fTriggerEvent; fTriggerEvent = NULL; }	
		
	Log << dolog;

}
void DummyFilter::process_start (UInt_t *error_code)
{
	Log << info << " Start the inner loop " << dolog; 
	
	Log.GetProcessMethod() = "process_block"; 
	(*error_code) = 0;
}

void DummyFilter::process_stop (UInt_t *error_code)
{
  std::cout << "process_stop called with GetPID()" << GetPID() << std::endl;
  *error_code = 0;
}
void DummyFilter::process_pause (UInt_t *error_code)
{
  std::cout << "process_pause called with GetPID()" << GetPID() << std::endl;
  *error_code = 0;
}
void DummyFilter::process_resume (UInt_t *error_code)
{
  std::cout << "process_resume called with GetPID()" << GetPID() << std::endl;
  *error_code = 0;
}
/*
void DummyFilter::process_unload (UInt_terror_code)
{
  std::cout << "process_unload called with GetPID()" << GetPID() << std::endl;
  *error_code = 0;
} */

#if HAVE_NARVAL_H == 1 && DUMMY_FILTER == 1
void process_config (const char *directory_path, UInt_t *error_code)
{
  DummyFilter::process_config (directory_path, error_code);
}

DummyFilter *process_register (UInt_t *error_code) { return new DummyFilter(); }

/*void process_unload (DummyFilter *algo_data, UInt_t *error_code)
{
  algo_data->process_unload (error_code);
}*/

void process_block (DummyFilter *algo_data,
		    void *input_buffer,
                    UInt_t size_of_input_buffer,
		    void *output_buffer,
		    UInt_t size_of_output_buffer,
		    UInt_t *used_size_of_output_buffer,
		    UInt_t *error_code)
{
  algo_data->process_block (input_buffer, size_of_input_buffer,
			    output_buffer, size_of_output_buffer,
			    used_size_of_output_buffer, error_code);
}

void process_initialise (DummyFilter *algo_data, UInt_t *error_code)
{
  algo_data->process_initialise (error_code);
}
void process_reset (DummyFilter *algo_data,UInt_t *error_code)
{
  algo_data->process_reset (error_code);
}
void process_start (DummyFilter *algo_data, UInt_t *error_code)
{
  algo_data->process_start (error_code);
}
void process_stop (DummyFilter *algo_data,UInt_t *error_code)
{
  algo_data->process_stop (error_code);
}
void process_pause (DummyFilter *algo_data,UInt_t *error_code)
{
  algo_data->process_pause (error_code);
}
void process_resume (DummyFilter *algo_data, UInt_t *error_code)
{
  algo_data->process_resume (error_code);
}
#endif




