/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** @file NarvalCollector.h real header file for narval/NarvalCollector.cpp 

*/

#ifndef ADF_NarvalCollector
#define ADF_NarvalCollector

#include <LogMessage.h>
#include <LogCollector.h>

namespace ADF {

//! Base class for a Log collector
/*!

*/
class NarvalCollector : public LogCollector
{               
protected:
    //! to convert log level from ADF to Narval conventions
    /*!
    
    	For Narval (situation 4/11/2009): 
    
    \code
   procedure Ada_Log_Message (Id : Integer;
                              Message : Interfaces.C.Strings.Chars_Ptr;
                              Level : Integer) is
      Ada_Message : constant String := "id=" & Id'Img & " " & Value
(Message);
   begin
      case Level is
         when 0 =>
            Debug_Out (Actor_Handle.Logger'Access, Ada_Message);
         when 1 =>
            Info_Out (Actor_Handle.Logger'Access, Ada_Message);
         when 2 =>
            Warn_Out (Actor_Handle.Logger'Access, Ada_Message);
         when 3 =>
            Error_Out (Actor_Handle.Logger'Access, Ada_Message);
         when others =>
            Fatal_Out (Actor_Handle.Logger'Access, Ada_Message);
      end case;
   end Ada_Log_Message;
   \endcode

    */      
    int ConvertLevel(LogMessage::ELevel lev);
    
public: 
    //!   
    NarvalCollector(const char *name, bool isdelbycental);  
    //!   
    virtual ~NarvalCollector();           
            
    //! Send messages to Narval system
    /*!
    	This is the method to overwritte to implement your own action
    */
    virtual void DoCollect(LogMessage &);   
}; 

} // namespace
#endif





