/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   aLong_t with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** @file CentralLog.h  

*/

#ifndef _CentralLog
#define _CentralLog
#define _CENTRALLOG_H _CentralLog

#include <string>
#include <list>

class CentralLog;

//! To properly destroy the singleton
/*!

*/
class CentralLogDestroyer 
{
public:
	CentralLogDestroyer(CentralLog *s = 0x0);
	~CentralLogDestroyer();

        void SetCentralLog(CentralLog *);
	
private:
        CentralLog *fCentralLog;
};

class LogCollector;   
class LogMessage;   
 
//! CentralLog is a singleton that should be destroyed at exit time
/*!

*/
class CentralLog 
{   
private:
    static CentralLog *gTheCentralLog; 
    static CentralLogDestroyer gTheCentralLogDestroyer; 
    
protected:     
    friend class CentralLogDestroyer;  

private:
    //! the current collector	    
    LogCollector *fCurrentCollector;
    //! list of available collectors	    
    std::list < LogCollector * > fListOfCollectors;
           
private:
    CentralLog();
    ~CentralLog();   
    
public: 
    //! CentralLog is a singleton	
    static CentralLog *theCentralLog();  
    
    //! to change the current collector	    
    bool ChangeCollector(std::string whichlog = std::string("Null")); 
    
    //! Check if the given collector already exists
    /*!
    */    
    bool IsCollector(std::string whichlog); 
           
    //! Return the current logcollector	
    LogCollector *GetCurrentLogCollector() const;
    
    //! Add a new LogCollector to the list of known collectors
    /*!
    	if a log with a similar name already exist, it is removed from the central log 
	and return by this method. It is then the charge of the user to delete it
	if the collector is deletable.
    */
    LogCollector *Add ( LogCollector * );  
    
    //! Remove a collector from the list of available collectors
    /*!
    */        
    // void Remove( std::string );

}; 

#endif





