/***************************************************************************
 *   Copyright (C) 2004-2006 by Olivier Stezowski & Christophe Theisen     *
 *   stezow(AT)ipnl.in2p3.fr, christophe.theisen(AT)cea.fr                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file ClickCollector.cpp compiled in libGWTOOLS.so */

#ifndef GW_CLICKCOLLECTOR_H
#include "ClickCollector.h"
#endif
#ifndef ROOT_TCanvas_H
#include "TCanvas.h"
#endif
#ifndef ROOT_TVirtualFitter_H
#include "TVirtualFitter.h"
#endif
#ifndef ROOT_TObject_H
#include "TObject.h"
#endif


#include <iostream>
#include <fstream>

using namespace std;
using namespace GwD;
 
// ROOT dictionnary
ClassImp(ClickCollector);

// set statics
ClickCollector *ClickCollector::gfCollector = 0;

ClickCollector::ClickCollector() : fLastPx(0), fLastPy(0), fCollectMode(kKeyPress), fMarker()
{	
	// set marker style for peaks
	fMarker.SetMarkerStyle(20); fMarker.SetMarkerColor(2); gfCollector = this;
}

void ClickCollector::DoCollect(Int_t event, Int_t px, Int_t py, TObject */*obj*/)
{
  	// Collect event.
  	// if event = fCollectMode ( = double click by default ) it adds a "click"
  	// to the list of clicks and displays a TCross at that position.
	if ( event == kMouseMotion ) {fLastPx = px; fLastPy = py; return; } // keep memory of the last position
	if ( event == fCollectMode && fIsActive ) {
		if ( gROOT->GetSelectedPad() ) {
			fMarker.DrawMarker(gROOT->GetSelectedPad()->AbsPixeltoX(fLastPx),
			gROOT->GetSelectedPad()->AbsPixeltoY(fLastPy));
			gROOT->GetSelectedPad()->Update();
		}
	}
}

ClickCollector * ClickCollector::TheCollector()
{
	if ( gfCollector == NULL ) new ClickCollector(); return gfCollector;
}

