/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file Events.h header file for useful events */

#ifndef GW_GWEVENTS_H
#define GW_GWEVENTS_H

#ifndef ROOT_Rtypes
#include "Rtypes.h"
#endif

#ifndef ROOT_TObject
#include "TObject.h"
#endif


namespace GwD {

/** 



@author Olivier Stezowski
*/
class EPoint : public TObject
{
private:
	Float_t E; // Energy 
	
	Float_t X; // X position
	Float_t Y; // Y position
	Float_t Z; // Z position

public:
	EPoint();
	virtual ~EPoint() {;}

	virtual void Clear(Option_t *);	
	
	virtual void Set(Float_t e, Float_t x, Float_t y, Float_t z) { E=e; X=x; Y=y; Z=z; }

	//! Root dictionary related
	ClassDef(EPoint,1); // A 3D point associated with an energy
};

class TrackedGamma : public TObject
{
private:
	Float_t E; // Energy 
	
	Float_t Theta1; // Theta first impact 
	Float_t Phi1;   // Phi first impact
	Float_t Theta2; // Theta second impact 
	Float_t Phi2;   // Phi second impact

public:
	TrackedGamma() : E(0.0) , Theta1(0.0), Phi1(0.0), Theta2(0.0), Phi2(0.0) {;}
	virtual ~TrackedGamma() {;}

	virtual void Set(Float_t e, Float_t t1, Float_t p1, Float_t t2, Float_t p2) {E=e; Theta1=t1; Phi1=p1; Theta2=t2; Phi2=p2;}

	//! Root dictionary related
	ClassDef(TrackedGamma,1); // Base for single tracked gamma
};

} // namespace GammaWare

#endif
