/***************************************************************************
 *   Copyright (C) 2004-2006 by Olivier Stezowski & Christophe Theisen     *
 *   stezow(AT)ipnl.in2p3.fr, christophe.theisen(AT)cea.fr                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file Peak.cpp compiled in libGWTOOLS.so */

#ifndef GW_PEAK_H
#include "Peak.h"
#endif

#include <iostream>
#include <fstream>

using namespace std;
using namespace GwD;
 
// ROOT dictionnary
ClassImp(Peak);

Peak::Peak() //Default constructor (dimansion = 1)
{
	fDim = 1;
	fPos = new TArrayD(1); 
	fRes = new TArrayD(1); 
	fInt = 0.0;
}

Peak::Peak(Int_t n) //Constructor (n dimensions)
{
	if (n <= 0 ) Peak();
	else {
		fDim = n;
		fPos = new TArrayD(n);
		fRes = new TArrayD(n);
		fInt = 0.0;
	}
}

Peak::Peak(const Peak& p):TObject(p)
{
	this->fDim = p.fDim;
	this->fPos = new TArrayD(this->fDim,(p.fPos)->GetArray());
	this->fRes = new TArrayD(this->fDim,(p.fRes)->GetArray());
	this->fInt = p.fInt;
}


Peak::~Peak(){
	delete fPos;
	delete fRes;
}

void Peak::Set(Int_t n)
{
	if (n <= 0) fDim = 1; else fDim = n;
	fPos->Set(fDim);
	fRes->Set(fDim);
}


void Peak::SetPosition(const Double_t x, const Double_t y, const Double_t z)
{
	Double_t *tab = new Double_t[3]; tab[0] = x; tab[1] = y; tab[2] = z; 
	fPos->Set(fDim,tab);
	delete [] tab;
}

void Peak::SetPosition(const Double_t *array)
{
	fPos->Set(fDim,array);
}

void Peak::SetResolution(const Double_t x, const Double_t y, const Double_t z)
{
	Double_t *tab = new Double_t[3]; tab[0] = x; tab[1] = y; tab[2] = z; 
	fRes->Set(fDim,tab);
	delete [] tab;
}

void Peak::SetResolution(const Double_t *array)
{
	fRes->Set(fDim,array);
}

void Peak::SetIntensity(const Double_t val)
{
	fInt = val;
}

Int_t Peak::GetSize(){
	return fDim;
}

Double_t Peak::GetPosition(const Int_t i)
{
	return fPos->At(i);
}

const Double_t* Peak::GetPosition()
{
	return fPos->GetArray();
}
 
Double_t Peak::GetResolution(const Int_t i)
{
	return fRes->At(i);
}

const Double_t* Peak::GetResolution()
{
	return fRes->GetArray();
}

Double_t Peak::GetIntensity()
{
	return fInt;
}









