/***************************************************************************
 *   Copyright (C) 2005-2006 by Christophe Theisen & Olivier Stezowski     *
 *   christophe.theisen(AT)cea.fr, stezow(AT)ipnl.in2p3.fr                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file Peak.h header file for the calibration facility */
 
#ifndef GW_PEAK_H
#define GW_PEAK_H

// root's includes
#ifndef ROOT_TARRAYD_H
#include <TArrayD.h>
#endif

#ifndef ROOT_TOBJECT_H
#include <TObject.h>
#endif


namespace GwD {

/**
Description of the class

\author Bertrand Rossé
*/
class Peak : public TObject 
{
public:
	Peak(); //Default constructor
	Peak(Int_t n); // constructor
	Peak(const Peak& p); //copy constructor
	virtual ~Peak(); // destructor
	//! Set the dimension. A new array for position (resolution) is created, the old contents copied to new array, then the old array is deleted.
	void Set(Int_t n); // Set the dimension
	void SetPosition(const Double_t x, const Double_t y=0.0, const Double_t z=0.0);
	void SetPosition(const Double_t *array);
	void SetResolution(const Double_t x, const Double_t y=0.0, const Double_t z=0.0);
	void SetResolution(const Double_t *array);
	void SetIntensity(const Double_t val);
	
	Int_t GetSize();
	Double_t GetPosition(const Int_t i);
	const Double_t* GetPosition();
	Double_t GetResolution(const Int_t i);
	const Double_t* GetResolution();
	Double_t GetIntensity();
	
private:
	Int_t fDim; //Dimension of the peak (1D, 2D, 3D, ...)
	TArrayD *fPos;  //Array of positions
	TArrayD *fRes;  //Array of resolutions
	Double_t fInt;	//Intensity
		
//! rootcint dictionary
    ClassDef(Peak,1); // a peak
};

// inline members

}

#endif



