
#include "RMTEmulator.h"

#include "TCanvas.h"
#include "TPaveLabel.h"
#include "TGraph.h"
#include "TLine.h"

using namespace Gw;

void RMTEmulator::Draw()
{
	BuildTopology();
	
	// Check the number of actors at each level and determe for which there is a maximum number of actors.
	UInt_t max = 0;
	for (UInt_t i = 0; i < fTopology.size(); i++ ) {
		if ( fTopology[i].size() > max )
			max = fTopology[i].size();
	}	
	
	// set a new canvas with the size to display all the topology 
	Int_t delta = 100; TCanvas *c = new TCanvas(GetName().data()); c->Range(0,0,delta*(fTopology.size()+1),delta*(max+1));
	
	// now  compute the positions of the different actors in 2D  
	UInt_t which_i,which_j, size_i, size_j, size_ii, size_jj; Float_t deltax = delta, deltay; TPaveLabel box; 
	//
	std::vector < Float_t > positionsX(max*fTopology.size(),0.0), positionsY(max*fTopology.size(),0.0); 
	//
	size_i = fTopology.size();
//	std::cout << " DEPTH " << size_i << std::endl;
	for (UInt_t i = 0; i < size_i; i++ ) {
		//
		size_j = fTopology[i].size();
		//
		if ( i == 0 ) { // producer, just draw a bow @ regular intervalles 
			
			deltay = delta * Float_t (max+1) / Float_t (fTopology[i].size()+1);
			
			for (UInt_t j = 0; j < size_j ; j++) {
				positionsX[i*max+j] = (i+1)*deltax;
				positionsY[i*max+j] = (j+1)*deltay;
				
				box.DrawPaveLabel( positionsX[i*max+j] - delta/4., positionsY[i*max+j] - delta/4.,
								   positionsX[i*max+j] + delta/4., positionsY[i*max+j] + delta/4.,fTopology[i][j]->GetName().data());
			}

			continue;
		}	
		
		// now check links with previous depth and compute the mean vertical position
		for (UInt_t j = 0; j < size_j ; j++) {
			//
			Float_t ymean = 0; Int_t nb_linked = 0; std::vector < std::pair<Int_t,Int_t> > is_linked_with; Bool_t is_linked; AnActorConnection *connection;
			// look in the lower levels 
			//
			is_linked_with.clear();
			is_linked = false;
			for (Int_t ii = i-1; ii >= 0; ii--) {
				size_jj = fTopology[ii].size();
				for (UInt_t jj = 0; jj < size_jj; jj++) {
					
					if ( fTopology[i][j]->IsLinked(fTopology[ii][jj],connection) != 0 ) {
						
						std::cout << fTopology[i][j]->GetName() << " is linked with " << fTopology[ii][jj]->GetName() << std::endl;
						std::cout << " FrameBlock " << connection->GetName() << std::endl;
						
//						std::cout << i << " " << j << " connected with " << ii << " " << jj << std::endl;
//						std::cout << " YMEAN " << ymean << " + " << positionsY[ii*max+jj] << std::endl;

						std::pair <Int_t, Int_t> p;
						p.first  = ii;
						p.second = jj;
						is_linked_with.push_back(p); is_linked = true; nb_linked++; ymean += positionsY[ii*max+jj]; 
						
					}
					else { /* std::cout << fTopology[i][j]->GetName() << " is NOT linked with " << fTopology[ii][jj]->GetName() << std::endl; */ }

				} // jj
				if ( is_linked ) { // no need to go deeper in the topology depth
					
//					break;
				}				
			} // ii
			positionsX[i*max+j] = (i+1)*deltax;
			positionsY[i*max+j] = ymean / nb_linked;	
			
			box.DrawPaveLabel( positionsX[i*max+j] - delta/4., positionsY[i*max+j] - delta/4.,
							   positionsX[i*max+j] + delta/4., positionsY[i*max+j] + delta/4.,fTopology[i][j]->GetName().data());

			// Draw connections
			for (Int_t i_linked = 0; i_linked < nb_linked; i_linked++) {
				if ( is_linked_with[i_linked].first == i-1 ) { // directly connected with the previous level
					TGraph *gr = new TGraph(2); 
					//
					gr->SetPoint(0,positionsX[is_linked_with[i_linked].first*max+is_linked_with[i_linked].second] + delta/4.,positionsY[is_linked_with[i_linked].first*max+is_linked_with[i_linked].second]);				
					gr->SetPoint(1,positionsX[i*max+j] - delta/4.,positionsY[i*max+j]);				
					// Set Name of the connection
					fTopology[i][j]->IsLinked(fTopology[is_linked_with[i_linked].first][is_linked_with[i_linked].second],connection);
					gr->SetName(connection->GetName().data());
					gr->SetEditable(false);
					gr->Draw("lp");
				}
				else {
					TGraph *gr = new TGraph(3); 
					//
					gr->SetPoint(0,positionsX[is_linked_with[i_linked].first*max+is_linked_with[i_linked].second] + delta/4.,positionsY[is_linked_with[i_linked].first*max+is_linked_with[i_linked].second]);				
					gr->SetPoint(1,positionsX[i*max+j] - 3*delta/4.,positionsY[is_linked_with[i_linked].first*max+is_linked_with[i_linked].second]);		
					gr->SetPoint(2,positionsX[i*max+j] -   delta/4.,positionsY[i*max+j]);		
					// Set Name of the connection
					fTopology[i][j]->IsLinked(fTopology[is_linked_with[i_linked].first][is_linked_with[i_linked].second],connection);
					gr->SetName(connection->GetName().data());
					gr->SetEditable(false);
					gr->Draw("lp");
				}
			}
		} // j
	} // i
}

Bool_t RMTActorConnection::Switch(FrameBlock *block_1, FrameBlock *block_2)
{
	Bool_t ok = false;
	/*
	Int_t size1 = block_1->GetSize(); Char_t *ad1 = block_1->SetAddress(0x0); Char_t *ad2 = block_2->SetAddress(0x0); 
	
	block_1->SetAddress(ad2);
	block_1->SetSize(0);
	block_2->SetAddress(ad1);
	block_2->SetSize(size1);	
	 */
	
	// safe method = copy ! ... less safe Link ... to be tested ...
	Char_t *ad1 = block_1->GetBufferIO()->Address(); UInt_t size1 = block_1->GetSize();
	//Char_t *ad2 = block_2->GetBufferIO()->Address();
	
	// copy 
	ok = block_2->SetBlock(ad1,size1,'-','c');
	if ( ok ) { 
		block_1->Rewind(ConfAgent::kWrite);
	}
	
	return ok;
}

Bool_t RMTActorConnection::IsReadyforConsumer() 
{
	Bool_t result = false;
	
	// has been set free by all task	
	if ( fBlockRight->IsOwner(0x0) ) {
		if ( fBlock->IsOwner(fConsumer) && fBlock->GetSize() ) {
			Switch(fBlock,fBlockRight);
			fBlock->SetOwner(0x0);
			
			result = true;
		}
	}
	return result;
	
	/*
	if ( fBlockRight->IsOwner(0x0) && fBlock->IsOwner(0x0) && fBlock->GetSize() > 0 )
		return true;
	
	return false; */
}
Bool_t RMTActorConnection::SetReadyforConsumer() 
{
	fBlockRight->SetOwner(fConsumer->GetActor()); 
	
	// TODO_REAL : should be rewind for READING
	fBlockRight->Rewind(ConfAgent::kRead);
	
	return true;
}
Bool_t RMTActorConnection::IsReadyforProducer()
{
	Bool_t result = false;
	
	// has been set free by all task
	if ( fBlockLeft->IsOwner(0x0) ) { 
		// means not used yet, so can be used
		if ( fBlockLeft->GetSize() == 0 )
			result = true;
		else {
			// already filled so check if it can be moved to fblock 
			if ( fBlock->IsOwner(0x0) ) {
				//
				if ( Switch(fBlockLeft,fBlock) ) { // should log somethng
					//
					fBlock->SetOwner(fConsumer);
				
					result = true;
				}
			}
		}
	}
	return result;
	/*
	if ( fBlockLeft->IsOwner(0x0) )
		return true;
	
	return false; */
}
Bool_t RMTActorConnection::SetReadyforProducer()
{	
	//
	fBlockLeft->SetOwner(fProducer->GetActor());	

	// TODO_REAL : should be rewind for WRITING
	fBlockLeft->Rewind(ConfAgent::kWrite);
	
	return true;
}


RMTActor::RMTActor(const Char_t *name) : ALoadedActor(name), fThread(0x0), fCondition(0x0)
{
	// so that thread is not detachable and join can be used
	fThread = new TThread(name, (TThread::VoidRtnFunc_t)&ProcessAsThread,(void*) this); 
//	fThread = new TThread(name, (void(*) (void *))&ProcessAsThread,(void*) this); 
	
}

RMTActor::~RMTActor()
{
//	TThread::Printf("RMTActor");

	TThread::Delete(fThread); delete fThread;
	
//	TThread::Printf("FIN RMTActor");

}

void RMTEmulator::LinkActors(ALoadedActor *left, ALoadedActor *middle, ALoadedActor *right)
{
	AnActorConnection *connection;
	
	// allocate a new connection and add it to the pool of connection
	if ( left ) {
		// look at the buffer size (name) in the globals
		connection = 
			new RMTActorConnection(left,middle,GetItem<UInt_t>("SizeDataBlockC",fGlobals)->Get());
		fAllConnections.push_back(connection);		
		
		left->AddOutput(connection); middle->AddInput(connection);
	}
	if ( right ) {
		// look at the buffer size (name) in the globals
		connection = 
			new RMTActorConnection(middle,right,GetItem<UInt_t>("SizeDataBlockC",fGlobals)->Get());
		fAllConnections.push_back(connection);		
		
		middle->AddOutput(connection); right->AddInput(connection);		
	}
}

void RMTActor::ProcessActor () 
{ 
	TThread::Printf("In RMTActor::ProcessActor, start thread of (RMTActor) %s ",fName.data());
//	std::cout << " Call of (RMTActor) " << GetName() << std::endl ;

try 
{

		
		// now call processing of the embedded actor
		GetThread()->Run();
	TThread::Printf("In RMTActor::ProcessActor, start thread of (RMTActor) %s DONE  ",fName.data());
		
		//fNarvalActor->ProcessBlock();
		
		// to add : check all input and output ... if freed, means DLActor is Idle
		/*
		 for (UInt_t i = 0; i < fInputs.size(); i++) {
		 //
		 if ( fInputs[i]->IsReadyForConsumer() ) {
		 }
		 }
		 for (UInt_t i = 0; i < fOutputs.size(); i++) {
		 
		 // the block to be treated
		 FrameBlock *block = fOutputs[i]->GetBlock('p');
		 //
		 if ( fOutputs[i]->IsReadyForProducer() ) {
		 
		 }
		 }	
		 */
		
	}
	catch (const std::exception& ex) 
	{
		std::cout << " ERROR " << ex.what() << std::endl ; 
	} 
	
}	
	/*
void RMTActor::Process () 
{ 

	std::cout << " Call of (RMTActor) " << GetName() << std::endl ;
	
	if ( GetName() == "P1" ) {
		gSystem->Sleep(3000);
	}
	if ( GetName() == "F1" ) {
		gSystem->Sleep(2000);
	}
	if ( GetName() == "F2" ) {
		gSystem->Sleep(100);
	}
	std::cout << " End of (RMTActor) " << GetName() << std::endl ;
	
	//fThread->Run(); 
}	
*/

void RMTActor::ProcessAsThread (void *void_actor)
{
	
	// thread function which calls user specified action Func0
	//TThread::SetCancelOn();
	//TThread::SetCancelDeferred();
	RMTActor * inst = (RMTActor *) void_actor;

	inst->GetActor()->ProcessBlocks();
	inst->GetCondition()->Broadcast();	
	
	/*
	TThread::SetCancelOn();
	TThread::SetCancelDeferred();
	RMTActor * inst = (RMTActor *) void_actor;

	// run process
	TThread::Printf("RMTEmulator::ProcessAsThread %s \n ",inst->GetName().data());
	inst->GetActor()->ProcessBlock(); // call the user defined threaded function
	TThread::Printf("RMTEmulator::ProcessAsThread %s \n ",inst->GetName().data());
	
//	if ( inst->GetCondition() )
		inst->GetCondition()->Broadcast();
	*/
	return ;
}

Short_t RMTActor::IsToBeRun()
{
	TThread::Lock();
	// 
	Short_t change = 1;

	// should be FramIO::kReady and/or ? thread state
	if ( GetThread()->GetState() == TThread::kRunningState ) {
		change = 2;
	}
	else {
		// check if one input/output is ready to be consumed/produced
		for (UInt_t i = 0; i < fInputs.size(); i++) {
			//
			if ( fInputs[i]->IsReadyforConsumer() ) {
				change = 0;
				fInputs[i]->SetReadyforConsumer();
			}
		}
		for (UInt_t i = 0; i < fOutputs.size(); i++) {
			//
			if ( fOutputs[i]->IsReadyforProducer() ) {
				change = 0;
				fOutputs[i]->SetReadyforProducer();
			}
		}		
	}
	TThread::UnLock();	

	return change;	
}

RMTEmulator::RMTEmulator(std::string name) : BaseEmulator(name) , fMutex(0x0), fCondition(0x0) 
{ 
	fMutex = new TMutex(true); 
	//fMutex = new TMutex(false); 
	
	fCondition = new TCondition(fMutex) ;
}

RMTEmulator::~RMTEmulator()
{ 
	TThread::Printf("RMTEmulator");
	TThread::Ps();

	for (UInt_t i = 0; i < fAllActors.size(); i++) {
		if ( fAllActors[i] )
		{ delete fAllActors[i]; fAllActors[i] = 0x0; }
	}	
	
	// delete cause a warning
	if ( fCondition )
		delete fCondition; 
	if ( fMutex )
		delete fMutex; 
}

void RMTEmulator::Join()
{
	for (UInt_t i = 0; i < fAllActors.size(); i++) {
		RMTActor *threaded_actor = dynamic_cast<RMTActor *> (fAllActors[i]);
		threaded_actor->GetThread()->Join();
	}	
}

//! Init the emulator
void RMTEmulator::Init( const NamedItem<std::string> &init_sequence )
{ 
	// to be sure the underlying topology is well constructed
	BuildTopology(); 
	// now distribute the condition so that the emulator waits for at least one thread to finish before restarting it.
	for (UInt_t i = 0; i < fAllActors.size(); i++) {
		RMTActor *threaded_actor = 
			dynamic_cast<RMTActor *> (fAllActors[i]);
		threaded_actor->SetCondition(fCondition);
	}
}

#include "TROOT.h"
//! Run max loop
void RMTEmulator::Run(Int_t max_loop)
{
	UInt_t nb_loop = 0;
	
	while ( nb_loop < max_loop ) {
		
		
		// start from lowest level i.e. producers
		for (UInt_t i = 0; i < fTopology.size(); i++ ) {
			for (UInt_t j = 0; j < fTopology[i].size() ; j++) {
				
				//fCondition->TimedWaitRelative(1);
				
				RMTActor *the_actor = (RMTActor *)fTopology[i][j];
				// Should be the actor->IsState() == kNewState
				// kRunning, kTerminate (interruption), kFinished (fini) see TThread
				Short_t cond = the_actor->IsToBeRun();
				//TThread::Lock();
				TThread::Printf("In RMTEmulator::Run, Check thread %s again : %d ",the_actor->GetName().data(),cond);

				if ( cond > 0 )
					TThread::Printf("In RMTEmulator::Run, No need to start thread %s again : %d ",the_actor->GetName().data(),cond);
				else {
					TThread::Printf("In RMTEmulator::Run, Thread %s starts : %d ",the_actor->GetName().data(),cond);
					the_actor->ProcessActor();
					//TThread::Lock();
					//the_actor->GetThread()->Run();
					//TThread::UnLock();
				}
			//	TThread::UnLock();

			} 
		} // @ depth i
		
		// TThread::Ps();
		//TThread::Printf(" ===> In RMTEmulator::Run, WAITING one of the actor has finished to complete loop %d ",nb_loop);	
		//TCondition::Wait();
		//
		//if ( nb_loop > 0 )
			fCondition->Wait();
		//TThread::Printf(" ===> In RMTEmulator::Run, WAITING one of the actor has finished to complete loop OK %d \n ",nb_loop);		
		
		if ( gROOT->IsInterrupted() ) {
			std::cout << " KEEP ON RUNNING" << std::endl; 
		}
		nb_loop++;
		
	//	gSystem->ProcessEvents();

	} // while
	TThread::Printf("In RMTEmulator::Run, Wait for last actors to finish ... ");
	Join();
	TThread::Printf("... DONE  ");
	TThread::CancelPoint();
}








