/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file NOMADBuilder.cpp compiled in libGWILL.so */

#include "NOMADBuilder.h"
#include "BufferIO.h"
#include "DefaultKeyFactory.h"

#include "TH3.h"
#include "TRandom2.h"

#include <fstream>

using namespace ADF; 

//#define LOCAL_DEBUG
 
// Should be re-calculate from the map file ... TO BE DONE


UShort_t NOMADBuilder::kGEThreshold = 1;
UShort_t NOMADBuilder::kACThreshold = 0;
UShort_t NOMADBuilder::kLAThreshold = 0;

// default is simpliest addback i.e. does not take diagonal (test : kACPattern[id1*4+id2])
Bool_t NOMADBuilder::kACPattern[4*4] = 
	{
		0,1,0,1,
		1,0,1,0,
		0,1,0,1,
		1,0,1,0,
	};

Int_t TotInSize = 0;
Int_t TotInEvent = 0;

#include "TH2.h"
TH2F *pattern;

// Window to search for coinc precisely. 
#define D_FINE_WIDTH 30
#define D_NB_OF_FINE_IN_ROUGH 1000

// number of fine_width used as security in case the sliding window is redefined using last time stamp event (lts) in sliding ou event in container.
// if lts then next sliding starts @ lts - D_SLIDING_SECURITY*D_FINE_WIDTH
// MUST Be less that  D_NB_OF_FINE_IN_ROUGH and obviously enough relative to delays between the different detectors
#define D_SLIDING_SECURITY 5

NOMADBuilder::NOMADBuilder() : 
	NarvalFilter(),
	fRandom(new TRandom2(0)),
	fCurrentIN(0x0),
	fMap(),
	fCEvent(0x0),
	fRunningMode(0),
	fOutputMode(11),	
	fFineSlidingWindowWidth(D_FINE_WIDTH), // should be twice real coinc window
	fTwiceCoincWindowWidth(2*D_FINE_WIDTH),
	fFineSlidingWindow(0UL,D_FINE_WIDTH),
	fNbOfFineWindowWidthInRoughSlidingWidth(D_NB_OF_FINE_IN_ROUGH),
	fRoughSlidingWindowWidth(D_NB_OF_FINE_IN_ROUGH*D_FINE_WIDTH),
	fRoughSlidingWindow(0UL,D_NB_OF_FINE_IN_ROUGH*D_FINE_WIDTH),
	fLocalTSOffset(0UL),
	fChainTSGAP(1000000UL),
	fNbEventsInSlicing(0),
	fNbEventsInContainer(0),
	fPendingEventsLowTS(0UL),
//	fPendingEventsTimeWith(),
	fListOfAllocatedEvents(),
	fListOfFreeEvents(),
	fRoughArrayOfSliding(),
	fStackOfCoinc(),
	fQueueOfPending(),
//	fPendingEvents(),
//	fPendingUID(),
//	fAllocatedListOfSliging(),
//	fListOfSliding(),
	fCoincPattern(),
	fCoincBuilderHisto(),
	fCoincBuilderEvent(),
	fNumberOfCoinc(),
	fListOfCoinc(),
	fUnknownEvent(),
	fGEEvent(),
	fACEvent(),
    fLAEvent(),
	fEvent(),
    fOutEvent(),
	fTotNumberOfEvents(0UL),
	fBadEvents(0UL),
	fOut(0x0)
{	
	Log.GetProcessName() = "NOMADBuilder"; 

	/*
	fNumberOfAlive = 0;
	fNumberOfPending = 0;
	fMaxPendingSize = 10;
	 */

}

void NOMADBuilder::InitOnceConfig()
{
	// in case the this parameter are set into .conf
	/*
	fFineSlidingWindow.first = 0; 
	fFineSlidingWindow.second = fFineSlidingWindowWidth;
	
	fTwiceCoincWindowWidth = 2*fFineSlidingWindowWidth;
	//
	fRoughSlidingWindowWidth = fNbOfFineWindowWidthInRoughSlidingWidth * fFineSlidingWindowWidth;
	fRoughSlidingWindow.first = 0; 
	fRoughSlidingWindow.second = fRoughSlidingWindowWidth;
	 */
	
	/*
	fPendingEvents.resize(fMap.maxUID);
	//
	for (size_t i = 0; i < fRoughSlidingWindowWidth; i++) {
		std::deque< NOMADEvent *> *a_queue = 
			new std::deque< NOMADEvent *>;
		fAllocatedListOfSliging.push( a_queue ); fListOfSliding.push_back( a_queue );
	}	
	 */
	//
	fListOfAllocatedEvents.resize(fMap.maxUID*1000); 
	for (size_t i = 0; i < fListOfAllocatedEvents.size(); i++) {
		fListOfFreeEvents.push(&fListOfAllocatedEvents[i]);
	}
	
	fRoughArrayOfSliding.resize(fNbOfFineWindowWidthInRoughSlidingWidth,0x0);
	//
	for (size_t i = 0; i < fNbOfFineWindowWidthInRoughSlidingWidth; i++) {
		std::vector <NOMADEvent *> *an_array_of_events = 
			new std::vector <NOMADEvent *>(100); // up to 100 element in one coinc window ... should be far enough !
		fStackOfCoinc.push( an_array_of_events ); 
	}		
	
	// 
	fCoincPattern.resize(fTwiceCoincWindowWidth);
	
	pattern = new TH2F("pattern","pattern",fTwiceCoincWindowWidth,0,fTwiceCoincWindowWidth,fTwiceCoincWindowWidth,0,fTwiceCoincWindowWidth); pattern->Draw();

	Int_t coinc_width = 10;
	for (size_t i = 0; i < fTwiceCoincWindowWidth; i++) {
		fCoincPattern[i].resize(fTwiceCoincWindowWidth,0);
		for (size_t j = 0; j < fTwiceCoincWindowWidth; j++) {
			Float_t delta; Int_t delta_i_j =  i - j;
			if ( j > i ) {
				delta = -1.0*delta_i_j;
			}
			else {
				delta = +1.0*delta_i_j;
			}
			if ( ( coinc_width - 1 ) < delta ) 
				fCoincPattern[i][j] = 0.0;
			else {
				pattern->Fill(i,j,coinc_width - delta);
				fCoincPattern[i][j] = coinc_width - delta;
			}/*
			if ( ( fFineSlidingWindowWidth - 1 ) < delta ) 
				fCoincPattern[i][j] = 0.0;
			else {
				fCoincPattern[i][j] = fFineSlidingWindowWidth - delta;
			}*/
//			std::cout << i << " " << j << " " << fCoincPattern[i][j] << std::endl;
		}
	}
	fCoincBuilderHisto.resize(fTwiceCoincWindowWidth);
	fCoincBuilderEvent.resize(fTwiceCoincWindowWidth);
	for (size_t i = 0; i < fTwiceCoincWindowWidth; i++) {
		fCoincBuilderEvent[i].resize(50,0x0); // up to 10 events per unit of of ts ... should be far enough
		fCoincBuilderEvent[i].clear();
	}
	
	fNumberOfCoinc.resize(2*fRoughSlidingWindowWidth);
	fNumberOfCoinc.clear();
	
	// fNumberOfCoinc.resize(fRoughSlidingWindowWidth);
	/*
	fNumberOfAlive = 0;
	for (size_t i = 0; i < fMap.maxUID; i++) {
		TString tmp = fMap.tsShift[i]->GetName();
		if ( tmp.Contains("NaD") ) {
			fPendingUID[i] = -1;
		}
		else {
			fNumberOfAlive++;
		}
	}					
	fMaxPendingSize = fNumberOfAlive; */
	
	// init statictics
	fNbEventsInSlicing = fNbEventsInContainer = 0; // fPendingUID.resize(fMap.maxUID,0); 
}

NOMADBuilder::~NOMADBuilder() 
{  
	// in principle not needed ... just in case reset has not been called by narval
	UInt_t error = 0u; process_reset(&error) ; 
} 

void  NOMADBuilder::Push(NOMADEvent *event)
{	
	Int_t slot; std::vector <NOMADEvent *> *a_vector_of_event;
	
#ifdef LOCAL_DEBUG	
	std::cout << " PUSH " <<  event->TS << " with current sliding " << fRoughSlidingWindow.first << " " << fRoughSlidingWindow.second << std::endl;
#endif
	
	
	/*
//	if ( fNbEventsInSlicing == 0u && fNbEventsInContainer == 0 ) {
	if ( fRoughSlidingWindow.first == 0 ) {
		
		if (  event->TS > fRoughSlidingWindow.first )
			printf("  --> ");
		
		fRoughSlidingWindow.first  = event->TS;
		fRoughSlidingWindow.second = event->TS + fRoughSlidingWindowWidth;
		
		slot = event->TS - fRoughSlidingWindow.first; // fListOfSliding[slot]->push_back( event );	
	
		Int_t slot_ = (slot) / fFineSlidingWindowWidth;
		// + new event : get en empty slot im the stacke of 
		a_vector_of_event = fStackOfCoinc.top(); 
		fStackOfCoinc.pop();
		fRoughArrayOfSliding[slot_] = a_vector_of_event; a_vector_of_event->clear(); a_vector_of_event->push_back(event);
		
		fNbEventsInSlicing++;
		
#ifdef LOCAL_DEBUG
		printf(" FIRST EVENT in sliding window %d \n",slot);
#endif
		return;
	}
	 */
	 	
	if ( event->TS < fRoughSlidingWindow.first ) { // back in time means new file as TS is reset for new files

#ifdef LOCAL_DEBUG		
		printf(" Add gap to timestamp since back to time \n");
#endif
		fLocalTSOffset = fRoughSlidingWindow.second + fChainTSGAP;
		event->TS += 
			fLocalTSOffset;
		
		//fListOfFreeEvents.push(event);
		//return;
	}
	if ( event->TS >= fRoughSlidingWindow.first && event->TS < fRoughSlidingWindow.second ) {
		
		slot = event->TS - fRoughSlidingWindow.first;
		
		Int_t slot_ = (slot) / fFineSlidingWindowWidth;
		if ( fRoughArrayOfSliding[slot_] == 0x0 ) {
			// + new event : get en empty slot im the stacke of 
			// fRoughArrayOfSliding[slot_] = 0;
			a_vector_of_event = fStackOfCoinc.top(); 
			fStackOfCoinc.pop();
			a_vector_of_event->clear(); 
			a_vector_of_event->push_back(event);
			
			fRoughArrayOfSliding[slot_] = a_vector_of_event; 
		}
		else { // only push
			fRoughArrayOfSliding[slot_]->push_back(event);
		}
		
#ifdef LOCAL_DEBUG
		printf(" EVENT in sliding window %d \n",slot);
#endif
		
		fNbEventsInSlicing++;
	}
	else {

		UShort_t uid = event->UID; // fPendingEvents[ uid ].push_back ( event ); 
		
		if ( fNbEventsInContainer == 0 ) { // to know whhen is expected the next pending event
			fPendingEventsLowTS = event->TS;
		}
		else {
			if ( event->TS < fPendingEventsLowTS )
				fPendingEventsLowTS = event->TS;
		}
		
		fQueueOfPending.push_back( event );
		
		fNbEventsInContainer++;

#ifdef LOCAL_DEBUG
		printf(" EVENT in container UID %d \n",uid);
#endif

		/*
		switch (fPendingUID[ uid ]) {
			case -1: // means has been set to dead but back to live
				fPendingUID[ uid ] = 0;
				// fNumberOfAlive++;
			case 0: // means at least one pending event for that detector
				// fNumberOfPending++;
			default:
//				std::cout << " " << fPendingUID[ uid ] << std::endl;
				fPendingUID[ uid ]++;
				break;
		}	
		 */
	}

}

/*
void  NOMADBuilder::Flush()
{

}
*/

/*
Bool_t NOMADBuilder::Front(UShort_t uid, NOMADEvent &_event)
{
	if ( fPendingEvents[ uid ].size() == 0 )
		return false;
	
	_event = fPendingEvents[ uid ].front();
	return true;
}

Bool_t NOMADBuilder::Pop(UShort_t uid, NOMADEvent &_event)
{
	if ( Front(uid,_event) ) {
		fPendingEvents[ uid ].pop_front();
		return true;
	}
	
	return false;
}
*/

UInt_t NOMADBuilder::SetInput()
{	
	Short_t CCrate, CBoard, CCell; Int_t CRawValue; ULong64_t CRawTS; Int_t w0, w1; ULong64_t ts_fast, ts_slow, ts_slow_tmp; 
	
	// stream composed of 2 words of 32 bits 	
	// decode each word
	(*fCurrentIN) >> w0 >> w1;
	//
	CBoard = (w0 & 0xF0000000) >> 28;
	CCell  = (w0 & 0x0E000000) >> 25;
	CCrate = (w1 & 0xC0000000) >> 30;
	
	// Be sure it does not exceed map 
	if ( !fMap.IsValid(CCrate,CBoard,CCell) ) {
		Log << error << " Non expected [crate,board,cell] read " << Form("[%d,%d,%d]",CCrate,CBoard,CCell) << dolog;
		fBadEvents++;
		return 1u;
	}	
	//
	CRawValue = w0 & 0x00007FFF;
	if ( !fMap.IsValid(CCrate,CBoard,CCell) ) {
		Log << error << " ADC is 0 for [crate,board,cell] " << Form("[%d,%d,%d]",CCrate,CBoard,CCell) << dolog;
		fBadEvents++;
		return 1u;
	}	
	//
	ts_slow_tmp = w0 & 0x01FF8000; ts_slow = (ts_slow_tmp << 15); ts_fast = w1 & 0x3FFFFFFF;
	CRawTS = ts_slow | ts_fast;
	//! take into account chaining of files 
	CRawTS += fLocalTSOffset;
	
	fTotNumberOfEvents++;
	
	if ( fListOfFreeEvents.size() ){
#ifdef LOCAL_DEBUG
		printf("SETINPUT List of free events %d \n",fListOfFreeEvents.size()); // to be done, expand allocated
#endif
		UShort_t uid = fMap.toUID[CCrate][CBoard][CCell]; 
		
		TParameter<Int_t> *shift = 
			(TParameter<Int_t> *)fMap.tsShift.At(uid);
		
		TF1 *calib = 
			(TF1 *)fMap.adcCalib.At(uid);
		
		fCEvent = fListOfFreeEvents.top(); 
		fListOfFreeEvents.pop();
		
		Int_t ts_shift = shift->GetVal();
		
		if ( ts_shift > 0 && CRawTS < ULong64_t(ts_shift) )
			fCEvent->TS = 0UL;
		else 
			fCEvent->TS  = CRawTS - ts_shift; 
		fCEvent->DTS = 0;
		fCEvent->UID = uid;
		fCEvent->E   = calib->Eval(CRawValue+fRandom->Uniform());
	}
	else {
		printf("NO MORE FREE EVENT AVAILABLE \n"); // to be done, expand allocated
		fCEvent = 0x0;
//		return 2u;
	}

	return 0u;
}


void NOMADBuilder::process_config (const Char_t *directory_path, UInt_t *error_code)
{
	// In case your algorithm needs to initiate globals, it should be done here
//	NarvalInterface::process_config(directory_path,error_code);
  
	if ( (*error_code) == 0u ) { // now init your stuff
		MainKeyFactory::theMainFactory().Register("Default");		
		MainFrameFactory::theMainFactory().Register("Default");
	}
}

UInt_t NOMADBuilder::Process()
{
	// process means if the presure is too high in the Container list, move the sliding window by one CoincWindow 	
	NOMADEvent *n_event = 0x0, *n_first_event = 0x0; std::deque< NOMADEvent *> *a_queue; Int_t nb_added_to_sliding = 0, nb_removed_from_sliding = 0;

//	Int_t fNbEventsInSlicing_ = fNbEventsInSlicing;

	// fill EventContainer, check if confitions for an event is there and then fill Event list
	Bool_t do_event = false;

	if ( fCEvent ) { // an input event, push it and in case it goes to the container with enough conditions, do event list
		Push(fCEvent);
		if (fNbEventsInContainer) {
			do_event = true;
		}
	}
	else {
		do_event = true;
		printf(" force flush \n");
	}	
	if ( do_event ) { 

#ifdef LOCAL_DEBUG
		std::cout << std::endl << " # in SLIDING " << fNbEventsInSlicing << " # in Container " << fNbEventsInContainer << " " ;
#endif
			
		Bool_t do_i_ev, do_i_evpp;
		
		/*
		Int_t tot_sliding = 0;
		for (size_t i_ev = 0; i_ev < fNbOfFineWindowWidthInRoughSlidingWidth; i_ev++) { // keep only two last slot, empty others
			if ( fRoughArrayOfSliding[i_ev] ) { // empty, no need to roll over
				tot_sliding += fRoughArrayOfSliding[i_ev]->size();
			}
		}
		if ( fNbEventsInSlicing != tot_sliding ) {
			printf("Ha hum \n ");
		}
		size_t lofc = fListOfCoinc.size(); */
		
		size_t go_out = 0;
		for (size_t i_ev = 0; i_ev < fNbOfFineWindowWidthInRoughSlidingWidth-1; i_ev++) { // keep only two last slot, empty others
	
			do_i_ev = do_i_evpp = false;
			
			if (fNbEventsInSlicing == 0) { // empty, no need to roll over
//				std::cout << " GO OUT AT " << i_ev << std::endl;
				go_out = i_ev;
				break;
			}
			if ( fRoughArrayOfSliding[i_ev] == 0x0 ) { // 				
//				printf("NO EVENTS \n");
				continue;
			}
//				printf("# EVENTS %d \n",fRoughArrayOfSliding[i_ev]->size());
			
			std::fill(fCoincBuilderHisto.begin(), fCoincBuilderHisto.end(), 0.0);
			
		
#ifdef LOCAL_DEBUG
			for (size_t i = 0; i < fTwiceCoincWindowWidth; i++) {
				if ( fCoincBuilderEvent[i].size() )
					std::cout << " AT THIS LEVEL should be 0 " << std::endl ;
			}
#endif

			do_i_ev = true;
			
			// number of events to be treated in both and full
			size_t nb_to_treat, nb_to_treat_i_ev, nb_to_treat_i_evpp; 
			nb_to_treat = nb_to_treat_i_ev = fRoughArrayOfSliding[i_ev]->size(); nb_to_treat_i_evpp = 0;
			
			ULong64_t slot;
			// build histogram used to build coincidences
			for (size_t kk = 0; kk < fRoughArrayOfSliding[i_ev]->size(); kk++) {
				slot = fRoughArrayOfSliding[i_ev]->at(kk)->TS - (fRoughSlidingWindow.first + i_ev*fFineSlidingWindowWidth);
				
#ifdef LOCAL_DEBUG
				if ( slot > fTwiceCoincWindowWidth ) {
					printf(" SLOT two High \n");
					std::cout << fRoughArrayOfSliding[i_ev]->at(kk)->TS << " " << fRoughSlidingWindow.first << std::endl;
				}
#endif
				for (size_t jj = 0; jj < fTwiceCoincWindowWidth; jj++) {
					fCoincBuilderHisto[jj] += fCoincPattern[slot][jj];
				}
				fCoincBuilderEvent[slot].push_back( fRoughArrayOfSliding[i_ev]->at(kk) );
			}
			// remove from sliding in any cases
			fNbEventsInSlicing -= 
				fRoughArrayOfSliding[i_ev]->size();	
			fRoughArrayOfSliding[i_ev]->clear(); fStackOfCoinc.push( fRoughArrayOfSliding[i_ev] ); fRoughArrayOfSliding[i_ev] = 0x0; 
			
			if ( fRoughArrayOfSliding[i_ev+1] ) { // add also next one in case an event is in coinc with the next slot
				
				do_i_evpp = true;
				
				nb_to_treat_i_evpp = 
					fRoughArrayOfSliding[i_ev+1]->size();
				
				nb_to_treat += nb_to_treat_i_evpp;
				
				// build histogram used to build coincidences
				for (size_t kk = 0; kk < fRoughArrayOfSliding[i_ev+1]->size(); kk++) {
					slot = fRoughArrayOfSliding[i_ev+1]->at(kk)->TS - (fRoughSlidingWindow.first + i_ev*fFineSlidingWindowWidth);
					for (size_t jj = 0; jj < fTwiceCoincWindowWidth; jj++) {
						fCoincBuilderHisto[jj] += fCoincPattern[slot][jj];
					}
					fCoincBuilderEvent[slot].push_back( fRoughArrayOfSliding[i_ev+1]->at(kk) );
				}
				fRoughArrayOfSliding[i_ev+1]->clear(); 
				// potentially could be used again in case events stay pendings because could be in coinc with next one
			}
			Bool_t new_coinc = true; Bool_t interrupt = false; size_t tot_nb_coinc = 0, nb_coinc = 0; Float_t val_coinc = 0.0;
			size_t stop_jj = 0;
			
#ifdef LOCAL_DEBUG
			std::cout << " START Building event around " << i_ev << " ----------- " << std::endl;
#endif			
			for (size_t jj = 0; jj < fTwiceCoincWindowWidth; jj++) {

				stop_jj = jj;
				
				if ( interrupt && new_coinc ) {
//					printf("# STOP @ %d \n",jj);
					break;
				}
				if ( jj == (fFineSlidingWindowWidth-1) )
					interrupt = true; // treatment only of event starting within coincwidth
				
				// test if the current coinc, if any, is to be closed
				if ( !new_coinc ) {
					if ( fCoincBuilderHisto[jj] < val_coinc ) { // close coinc.

#ifdef LOCAL_DEBUG
						if ( TotInSize != TotInEvent ) {
							printf("rr \n ");
						}
						std::cout << " Close coinc " << nb_coinc << " @ slot " << jj << std::endl;
						std ::cout << "Close coinc Number of event " << TotInSize << " expected " << TotInEvent << std::endl; 
#endif			
						
						TotInSize+=nb_coinc;
						
						fNumberOfCoinc.push_back(nb_coinc);
						nb_coinc = 0;
						new_coinc = true;

					}
				}
				if ( fCoincBuilderEvent[jj].size() == 0 ) {
					continue;
				}
				// open a new coinc
				if ( new_coinc ) {
#ifdef LOCAL_DEBUG					
					std::cout << " Open coinc @ slot " << jj << std::endl;
#endif			
					
					val_coinc = fCoincBuilderHisto[jj];
					nb_coinc  = 0; 
					new_coinc = false;
				}
				// now move event in ListOfCoinc
				for (size_t kk = 0; kk < fCoincBuilderEvent[jj].size(); kk++) {
					
					if ( fCoincBuilderEvent[jj].at(kk) == 0x0 ) {
						printf("WARNING PUSH EMPTY EVENT \n ");
					}
					fListOfCoinc.push_back( fCoincBuilderEvent[jj].at(kk) );
					
#ifdef LOCAL_DEBUG										
					TotInEvent++;
					std ::cout << "Inject coinc Number of event " << TotInSize << " expected " << TotInEvent << std::endl; 
					std::cout << fCoincBuilderEvent[jj].at(kk)->TS << " @@@ " << jj << std::endl;
#endif			

					tot_nb_coinc++;
					nb_coinc++;
				}
#ifdef LOCAL_DEBUG														
				std::cout << fCoincBuilderEvent[jj].size() << " event injected " << nb_coinc << std::endl;
#endif			

				fCoincBuilderEvent[jj].clear();
				
			} // jj build coincidences

			if ( stop_jj == fTwiceCoincWindowWidth - 1 && nb_coinc ) { // force closing
#ifdef LOCAL_DEBUG																		
				printf(" Force closing \n ");
#endif			
				
				fNumberOfCoinc.push_back(nb_coinc);
				TotInSize+=nb_coinc;
			}
			// re-inject events if still some in i_ev+1
			if ( !( tot_nb_coinc == nb_to_treat ) ) {
				
#ifdef LOCAL_DEBUG														
				printf(" Event re-injected STOP @ %d size %d  \n ",stop_jj,fTwiceCoincWindowWidth);
#endif			
				for (size_t jj = stop_jj ; jj < fTwiceCoincWindowWidth; jj++) {
					if ( fCoincBuilderEvent[jj].size() == 0 ) {
						continue;
					}
#ifdef LOCAL_DEBUG														
					if ( fRoughArrayOfSliding[i_ev+1]->size() ) {
						printf("Should BE EMPTYYY \n");
					}
#endif			
					for (size_t kk = 0; kk < fCoincBuilderEvent[jj].size(); kk++) {
						fRoughArrayOfSliding[i_ev+1]->push_back( fCoincBuilderEvent[jj][kk] );
					}
					fCoincBuilderEvent[jj].clear();
				}
				fNbEventsInSlicing -= (tot_nb_coinc-nb_to_treat_i_ev);
			}
			else {
				if ( fRoughArrayOfSliding[i_ev+1] ) {
					fNbEventsInSlicing -= nb_to_treat_i_evpp;	
					fStackOfCoinc.push( fRoughArrayOfSliding[i_ev+1] ); fRoughArrayOfSliding[i_ev+1] = 0x0; 
				}
			}
			

#ifdef LOCAL_DEBUG
			std::cout << " STOP Building event around " << i_ev << " ----------- " << std::endl;
#endif					

		}
		
#ifdef LOCAL_DEBUG
//		std::cout << " size list of coinc " << lofc << " " << fListOfCoinc.size();
		size_t bb = 0;
		for (size_t i_ev = go_out; i_ev < fNbOfFineWindowWidthInRoughSlidingWidth; i_ev++) { // keep only two last slot, empty others
			if ( fRoughArrayOfSliding[i_ev] ) { // empty, no need to roll over
//				printf("%d not empty ! ",i_ev);;
				bb += fRoughArrayOfSliding[i_ev]->size();
			}
		}
		printf("BIlan %d %d ",bb,fNbEventsInSlicing);;
#endif					
		
		// move event @ the end to the beginning and in case the pending and redefine sliding window
		Bool_t redo_sliding = false;
		if ( fRoughArrayOfSliding[fNbOfFineWindowWidthInRoughSlidingWidth-1] ) {
			
#ifdef LOCAL_DEBUG
			std::cout << " CHANGE SLIDING using last event " << fRoughArrayOfSliding[fNbOfFineWindowWidthInRoughSlidingWidth-1]->size() << std::endl;
#endif		
			/*
			for (size_t nn = 0; nn = fRoughArrayOfSliding[fNbOfFineWindowWidthInRoughSlidingWidth-1]->size(); nn++) {
				std::cout << fRoughArrayOfSliding[fNbOfFineWindowWidthInRoughSlidingWidth-1]->at(nn)->TS << std::endl;
			}
			 */
	
			fRoughSlidingWindow.first  = fRoughSlidingWindow.first + (fNbOfFineWindowWidthInRoughSlidingWidth-D_SLIDING_SECURITY)*fFineSlidingWindowWidth; 
			fRoughSlidingWindow.second = fRoughSlidingWindow.first + fRoughSlidingWindowWidth;
			
			fRoughArrayOfSliding[D_SLIDING_SECURITY-1] = fRoughArrayOfSliding[fNbOfFineWindowWidthInRoughSlidingWidth-1];

			/*
			fRoughSlidingWindow.first  = fRoughSlidingWindow.first + (fNbOfFineWindowWidthInRoughSlidingWidth)*fFineSlidingWindowWidth; 
			fRoughSlidingWindow.second = fRoughSlidingWindow.first + fRoughSlidingWindowWidth;	
				*/
			

			
			redo_sliding = true;
			
			fRoughArrayOfSliding[fNbOfFineWindowWidthInRoughSlidingWidth-1] = 0x0 ;
				
#ifdef LOCAL_DEBUG
			std::cout << " CHANGE SLIDING using last event " << fRoughSlidingWindow.first << " " << fRoughSlidingWindow.second << std::endl;
#endif
		}		
		
		// used pending otherwise move to next
		size_t nb_pending = fQueueOfPending.size();
		
		if ( fNbEventsInContainer != nb_pending ) {
			printf("warning \n");
		}
			
				
		if ( !redo_sliding ) {
			if ( nb_pending ) {
				ULong64_t min_ts = kMaxULong64;
				for (size_t i = 0; i < nb_pending; i++) {
					if ( fQueueOfPending[i]->TS < min_ts )
						min_ts = fQueueOfPending[i]->TS;
				}
				fRoughSlidingWindow.first  = min_ts - D_SLIDING_SECURITY*fFineSlidingWindowWidth; // for security in case delayed event are still to com
				fRoughSlidingWindow.second = fRoughSlidingWindow.first + fRoughSlidingWindowWidth;		
#ifdef LOCAL_DEBUG
				std::cout << " CHANGE SLIDING using pending event " << fRoughSlidingWindow.first << " " << fRoughSlidingWindow.second << std::endl;
#endif
			}
			else {
				fRoughSlidingWindow.first  = fRoughSlidingWindow.first + (fNbOfFineWindowWidthInRoughSlidingWidth-D_SLIDING_SECURITY)*fFineSlidingWindowWidth; 
				fRoughSlidingWindow.second = fRoughSlidingWindow.first + fRoughSlidingWindowWidth;	
#ifdef LOCAL_DEBUG
				std::cout << " CHANGE SLIDING by moving by one unit " << fRoughSlidingWindow.first << " " << fRoughSlidingWindow.second << std::endl;
#endif
			}
		}
		// now try and move pending in new sliding
		std::vector  < NOMADEvent * > tmp(100); 	
		tmp.clear();
		
		for (size_t i_ev = 0; i_ev < nb_pending; i_ev++) {
			
			Bool_t is_to_be_keep = true;
			
			if ( fQueueOfPending[i_ev]->TS >= fRoughSlidingWindow.first ) {
				
				Int_t slot = fQueueOfPending[i_ev]->TS - fRoughSlidingWindow.first; 
					
				if ( fQueueOfPending[i_ev]->TS < fRoughSlidingWindow.second ) { // in new window
					fNbEventsInSlicing++;
					Int_t slot_ = (slot) / fFineSlidingWindowWidth;
					if ( fRoughArrayOfSliding[slot_] == 0x0 ) {
						std::vector <NOMADEvent *> *a_vector_of_event = fStackOfCoinc.top(); 
						fStackOfCoinc.pop();
						a_vector_of_event->clear(); 
						a_vector_of_event->push_back(fQueueOfPending[i_ev]);
						
						fRoughArrayOfSliding[slot_] = a_vector_of_event; 
						
					}
					else { // only push
						fRoughArrayOfSliding[slot_]->push_back(fQueueOfPending[i_ev]);
					}
					is_to_be_keep = false;
				}
			}
			else {
				fListOfFreeEvents.push( fQueueOfPending[i_ev] ) ;
				printf(" Pending before current sliding \n");
			}	
			if ( is_to_be_keep ) {
				tmp.push_back(fQueueOfPending[i_ev]);
			}
			if ( fNbEventsInContainer == 0u ) {
				printf("warning \n");
			}
		//	fNbEventsInContainer--;
		}
		fQueueOfPending.clear();
		for (size_t i = 0; i < tmp.size(); i++) {
			fQueueOfPending.push_back( tmp[i] ) ;
		}
		fNbEventsInContainer = fQueueOfPending.size();
		
		/*
		for (Int_t i = 0; i < fRoughSlidingWindowWidth; i++) {
			
			Int_t maxi = fRoughSlidingWindowWidth - fFineSlidingWindowWidth; Int_t previous_ts_i = 0, current_ts_i = 0 ;
			
			if (fNbEventsInSlicing == 0) { // empty, no need to roll over
				break;
			}
			current_ts_i = i; 
			
			a_queue = fListOfSliding.front(); size_t q_size = a_queue->size(); 
			if ( q_size ) {
#ifdef LOCAL_DEBUG
				std::cout << std::endl << "An event built of size " << a_queue->size() << " slot " << i << " " ;
#endif
//				fListOfCoinc.push_back((NOMADEvent *)(q_size));
				if ( (i - previous_ts_i) < fFineSlidingWindowWidth ) { // coinc with previous
//					if ( fNumberOfCoinc.empty() )
//						fNumberOfCoinc.push_back(q_size);
//					else 
//						fNumberOfCoinc.back() += q_size; 
					fNbEventsInSlicing -= q_size;
				}
				else { 
					if ( i < maxi ) {
						previous_ts_i = i; 
//						fNumberOfCoinc.push_back(q_size); fNbEventsInSlicing -= q_size; // a new coinc
					}
					else { 
#ifdef LOCAL_DEBUG
						printf("Break with %d \n",i);
#endif						
						break;
					}
				}
				while ( q_size ) {
					n_event = a_queue->front(); 
#ifdef LOCAL_DEBUG					
					std::cout << n_event->TS << " " << n_event->UID << " " << n_event->E << " ";
#endif
					a_queue->pop_front();
					q_size--;
					nb_removed_from_sliding++;
					
					// added to the event queue ! 
					// NEW
//					fListOfCoinc.push_back ( n_event );
				}
			}
			fListOfSliding.pop_front(); fListOfSliding.push_back( a_queue ); 
		}
		 */

		/*
		// last event 
		if ( fNbEventsInSlicing_ ) {
#ifdef LOCAL_DEBUG	
			std::cout << " CHANGE SLIDING OL " <<  current_ts_i << " with current sliding " << fRoughSlidingWindow.first << " " << fRoughSlidingWindow.second << std::endl;
#endif
			fRoughSlidingWindow.first  = fRoughSlidingWindow.first + current_ts_i; 
			fRoughSlidingWindow.second = fRoughSlidingWindow.second + current_ts_i ;
#ifdef LOCAL_DEBUG	
			std::cout << " CHANGE SLIDING NE " <<  current_ts_i << " with current sliding " << fRoughSlidingWindow.first << " " << fRoughSlidingWindow.second << std::endl;
#endif
		}
		else {
			// no more event in the window, next one is the container
			fRoughSlidingWindow.first  = fPendingEventsLowTS; 
			fRoughSlidingWindow.second = fPendingEventsLowTS + fRoughSlidingWindowWidth ;
			
		}
		 */
		
		/*
		if ( fNbEventsInContainer ) {
			// put the pending event in the Sliding window 
			fPendingEventsLowTS = fRoughSlidingWindow.second ;
			for (Int_t i = 0; i < fMap.maxUID; i++) {
								
				if ( fPendingUID[i] < 1 ) {
					continue;
				} 
				n_event = fPendingEvents[i].front(); 
				
				Int_t slot = n_event->TS - fRoughSlidingWindow.first; 
				
				if ( slot < 0 ) {
#ifdef LOCAL_DEBUG	
					std::cout << " negative slot " << n_event->TS << " with current sliding " << fRoughSlidingWindow.first << " " << fRoughSlidingWindow.second << std::endl;;
#endif	
					fPendingEvents[i].pop_front(); fNbEventsInContainer--; fPendingUID[i]--;
					continue;
				}
				if ( slot < fRoughSlidingWindowWidth ) {
					fPendingEvents[i].pop_front(); fNbEventsInContainer--; fPendingUID[i]--;
					fListOfSliding[slot]->push_back( n_event );		
					fNbEventsInSlicing++;
				}
				else {
					if ( n_event->TS < fPendingEventsLowTS )
						fPendingEventsLowTS = n_event->TS ;
				}
			}			
		}
		*/
	}
	
	return 0u;
}

UInt_t NOMADBuilder::SetOutput(ADF::FrameBlock & out)
{	
	// size requested per item streamed out. 
	// KEY [ FS(ushort:2bytes) | TS(ulong64:8bytes)] 
	//   DATA [ uid(ushort:2bytes) | e(Float:4bytes) ] if not correlations
	//   DATA [Delta TS(Int:4bytes)] [ uid(ushort:2bytes) | e(Float:4bytes) ] if correlations, Delta TS is a correction to the timestamp
	Int_t key_size = 10, data_size_item = 8;
	switch (1) {
		case 0:
			data_size_item = 6; // no events means 
			break;			
		case 1:
			data_size_item = 8; // 
			break;
		default:
			break;
	}
	
	// frame to stream out the current
	RawFrame *f = 
		(RawFrame *)fOut->GetFrame(); 
	BufferIO &bio = f->RawBuffer();
	
	/*
	std ::cout << "Number of Coinc " << fNumberOfCoinc.size() << std::endl; size_t tot = 0;
	for (size_t i = 0; i < fNumberOfCoinc.size() ; i++) {
		tot +=  fNumberOfCoinc[i];
	}
	std ::cout << "TOT Number of event " << tot << " expected " << fListOfCoinc.size() << std::endl; 
	std ::cout << "TOTTOT Number of event " << TotInSize << " expected " << TotInEvent << std::endl; 
	 */
	
	while ( fNumberOfCoinc.size() ) {

		// next event size in unit of nomad event
		Int_t size_r = 
			fNumberOfCoinc.front(); 
		fNumberOfCoinc.pop_front();	
			
		// build collection depending on the type of event : Ge, AC, full, unknown etc 
		fUnknownEvent.clear();
		fGEEvent.clear(); 
		fACEvent.clear();
		fLAEvent.clear();
		fEvent.clear();
		fOutEvent.clear();
		
		ULong64_t key_ts, key_ts_all = 0UL, key_ts_ge = 0UL, key_ts_ac = 0UL, key_ts_la = 0UL;
		for (Int_t i = 0; i < size_r; i++) {
			NOMADEvent *an_event = 
				fListOfCoinc.front();
			fListOfCoinc.pop_front();
			
			key_ts_all += an_event->TS;
			
//			if ( an_event->UID >= kGERange.second.first && an_event->UID < kGERange.second.second ) {
            if ( fMap.allDigitizer[an_event->UID].Family == 0 ) {
				key_ts_ge += an_event->TS;
				fGEEvent.push_back(an_event);
			}
			else {
				if ( fMap.allDigitizer[an_event->UID].Family == 1 ) {
					key_ts_ac += an_event->TS;
					fACEvent.push_back(an_event);
				}
                else {
                    if ( fMap.allDigitizer[an_event->UID].Family == 2 ) {
                        key_ts_la += an_event->TS;
                        fLAEvent.push_back(an_event);
                    }
                    else {
                        std::cout << "UNKNOWN " << an_event->TS << " " << an_event->UID << " " << an_event->E << " ";
                        fUnknownEvent.push_back(an_event);
                    }
                }
			}
			fEvent.push_back(an_event);
			
			fListOfFreeEvents.push(an_event);
			//std::cout << an_event->TS << " " << an_event->UID << " " << an_event->E << " ";
//			printf("SETOUTPU List of free events %d \n",fListOfFreeEvents.size()); 

		}		
		
		// Do time / compton rejections and addback if required
		Bool_t marked = false, do_stream_out = false;
		
		switch (fOutputMode) {
                
            fOutEvent.clear();

			case 0: // keep everything, only set relative time compare to the global calculed by La in case their are, otherwize global
                if ( fLAEvent.size() ) {
                    key_ts = key_ts_la / fLAEvent.size();
                }
				else key_ts = key_ts_all / size_r;
                
				for (size_t i = 0; i < size_r; i++) { // just time
					if ( fEvent[i]->TS < key_ts ) {
						fEvent[i]->DTS = -Int_t(key_ts - fEvent[i]->TS);
					}
					else 
						fEvent[i]->DTS = +Int_t(fEvent[i]->TS - key_ts);
                    
                    fOutEvent.push_back( fEvent[i] );

//					std::cout << i << " key " <<  key_ts << " " << fEvent[i]->DTS << std::endl;
				}
				do_stream_out = true;
				break;
				
			case 1: // compton rejection, no addback
                
                // treat TS of the event
                if ( fLAEvent.size() ) {
                    key_ts = key_ts_la / fLAEvent.size();
                }
				else {
                    if ( fGEEvent.size() )
                        key_ts = key_ts_ge / fGEEvent.size();
                    else
                        key_ts = key_ts_all / size_r;
                }
				
                // Ge part
				for (size_t i = 0; i < fGEEvent.size(); i++) {

					if ( fGEEvent[i]->TS < key_ts ) {
						fGEEvent[i]->DTS = -Int_t(key_ts - fGEEvent[i]->TS);
					}
					else 
						fGEEvent[i]->DTS = +Int_t(fGEEvent[i]->TS - key_ts);
					
					marked = false;
					for (size_t j = 0; j < fACEvent.size(); j++) {
						if ( (fMap.allDigitizer[fGEEvent[i]->UID].Group) == (fMap.allDigitizer[fACEvent[j]->UID].Group) ) {
							marked = true;
							break;
						}
					}
					if ( !marked ) { // 
						fOutEvent.push_back( fGEEvent[i] );
					}
				}
				if ( !(fOutEvent.size() < kGEThreshold) )
					do_stream_out = true;
                
                // Add LA part
                if ( do_stream_out ) {
                    if ( !(fLAEvent.size() < kLAThreshold) ) {
                        for (size_t i = 0; i < fLAEvent.size(); i++) {
                            
                            if ( fLAEvent[i]->TS < key_ts ) {
                                fLAEvent[i]->DTS = -Int_t(key_ts - fLAEvent[i]->TS);
                            }
                            else
                                fLAEvent[i]->DTS = +Int_t(fLAEvent[i]->TS - key_ts);
                            
                            fOutEvent.push_back( fLAEvent[i] );
                        }
                    }
                    else do_stream_out = false;
                }
                   
				break;
			case 10: // compton rejection, full addback
                
                // treat TS of the event
                if ( fLAEvent.size() ) {
                    key_ts = key_ts_la / fLAEvent.size();
                }
				else {
                    if ( fGEEvent.size() )
                        key_ts = key_ts_ge / fGEEvent.size();
                    else
                        key_ts = key_ts_all / size_r;
                }
				
				for (size_t i = 0; i < fGEEvent.size(); i++) {
					if ( fGEEvent[i]->E < 0 )
						continue; // previously added back
					
					if ( fGEEvent[i]->TS < key_ts ) {
						fGEEvent[i]->DTS = -Int_t(key_ts - fGEEvent[i]->TS);
					}
					else 
						fGEEvent[i]->DTS = +Int_t(fGEEvent[i]->TS - key_ts);
					
					marked = false;
					for (size_t j = 0; j < fACEvent.size(); j++) {
						if ( (fMap.allDigitizer[fGEEvent[i]->UID].Group) == (fMap.allDigitizer[fACEvent[j]->UID].Group) ) {
							marked = true;
							break;
						}
					}
					if ( !marked ) { // full addback
						for(size_t j = i+1; j < fGEEvent.size(); j++) {
							if ( fGEEvent[j]->E > 0 && ( fMap.allDigitizer[fGEEvent[i]->UID].Group == fMap.allDigitizer[fGEEvent[j]->UID].Group ) ) {
								fGEEvent[i]->E += fGEEvent[j]->E;
								fGEEvent[j]->E  = -1;
								continue;
							}
						}
						fOutEvent.push_back( fGEEvent[i] );
					}
				}
				if ( !(fOutEvent.size() < kGEThreshold) )
					do_stream_out = true;
                
                // Add LA part
                if ( do_stream_out ) {
                    if ( !(fLAEvent.size() < kLAThreshold) ) {
                        for (size_t i = 0; i < fLAEvent.size(); i++) {
                            
                            if ( fLAEvent[i]->TS < key_ts ) {
                                fLAEvent[i]->DTS = -Int_t(key_ts - fLAEvent[i]->TS);
                            }
                            else
                                fLAEvent[i]->DTS = +Int_t(fLAEvent[i]->TS - key_ts);
                            
                            fOutEvent.push_back( fLAEvent[i] );
                        }
                    }
                    else do_stream_out = false;
                }
                
				break;
                
			case 11: // compton rejection, simpliest addback i.e. 
                
                // treat TS of the event
                if ( fLAEvent.size() ) {
                    key_ts = key_ts_la / fLAEvent.size();
                }
				else {
                    if ( fGEEvent.size() )
                        key_ts = key_ts_ge / fGEEvent.size();
                    else
                        key_ts = key_ts_all / size_r;
                }
				
				for (size_t i = 0; i < fGEEvent.size(); i++) {
					if ( fGEEvent[i]->E < 0 )
						continue; // previously added back
					
					if ( fGEEvent[i]->TS < key_ts ) {
						fGEEvent[i]->DTS = -Int_t(key_ts - fGEEvent[i]->TS);
					}
					else 
						fGEEvent[i]->DTS = +Int_t(fGEEvent[i]->TS - key_ts);
					
					marked = false;
					for (size_t j = 0; j < fACEvent.size(); j++) {
						if ( (fMap.allDigitizer[fGEEvent[i]->UID].Group) == (fMap.allDigitizer[fACEvent[j]->UID].Group) ) {
							marked = true;
							break;
						}
					}
					if ( !marked ) { //  addback
						for(size_t j = i+1; j < fGEEvent.size(); j++) {
							if ( fGEEvent[j]->E > 0
								 && ( fMap.allDigitizer[fGEEvent[i]->UID].Group == fMap.allDigitizer[fGEEvent[j]->UID].Group ) 
								 && ( kACPattern[ fMap.allDigitizer[fGEEvent[i]->UID].Item*4 + fMap.allDigitizer[fGEEvent[j]->UID].Item ] ) ) 
							{
								fGEEvent[i]->E += fGEEvent[j]->E;
								fGEEvent[j]->E  = -1;
								continue;
							}
						}
						fOutEvent.push_back( fGEEvent[i] );
					}
				}
				if ( !(fOutEvent.size() < kGEThreshold) )
					do_stream_out = true;
				break;
                
                // Add LA part
                if ( do_stream_out ) {
                    if ( !(fLAEvent.size() < kLAThreshold) ) {
                        for (size_t i = 0; i < fLAEvent.size(); i++) {
                            
                            if ( fLAEvent[i]->TS < key_ts ) {
                                fLAEvent[i]->DTS = -Int_t(key_ts - fLAEvent[i]->TS);
                            }
                            else
                                fLAEvent[i]->DTS = +Int_t(fLAEvent[i]->TS - key_ts);
                            
                            fOutEvent.push_back( fLAEvent[i] );
                        }
                    }
                    else do_stream_out = false;
                }
                
			default:
				break;
		}
				
		if ( !do_stream_out ) { // required number of Clean/addback GeEvent  ! 
//			std::cout << " REJECTED " << std::endl;
			continue;
		}		
		// not enough place to stream out, pop front the current event using copy construcor
		Int_t needed_size_to_stream = fOutEvent.size() * data_size_item + key_size;
		f->FastReset();
		
		if ( bio.Reserve(needed_size_to_stream) && out.Reserve(needed_size_to_stream,true) ) {
			for(size_t i = 0; i < fOutEvent.size(); i++) {
				bio << (Int_t)fOutEvent[i]->DTS << fOutEvent[i]->UID << fOutEvent[i]->E;
			}			
			UIDKey *key = (UIDKey *)f->GetKey();
			key->SetUID(key_ts);
			f->Write();
			
			out.AddFrame((*f));		
		}
		else {
			if ( fEvent.size() ) {
				for(size_t i = 0; i < fEvent.size(); i++) {
					NOMADEvent *ev = 
						fListOfFreeEvents.top();
					fListOfFreeEvents.pop();
					
					(*ev) = (*fEvent[i]);
					fListOfCoinc.push_front(ev); 
				}
				fNumberOfCoinc.push_front(fEvent.size());
				
			//	fListOfCoinc.push_front((NOMADEvent *)(fEvent.size()));
				
				std::cout << " RE INJECTED " << std::endl;

			}
			return 1u;
		}
		// 
	}
	
	return 0u;	
}

UInt_t NOMADBuilder::ProcessBlock(ADF::FrameBlock &in, ADF::FrameBlock &out)
{
	Log.ClearMessage(); Log.SetProcessMethod("process_block"); UInt_t result = 0u;

	in.Rewind(ConfAgent::kRead); out.Rewind(ConfAgent::kWrite);
	
	// if not input && coinc queue, flush + Setoupt
	
	fCurrentIN = in.GetBufferIO();
	//
	if ( in.GetSize() ) {
		while ( 1 ) {
			if ( fCurrentIN->FreeSize() < 4u ) {
				in.SetEoB(true);
				break;
			}
			UInt_t inp = SetInput();
			switch ( inp ) {
				case 0u:
					if ( Process() == 0u ) {
						if ( SetOutput(out) == 0u ) {
						}
					}					
					break;
				case 1u:
					break; // just bad event, do nothing but try next event
				default:
					result = 1u;
					break;
			}
		} 
	}	
	else {
		fCEvent = 0x0;
		if ( fNbEventsInSlicing || fNbEventsInContainer ) {
			if ( Process() == 0u ) {
				if ( SetOutput(out) == 0u ) {
				}
			}
		}
		in.SetEoB(true);
	}

//	printf("NOMADBuilder::ProcessBlock %d %d %d %d \n", in.GetOffset(),out.GetOffset(), in.GetSize(),out.GetSize());
	
	if ( !in.IsEoB() ) {
		Log << warning << " Out is full while input not yet completely treated " << nline;
	}
	out.SetEoB(true);
	
	// send messages to the central log
	Log << dolog;
	return 0u;
}	      


void NOMADBuilder::process_initialise (UInt_t *error_code)
{
	*error_code = 0u; Log.ClearMessage(); Log.SetProcessMethod("process_initialise"); 
	//
	
	// read an input file to get the base filename and the path  
	std::string conffile = 
		GetConfPath() + "NOMADBuilder.conf";
	
	std::ifstream filein(conffile.data());
	if ( filein.is_open() == true ) { 
		
		std::string keyword, tmp; TString nomad_map;
	
		getline(filein,tmp);
		tmp += " ";
		while ( filein.good() && !filein.eof() ) { // read input stream line by line
			//
			if ( tmp[0] == '#' ) {
				getline(filein,tmp);
				tmp += " ";
				continue; 
			} // this line is a comment 
			
			std::istringstream decode(tmp);
			decode.clear();
			decode >> keyword;
			if ( keyword == "NOMADMap" ) {
				decode >> nomad_map;
				
				nomad_map.Prepend(GetConfPath().data());
				
				if ( !fMap.Load(nomad_map.Data()) ) {
					Log << warning << " cannot load configuration file " << nomad_map.Data() << dolog;
					*error_code = 1u;
					return;
				}
			}
			if ( keyword == "OutputMode" ) {
				decode >> fOutputMode;
				Log << info << " SetOuputMode to " << fOutputMode << nline;
			}		
			if ( keyword == "GeThreshold" ) {
				decode >>  kGEThreshold;
				Log << info << " GeThreshold (take into account addback, AC if required) set to " << kGEThreshold << nline;
			}
            if ( keyword == "LaThreshold" ) {
				decode >>  kLAThreshold;
				Log << info << " LaBr3Threshold set to " << kLAThreshold << nline;
			}
			if ( keyword == "RunningMode" ) {
				decode >> fRunningMode ;
				Log << info << " RunningMode to " << fRunningMode << nline;
			}
			getline(filein,tmp);
			tmp += " ";
		}			
		filein.close();
	}
	
	// everyting ok, set collections
	InitOnceConfig();
	
	// print in log the current conditions
	// TO BE DONE
	
	// Output frame produced which is a raw data : 
	// [ FS(ushort:2bytes) | TS(ulong64:8bytes)] [ uid(ushort:2bytes) | e(Float:4bytes) ]
	//
	fOut = MainFrameFactory::theMainFactory().
			NewSharedFrame(FactoryItem("Default","UID",Version(1,1)),FactoryItem("Default","data",Version(0,1)));
	
	Log << dolog;		
}


void NOMADBuilder::process_reset (UInt_t * /*error_code*/)
{
	Log.SetProcessMethod("process_reset"); 
	
	// some log
	Log << " Number of events treated "  << fTotNumberOfEvents << dolog;
	
	if (fOut) {
		delete fOut; fOut = 0x0;
	}

}




