/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/** @file NOMADFile.h header file for NOMADFile.cpp */

#ifndef ILL_NOMADFile
#define ILL_NOMADFile

#include "NarvalInterface.h"
#include "FrameBlock.h"
#include "XtermProgressBar.h"

#include <list>
#include <utility>

//! NOMADFile
/*!
 
 */
class NOMADFile : public ADF::NarvalProducer
{   	
private:
	//! current file 
	FILE *fCurrentFile;
	//! current file number in the vector fFile2Read
	UInt_t fCurrentFileNumber;	
	//! list of file to be read. Set @ init
	std::vector < std::string > fFiles2Read;

private:
	//! Total Number of event expected (from header) 
	ULong64_t fTotEventsExpected;
	//! Total Number of bytes read
	ULong64_t fTotBytesRead;	
	//! Number of event expected (from header) for the current ile 
	ULong64_t fEventsExpected;
	//! Total Number of bytes read
	ULong64_t fEventsRead;	
	
private:
	Gw::XtermProgressBar fProgressBar;
	
protected:	
	Bool_t NewFile();
	
public:
	NOMADFile();
 	virtual ~NOMADFile(); 
	
	//! to init globals (static) from a directory
	static void process_config (const Char_t *, UInt_t *) ;
	
	//! Constructor implementation 	
	virtual	void process_initialise (UInt_t *error_code);
	
	//! Destructor implementation  
	virtual	void process_reset (UInt_t *error_code);
	
	//! Produce one block of data  
	virtual UInt_t ProcessBlock (ADF::FrameBlock &) ;	
};

#endif
