/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** @file NOMADFilter.h header file for NOMADFilter.cpp */

#ifndef _NOMADFilter
#define _NOMADFilter

#include "NarvalInterface.h"
#include "NOMADMap.h"

#include <queue>

class TRandom;

//! NOMADFilter 
/*!
 Process the data stream provided by the NOMAD DAQ and transform it. 
 For that it requires a configuration file.
 
 What is done is 
 - to transform a couple board,cell (electronic point of view) into groupe,item (useful data point of view)
 - for each couple groupe/item, it applies a shift in TS to align them in case of delays
 - it calibrates the energy, giving the true energy
 
 \author O.Stezowski
*/
class NOMADFilter : public ADF::NarvalFilter
{  
private:
	TRandom *fRandom;

protected:
	//! input buffer that contains the NOMAD stream composed of 2 consecutives words of 32 bits.
	ADF::BufferIO *fCurrentIN;
	
protected:
	NOMADMap fMap;
	
protected:
	Short_t fCCrate;
	Short_t fCBoard;
	Short_t fCCell;
	Int_t   fCRawValue;
	ULong64_t fCRawTS;
	
protected:
	Short_t fCGroup;
	Short_t fCItem;
	Float_t fCCalibValue;
	ULong64_t fCCalibTS;
	
protected:
	ADF::SharedFP *fOut;
	
protected:
	ULong64_t fTotNumberOfEvents;
	ULong64_t fBadEvents;
	
protected:
	//! read the data from the trigger and put them in local variable
	virtual UInt_t SetInput();
	//! process this with the current conditions
	virtual UInt_t Process();	
	//! compute output of the process 
	virtual UInt_t SetOutput();
	
public:
 	NOMADFilter();
 	virtual ~NOMADFilter(); 
			
	//! to init globals (static) from a directory
	static void process_config (const Char_t *, UInt_t *) ;
	
	//! Constructor implementation 	
	virtual	void process_initialise (UInt_t *error_code);
	
	//! Destructor implementation  
	virtual	void process_reset (UInt_t *error_code);
		
	//! To process just a block of data
	/*!
	*/	
	virtual UInt_t ProcessBlock(ADF::FrameBlock &, ADF::FrameBlock &);	
	
};

#endif
