/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/** @file NOMADGene.h header file for NOMADGene.cpp */

#ifndef ILL_NOMADGene
#define ILL_NOMADGene

#include "NarvalInterface.h"
#include "FrameBlock.h"
#include "NOMADMap.h"
#include "XtermProgressBar.h"

#include <list>
#include <utility>

 class TRandom;

//! NOMADGene
/*!
 
 */
class NOMADGene : public ADF::NarvalProducer
{   	
private:
	TRandom *fRandom;

protected:
	NOMADMap fMap;

private:
	//! Total Number of event to simulate
	ULong64_t fNumberOfEventsToCreate;
	//! Total Number of event to simulate
	ULong64_t fNumberOfEventsSimulated;	
	//! rate means delta T between two events
	UInt_t fDTRate;
	//! width of the window to generate event
	UInt_t fDTCoinc;	
	//! 
	ULong64_t fCurrentTS;	

	//! # number of fold to 
	UInt_t fFold;
	
private:
	Gw::XtermProgressBar fProgressBar;
	
public:
	NOMADGene();
 	virtual ~NOMADGene(); 
	
	//! to init globals (static) from a directory
	static void process_config (const Char_t *, UInt_t *) ;
	
	//! Constructor implementation 	
	virtual	void process_initialise (UInt_t *error_code);
	
	//! Destructor implementation  
	virtual	void process_reset (UInt_t *error_code);
	
	//! Produce one block of data  
	virtual UInt_t ProcessBlock (ADF::FrameBlock &) ;	
};

#endif
