/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/** @file NOMADFilter.h header file for NOMADFilter.cpp */

#ifndef _NOMADMap
#define _NOMADMap

#include "TObjArray.h"
#include "TF1.h"
#include "TParameter.h"

/*! mapping electronic as delivered by NOMAD and mapping for physics
 */
class NOMADMap
{	
public:
	// from the available # of bits, MAX is 4 - 16 - 8
	// from the current situation @ ILL 2 - 5 - 8
	static const Short_t MAX_NUMBER_OF_CRATE =  4;
	static const Short_t MAX_NUMBER_OF_BOARD = 16;
	static const Short_t MAX_NUMBER_OF_CELLS =  8;
	
public:
	//! Mapping crate,board,cell versus unique id
	UShort_t toUID[MAX_NUMBER_OF_CRATE][MAX_NUMBER_OF_BOARD][MAX_NUMBER_OF_CELLS]; 
	
public:
	//! Max number of UID
	Int_t maxUID; 
	
public:
	//! identify a unique digitizer in the daq
	struct Digitizer 
	{
		//! pure DAQ part
		Short_t Crate;
		Short_t Board;
		Short_t Cell;
		//! physicist
        Short_t Family;
		Short_t Group;
		Short_t Item;
		
		Digitizer() : Crate(-1), Board(-1), Cell(-1), Family(-1), Group(-1), Item(-1)
            {;}
	};
	std::vector < Digitizer > allDigitizer; 
	
public: 
	//! TParameter to align all TS. Slot is uid
	TObjArray tsShift;
	//! functions to calibrate the different adc. Slot is uid
	TObjArray adcCalib;
	
public:
 	NOMADMap();
 	virtual ~NOMADMap()
	{
		;
	}
	
	//! check if crate,board,cell is in the limits
	Bool_t IsValid(Short_t crate, Short_t board, Short_t cell) const
	{
		if ( crate < MAX_NUMBER_OF_CRATE && board < MAX_NUMBER_OF_BOARD && cell < MAX_NUMBER_OF_CELLS ) 
			return true;
		return false;
	}
    	
	//! load map from a file. return true if ok
	Bool_t Load(const Char_t *fname = "NOMADMap.conf");
	
	//! Save map in a file. 
	/*! 
	 - if opt is init, it gives a default i.e. all offset = 0 and calibration functions are indentity
	 - if maxUID is 0, it generates a full map otherwise save the current definition
     
     NOTE : the family field is not saved in the file but for each active element deduced from the name 
     i.e. 
     - Ge 0
     - AC 1
     - LaBr3 2
     - Unknown -1
	 */
	Bool_t Save(const Char_t *fname = "NOMADMap.conf", Option_t *opt = "") const;
	
};

class NOMADEvent 
{
public:
	// timestamp for one detector
	ULong64_t TS;
	//
	Long64_t DTS; // TS difference with a reference TS
	UShort_t UID; // unique detector id
	Float_t    E; // energie
	
public:
	NOMADEvent() : TS(0UL), DTS(0L), UID(0), E(0.0)
		{;}
	NOMADEvent( const NOMADEvent& other )
	{
		this->TS  = other.TS;
		this->DTS = other.DTS;
		this->UID = other.UID;
		this->E   = other.E;		
	}	
	virtual ~NOMADEvent()
	{ 
	//	printf("DES");
	}
	

};

#endif