/***************************************************************************
 *   Copyright (C) 2004-2006 by Olivier Stezowski & Christian Finck        *
 *   stezow(AT)ipnl.in2p3.fr, cfinck(AT)ires.in2p3.fr                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/** \file ALoadedActor.cpp compiled in libADF.so */

#include "ALoadedActor.h"
#include <stdexcept>

//#define L_DEBUG 

using namespace Gw;

Gw::AConfigurator::AConfigurator(ALoadedActor *lactor, ConfigFunction f) :
	fError(0u),
	fRefToLoadedActor(lactor),
	fConfigFunction(f)
{
	;
}

void Gw::AConfigurator::Configure()
{		
	// now call the conf method, get path from the loaded actor
	NamedItem<std::string> conf_path("ConfDir","./"); 
	//
	fRefToLoadedActor->GetGlobal(conf_path);
	//
	std::string path = 
		conf_path.Get();
	fConfigFunction(path.data(),&fError);
}


Gw::ALoadedActor::ALoadedActor(const Char_t *name) : 
	fLog("ALoadedActor"),
	fName(name), 
	fGlobals(),
	fGeneration(0x0),
	fConfigurator(0x0),
	fInputs(), 
	fOutputs(),
	fDepth(-1),
	fActor(0x0),
	fActorType( kUnknown ),
	fActorStatus( kUndefined ),
	fLastError(0u)
{
	;
}

AConfigurator *Gw::ALoadedActor::SetConfigurator(AConfigurator *new_conf)
{
	AConfigurator *old_conf = fConfigurator; fConfigurator = new_conf;
	//
	return old_conf;
}


//! Add one input line to the actor. It does not check if possible ... it has to be done by the user before calling this using GetMaxInput/GetMaxOutput
/*!
 */	
void Gw::ALoadedActor::AddInput(AnActorConnection *aconnection)
{
	// add the connection to this and attach the right block of the connection
	// to the embedded actor
	fInputs.push_back(aconnection);
}

void Gw::ALoadedActor::AddOutput(AnActorConnection *aconnection)
{
	// add the connection to this and attach the right block of the connection
	// to the embedded actor
	fOutputs.push_back(aconnection); 
}

void Gw::ALoadedActor::ComputeDepth(Int_t &depth)
{
	if ( GetNbInput() == 0 )
		return; 
	
	depth++;
	
	Int_t tmp_depth = depth, max_depth = 0;
	
	for (Int_t i = 0; i < GetNbInput() ; i++) {
		ALoadedActor *node = GetInput(i); 
		
		tmp_depth = depth;
		node->ComputeDepth(tmp_depth);
		
		if ( tmp_depth > max_depth ) {
			max_depth = tmp_depth;
		}
	}
	depth = max_depth;
}

void Gw::ALoadedActor::SetGlobals( const std::vector < ANamedItem *> &globals )
{
	for (size_t i = 0; i < globals.size(); i++) {
		Bool_t add_new = true;
		for (size_t j = 0; j < fGlobals.size(); j++) {
			// 
			if ( globals[i]->GetName() == fGlobals[j]->GetName() ) {
					// change is possible means same name same type
				if ( fGlobals[j]->Set(globals[i]) ) {
					add_new = false;
					break;
				}
			}
		}
		if ( add_new ) {
			fGlobals.push_back( globals[i]->MakeAClone() );
		}

	}
	if ( fGeneration == 0x0 ) {
		for (size_t j = 0; j < fGlobals.size(); j++) {
			if ( fGlobals[j]->GetName() == std::string("Generation") ) {
				fGeneration = dynamic_cast< NamedItem<Short_t> * > ( fGlobals[j] );
			}
		}
	}
}

void Gw::ALoadedActor::SetGlobal( const ANamedItem &global )
{
	Bool_t add_new = true;
	// 
	for (size_t j = 0; j < fGlobals.size(); j++) {
		// 
		if ( global.GetName() == fGlobals[j]->GetName() ) {
			// change is possible means same name same type
			if ( fGlobals[j]->Set(&global) ) {
				add_new = false;
				break;
			}
		}
	}
	if ( add_new ) {
		fGlobals.push_back( global.MakeAClone() );
	}
	if ( fGeneration == 0x0 ) {
		for (size_t j = 0; j < fGlobals.size(); j++) {
			if ( fGlobals[j]->GetName() == std::string("Generation") ) {
				fGeneration = dynamic_cast< NamedItem<Short_t> * > ( fGlobals[j] );
			}
		}
	}
}

Bool_t Gw::ALoadedActor::GetGlobal( ANamedItem &global ) const
{
	Bool_t is_set = false;
	// 
	for (size_t j = 0; j < fGlobals.size(); j++) {
		// 
		if ( global.GetName() == fGlobals[j]->GetName() ) {
			// change is possible means same name same type
			if ( global.Set(fGlobals[j]) ) {
				is_set = true;
				break;
			}
		}
	}
	return is_set;
}

Short_t Gw::ALoadedActor::IsLinked(ALoadedActor *actor, AnActorConnection *&connection) const
{
	Short_t result = 0;
	
	//	std::cout << "Input size " << fInputs.size() << " for " <<  << std::endl;
	for (UInt_t i = 0; i < fInputs.size(); i++) {
		if ( fInputs[i]->GetProducer() == actor ) {
			connection = fInputs[i];
			result = -1;
			
			return result;
		}
	}
	
	//	std::cout << "Output size " << fOutputs.size() << std::endl;
	for (UInt_t i = 0; i < fOutputs.size(); i++) {
		if ( fOutputs[i]->GetConsumer() == actor ) {
			connection = fOutputs[i];
			result = +1;
			
			return result;
		}
	}	
	connection = 0x0; return result;
}

void Gw::ALoadedActor::ConfigureActor()
{
	fConfigurator->SetError(0u);
	fConfigurator->Configure();
	
	SetLastError(fConfigurator->GetError());
}

void Gw::ALoadedActor::RegisterActor()
{
	fLog.SetProcessMethod("RegisterActor()");
	
	if ( fConfigurator == 0x0 )
		throw std::bad_alloc();
		
	fConfigurator->SetError(0u);
	fActor = 
		fConfigurator->Register();
	
	if ( fActor == 0x0 || fConfigurator->GetError() != 0u ) {
		SetLastError(fConfigurator->GetError());
		//
		throw std::bad_alloc();
	}
	
	if ( fActor->GetMaxInput() == 0 && fActor->GetMaxOutput() == 0 ) { // virtual or not well defined
		//
		fLog << error << "The registered actor " << GetName() << " cannot have any input and output " << dolog;
		//
		fActorType = kUnknown;
		SetLastError(fConfigurator->GetError());
		//
		return;
	}
	SetLastError(0u);
	if ( fActor->GetMaxInput() > 1 || fActor->GetMaxOutput() > 1 ) { // Shunter
		UInt_t wrong = 0u;
		// input and output are attached only for generation > 2 for which the proper narval interface is defined and the Shunter  
		if ( fGeneration->Get() > 0 ) {
			for (size_t i = 0; i < fInputs.size(); i++) {
				if ( fActor->AttachInput(fInputs[i]->GetBlock('c') ) < 0 ) {// cannot be attached
					wrong = 1;
					break;
				}
			}
			for (size_t i = 0; i < fOutputs.size(); i++) {
				if ( fActor->AttachOutput(fOutputs[i]->GetBlock('p') ) < 0 ) {// cannot be attached
					wrong = 2;
					break;
				}
			}	
		}
		else wrong = 1; // to low generation for that
		//
		if  ( wrong == 0 ) 
			fActorType = kShunter;
		else {
			SetLastError(1u);			
		}
	}
	else {
		if ( fActor->GetMaxInput() == 0 ) { // producer
			if ( fInputs.size() != 0 || fOutputs.size() != 1 ) {
				fLog << error << "The registered producer " << GetName() << " has not the right number of [input and/or output] [" << fInputs.size() << "," << fOutputs.size() << "]" << nline;
				SetLastError(1u);
			}
			else {
				if (  fGeneration->Get() > 0 ) {
					fActor->AttachOutput(fOutputs[0]->GetBlock('p'));
				}
				fActorType = kProducer;
			}
		}
		else {
			if ( fActor->GetMaxOutput() == 0 ) { //consumer
				if ( fInputs.size() != 1 || fOutputs.size() != 0 ) {
					fLog << error << "The registered consumer " << GetName() << " has not the right number of [input and/or output] [" << fInputs.size() << "," << fOutputs.size() << "]" << nline;
					SetLastError(1u);
				}
				else {
					if ( fGeneration->Get() > 0 ) {
						fActor->AttachInput(fInputs[0]->GetBlock('c'));
					}
					fActorType = kConsumer;
				}
			}
			else {
				if ( fInputs.size() != 1 || fOutputs.size() != 1 ) {
					fLog << error << "The registered filter " << GetName() << " has not the right number of [input and/or output] [" << fInputs.size() << "," << fOutputs.size() << "]" << nline;
					SetLastError(1u);
				}
				else {
					if ( fGeneration->Get() > 0 ) {
						fActor->AttachInput(fInputs[0]->GetBlock('c'));
						fActor->AttachOutput(fOutputs[0]->GetBlock('p'));
					}
					fActorType = kFilter;		
				}
			}
		}
	}		
	fLog << dolog;
}

void Gw::ALoadedActor::InitActor()
{
	UInt_t error_code = 0u; 
	fActor->process_initialise(&error_code);
	
	SetLastError(error_code);
}	

void Gw::ALoadedActor::ProcessActorGeneration_2_1_0 (Short_t generation)
{
	NarvalProducer *p; NarvalConsumer *c; NarvalFilter *f; FrameBlock *block_in, *block_out; UInt_t error_code = 0u, size_in, size_out, used_size_out; char  *input_buffer, *output_buffer;
	
	switch ( generation ) {
		case -2:
		//	std::cout << " Generation -2" << std::endl;
		case -1:
		//	std::cout << " Generation -1" << std::endl;
			switch (fActorType) {
				case kProducer:
					p = dynamic_cast<NarvalProducer *> (fActor); block_out = fOutputs[0]->GetBlock('p');
					
					output_buffer = block_out->GetBufferIO()->Address();
					size_out = block_out->GetMaxSize();
					//std::cout << " proceess_produc 1" << std::endl;
					p->process_block(output_buffer,size_out,&used_size_out,&error_code);
						
					//std::cout << " proceess_produc 2" << size_out << " " << used_size_out << std::endl;

					// the filled memory is again re attached to the block. It takes care of setting correctly size, eob etc ...
					// do not rewind since it is passed to the next actor
					block_out->SetBlock(output_buffer,used_size_out,'-');
					if ( used_size_out > 0u ) 
						SetStatus(ALoadedActor::kHasRun);
		
					break;
					
				case kConsumer:
					c = dynamic_cast<NarvalConsumer *> (fActor); block_in = fInputs[0]->GetBlock('c');
					
					input_buffer = block_in->GetBufferIO()->Address();
					size_in = block_in->GetSize();
					//std::cout << " proceess_consu 1" << std::endl;
					if ( size_in > 0u ) {
						c->process_block(input_buffer,size_in,&error_code);
						SetStatus(ALoadedActor::kHasRun);
					}
					else SetStatus(ALoadedActor::kIdle);
						
					//std::cout << " proceess_consu 2" << size_in << std::endl;
					break;
					
				case kFilter:
					f = dynamic_cast<NarvalFilter *> (fActor); block_in = fInputs[0]->GetBlock('c'); block_out = fOutputs[0]->GetBlock('p');
					
					input_buffer = block_out->GetBufferIO()->Address();
					size_in = block_in->GetSize();					
					output_buffer = block_out->GetBufferIO()->Address();
					size_out = block_out->GetMaxSize();
					
					if ( size_in > 0u ) {
						f->process_block(input_buffer,size_in,output_buffer,size_out,&used_size_out,&error_code);
						if ( used_size_out > 0u ) 
							SetStatus(ALoadedActor::kHasRun);
					}
					else SetStatus(ALoadedActor::kIdle);
					
					// the filled memory is again re attached to the block. It takes care of setting correctly size, eob etc ...
					// do not rewind since it is passed to the next actor
					block_out->SetBlock(output_buffer,used_size_out,'-');
					
					break;						
				default:
					throw std::logic_error("Unknown actor for emulator generation 0,1");
					break;
			}	
			break;
		case 0:
		//	std::cout << " Generation 0" << std::endl;
			
		{
			FrameBlock *block, *block_in, *block_out;
			switch (fActorType) {
				case kProducer:
					block = fOutputs[0]->GetBlock('p'); block->Rewind(ConfAgent::kWrite);
					if ( GetStatus() == kDoRunning ) {
						p = dynamic_cast<NarvalProducer *> (fActor); 
						SetStatus(kRunning);
						error_code = p->ProcessBlock((*block));
#ifdef L_DEBUG
						std::cout << "Actor " << GetName() << " has produced " <<  block->GetSize() << " bytes " << std::endl;
#endif
						if ( block->GetSize() ) { // producer are specific, could produce empty 
							SetStatus(ALoadedActor::kHasRun);
						}
						/*
						if ( error_code > 0u ){
							SetStatus(kFinished);
						}
						else {
							if ( block->GetSize() ) {
								SetStatus(ALoadedActor::kHasRun);
							}
							SetStatus(ALoadedActor::kIdle);
						}
						 */
					} 						
					break;
				case kConsumer:
					c = dynamic_cast<NarvalConsumer *> (fActor); block_in = fInputs[0]->GetBlock('c');	
#ifdef L_DEBUG
					std::cout << "cunsumer " << GetName() << " has " <<  block_in->GetSize() << " bytes to consume " << std::endl;
#endif
					if ( block_in->GetSize() ) {
						SetStatus(kRunning);
						error_code = c->ProcessBlock((*block_in));
						if ( block_in->IsEoB() )
							SetStatus(ALoadedActor::kHasRun);
						/*
						if ( error_code > 0u ) {
							SetStatus(kFinished);
						}
						else {
							SetStatus(ALoadedActor::kHasRun);
						}*/
					}
					break;
				case kFilter:
					f = dynamic_cast<NarvalFilter *> (fActor); block_in = fInputs[0]->GetBlock('c'); block_out = fOutputs[0]->GetBlock('p');
#ifdef L_DEBUG
					std::cout << "Filter " << GetName() << " has " <<  block_in->GetSize() << " bytes to consume " << std::endl;
#endif
					
					SetStatus(ALoadedActor::kRunning);
					error_code = f->ProcessBlock((*block_in),(*block_out));
					
#ifdef L_DEBUG
					std::cout << "Filter " << GetName() << " has produced " <<  block_out->GetSize() << " bytes " << std::endl;
#endif
					if ( block_out->GetSize() )
						SetStatus(ALoadedActor::kHasRun);
					else {
						;
					}

/*					
					if ( block_in->GetSize() ) {
						SetStatus(ALoadedActor::kRunning);
						error_code = f->ProcessBlock((*block_in),(*block_out));
						
#ifdef L_DEBUG
						std::cout << "Filter " << GetName() << " has produced " <<  block_out->GetSize() << " bytes " << std::endl;
#endif
						if ( block_out->GetSize()  )
							SetStatus(ALoadedActor::kHasRun);

						if ( error_code > 0u ) {
							SetStatus(kFinished);
						}
						else {
							if ( block_out->GetSize() )
								SetStatus(ALoadedActor::kHasRun);
						}	
 
					}
					else {
						block_out->Rewind(ConfAgent::kWrite); // force empty out
					}
			*/
					break;						
				default:
					throw std::logic_error("Unknown actor for emulator generation 0");
					break;
			}			
		}
			break;				
		default:
			break;
	}
	if ( error_code )
		SetStatus(ALoadedActor::kDead);
	
	SetLastError(error_code);
}

void Gw::ALoadedActor::ProcessActor () 
{ 
//	std::cout << " Call of (ProcessActor) " << fName << std::endl ;
	
	// to be used to check if the actors has done something 
	std::vector <UInt_t> inputs_off, outputs_off;
	
	// to deal with generation of Emulator
	Short_t generation = fGeneration->Get();
	
	try 
	{
		// to deal with generation for which the interface depends of the kind of actor
		if ( generation < 1 ) {
			ProcessActorGeneration_2_1_0(generation);
			return;
		}

		// loop on input for that node and check if they are free i.e. Owner is 0x0
		// if so and the buffer is full, Rewind it to be ready to be read / written
		
		UInt_t ok_condition = fInputs.size() + fOutputs.size(), ok_val = 0u, nb_lazy = 0u, identical; inputs_off.clear(); outputs_off.clear();
		for (UInt_t i = 0; i < fInputs.size(); i++) {
		 //
			inputs_off.push_back(fInputs[i]->GetBlock('c')->GetOffset());
			if ( fInputs[i]->IsLazy() ) {
				nb_lazy++;
			}
			if ( fInputs[i]->IsReadyforConsumer() ) {
				if ( !fInputs[i]->IsLazy() )
					ok_val++;
				fInputs[i]->SetReadyforConsumer();
			}
		}
		for (UInt_t i = 0; i < fOutputs.size(); i++) {
		 //
			outputs_off.push_back(fOutputs[i]->GetBlock('p')->GetOffset());

			if ( fOutputs[i]->IsLazy() ) {
				nb_lazy++;
			}
			if ( fOutputs[i]->IsReadyforProducer() ) {
				if ( !fOutputs[i]->IsLazy() ) 
					ok_val++;
				fOutputs[i]->SetReadyforProducer();
			}
		}		
		ok_condition -= nb_lazy;
		
		// now call processing of the embedded actor
		if ( ok_condition == ok_val ) {
			fActor->ProcessBlocks();
			
			// check if at least one of the input / output has changed 
			
			identical = 0u;
			for (UInt_t i = 0; i < fInputs.size(); i++) {
				if ( inputs_off[i] == fInputs[i]->GetBlock('c')->GetOffset() )
					identical++;
			}			
			for (UInt_t i = 0; i < fOutputs.size(); i++) {
				if ( outputs_off[i] == fOutputs[i]->GetBlock('p')->GetOffset() )
					identical++;
			}		
			if ( identical == (inputs_off.size()+outputs_off.size()) ) // additional check with poistion before / now of the offset in all the blocks
				SetStatus(ALoadedActor::kIdle); 			
			else
				SetStatus(ALoadedActor::kHasRun); 
			
		//	SetStatus(ALoadedActor::kHasRun); 
		}
		else {
			SetStatus(ALoadedActor::kIdle); // nothing has been done ==> kIdle
//			if ( ok_condition == 0 ) 
		}
		// to add : check all input and output ... if freed, means DLActor is Idle
		/*
		 for (UInt_t i = 0; i < fInputs.size(); i++) {
		 //
		 if ( fInputs[i]->IsReadyForConsumer() ) {
		 }
		 }
		 for (UInt_t i = 0; i < fOutputs.size(); i++) {
		 
		 // the block to be treated
		 FrameBlock *block = fOutputs[i]->GetBlock('p');
		 //
		 if ( fOutputs[i]->IsReadyForProducer() ) {
		 
		 }
		 }	
		 */
		
		// SetLastError(error_code);
	}
	catch (const std::exception& ex) 
	{
		std::cout << " ERROR " << ex.what() << std::endl ; 
	}
}	


