/***************************************************************************
 *   Copyright (C) 2004-2006 by Olivier Stezowski & Christian Finck        *
 *   stezow(AT)ipnl.in2p3.fr, cfinck(AT)ires.in2p3.fr                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/** \file AnActorConnection.h header file for AnActorConnection.cpp */

#ifndef Gw_AnActorConnection
#define Gw_AnActorConnection

#include <string>
//

#include "ADFConfig.h"
#include "FrameBlock.h"

namespace Gw 
{
	
class ALoadedActor;

//! Base class that connects two actors in a general topology
/*!
	\todo set lazy in the topology using globals. At the same time possibilities to set expandable blocks.
 */
class AnActorConnection 
{
private:
	//! > 0 means lazy
	Short_t fLazy;

public:
	AnActorConnection();
	virtual ~AnActorConnection();
	
	//! name of the connection ... better if unique in a emulator
	virtual std::string GetName() const 
	{
		std::string default_name = "AnActorConnection" ; 
		return default_name; 
	}
	//! name of the connection ... better if unique in a emulator
	virtual Bool_t IsLazy(Short_t lazy = 0u) const 
	{
		return fLazy > lazy; 
	}
	//! set this connection lazy i.e. not absolutely necessary for an actor
	/*!
		
	*/
	virtual void SetLazy(Short_t lazy = 1u) 
	{
		fLazy = lazy; 
	}	
	//! to get back the producer 
	virtual ALoadedActor *GetProducer() = 0;
	//! to get back the consumer 
	virtual ALoadedActor *GetConsumer()= 0;
	
	//! test if some data are ready to be sent to the consumer (right part of the connection) 
	virtual Bool_t IsReadyforConsumer() 
	{ return false; }
	//! Prepare the connection so that data can be consumed by the right actor: return false if someting wrong
	virtual Bool_t SetReadyforConsumer()
	{ return false; }
	
	//! test if some data can be collected be the producer (left part of the connection) 
	virtual Bool_t IsReadyforProducer() 
	{ return false; }
	
	//! Prepare the connection so that data can be produced by the left actor	: return false if someting wrong
	virtual Bool_t SetReadyforProducer()
	{ return false; }
	
	//! return the address of the block - opt = p => producer, c => consumer (different in case the connection is a pipeline) 
	virtual ADF::FrameBlock *GetBlock(const Char_t opt) = 0;
};
	
} // end namespace

#endif
