
#include "DEmulator.h"
#include <stdexcept>

using namespace Gw;

Bool_t DActorConnection::IsReadyforConsumer()
{
	// this connection is already owned by the consumer actor, so return true only if the block has not been fully consumed
	if ( fBlock->IsOwner(fConsumer->GetActor()) ) {
		if ( fBlock->IsEoB() ) 
			return false;
		else 
			return true;
	}
	if ( fBlock->IsOwner(0x0) && fBlock->GetSize() > 0 )
		return true;
	
	return false;
}
Bool_t DActorConnection::SetReadyforConsumer() 
{
	// connection already owned by the consumer
	if ( fBlock->IsOwner(fConsumer->GetActor()) ) {
		if ( fBlock->IsEoB() ) {
			fBlock->SetOwner(0x0);
			return false;
		}
		else 
			return true;
	}
	
	// new block coming from the producer part, so set consumer has owner and rewinf to be ready for reading
	fBlock->SetOwner(fConsumer->GetActor());
	// rewind for READING
	fBlock->Rewind(ConfAgent::kRead);
	
	return true;
}
Bool_t DActorConnection::IsReadyforProducer() 
{
	// this connection is already owned by the producer actor, so return true only if the block has not been fully filled
	if ( fBlock->IsOwner(fProducer->GetActor()) ) {
		if ( fBlock->IsEoB() ) 
			return false;
		else 
			return true;
	}
	
	if ( fBlock->IsOwner(0x0) )
		return true;
	
	return false;
}
Bool_t DActorConnection::SetReadyforProducer()
{
	// connection already owned by the producer
	if ( fBlock->IsOwner(fProducer->GetActor()) ) {
		if ( fBlock->IsEoB() ) {
			fBlock->SetOwner(0x0);
			return false;
		}
		else 
			return true;
	}
	
	// new block to be filled	
	fBlock->SetOwner(fProducer->GetActor());	
	// rewind for WRITING
	fBlock->Rewind(ConfAgent::kWrite);
	
	return true;
}

void DEmulator::LinkActors(ALoadedActor *left, ALoadedActor *middle, ALoadedActor *right)
{
	AnActorConnection *connection;
	
	// allocate a new connection and add it to the pool of connection
	if ( left ) {
		// look at the buffer size (name) is the globals
		const Char_t *opt = (GetItem<std::string>("TypeDataBlockC",fGlobals)->Get()).data();
		connection = 
			new DActorConnection(left,middle,GetItem<UInt_t>("SizeDataBlockC",fGlobals)->Get(),opt);
		fAllConnections.push_back(connection);		
		
		left->AddOutput(connection); middle->AddInput(connection);
	}
	if ( right ) {
		// look at the buffer size (name) is the globals
		const Char_t *opt = (GetItem<std::string>("TypeDataBlockP",fGlobals)->Get()).data();
		connection = 
			new DActorConnection(middle,right,GetItem<UInt_t>("SizeDataBlockP",fGlobals)->Get(),opt);
		fAllConnections.push_back(connection);		
		
		middle->AddOutput(connection); right->AddInput(connection);		
	}		
}








