/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file Data.h template that defines a general Data
*
*   In a root session, Data is available with the following data type (Data_t):
*  - Data<Char_t>
*  - Data<Short_t>
*  - Data<UShort_t>
*  - Data<Int_t>
*  - Data<UInt_t>
*  - Data<Float_t>
*  - Data<Double_t>
*/
 
#ifndef GW_DATA_H
#define GW_DATA_H

#ifndef ROOT_TString
#include <TString.h>
#endif

#include <iostream>
#include <sstream>

#ifndef GW_INFODATA_H
#include "InfoData.h"
#endif


namespace Gw {

/**
A single value data.

Any data that can be determined experimentally and has thus all possible flags 
defined in the InfoData base class.

@author Olivier Stezowski
*/
template<typename Data_T> class Data : public InfoData
{
protected:
    Data_T fValue;	// data value
    
public:
    Data() { fValue = 0 ; }
    Data(Data_T data) { SetValue(data); }
    Data(const Data & data) : InfoData(data) { fValue = data.fValue; }
    virtual ~Data() { ; }
    
    //! get the value, can be overloaded
    virtual Data_T Get() const { return fValue; }
    //! set the value, can be overloaded
    virtual void Set(Data_T data) { fValue = data; }   
    
    //! set the value, can be overloaded
    virtual void Set(const char *s) 
    {
    	std::istringstream input; input.clear(); TString st = s;
	switch( WhatIs(s) ){
		case kKnown:
			input.str(s); input >> fValue;  
			if ( input.fail() ) SetInfo(kUnknown);
			else SetInfo(kKnown);
			break;
		case kUnknown:
			SetInfo(kUnknown);
			break;	
		case kAbout:
			st.ReplaceAll('~',' '); 
			input.str(st.Data()); input >> fValue;  
			if ( input.fail() ) SetInfo(kUnknown);
			else SetInfo(kAbout);	
			break;				
		case kTentative:
			st.ReplaceAll('(',' '); 
			st.ReplaceAll(')',' '); 
			input.str(st.Data()); input >> fValue; 
			if ( input.fail() ) SetInfo(kUnknown);
			else SetInfo(kTentative);	
			break;
		case kTheo:
			st.ReplaceAll('[',' '); 
			st.ReplaceAll(']',' '); 
			input.str(st.Data()); input >> fValue; 
			if ( input.fail() ) SetInfo(kUnknown);
			else SetInfo(kTheo);	
			break;
		default:
			SetInfo(kUnknown);
			break;
	}
    }         
    
    //! get the value, cannot be overloaded 
    Data_T GetValue() const { return fValue; }
    //! set the value, cannot be overloaded
    void SetValue(Data_T data) { fValue = data; }  
    
    // virtual Data_T & operator () () { return fValue; }
    virtual Data_T & operator [] (int /*i*/) { return fValue; }
       
    //! rootcint dictionary
    ClassDef(Data,1); // A general data
};
// inline members
}

#endif
