/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   agata(AT)ipnl.in2p3.fr                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file Env.cpp compiled in libGWCORE.so */

#ifndef ROOT_TString
#include <TString.h>
#endif

#ifndef GW_CONFIG_H
#include "GwConfig.h"
#endif

#ifndef GW_ENV_H
#include "Env.h"
#endif

#include "GwLogMessage.h"

#include "TGClient.h"
#include "TGMimeTypes.h"

ClassImp(Gw::Env);

using namespace Gw;

const char *Env::GetVersion() 
	{ return GW_Version;}
 
const char *Env::GetPath(const char *whichpath)
{
	TString which = whichpath;
	
	if ( which.Contains("Version",TString::kIgnoreCase) ) {
		return GW_Version;
	}
	if ( which.Contains("LibraryPath",TString::kIgnoreCase) ) {
		return GW_LibraryPath;		
	}
	if ( which.Contains("BinaryPath",TString::kIgnoreCase) ) {
		return GW_BinaryPath;
	}
	if ( which.Contains("IncludePath",TString::kIgnoreCase) ) {
		return GW_IncludePath;
	}
	if ( which.Contains("IconsPath",TString::kIgnoreCase) ) {
		return GW_IconsPath;
	}
	if ( which.Contains("NdbPath",TString::kIgnoreCase) ) {
		return GW_NdbPath;
	}
	if ( which.Contains("MacrosPath",TString::kIgnoreCase) ) {
		return GW_MacrosPath;
	}
	
	return "NOT_DEFINED";	
}	

void Env::SetDebug(short debug)
{ 
	LogMessage::SetDebug(debug);
}

void  Env::AddRMimeType()
{
	static Bool_t has_done = false; TGMimeTypes* mineTypes = gClient->GetMimeTypeList();
	
	if ( has_done == false ) {
		char action[256]  = {0}; Int_t added = 0;
		if ( !mineTypes->GetAction("Gw::GCond1D",action) ) {
			mineTypes->AddType("[gw/gwgcond1d]"," Gw::GCond1D", "selection_t.xpm", "selection_t.xpm", "->Draw()");
			added++;
		}	
		if ( !mineTypes->GetAction("Gw::GCond2D",action) ) {
			mineTypes->AddType("[gw/gwgcond2d]"," Gw::GCond2D", "cut.xpm", "cut.xpm", "->Draw()");
			added++;
		}
		if ( !mineTypes->GetAction("*.spe",action) ) {
			mineTypes->AddType("[root/file]"," *.eg *.spe *.s", "hdb_s.xpm", "hdb_t.xpm", "Gw::HistoConverter::DrawSpectrum(\"%s\")");
			added++;
		}
		if ( added > 0 ) {
			has_done = true;
			mineTypes->SaveMimes();
		}	
	}
}



