/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   agata(AT)ipnl.in2p3.fr                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file Env.h to get information about the gammaware configuration
*
* Available are (const char *):
*	- Gw::Version
*	- Gw::LibraryPath
*	- Gw::BinaryPath
*	- Gw::IncludePath
*	- Gw::IconsPath
*	- Gw::NdbPath
*	- Gw::MacrosPath
*
*/
 
#ifndef GW_ENV_H
#define GW_ENV_H

#ifndef ROOT_Rtypes
#include <Rtypes.h>
#endif

#include <CentralLog.h>

//! Gammaware namespace 
/*!
	All gammaware classes are protected by the Gw namespace
*/
namespace Gw {

class Env 
{
public:
	Env() 
		{;}
	virtual ~Env()
		{;}	
		
	//! check mime type and add those proper to gw
	static void AddRMimeType();
	
	//! to get a particular GW environment variable
	/*!
	Currently, the following path are defined:
	
		- Version
		- LibraryPath
		- BinaryPath
		- IncludePath
		- IconsPath
		- NdbPath
		- MacrosPath
	*/	
	static const char *GetPath(const char *);	
	
	//! to get the current gammaware version
	static const char *GetVersion();
	
	//! Change the current log collector
	static Bool_t ChangeCollector(std::string whichlog = std::string("Null"))
	{ 
		return CentralLog::theCentralLog()->ChangeCollector(whichlog);
	}
	
	//! change the global level 
	static void SetDebug(short) ;
	
	//! Root dictionary related
	ClassDef(Env,0); // Gammaware environment	
};

} // namespace
#endif
