/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file GwLogCollector.cpp compiled in GwCore */

#ifndef Gw_LogMessage
#include "GwLogMessage.h"
#endif

#ifndef Gw_CentralLog
#include "GwCentralLog.h"
#endif

#ifndef Gw_LogCollector
#include "GwLogCollector.h"
#endif

#include <ctime>

using namespace Gw;

LogCollector::LogCollector(const char *name, bool isdelbycental) : 
	fName(name), 
	fMinLevel(LogMessage::kError),
	fMaxLevel(LogMessage::kInfo),	
	fIsDeleteByCentralLog(isdelbycental)
{
	
} 

LogCollector::~LogCollector() 
{

}  

LogCollector *LogCollector::theCollector()
{
	return CentralLog::theCentralLog()->GetCurrentLogCollector();
}    

#include <fstream>
#include <ios>
OStreamCollector::OStreamCollector(const char *name, bool isdelbycental) : 
	LogCollector(name,isdelbycental),  
	fOutputStream(0x0)
{
	std::string tmp = name;
	
	if ( tmp == "cout" ) { 
		fOutputStream = &std::cout;
	}
	if ( tmp == "cerr" ) { 
		fOutputStream = &std::cerr;
	}
	if ( tmp == "clog" ) { 
		fOutputStream = &std::clog;
	}	
	if ( fOutputStream == 0x0 ) 
		{ fOutputStream = new std::ofstream(GetName().data(),std::ios_base::app) ; }
}
OStreamCollector::~OStreamCollector() 
{
	bool donot = fOutputStream == &std::cout || fOutputStream == &std::cerr || fOutputStream == &std::clog ;
	if ( !donot ) 
		if ( fOutputStream ) delete fOutputStream;
}

void OStreamCollector::DoCollect(LogMessage &alogmsg)
{
	if ( alogmsg.GetLevel() < fMinLevel || alogmsg.GetLevel() > fMaxLevel )
		return;
	
	std::string message = alogmsg.Message();
	if ( message.size() < 1 )
		return;
			
	if ( fOutputStream ) {
		// header of the message
		(*fOutputStream) << alogmsg.Header();
		// the message	 
		(*fOutputStream) << message;
		// end of the message
		if ( message[message.size()-1] != '\n' ) 
			(*fOutputStream) << std::endl;
		(*fOutputStream) << alogmsg.Footer();
		
		// flush to be synchronized
		fOutputStream->flush();	
	} 
}




