/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file InfoData.h header file for InfoData */

#ifndef GW_INFODATA_H
#define GW_INFODATA_H

#ifndef ROOT_TObject
#include <TObject.h>
#endif

namespace Gw {

/**
Base class to get/set some flag informations concerning any data.
A data is something that can be measured or obtained experimentally and has, through this InfoData class,
some qualitative flags. For the moment: \n
 -# kKnown 
 -# kUnknown
 -# kTentative:  The data is based on some hypothesis
 -# kTheo:  The data is known only theoretically
 -# kAbout: The data is known roughly 
 
 ... but it could be extended.
 
@author Olivier Stezowski
*/
class InfoData : public TObject
{
protected:
    UShort_t fInfo;	// set a flag to a given data

public:
    /**
       flags that qualify a given data
    */
    enum EData { kKnown = 0, kUnknown = 1, kTentative = 2, kTheo = 3, kAbout = 4 } ;
    
public:
    InfoData();
    explicit InfoData(InfoData::EData info) { SetInfo(info); }
    InfoData(const InfoData &);
    virtual ~InfoData();
    
    //! It deduces from a string the kind of data
    /*!
    	
    */
    static EData WhatIs(const char *);
    
    //! to get some information about this data 
    virtual bool IsData(UShort_t) const;
    
    //! to set some information about this data   
    virtual void SetInfo(InfoData::EData);
    
    //! to set some information about this data   
    virtual void Set(const char *st) { fInfo = WhatIs(st) ; }   

    //! rootcint dictionary
    ClassDef(InfoData,1); // InfoData base class
};
// inline members
inline bool InfoData::IsData(UShort_t info) const { return fInfo == info ;}
inline void InfoData::SetInfo(InfoData::EData info) { fInfo = info; }

}

#endif
