/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file Random.cpp compiled in libGWPHYSICS.so */

#ifndef GW_RANDOM_H
#include "Random.h"
#endif

#ifndef ROOT_TList
#include <TList.h>
#endif

#ifndef ROOT_TRandomom2
#include <TRandom2.h>
#endif

#ifndef ROOT_TRandomom3
#include <TRandom3.h>
#endif

using namespace Gw;

ClassImp(Random);

Random  *Random::fgRandom = NULL;
    
Random::Random()
{
	TRandom *r; fROOT = NULL;
	
	// default list
	fList = new TList(); fList->SetOwner(kTRUE);
		
	r = new TRandom(); r->SetName("basic"); fList->Add(r);	
	r = new TRandom(0); r->SetName("basic_clock"); fList->Add(r);
	r = new TRandom2(); r->SetName("medium"); fList->Add(r);
	r = new TRandom(0); r->SetName("medium_clock"); fList->Add(r);
	r = new TRandom3(); r->SetName("large"); fList->Add(r);
	r = new TRandom3(0); r->SetName("large_clock"); fList->Add(r);

	// current generator, basic	
	SetCurrent();
}

Random::~Random()
{
	// delete the created objects and restore gRandomom
	delete fList; gRandom = fROOT;
}

Random *Random::Instance() 
{ 
	if ( fgRandom == NULL ) fgRandom = new Random();
	return fgRandom; 
}

void Random::SetCurrent(const char *name)
{
	TObject *r;
	
	if ( (r = fList->FindObject(name)) ) { // ok name find in the list
		fCurrent = dynamic_cast<TRandom *>(r); 
		
		// keep a copy of gRandomom and synchronise gRandomom with this
		if ( fROOT == NULL ) fROOT = gRandom; gRandom  = fCurrent;
	}	
}

void Random::Add(const char *name, TRandom *r)
{
	if ( fList->FindObject(name) == NULL && r != NULL ) { r->SetName(name); fList->Add(r); }
}




