/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/** @file SingletonDestroyer.h  
 
 */

#ifndef Gw_SingletonDestroyer
#define Gw_SingletonDestroyer

//! To properly destroy a singleton
/*!
	The way it works is the following : 
	Add to your class a 
 
	static SingletonDestroyer<MyType> gTheSingletonDestroyer; 
 
	(not a pointer but a real object !)
	Once the pointer of the singleton is created, pass it to the destroyer using SetSingleton
 
	Once the program is finished, the destructo of the destroyer is called. This one delete the singleton 
 
*/
template<typename Data_T> class SingletonDestroyer 
{
public:
	SingletonDestroyer(Data_T *t = 0x0) : fTheSingletonToDestroy(t) 
		{;}
	virtual ~SingletonDestroyer()
	{
		if ( fTheSingletonToDestroy )
			delete fTheSingletonToDestroy;
		
		fTheSingletonToDestroy = 0x0;
	}
	void SetSingleton(Data_T *t)
	{
		fTheSingletonToDestroy = t;
	}
	
private:
	Data_T *fTheSingletonToDestroy;
};

#endif

