#ifndef BASECALIB_H
#define BASECALIB_H

//#include<stdio>
#include<fstream>
#include<iostream>
#include<iomanip>
using namespace std;


#include<TH1.h>
#include<TF1.h>
#include<TGraphErrors.h>
#include<TROOT.h>
#include<TObjArray.h>


/**

BaseCalib is an abstract class. It provides basic method to perform a calibration of Germanium
spectra.  

@author Bertrand Rossé
*/

class BaseCalib {

public:
static const Int_t	NBCANAUX;  
static const Float_t	SCALEENERGY;	//Scale 6 MeV
static const Float_t	KEVCANAL;	//We want a calibrated histogram with KEVCANAL keV/canal
static const Float_t	RANGE;		//Range to fit a peak --> Fit between Max-RANGE and Max+RANGE in keV
static const Int_t	MINCOUNTPEAK;	//Minimal counts in the maximum of the peak to do the calibration 

public:
	BaseCalib();
        virtual ~BaseCalib();


	void OpenLogFile();  //Open the Log file
	void CloseLogFile(); //Close the Log File
	void OpenErrFile();  //Open the Err file
	void CloseErrFile(); //Close the Err File
	void OpenOutFile();  //Open the Out file
	void CloseOutFile(); //Close the Out File
	//Bool_t CheckFindPeak(TH1 *);

	void Setup();
	Bool_t GetOut() const;  //return true if a file .out is opened
	Bool_t GetLog() const;  //return true if a file .log is opened
	Bool_t GetErr() const;  //return true if a file .err is opened

	Bool_t GetCheck() const; //return true if you want just a check of calibration
	void SetCheck(const Bool_t);

	TF1 *GetGaussian(); //return the current gaussian function
	TF1 *GetSkewedGaussian(); //return the current SkewedGaussian function

	void Fit1Peak(TH1D*, const Char_t *option = "Gauss"); //To fit a peak in a TH1 histogram
	void DoBg(TH1D*); //Do the Background substraction in the TH1 histogram

	//virtual const Char_t* DoCalib(TH1D*, TF1*, const Char_t*) = 0;

	void FitGraph(TGraphErrors *, TF1 *); //To fit a graph with a formula
	
	void Resolution(TObjArray *, Float_t threshold = 0.5); //resolution test for a TObjArray of peaks
	
	void AlignMax(TObjArray *, TObjArray*, Int_t, Float_t xmin=0, Float_t xmax=0); //Perform alignment of the maximum of a TH1D on a value

	void Display(TObjArray *, Int_t, Int_t, Float_t xmin=0, Float_t xmax=0);
	
	void DisplayAlign(TObjArray *, Float_t, Float_t xmin=0, Float_t xmax=0);
	
	void CheckShift(TObjArray *, TObjArray *, Float_t xmin=0, Float_t xmax=0);
protected:
	Bool_t fOut, fLog, fErr;   //Status of output files, if kTRUE --> file is opened, else file is closed
	ofstream fLogFile, fErrFile, fOutFile;   //flux for the output files

	Bool_t fCheck; // if fCheck = kTRUE -->Just perform the calibration without change the formula

	TObjArray *fCollHistoCalib; //Collection for calibrated histograms

	TF1 *fGaussian;  //Gaussian function
	TF1 *fSkewedGaussian;  //SkewedGaussian function (See Radford website for more details)
	
	Int_t fNbSpectra; //Number of spectra to calibrate
	Int_t fNbGood;    //Number of good calibrations
	Int_t fNbWrong;   //Number of failed calibrations


public :
	TString FileNextName(const Char_t *, const Char_t *);
};

#endif
