#ifndef GW_BUFFERGANIL_H
#define GW_BUFFERGANIL_H

#define DATELENGTH 30

#include <TString.h>
#include <Buffer.h>

#define MAXCLOVER  16
#define MAXDIAMANT 56
#define NBCRIS      4
#define NBSEDX    128
#define NBSEDY     48

#define MAXGROUPTABLE 256

class BufferGANIL 
{
public:
	static const short gMGroupTable = MAXGROUPTABLE;
	
	static short *gGroupTableECC;	//! group detector table for ECC cards
	static short *gGroupTableESS;	//! group detector table for ESS cards		
	static short *gGroupTableDIA;	//! group detector table for Diamant cards
	static short *gGroupTableGOCCE;	//! group detector table for COCCE cards	
	static short *gGroupTableVAMOS;	//! group detector table for VAMOS cards

	static int    gNbECC;		//! Number of clovers GE in the setup
	static int    gNbESS;		//! Number of SHIELDED clovers in the setup	
	static int    gNbDIA;		//! Number of DIAMANT CELLS in the setup	
	static int    gNbGOCCE;		//! Number of SEGEMENTED clovers in the setup	
	static int    gNbVAMOS;		//! Number of VAMOS in the setup	
				
	static short gItemDIAE;		//! item for Diamant energy
	static short gItemDIAT;		//! item for Diamant TAC
	static short gItemDIAID;	//! item for Diamant ID
		
	static short gItemHR[MAXGROUPTABLE];	//! item for energy GE HR
	static short gItemLR[MAXGROUPTABLE];	//! item for energy GE LR
	static short gItemT[MAXGROUPTABLE];	//! item for GE TAC
		
	static short gItemSE;		//!
	static short gItemSTAC;		//!
	static short gItemSP;		//!	
	static short gItemST[MAXGROUPTABLE];	//! item for ID
	
	static short gItemSEDX0[MAXGROUPTABLE];//!
	static short gItemSEDY0[MAXGROUPTABLE];//!	
	static short gItemSEDX1[MAXGROUPTABLE];//!
	static short gItemSEDY1[MAXGROUPTABLE];//!
	static short gItemICHA[MAXGROUPTABLE];//!	
	static short gItemICHB[MAXGROUPTABLE];//!		
	static short gItemQSED0[MAXGROUPTABLE];//!		
	static short gItemQSED1[MAXGROUPTABLE];//!			
	static short gItemVAMOST[MAXGROUPTABLE];//!		
	static short gItemQPLG;//!			
	static short gItemQPLD;//!			
										
	static short gMaskDIAE;		//! mask for Diamant energy
	static short gMaskDIAT;		//! mask for Diamant TAC
	static short gMaskDIAID;	//! mask for Diamant ID
		
	static short gMaskHR[MAXGROUPTABLE];	//! mask for energy GE HR
	static short gMaskLR[MAXGROUPTABLE];	//! mask for energy GE LR
	static short gMaskT[MAXGROUPTABLE];	//! mask for GE TAC
		
	static short gMaskSE;		//!
	static short gMaskSTAC;		//!
	static short gMaskSP;		//!	
	static short gMaskST[MAXGROUPTABLE];	//! mask for ID
	
	static short gMask[16];		//! all possible mask gMask[5] means first 5 bits to 1
	
	static unsigned short fStartSubEventTokenEXO;	//!  	
	static unsigned short fStartSubEventTokenVAM;	//!  	
	
	static const unsigned short fStartEventToken    = 0xFF60;
	static const unsigned short fEndDataBlockToken  = 0xFF00;

private:
	int  fRun; 		// run number associated with this buffer
	int  fBlock; 		// block number associated with this buffer
	char fDate[DATELENGTH];	// date of the creation of the buffer
	
	short  fNbEvents;	// number of events written in this block
	int  fEventsSize;	// size (in 2 byes unit) of events written in this block
	short  fNbEventsR;	// number of events read from this block
	int  fEventsSizeR;	// size (in 2 byes unit) of events read from this block
				
	int fEventSize;		// size of the current event
	unsigned short fEvent[16384];	// buffer to hold the current event
	
	bool fIsDAQ;
	
   Gws::Buffer *fBuffer;	// buffer to be scanned
	
public:
   BufferGANIL(Gws::Memory::EEndian e = Gws::Memory::kLittle, size_t s = 16*Gws::KBYTE);
	~BufferGANIL();
	
	void SetDAQ(bool is) { fIsDAQ = is; }
	
	bool IsType(const char *);
//	bool IsEndian(Gws::Memory::EEndian);

	static void InitTable();	
	static void InitTable(TString &);
	static void InitTable(Gws::Buffer *);
	static void InitTable(const char *filename) ;
	static void SaveTable(const char *filename = "bufferGANIL.vxitoposition");
		
	int GetRun() const;
	void SetRun(int run) { fRun = run;}
	const char *GetDate() const;
	
	Gws::Buffer * GetBuffer();
	
	bool InitReading();	
	unsigned short * NextEvent(unsigned short &);
};

// BufferGANIL inline members /////////////////////////////////////////////////
inline Gws::Buffer * BufferGANIL::GetBuffer() { return fBuffer; }
inline int BufferGANIL::GetRun() const { return fRun; }
inline const char *BufferGANIL::GetDate() const { return fDate; }
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


#endif
