///////////////////////////////////////////////////////////////////////////////
//									                                 //
//  DAQGANIL      						         	                             //
//									                                 //
//  Interface to a system stream.		     	                             //
//  Implementation is the real interface. There is a specific constructor    //
//  to work with already open streams. In case of unix like system 		     //
//  DAQGANIL() (I know nothing about any other system !). Otherwise, opening    //
//  is done by a name (see Open).									         //
//  As all classes interfacing the system, this class is protected by a      //
//  namespace gws	     			                                 //
//									                                 //
//  zouzou - April 2002 						                             //
//									                                 //
///////////////////////////////////////////////////////////////////////////////

////// INCLUDES /////////////////////////////////////////////////////////////////

// GANIL includes
#include <gen_type.h>
#include <acq_tcp_erreur.h>
#include <acq_tcp_general.h>
#include <acq_tcp_parametres_def.h>
#include <acq_tcp_status_struc.h>
#include <acq_tcp_struct.h>
#include <gan_acq_buf.h>
#include <acq_tcp_fct_utilisateur.h>
#include <acq_codes_actions.h>
#define PR_FLAG(texte, booleen) printf( " <" #texte " %s>",(booleen) ? "on" : "off")
	
namespace {
in2p3_buffer_struct fBuffin2p3;	// to get the next buffer
}

#include "Base.h"
#include "DAQGANIL.h"
#include "TROOT.h"
#include "TFile.h"
#include "TStopwatch.h"

TStopwatch dtime;

using namespace gws; 

#include <iostream>
#include <fstream>
using std::ofstream;

//// IMPLEMENTATION ///////////////////////////////////////////////////////////

DAQGANIL::DAQGANIL(BufferGANIL *buf) 
{	
// Default constructor
	
	fRate = 0; fDAQBufNumber = fDAQBufNumber0 = 0;
	fBufGANIL = buf;
}

DAQGANIL::~DAQGANIL() 
{
	if ( IsOpen() ) { Close(); SetID(-1); }
}

void DAQGANIL::Open(const char* name, int openmode, const mode_gw permissions) throw(Exception)
{ 
// check out if the correct environment variables exist to spy on the GANIL DAQ
  
    char *s; 
    s = ::getenv("acq_tcp_serv_name");
    if ( s == NULL ) throw Exception("Cannot access DAQ, check out acq_tcp_serv_name","nosys");
    s = ::getenv("acq_tcp_serv_port");    
    if ( s ==  NULL ) throw Exception("Cannot access DAQ, check out acq_tcp_serv_port","nosys");    

    SetID(1);
}

void DAQGANIL::Close() throw(Exception) 
{ 
     char str[256]; int status;
     if ( acq_tcp_close_connection() == -1 ) {
	    acq_tcp_get_messerr ( &status, str );
	    cout << " Cannot close properly the connection to the DAQ " << str << "\n";
     }
}


int DAQGANIL::Read(Buffer *b) throw(Exception) 
{    
// OK
    acq_tcp_status_struc buffer; // to get status of the acquisition

    int bread = 0;
    int stat, status, nbbuffer; char str[256];
    
    status = acq_tcp_get_acquis_status ( &buffer ); 
    if ( status == 0 && ( buffer.utile_stat.on == 0  || buffer.utile_stat.store == 0) ) return 0;
    
    fBufGANIL->SetRun(buffer.utile_stat.norun);
    status = acq_tcp_get_ctrl_buffer (&nbbuffer, &fBuffin2p3, 16384, &bread);
    
    // test status
    switch ( status ) {
	case 0:
		fBytesRead += 16384; fBuffersRead++;
		
		if ( fDAQBufNumber0 == 0 )  fDAQBufNumber0 = nbbuffer;
		else fDAQBufNumber = nbbuffer;
		
		Memory::Copy((char *)fBuffin2p3.les_donnees.cas.Buff,b->GetBuffer(),16384);	
	break;
	case 2:
              // cout << "Buffer already read" << fDAQBufNumber << " " << status << endl;
	      bread = 0;
        break;
 	default:
	      acq_tcp_get_messerr ( &stat, str ); 
	      // cout <<   str   << endl;
	      bread = 0;        
    	break;
    }
/*    
    if ( status == 0 ) {
		fBytesRead += 16384; fBuffersRead++;
		
		if ( fDAQBufNumber0 == 0 )  fDAQBufNumber0 = nbbuffer;
		else fDAQBufNumber = nbbuffer;
		
		Memory::Copy((char *)fBuffin2p3.les_donnees.cas.Buff,b->GetBuffer(),16384);
		//Memory::Copy((char *)&fBuffin2p3,b->GetBuffer(),16384);
	
		ofstream sortie("onedacblock",ios::out);		
		if ( sortie ) {
			printf("dump a block"); sortie.write(b->GetBuffer(),16384);
		}
		sortie.close(); bread=0; 
    }
    else {
 	    	acq_tcp_get_messerr ( &stat, str ); throw Exception(str,"nodata");     
		// bread = 0;
    }
*/
    
    return bread; 
}

void DAQGANIL::Reset(const char *option)
{
	fDAQBufNumber0 = fDAQBufNumber = 0; StreamIO::Reset(option);
}


int DAQGANIL::Write(Buffer *b) throw(Exception)  
{ 
// OK
    throw Exception("Your are not allowed to write on the GANIL DAQ","nosys");
    return 0;
}

void DAQGANIL::Status(ostream &out)
{
	int stat; char str[256]; int rate = 0;
 	acq_tcp_status_struc buffer; // to get status of the acquisition

   	int status = acq_tcp_get_acquis_status ( &buffer );

   	if ( status == 0 )
   	{
      	  out << "\n Experiment : " << buffer.utile_stat.experiment_name << "\n"; 

	  out << "\n - Info - Acquistion flags :\n    ";

	  PR_FLAG( acquisition, buffer.utile_stat.on);
	  //  PR_FLAG( control, buffer.utile_stat.control);
	  PR_FLAG( storage, buffer.utile_stat.store);
	  // PR_FLAG( full control, buffer.utile_stat.fullcontrol);
	  printf("\n    ");
	  //  PR_FLAG( user control, buffer.utile_stat.usercontrol);
	  // PR_FLAG( replay, buffer.utile_stat.replay);
	  // PR_FLAG( simulation, buffer.utile_stat.simul);
	  // PR_FLAG( debug, buffer.utile_stat.debug);

	  printf ("\n - Info - Status of all acquisition processus :\n    ");
	  PR_FLAG( acq , buffer.utile_stat.prc_acquis_status_ok);
	  PR_FLAG( read, buffer.utile_stat.prc_read_status_ok);
	  // PR_FLAG( ctrl, buffer.utile_stat.prc_ctrl_status_ok);
	  PR_FLAG( store, buffer.utile_stat.prc_store_status_ok); 

	  out << "\n";
// 	  out << "\n current run number is   : " << buffer.utile_stat.norun;
	  out << "\n current event number is : " << buffer.utile_stat.nbctevent;
	  if ( (fDAQBufNumber -fDAQBufNumber0) > 0 ) 
	       out << "\n current block number is: "<< fDAQBufNumber << " <"
	           << 100. * Float_t(fBuffersRead) / ( fDAQBufNumber -fDAQBufNumber0) << "> " << endl;
	  if (buffer.utile_stat.on) {
	    out << "\n\n\t -----> Data rate is " << buffer.utile_stat.acq_data_rate << " Kbytes/sec " << endl;
	    rate = buffer.utile_stat.acq_data_rate;
	  }
	  else {
	    out << " ----->  Data rate is 0 "; 
	  }
	}
	else {
	  if ( status == -100 ) {
		out << "\n - ! - : TCP/IP server unavailable " ; 
	  }
	  else {    
 	    	acq_tcp_get_messerr ( &stat, str );
		out << "\n Error : get_acquis_status : " << str;
 	  }
	  
	}
}


