#ifndef GW_RAWEVENT_H
#define GW_RAWEVENT_H

#include <TH1.h>
#include <TClonesArray.h>
#include <TClass.h>

//#define MAXCR 4

//! base class for an RAWevent
/*!
	detail
*/
class RawEvent : public TObject
{
	
public:
	RawEvent() ;
	virtual ~RawEvent();

	virtual Bool_t Fill(UShort_t *, UShort_t) { return kFALSE; }	

	ClassDef(RawEvent,1) // Base class for a raw event
};

class CellCsI : public RawEvent
{
public:
	Short_t   Which;	// Cell identification 
	UShort_t  Energy;	// Energy
	UShort_t  Time;		// Tie
	UShort_t  Id;           // I
		
public:
	CellCsI(); 
	virtual ~CellCsI();
	
	virtual void Clear(Option_t *);	
	virtual Bool_t Fill(UShort_t *, UShort_t) { return kFALSE; }	
	
 	virtual void ls(Option_t* option) const;	
		
	ClassDef(CellCsI,1) // a single CsI element		
};
inline void CellCsI::Clear(Option_t *opt) { Which = -1;  Energy = Time = Id = 0 ; }
inline void CellCsI::ls(Option_t* option) const { printf("%d %d %d %d \n",Which,Energy,Time,Id) ; }	
	
class ExoClover : public RawEvent
{
public:
	Short_t  Which;		// Clover identification 
	
	UShort_t Time[4];	// time for each cristal 
	UShort_t EnergyHR[4]; 	// energy High Resolution
	UShort_t EnergyLR[4]; 	// energy Low Resolution

	Bool_t PileUp[4]; 		// true if pileup
	Bool_t Compton[4]; 	// true if compton

	UShort_t ShieldE;		// energy of the shield
	UShort_t ShieldT;		// time of the shield
	UShort_t ShieldP;		// pattern of the shield

	UShort_t TimeS[4]; 	//! time for individual shield elements

	UShort_t GocceE[4][4];// energy for segments
	UShort_t GocceT[4][4];// TAC for segments
		
public:
	ExoClover();
	ExoClover(const ExoClover &); 
	virtual ~ExoClover();
	
	virtual void Clear(Option_t *);	
	virtual Bool_t Fill(UShort_t *, UShort_t) { return kFALSE; }	
		
	ClassDef(ExoClover,1) // a ExogamR clover detector		
};	
inline void ExoClover::Clear(Option_t *opt) { 
	Which = -1; ShieldE = ShieldT = ShieldP = 0; 

	Time[0] = Time[1] = Time[2] = Time[3] = 0;
	EnergyHR[0] = EnergyHR[1] = EnergyHR[2] = EnergyHR[3] = 0;
	EnergyLR[0] = EnergyLR[1] = EnergyLR[2] = EnergyLR[3] = 0;

	PileUp[0] = PileUp[1] = PileUp[2] = PileUp[3] = kFALSE;
	Compton[0] = Compton[1] = Compton[2] = Compton[3] = kFALSE;

	TimeS[0] = TimeS[1] = TimeS[2] = TimeS[3] = 0;
	
	GocceE[0][0] = GocceE[0][1] = GocceE[0][2] = GocceE[0][3] = 0;
	GocceE[1][0] = GocceE[1][1] = GocceE[1][2] = GocceE[1][3] = 0;
	GocceE[2][0] = GocceE[2][1] = GocceE[2][2] = GocceE[2][3] = 0;
	GocceE[3][0] = GocceE[3][1] = GocceE[3][2] = GocceE[3][3] = 0;
	
	GocceT[0][0] = GocceT[0][1] = GocceT[0][2] = GocceT[0][3] = 0;
	GocceT[1][0] = GocceT[1][1] = GocceT[1][2] = GocceT[1][3] = 0;
	GocceT[2][0] = GocceT[2][1] = GocceT[2][2] = GocceT[2][3] = 0;
	GocceT[3][0] = GocceT[3][1] = GocceT[3][2] = GocceT[3][3] = 0;
}

class DiamantR : public RawEvent
{
protected:
	Int_t NbCell;		// number of cells fired in this event 
	TClonesArray *Cells;	//-> an DiamantR event consists in several CellCsI
	
public:
	DiamantR();
	virtual ~DiamantR() ;

	Int_t GetNbCell() { return NbCell; }
	TClonesArray *GetCells() { return Cells; }
	
	virtual void Clear(Option_t *);	
	virtual Bool_t Fill(UShort_t *, UShort_t) ;	
	
	ClassDef(DiamantR,1); // Raw Diamant is an array of CellCsI
};


class ExogamR : public RawEvent
{
protected:
	Int_t NbClover;	// number of clover fired in this event 
	TClonesArray *Clovers;	//-> an ExogamR event consists in several clovers
		
public:
	ExogamR();
	virtual ~ExogamR() ;
	
	Int_t GetNbClover() { return NbClover; }
	TClonesArray *GetClovers() { return Clovers; }

	virtual void Clear(Option_t *);	
	virtual Bool_t Fill(UShort_t *, UShort_t) ;	
	
	ClassDef(ExogamR,1); // ExogamR is an array of Clovers
};

class VariableArrayF : public TObject
{
public:
	UShort_t Mult;	    //
	Float_t  Data[100]; //[Mult] 
public:
	VariableArrayF();
	virtual ~VariableArrayF();
	
	virtual void Clear(Option_t *);
		
	ClassDef(VariableArrayF,1);
};

class GammaTheta : public TObject
{
public:
	UShort_t Mult;		// Multiplicty
	Float_t Energy[100];	//[Mult] Gamma-ray energies
	Float_t Theta[100];	//[Mult] Corresponding angles 
	
public:
	GammaTheta();
	virtual ~GammaTheta();
	
	virtual void Clear(Option_t *);
	
	ClassDef(GammaTheta,1);	// Gamma-ray energies and emission angles	
}; 

class BetaF : public TObject
{
public:
	Float_t Beta;	// 
	
public:
	BetaF();
	virtual ~BetaF();
	
	virtual void Clear(Option_t *);
	
	ClassDef(BetaF,1); // Doppler correction
};

class GammaTAC : public TObject
{
public:
  UShort_t Mult;  //Multiplicity
  Float_t Energy[100]; //[Mult] Energies of gamma rays
  UShort_t TAC[100];	//[Mult] Times of gamma rays
  Float_t Theta[100];   //[Mult] Angles theta of gamma rays 
  Float_t Phi[100];     //[Mult] Angles phi of gamma rays 

public:
	GammaTAC();
	virtual ~GammaTAC();

	virtual void Clear(Option_t *);

	ClassDef(GammaTAC,1);
}; 

class LCP : public TObject
{
public:
  UShort_t Proton;  // Number of protons in the event
  UShort_t Alpha;   // Number of alphas in the event
  UShort_t Radioactivity1;  //Number of counts in the gamma blop (high PID) 
  UShort_t Radioactivity2;  //Number of counts in the gamma blop (low PID) 
  UShort_t Others;  // Number of others in the event
  

public:
	LCP();
	virtual ~LCP();

	virtual void Clear(Option_t *);

	ClassDef(LCP,1);
};


class Pad : public RawEvent
{
public:
	Short_t  Which;	// pad number 
	UShort_t Q; 	// charge
public:
	Pad() { Pad::Class()->IgnoreTObjectStreamer(); Clear(); }
	virtual ~Pad() { ; }

	virtual void Clear(Option_t *opt = "");	
	virtual Bool_t Fill(UShort_t *, UShort_t) { return kFALSE; }	
			
	ClassDef(Pad,1); // a pad is a charge somewhere
};
inline void Pad::Clear(Option_t *opt) { Q = 0; Which = -1; }	
	
#define NBSED   2
#define NBICH   2
#define MAXICH  7
#define MAXSEDX 128
#define MAXSEDY 48

class VamosR : public RawEvent
{
public:
	UShort_t NbICH[NBICH];	// Number of pad fired in ionisation chamber 0 and 1
	UShort_t NbSEDX[NBSED];	// Number of x pad fired in sed 0 and 1
	UShort_t NbSEDY[NBSED];	// Number of y pad fired in sed 0 and 1
	
	TClonesArray *ICH0;	//-> Array of Pads in chamber 1	
	TClonesArray *ICH1;	//-> Array of Pads in chamber 2	
	TClonesArray *SED0X;	//-> Array of Pads in SED 1X			
	TClonesArray *SED1X;	//-> Array of Pads in SED 2X	
	TClonesArray *SED0Y;	//-> Array of Pads in SED 1Y			
	TClonesArray *SED1Y;	//-> Array of Pads in SED 2Y
	
	UShort_t QSED0[3];	//
	UShort_t QSED1[3];	//
					
	UShort_t TSED0_HF;	//
	UShort_t TSED1_HF;	//
	UShort_t TSED0_SED1;	//
	
	UShort_t TPLG_SED0;	//
	UShort_t TPLG_SED1;	//
	UShort_t TPLG_PLD;	//
		
	UShort_t TPLG_HF;	//

	UShort_t TSED2_GAM;	//
		
	UShort_t QPL_G;		//
	UShort_t QPL_D;		//
public:
	VamosR();
	virtual ~VamosR() ;

	virtual void Clear(Option_t *);	
	virtual Bool_t Fill(UShort_t *, UShort_t) ;	
	
	ClassDef(VamosR,2); // Vamos
};

//! base class for an Clean Vamos
/*!
	detail
*/
class VamosC : public TObject
{
public:
	Float_t  X1;
	Float_t  Y1;
	Float_t  X2;
	Float_t  Y2;
	Float_t  Xfoc;
	Float_t  Yfoc;
	Float_t  Thetafoc;
	Float_t  Phifoc;
	Float_t  BRho;
	Float_t  Thetaphys;
	Float_t  Phiphys;
	Float_t  Tof;
	Float_t  vof;
	Float_t  deltaE[7];
	Float_t  Eres[7];
public:
	VamosC() ;
	virtual ~VamosC();

	virtual void Clear(Option_t *);
	
	ClassDef(VamosC,1) // Base class Vamos
};

#endif
