
#include <iostream>
#include <fstream>
using namespace std;

#include "TSystem.h" 
#include "TDatime.h"
#include "Event.h" 
#include "TTree.h" 
#include "TFile.h" 
#include "TTimer.h" 

#include "StreamIO.h"
#include "BufferGANIL.h"
#include "DAQGANIL.h"

#include "RawToTree.h"

using namespace gws;   
using namespace std;   

ClassImp(MetaTree);

MetaTree::MetaTree() : TNamed()
{
	Clear();
}

void MetaTree::Clear(Option_t *option)
{
	TString opt = option;
	if ( opt.Contains("stat") ) {
		Run = -1;
		EventsOut = 0; EventsIn = 0;
		ExoEvents = VamosEvents = DiamantEvents = 0;
	}
	else { 
		Run = -1;
		EventsOut = 0; EventsIn = 0;
		ExoEvents = VamosEvents = DiamantEvents = 0;
		
		TreeName.SetString("EmptyTree"); TreeTitle.SetString(""); 	
		TreeFile.SetString("RawToTree.root");
	}
}	

class RawToTree::Implementation {
public:
   enum EStatus { kBad = 0, kInit = 1, kUpdate = 2, kRunning = 3, kSuspended = 4, kTimerOn = 5 } ;
 
private:     
   //! input stream
   gws::StreamIO *fInput; 
   //! input buffer
   BufferGANIL    fBuffer;  
   
   //! status of this application
   Int_t fStatus;	

   //! list of 'sub-events'
   TObjArray fEvents;	
   //! tree to store events  
   TTree *fTree;
   //!  tree save any fUpdate events
   Int_t   fUpdate;	
   
   // Int_t   fMetaTree.Run; 
   // TString fCurrentTFileName;
   //! # of event read since the last update   
   // Int_t fMetaTree.EventsIn[0];	
   //! # of event read since the last update
   // Int_t fMetaTree.EventsOut;	
   // Base name for trees
   // TString fNameTree;   
   // Base title for trees	
   // TString fTitleTree;   	
   
   //! to store meta data about the created trees.
   MetaTree fMetaTree;
   
   //! log file
   ofstream fLog;	   
   ofstream fCatalogue;	

   Bool_t NextOut(const Char_t *);
   void Status(ostream &);
   
   TTimer fTimer;
   
public:
   Implementation();
   ~Implementation();
   
    void AttributesTree(const Char_t *nametree = "RawData", const Char_t *titletree = "Exo_Diam_Vamos",
    			Int_t upd = 10000000, Long64_t maxtreesize = 1900000000);
    void Init(Option_t *opt = "");
    void Run(Option_t *opt = "");
    void Update(Option_t *opt = "");

    void Help(Option_t *opt = "");
    void Status(Option_t *opt = "");

    void Suspend(Option_t *opt = "");
    void End(Option_t *opt = "");
    
    void StopTimer() { fTimer.TurnOff(); }
};

RawToTree::Implementation::Implementation() 
{
	// application status
	fStatus = RawToTree::Implementation::kBad; 
	
	// log and catalogue files
	fLog.open("RawToTree.log",ios::app); 
	fCatalogue.open("RawToTree.catalogue",ios::out|ios::app);
	
	// input file	
	fInput = NULL;
	
	// current statistics = Meta Data for tree file
	fMetaTree.Clear(); fMetaTree.SetNameTitle("E404as","October2004"); 
	
	// tree related
	fTree = NULL; TTree::SetBranchStyle(1); // use the new Bronch style
	fEvents.SetOwner(kTRUE);	
	AttributesTree(); 
	
	Help(""); fTimer.SetCommand("scan->Run();");
}

RawToTree::Implementation::~Implementation()
{	
	fEvents.Clear();
	if ( fInput ) {delete fInput; fInput = NULL; } 			
	if ( fCatalogue ) fCatalogue.close();  if ( fLog ) fLog.close(); 
}

void RawToTree::Implementation::AttributesTree(const Char_t *nametree, const Char_t *titletree, Int_t upd, Long64_t maxtreesize )
{
	if ( fStatus != RawToTree::Implementation::kBad ) {
		printf(" Attributes of the output stream can be changed only when RawToTree is not initialized \n");
	}
	else {
	
		TDatime *d = new TDatime; 
		fLog << endl;
		fLog << " *** " << d->AsSQLString() << " New Attributes for TTree " << endl;
		fLog << "  Tree name: "  << nametree << endl;
		fLog << "  Tree title: " << titletree << endl;
		fLog << "  Update any " << upd << " events " << endl;
		fLog << "  New ROOT max file size:  " << maxtreesize << endl;
		delete d;
			
		// fNameTree = nametree; fTitleTree = titletree;
		fMetaTree.TreeName.SetString(nametree);
		fMetaTree.TreeTitle.SetString(titletree);
				
   		fUpdate = upd;
		TTree::SetMaxTreeSize(maxtreesize);
	}
	
}

Bool_t RawToTree::Implementation::NextOut(const Char_t *filename)
{
	TFile *f;
	TString out = filename, rout;
	
	if ( out.EndsWith(".root") ) { out.ReplaceAll(".root",""); }
	
	Bool_t oktree = kFALSE; Int_t fnumber = 0;
	for( Int_t i = 0; i < 999; i++ ){ // try to find the next file
		
		if ( i == 0 ) rout = out + ".root";
		else { rout = out; rout += "_"; rout += i; rout += ".root"; }
			
		if ( gSystem->AccessPathName(rout.Data()) > 0 ) { oktree = kTRUE; fnumber = i; break; }
	}
	if ( gDebug > 0 ) printf("In RawToTree::NextOut, next root file %s \n",rout.Data());
		
	f = new TFile(rout.Data(),"NEW");
	if ( f ) { // now new tree
		fTree = new TTree(fMetaTree.TreeName.GetName(),fMetaTree.TreeTitle.GetName());
		
		fEvents.Clear();
		
		if ( fMetaTree.TreeTitle.GetString().Contains("Exo",TString::kIgnoreCase ) )  
			fEvents.Add(new ExogamR());
		if ( fMetaTree.TreeTitle.GetString().Contains("Vam",TString::kIgnoreCase ) )  
			fEvents.Add(new VamosR());	
		if ( fMetaTree.TreeTitle.GetString().Contains("Diam",TString::kIgnoreCase ) ) 
			fEvents.Add(new DiamantR());

		// add events depending on the tree name
		if ( fTree ) { 
			fTree->Branch(&fEvents); fTree->SetFileNumber(fnumber);
		} 
		else {  oktree = kFALSE; delete f; }	
	}
	else { oktree = kFALSE; }
	
	return oktree;
}

void RawToTree::Implementation::Init(Option_t *op)
{
	TString opt = op, in, out;
	
	// from option, get the input name and the output name
	TObjArray *token = opt.Tokenize(" ");
	if ( token->GetEntries() != 2 ) { 
		printf("Format to initialize this application: input_name output_name \n");
		delete token; return;
	}
	else {
		in = token->At(0)->GetName(); out = token->At(1)->GetName(); delete token;
	}
	if ( gDebug > 0 ) printf("INPUT: %s, OUTPUT: %s \n",in.Data(),out.Data());
	
	if( fStatus != RawToTree::Implementation::kBad ){
		printf(" This application has already been initialized, close it before \n");
	}
	else {
		if ( fInput != NULL ) { delete fInput; fInput = NULL; } 
		if ( in.BeginsWith("DAQ") ) {
			fInput = new gws::DAQGANIL(&fBuffer); // DAQ Ganil
			fBuffer.SetDAQ(true);			
			fBuffer.InitTable("/space/users/evamos/E404aS/config/ACTIONS_E404AS.CHC_PAR");
		}
		else 
			fInput = new gws::StreamIO(); // file or DLT
			
		// try to open input string
		try {  fInput->Open(in.Data(),gws::IO::kIn); }
		catch ( gws::Exception & e) 
		{
			printf(" - ! - Exception %s \n ",e.what()); delete fInput; fInput = NULL; 
			return; 
		}
		
		// try to open a root file
		if ( NextOut(out.Data()) ) {
			fStatus = RawToTree::Implementation::kInit;
				
			fMetaTree.Clear("stat");
			fMetaTree.TreeFile.SetString(fTree->GetCurrentFile()->GetName());
			// fCurrentTFileName = fTree->GetCurrentFile()->GetName(); 
		}	
	}
}

void RawToTree::Implementation::Suspend(Option_t *op)
{
	TString opt = op;
	if ( fStatus == RawToTree::Implementation::kTimerOn ) {
		StopTimer(); 
		fStatus = RawToTree::Implementation::kSuspended;
	}
	if ( fStatus == RawToTree::Implementation::kRunning ) { fStatus = RawToTree::Implementation::kSuspended; }
}

void RawToTree::Implementation::Run(Option_t *op)
{
// 
   Bool_t ok2tree[10];
   UShort_t size_event = 0, *data_event; 
   int bytesread, nbtry, nb_branches = fEvents.GetEntries(); RawEvent *ev;
   
   // printf("Number of branches in the TTree %d \n",nb_branches);
   TString opt = op;
   if ( fStatus == RawToTree::Implementation::kBad ) {
	printf("Cannot Run this application, it has not been initialised ! \n "); return;
	
   }
   if ( fStatus == RawToTree::Implementation::kRunning ) {
	printf(" **** This application is already running ! **** \n "); return;
   }
   fStatus = RawToTree::Implementation::kRunning;

//   printf("\n");
//   printf("     <<< Looping on events >>>");
//   printf("\n");
//   TDatime *d1 = new TDatime; 
//   fLog << " *** " << d1->AsSQLString() << " Start looping " << endl;
//   delete d1;
   
   nbtry = 0;
   while ( fStatus == RawToTree::Implementation::kRunning ) {
   
	// try to get the next block
	//printf(" Read \n"); 	
	try { bytesread = fInput->Read(fBuffer.GetBuffer()); }
	catch ( exception & e) {
		  printf(" gws::Exception %s \n ",e.what()); fLog << " gws::Exception " << e.what() << endl;
		  fStatus = RawToTree::Implementation::kSuspended; break; 
	}
	if ( bytesread == 0 ) { 
		fTimer.Start(5000, kTRUE); 
		if ( nbtry > 3 ) { fStatus = RawToTree::Implementation::kSuspended; break; }
		printf(" No data read from the input, I will try again in one second \n"); 
		fStatus = RawToTree::Implementation::kTimerOn; break;
	}
	// printf(" Read \n"); 		
	// process events in this block
	
	if ( fBuffer.InitReading() == false ) { continue;}
	
	// new run, write it in the catalogue file
	
	// printf(" Current run \n"); 
	if ( fBuffer.GetRun() != fMetaTree.Run ) { // the run number has just changed
	
		if ( fMetaTree.Run != -1 ) {
			fStatus = RawToTree::Implementation::kSuspended;
			Update();
			fStatus = RawToTree::Implementation::kRunning;			
		}
		
		fMetaTree.Run = fBuffer.GetRun() ;
		fMetaTree.TreeFile.SetString(fTree->GetCurrentFile()->GetName());
		fMetaTree.Date.SetString(fBuffer.GetDate());
		// fCurrentTFileName = fTree->GetCurrentFile()->GetName();
		fCatalogue << fMetaTree.Run << "\t \t" 
		           << fTree->GetCurrentFile()->GetName() << "\t \t"
			   << fBuffer.GetDate() << endl;
	}
	//printf(" Current run \n"); 

	while ( (data_event = fBuffer.NextEvent(size_event)) != NULL ) {
		// printf("in fMetaTree.EventsIn[0] %d\n",fMetaTree.EventsIn[0]);
	 	fMetaTree.EventsIn++;
		for (Int_t i = 0; i < nb_branches; i++ ) {
			ev = (RawEvent *)fEvents.At(i); ok2tree[i] = ev->Fill(data_event,size_event);
			
			if ( i == 0 && ok2tree[i]) fMetaTree.ExoEvents++;
			if ( i == 1 && ok2tree[i]) fMetaTree.VamosEvents++;
			if ( i == 2 && ok2tree[i]) fMetaTree.DiamantEvents++;			
		}
		if ( ok2tree[0] && ok2tree[1] || ok2tree[0] && ok2tree[2] ) { fMetaTree.EventsOut++; fTree->Fill(); } 
	} // data_event	 
	
	// root file has changed, write it in the catalogue
	//printf(" Current file \n"); 
	if (  !fMetaTree.TreeFile.GetString().Contains(fTree->GetCurrentFile()->GetName()) ) {
		//fCurrentTFileName = fTree->GetCurrentFile()->GetName();
		fMetaTree.TreeFile.SetString(fTree->GetCurrentFile()->GetName());
		fCatalogue << fMetaTree.Run << "\t \t" 
		           << fTree->GetCurrentFile()->GetName() << "\t \t"
			   << fBuffer.GetDate() << endl;		
	}
	//printf(" process \n"); 	
	// check if application is suspended
	gSystem->ProcessEvents();
	//printf(" Current \n"); 		
  } // kRunning
  
  //TDatime *d2 = new TDatime; 	
  //fLog << " *** " << d2->AsSQLString() << " Stop looping " << endl;
  // delete d2;
 
  /* 
  printf("\n"); 
  printf("   <<< looping has been interrupted, run to resume >>> "); 
  printf("\n");  
  */
}


void RawToTree::Implementation::Update(Option_t *op)
{
	TString opt = op, tmp; TFile *f=NULL;
	if ( fStatus == RawToTree::Implementation::kSuspended ) { // write tree and increase file number

		
		
		TDatime *d = new TDatime; 
		fLog << " *** " << d->AsSQLString() << " *** " << endl;
		fLog << " Update, close " << f->GetName() << endl;
		delete d;
		
		Status(fLog);
		
		// save MetaData
		f = new TFile("RawToTree.xml");
		f->cd();
		fMetaTree.Write(); f->Close(); // delete f;
		
		if ( fMetaTree.EventsOut > 0 ) { 
			printf(" Save tree before closing file %s \n", f->GetName() ); f->Write(); 
		}
		f->Close(); // delete f;
		
		//printf("toto \n");
		if ( tmp.Contains("_") ) tmp.Remove((tmp.Last('_'))); 
		NextOut(tmp.Data());
		
		fMetaTree.Clear("stat");
	}
}

void RawToTree::Implementation::Status(Option_t *op)
{
	TString opt = op; 		
	cout << "\n******************************************* \n";
	Status(cout);
	cout << "******************************************* \n";		
}

void RawToTree::Implementation::End(Option_t *op)
{
	TString opt = op; TFile *f;
	
	if ( fStatus == RawToTree::Implementation::kSuspended ) { // write tree and close root file
		
		// save MetaData
		f = TFile::Open("RawToTree.xml","UPDATE");
		f->cd();
		fMetaTree.Write(); f->Close(); // delete f;
				
		TDatime *d = new TDatime; 
		fLog << " *** " << d->AsSQLString() << " *** " << endl;
		fLog << " End, closing " << f->GetName() << endl;
		delete d;
		
		Status(fLog);
		
		f = fTree->GetCurrentFile(); 
		if ( fMetaTree.EventsOut > 0 ) { 
			printf(" I am writting the current tree before closing the root file %s \n", f->GetName()); 
			f->Write(); 
		}
		f->Close(); fInput->Close(); fInput = NULL;
		
		fMetaTree.Clear();
		fStatus = RawToTree::Implementation::kBad; cout << " ... Bye ! " << "\n";		
	}
}

void RawToTree::Implementation::Help(Option_t *op)
{
// display availabled commands.
	TString opt = op;
	cout << "\n";
	cout << " --- Help on RawToTree --- " << "\n" ; 
	cout << "      RawToTree : Program to read raw events and store them in a ROOT file " << "\n" ;
	cout << "      Commands availables : " << "\n";
	cout << "       * Help()   : write this text " << "\n";
	cout << "       * Init()   : to init input file and output file " << "\n";
	cout << "           Ex: Init('/dev/st0 myroot.root') " << "\n";
	cout << "       * AttributesTree() : to set some TTree attributes " << "\n";
	cout << "       * End()            : to stop this application " << "\n";
	cout << "       * Suspend()        : to suspend this application " << "\n";           
	cout << "       * Status()         : write status of the application " << "\n";        
	cout << " --- End Help on RawToTree --- " << "\n" ;
	cout << "\n";
}

void RawToTree::Implementation::Status(ostream &out)
{
	switch( fStatus ){
		case RawToTree::Implementation::kBad :
		  out << "Application has not been initialized \n";
		  break;
		  
		case RawToTree::Implementation::kInit :
		  out << "Application has just been initialized, you can run it \n";
		  fInput->Status(out);
		  out << endl;	  
		  out << " Current run number is: " <<  fMetaTree.Run << endl;
		  out << " Current root file  is: " <<  fMetaTree.TreeName.GetName() << endl;			  
		  out << " Number of events read: " <<	fMetaTree.EventsIn << endl;
		  out << " Number of events written: " << fMetaTree.EventsOut << endl;  
		  break;
		  		  	  
		case RawToTree::Implementation::kRunning :
		  out << "Application is running \n";	
		  fInput->Status(out);	
		  out << endl;	  
		  out << " Current run number is: " << fMetaTree.Run << endl;
		  out << " Current root file  is: " << fMetaTree.TreeName.GetName() << endl;			  
		  out << " Number of events read: " <<	fMetaTree.EventsIn << endl;
		  out << " Number of events written: " << fMetaTree.EventsOut << endl;
		  break;
		  		  
		case RawToTree::Implementation::kSuspended :
		  out << "Application is suspended, resume it with run \n";	
		  fInput->Status(out);	
		  out << endl;	  
		  out << " Current run number is: " << fMetaTree.Run << endl;
		  out << " Current root file  is: " << fMetaTree.TreeName.GetName() << endl;
		  out << " Number of events read: " << fMetaTree.EventsIn << endl;
		  out << " Number of events written: " << fMetaTree.EventsOut << endl;		   
		  break;
		  	
		 case RawToTree::Implementation::kTimerOn : 
		  out << "Application is running [idle]\n";	
		  fInput->Status(out);	
		  out << endl;	  
		  out << " Current run number is: " << fMetaTree.Run << endl;
		  out << " Current root file  is: " << fMetaTree.TreeName.GetName() << endl;
		  out << " Number of events read: " << fMetaTree.EventsIn << endl;
		  out << " Number of events written: " << fMetaTree.EventsOut << endl;		   
		  break;			 
			  	  
	}
}

ClassImp(RawToTree);

RawToTree::RawToTree()
{
	fImplementation = new RawToTree::Implementation();
}

RawToTree::~RawToTree()
{
	delete fImplementation;
}

void RawToTree::AttributesTree(const Char_t *nametree, const Char_t *titletree, Int_t upd, Long64_t maxtreesize)
{ fImplementation->AttributesTree(nametree,titletree,upd,maxtreesize);}

void RawToTree::Init(Option_t *opt) {fImplementation->Init(opt);}
void RawToTree::Run(Option_t *opt){fImplementation->Run(opt);}
void RawToTree::Update(Option_t *opt){fImplementation->Update(opt);}

void RawToTree::Help(Option_t *opt){fImplementation->Help(opt);}
void RawToTree::Status(Option_t *opt){fImplementation->Status(opt);}

void RawToTree::Suspend(Option_t *opt){fImplementation->Suspend(opt);}
void RawToTree::End(Option_t *opt){fImplementation->End(opt);}
void RawToTree::StopTimer(){fImplementation->StopTimer();}

