#ifndef GW_BASEFILTER_H
#define GW_BASEFILTER_H

#ifndef ROOT_TBuffer
#include <TBuffer.h>
#endif

#ifndef ROOT_TList
#include <TList.h>
#endif

#ifndef GW_RANDOM_H
#include <Random.h>
#endif

namespace Gw {

//! This class is a base class for any experimental filter
/*!
	In the GammaWare, the physics generators give cascades of links stored in
	ROOT SeqCollection. This is the general input to the experimental filters that
	degrade the true cascades to give experimental like events.
	
	Design following a strategy pattern.

\author Olivier Stezowski \date 11/2005
*/
class BaseFilter : public TNamed
{
protected:
	Random *fRand;	// unique Gw::Rand instance 
	
public:
	BaseFilter();
	virtual ~BaseFilter();

	//@{
	//! Apply this filter to the list of links
	/*!
		It applies this filter to the collection and write to buffer only the
		energy of the particle. It returns the number of values written in the buffer
	*/
	virtual Int_t ApplyE(const TSeqCollection &, TBuffer &) = 0;
	virtual Int_t ApplyE(const TSeqCollection &, Float_t *) = 0;
	//@}
		
	//! Root dictionnary related
	ClassDef(BaseFilter,0); // An experimental filter to real gamma-ray cascades
};
// BaseFilter inline members //////////////////////////////////////////////////////
// End BaseFilter inline members //////////////////////////////////////////////////

}
#endif
