/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file GEM.cpp compiled in libGWGEM.so */
 
#ifndef GW_GEM_H
#include "GEM.h"
#endif

#ifndef GW_TESTFEEDING_H
#include "TestFeeding.h"
#endif

using namespace Gw;

ClassImp(GEM);

GEM::GEM()
 : BaseGEM()
{
	fFeeding = NULL; 
}

GEM::~GEM()
{
	GEM::Clear(""); if ( fFeeding ) delete fFeeding; fFeeding = NULL;
}


Link *GEM::SetFeeding(const char *feedingtype)
{
	TString type = feedingtype;
	
	Link *tmp = fFeeding; fFeeding = NULL;
	
	// test, no physics in there ... just an example
	if ( type.Contains("TestFeeding") ) { fFeeding = new TestFeeding(); }
	
	return tmp;	
}

void GEM::SetExI(Float_t E, Float_t I)
{
	if ( fFeeding ) {
		NuclearLevel *toplevel = (NuclearLevel *)fFeeding->GetIL();
		toplevel->GetEnergy().SetValue(E); toplevel->GetSpin().Set(Int_t(I)); 
	}
	else cout << "  ** WARNING ** in GEM::SetExI, no feeding link defined ! " << endl;
}
    
void GEM::Clear(Option_t *o)
{
	TString opt = o; BaseGEM::Clear(o);
}

void GEM::DoCascade(TSeqCollection &cas, Option_t *o)
{
	Level *entrypoint; TString opt = o;
	
	BaseGEM::DoCascade(cas,o); // determine a cascade in the level scheme
	
	if ( fFeeding ) {
		entrypoint = ((Link *)cas.Last())->GetIL();
		//entrypoint->ls();
		fFeeding->SetFL( entrypoint ); fFeeding->DoCascade(cas);
	}
	
	return;
}



/*
void GEM::DoCascade(TSeqCollection *cas, Option_t *o)
{
	TObject *obj; Link *link;

	cas->Clear("nodelete"); // Reset the previous cascade.
	
	Int_t which_first;

	// select first gamma
	link = (Link *)fRand0.Rand(which_first); 
	link->DoCascade(*cas);

	// start path down
	obj = ((RandObj *)fRandDown.At( which_first ))->Rand();
	while ( obj != &fEoC ) {
		link = (Link *)fRand0.At(obj->GetUniqueID()) ;
		link->DoCascade(*cas);
		obj = ((RandObj *)fRandDown.At( obj->GetUniqueID() ))->Rand();		
	}

	// start path up
	obj = ((RandObj *)fRandUp.At( which_first ))->Rand();
	while ( obj != &fEoC  ) {
		link = (Link *)fRand0.At(obj->GetUniqueID()) ;	
		link->DoCascade(*cas);
		obj = ((RandObj *)fRandUp.At( obj->GetUniqueID() ))->Rand();
	}
}
*/
