/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file TestFeeding.h header file for a GEM (gamma-rays generator) */
 
#ifndef GW_TESTFEEDING_H
#define GW_TESTFEEDING_H

#ifndef GW_LINK_H
#include "Link.h"
#endif

#ifndef GW_NUCLEARLEVEL_H
#include "NuclearLevel.h"
#endif

#ifndef GW_RANDOM_H
#include <Random.h>
#endif

#ifndef ROOT_TObjArray
#include <TObjArray.h>
#endif

namespace Gw {

class TestFeeding : public Link
{
private:
	Random *fRand;		//! Unique Rand instance
	
protected:
	NuclearLevel fTop;	//! starting point for any decay 

private:
	Int_t fNbE1;		//! number of E1 simulated in the last cascade
	Int_t fNbE2;		//! number of E2 simulated in the last cascade
	TObjArray *fE1;		//! array of E1 transitions 
	TObjArray *fE2;		//! array of E2 transitions 
	
public:
	TestFeeding();
	virtual ~TestFeeding();
	
	//! to modify the entry point, if needed
	NuclearLevel & GetFeeding() { return fTop; }
	
	//! already exist and always the same
	virtual Level * SetIL(Level *initial) { return initial; }
	
	//! do cascade 
	virtual void DoCascade(TSeqCollection &col, Option_t *opt = "");
};

}
#endif
