/***************************************************************************
 *   Copyright (C) 2004-2006 by Olivier Stezowski & Christian Finck        *
 *   stezow(AT)ipnl.in2p3.fr, cfinck(AT)ires.in2p3.fr                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/** \file ASymCorrelatedSpace.cpp compiled in libGWPHYSICS.so */

#ifndef Gw_ASymCorrelatedSpace
#include "ASymCorrelatedSpace.h"
#endif

#ifndef ROOT_TH1D
#include "TH1D.h"
#endif

#ifndef ROOT_TList
#include "TList.h"
#endif

#ifndef ROOT_TString
#include "TString.h"
#endif

using namespace Gw;

//
ClassImp(ASymCorrelatedSpace)


//__________________________________________________________
ASymCorrelatedSpace::ASymCorrelatedSpace() 
  : CorrelatedSpace()
{ 
  // default constructor
  fLog.GetProcessName() = "ASymCorrelatedSpace";
}

//__________________________________________________________
ASymCorrelatedSpace::ASymCorrelatedSpace(const char* name, const char* title) 
: CorrelatedSpace(name, title)
{ 
  fLog.GetProcessName() = "ASymCorrelatedSpace";
}

//__________________________________________________________
ASymCorrelatedSpace::~ASymCorrelatedSpace()
{
  // default destructor
}

//__________________________________________________________
Int_t ASymCorrelatedSpace::GetAxisNumber(Option_t* name) const
{
  
  TString tmp(name);
  tmp.ToLower();
  
  switch (tmp[0]) {
	case 'x':
	  return 0;

	case 'y':
	  return 1;
	  
	case 'z':
	  return 2;
	  
	case 't':
	  return 3;
	  
	default:
	  return tmp[0] - '0';
  }
  
}

//__________________________________________________________
void ASymCorrelatedSpace::SetDimension(Int_t dimension)
{
  fDimension = dimension;  
  
  for (Int_t i = 0; i < fDimension; ++i) {
	TH1D* h1 = new TH1D(Form("hp%d", i+1), Form("Axis %d Projection", i+1), 1, 0, 1);
	fHistoList->Add(h1);
	
	TH1D* h1b = new TH1D(Form("hpBkg%d", i+1), Form("Axis %d Bkg Projection", i+1), 1, 0, 1);
	fHistoBkgList->Add(h1b);
	
	TH1D* h1t = new TH1D(Form("hpTot%d", i+1), Form("%s Axis %d Total Projection", GetName(), i+1), 1, 0, 1);
	fHistoTotList->Add(h1t);
  }
}


//__________________________________________________________
void ASymCorrelatedSpace::Project(Option_t* axis, Option_t* gateName) 
{ 
  CorrelatedSpace::Project(axis, gateName); 
}

