


#ifndef Gw_BaseLSPlayer
#define Gw_BaseLSPlayer

#include "GwLogMessage.h"


#ifndef ROOT_TGaxis
#include "TGaxis.h"
#endif

#ifndef ROOT_TNamed
#include "TNamed.h"
#endif

#ifndef Gw_LevelScheme
#include "LevelScheme.h"
#endif

class TCanvas;
class TGaxis;
class TGToolTip;
class TBox;
class TContextMenu;

namespace Gw {

/** LSaxis extented class of TGaxis, allows zooming
  
*/

class LSaxis : public TGaxis
{
public:
   LSaxis();
   virtual ~LSaxis() {;}
   
   //! Handle Movement
   void ExecuteEvent(Int_t eventType, Int_t eventX, Int_t eventY);
   
   //! Unzoom
   void UnZoom(); //*MENU
   
   //! Set range axis
   void SetRange(Int_t ymin, Int_t ymax, Int_t xmin, Int_t xmax); //*MENU* *ARGS={ymin=>fLastY1Zoom,ymax=>fLastY2Zoom,xmin=>fLastX1Zoom,xmax=>fLastX2Zoom}*
   
   //! Set Y1
   void SetY1(Double_t y1);
   
   //! Set Y2
   void SetY2(Double_t y2);
   
   //! Get last zoom Y1
   Int_t GetLastY1Zoom() const { return fLastY1Zoom; }
   
   //! Get last zoom Y2
   Int_t GetLastY2Zoom() const { return fLastY2Zoom; }
   
   //! Get last zoom X1
   Int_t GetLastX1Zoom() const { return fLastX1Zoom; }
   
   //! Get last zoom X2
   Int_t GetLastX2Zoom() const { return fLastX2Zoom; }
   
private:
   Int_t fLastX;   // last x mouse position
   Int_t fLastY1;  // last y1 position
   Int_t fLastY2;  // last y2 position 
   
   Bool_t fFirst1Zoom;  // flag for first zoom in Y1
   Bool_t fFirst2Zoom;  // flag for first zoom in Y2
   Int_t  fLastY1Zoom;  // last zoom position in Y1
   Int_t  fLastY2Zoom;  // last zoom position in Y2
   Int_t  fLastX1Zoom;  // last zoom position in X1
   Int_t  fLastX2Zoom;  // last zoom position in X2
   
   ClassDef(LSaxis,0) 	  
};
   
//-------------------------------------------------------------------------  
   
class Level;
class Link;
class Cascade;  
  
/** BaseLSPlayer to work on level scheme
 
 Level scheme has by default a player that's at least draw the scheme.
 This class sets the pad range and connect the pad to the ProcessEvent method.
 The readers are also handle in this class.
 
 A number of pressed keys start an action:
  's': select a link/level/cascade
  '1/x': select a link for a gate on x-axis 
  '2/y': select a link for a gate on y-axis 
  '3/z': select a link for a gate on z-axis 
  'c': collect selected links (deprecated) 
  'l': popup add link dialogue box
  'e': popup add level dialogue box 
  't': Toggle on/off active LS menu on canvas, and not only on the energy axis			
  'i': Toggle on/off information tips that appears when passing over an object 
*/

class BaseLSPlayer : public TNamed
{
protected:
  //! default constructor
  BaseLSPlayer(LevelScheme* levelScheme);
  
public:
  virtual ~BaseLSPlayer();
      
  enum EDrawStyle { kRadware, kEnsdf, kUnknown};

  //!  set default size for level scheme
  static void SetDefaultSize(Float_t xmin, Float_t xmax, Float_t ymin, Float_t ymax) { 
    fgDefaultSize[0] = xmin; fgDefaultSize[1] = xmax;
    fgDefaultSize[2] = ymin; fgDefaultSize[3] = ymax;    
  }
  
  //! Get default size
  static Float_t* GetDefaultSize() {return fgDefaultSize;}
  
  //!  set default widths
  static void SetDefaultWidth(Float_t width) { fgDefaultWidth = width;}
  
  //! Get default width
  static Float_t GetDefaultWidth() {return fgDefaultWidth;}
  
  //!  set default color for axis toggle
  static void SetDefaultToggleColor(Int_t color) { fgDefaultToggleColor = color;}

  //!  get default color for axis toggle
  static Int_t GetDefaultToggleColor() {return fgDefaultToggleColor;}

  
  //! Get size from range
  static Float_t GetXMin(Float_t xRmin, Float_t xRmax) { return (xRmin*1.1 + 0.2*xRmax)/1.3; }
  static Float_t GetXMax(Float_t xRmin, Float_t xRmax) { return (xRmin*0.1 + 1.2*xRmax)/1.3; }
  
  static Float_t GetYMin(Float_t yRmin, Float_t yRmax) { return (yRmin*1.1 + 0.1*yRmax)/1.2; }
  static Float_t GetYMax(Float_t yRmin, Float_t yRmax) { return (yRmin*0.1 + 1.1*yRmax)/1.2; }
  
  //! Get range from size
  static Float_t GetXRmin(Float_t xmin, Float_t xmax) { return xmin-0.2*(xmax-xmin); }
  static Float_t GetXRmax(Float_t xmin, Float_t xmax) { return xmax+0.1*(xmax-xmin); }

  static Float_t GetYRmin(Float_t ymin, Float_t ymax) { return ymin-0.1*(ymax-ymin); }
  static Float_t GetYRmax(Float_t ymin, Float_t ymax) { return ymax+0.1*(ymax-ymin); }
   
  //! Get position of labels in Y
  static Float_t GetYLabels(Float_t ymin, Float_t ymax) { return GetYRmin(ymin, ymax)*0.4; }
   
  //! Get position of axis in X
  static Float_t GetXAxis(Float_t xmin, Float_t xmax) { return xmin-(xmax-xmin)*0.1; }
  
  //! Connect the Canvas to this to collect events
  Bool_t Connect(TCanvas *c = 0x0); 
  
  //! Connect the Canvas to this to collect events
  Bool_t Disconnect(TCanvas *c = 0x0);   
  
  //! Handle Movement
  void HandleMovement(Int_t eventType, Int_t eventX, Int_t eventY, TObject* select);
  
  //! Popup DoProject menu
  virtual void PopupDoProject() {return;} 
    
  //! Popup AddLink menu
  virtual void PopupAddLink() {return;} 
  
  //! Popup AddLevel menu
  virtual void PopupAddLevel() {return;} 
  
  //! Popup AddFirstLevel menu
  virtual void PopupAddFirstLevel() {return;} 
   
   //! Popup AddGroundLevel
   virtual void PopupAddGroundLevel() {return;} 

  //! Popup write LS menu
  virtual void PopupWriteLS() {return;} 
  
  //! Select all objects
  virtual void Select() {return;}
  
  //! Deselect all objects
  virtual void DeSelect() {return;}
   
  //! Intercept Delete
  virtual void Delete(Option_t* /*opt*/) {return;}
  
  //! Collect gates
  virtual void ShowGate(Option_t* /*opt*/ = "gate") { return; }
  
  //! Draw projection total
  virtual void DrawProjTot(Option_t* /*opt*/ = "") { return; }
  
   //! Get current links
  virtual Cascade* GetSelectedLinks() { return fSelectedLinks; }
  
  //! Get current liks
  virtual LevelScheme* GetLevelScheme() { return fLevelScheme; }

  //! Collect selected links
  virtual Int_t CollectLinks(Option_t* o = "");
 
  //! Collect selected leveks
  virtual Int_t CollectLevels(Option_t* o = "");
  
  //! Collect selected cascades
  virtual Int_t CollectCascades(Option_t* o = "");
  
  //! to draw this level scheme 
  virtual void Draw(Option_t *opt = "");
  
  //! Compute distance to object
  virtual Int_t DistancetoPrimitive(Int_t px, Int_t py);
  
  //! get style
  virtual Bool_t IsDrawStyle(EDrawStyle style) { return fDrawStyle == style; }
  
  //! set style
  virtual void SetDrawStyle(EDrawStyle style) { fDrawStyle = style; }
  
  //! do style
  virtual void DoStyle(EDrawStyle style);
  
  //! for backward compatibility
  virtual Int_t ReadAGS(const Char_t *filename, Option_t *opt = "152Dy");
  
  //! for backward compatibility
  virtual Int_t ReadENSDF(const Char_t *, Option_t *);   
  
  //! to init this level scheme from an existing formatted file (ENSDF, Radware ..)
  virtual Int_t ImportLevelScheme(const Char_t *, Option_t *);    
  
protected:

  //! set/get fLastX/fLastY
  virtual Int_t GetLastX() {return fLastX;}
  virtual Int_t GetLastY() {return fLastY;}
  virtual void  SetLastXY(Int_t lastX, Int_t lastY) {fLastX = lastX; fLastY = lastY;}
  //! Print out
  virtual void  Print(Option_t* /*opt*/) const {return;} // for the moment
  
  //! set text for tool tip info
  virtual void  SetTipText(TObject* select);
  
  TList&  GetLevelList()   { return fLevelScheme->GetLevelList();   }
  TList&  GetLinkList()    { return fLevelScheme->GetLinkList();    }
  TList&  GetCascadeList() { return fLevelScheme->GetCascadeList(); }
  TLatex& GetNameLatex()   { return fLevelScheme->GetNameLatex();   }
  
  Link*    GetCurrentLink()    { return fLevelScheme->GetCurrentLink();    }
  Level*   GetCurrentLevel()   { return fLevelScheme->GetCurrentLevel();   }
  Cascade* GetCurrentCascade() { return fLevelScheme->GetCurrentCascade(); }
  
  static Float_t fgDefaultSize[];   ///< size of the level scheme per default
  static Float_t fgDefaultWidth;   ///< default width for level, box etc...
  static Int_t   fgDefaultToggleColor;   ///< default color of axis when toggle is on

private:
  
  Int_t fLastX;      //! last x event position
  Int_t fLastY;      //! last y event position
  
  TBox* fBox;        // just a box for tool tip

protected:
  
  Cascade* fSelectedLinks;     //! pointer to the selected links
  
  Cascade* fSelectedLevels;    //! pointer to the selected levels
  
  TList*   fSelectedCascades;  //! pointer to the selected levels
   
  TList*   fNucleiList;        // list of nuclei 

  LevelScheme* fLevelScheme;   //! pointer on level scheme
  
  EDrawStyle fDrawStyle;       // drawing style
    
  LSaxis* fAxis;               // menu 
  
  TGToolTip* fInfoTip;         // info tool tip

  Bool_t fIsOnAxis;            // flag to have selection only on axis or whole pad
  
  Bool_t fIsTipsOn;            // flag to activated info tips on objects
  
  Bool_t fIsLSSelected;        // flag for selection of LS

  TContextMenu* fContextMenu;  // context menu in canvas
  
  Color_t fCColorSelectedLink; // current color for selected link  
   
  Float_t fPositionLabels;     // position in Y for labels
   
  mutable LogMessage fLog;     //! log message
  
  ClassDef(BaseLSPlayer, 0); // Base class for facilities to play with level scheme  
};
   
} // end namespace

#endif
