

#ifndef Gw_BaseLevelSchemeReader
#define Gw_BaseLevelSchemeReader

#ifndef ROOT_TObject
#include "TObject.h"
#endif

#ifndef ROOT_TObjArray
#include <TObjArray.h>
#endif

#ifndef ROOT_TString
#include "TString.h"
#endif

#include "GwLogMessage.h"

#ifndef GW_LEVELSCHEME_H
#include "LevelScheme.h"
#endif

namespace Gw {

/** A BaseLevelSchemeReader class to read level scheme files

*/
class BaseLevelSchemeReader : public TObject 
{

public:
  
  BaseLevelSchemeReader(const char* filename, Option_t* opt);
  virtual ~BaseLevelSchemeReader();
    
  //! imported level scheme file
  virtual Int_t Import(LevelScheme& levelScheme);

  //! check file before imported level scheme
  virtual Int_t Check()                              = 0;
    
  //! fill labels
  virtual void FillLabels( LevelScheme& levelScheme) = 0;
  

protected:
   
  TString fFileName; ///< name of level scheme file
  TString fOptions;  ///< options
  
  TObjArray fBand;   ///< list of cascades
  TObjArray fLevel;  ///< list of levels
  TObjArray fLink;   ///< list of links
  TObjArray fText;   ///< list of texts

  Bool_t fIsChecked; ///< flag for reading is ok
  
  mutable LogMessage fLog; //!< log message
  
protected:
  
  // fill level scheme once it has been checked
  virtual void Fill(LevelScheme& levelScheme);
  
};

} // end namespace
#endif
