/***************************************************************************
 *   Copyright (C) 2004-2006 by Olivier Stezowski & Christian Finck        *
 *   stezow(AT)ipnl.in2p3.fr, cfinck(AT)ires.in2p3.fr                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/** \file BasePeak.cpp compiled in libGWTOOLS.so */

#ifndef Gw_BasePeak
#include "BasePeak.h"
#endif

Gw::BasePeak::GateColor Gw::BasePeak::gGateColor = { kRed-9, kBlue-7, kGreen-8, kYellow-6 };

using namespace Gw;

ClassImp(BasePeak)

//__________________________________________________________
BasePeak::BasePeak(const char* name, const char* title) : TNamed(name, title),
	fHeight(0.0),
	fIntensity(0.0),
	fDimension(0),
	fIsFitOn(false),
	fIsCombined(false)
{ 
  //Default Constructor

}

//__________________________________________________________
BasePeak::~BasePeak()
{
// default destructor
}


//__________________________________________________________
BasePeak::BasePeak(const BasePeak& p) : TNamed(p),
	fHeight(p.fHeight),
	fIntensity(p.fIntensity),
	fDimension(p.fDimension),
	fIsFitOn(p.fIsFitOn),
	fIsCombined(p.fIsCombined)
{
  // copy constructor
}


//__________________________________________________________
BasePeak& BasePeak::operator=(const BasePeak& p)
{
	if (this == &p) 
			return *this;

    ((TNamed &)p).TNamed::Copy((*this));
        
	fHeight     = p.fHeight;
	fIntensity  = p.fIntensity;
	fDimension  = p.fDimension;
	fIsFitOn    = p.fIsFitOn;
	fIsCombined = p.fIsCombined;

	return *this;
}

//__________________________________________________________
Bool_t BasePeak::IsvalideGateColor(Color_t co)
{
	if (co == gGateColor.k0) return true;
	if (co == gGateColor.k1) return true;
	if (co == gGateColor.k2) return true;
	if (co == gGateColor.k3) return true;
	
	return false;
}

//__________________________________________________________
Color_t BasePeak::GetGateColor(Int_t idx)
{
	switch (idx) {
		case 0:
			return gGateColor.k0;
		case 2:
			return gGateColor.k1;	
		case 3:
			return gGateColor.k2;
		case 4:
			return gGateColor.k3;
		default:
			return -1;
	}
	
}

//__________________________________________________________
Int_t BasePeak::GetGateAxis(Color_t co)
{
	if (co == gGateColor.k0) 
		return 0;
	else if (co == gGateColor.k1) 
		return 1;
	else if (co == gGateColor.k2) 
		return 2;
	else if (co == gGateColor.k3) 
		return 3;
	else 
		return -1;
}

//__________________________________________________________
void BasePeak::FitOn(Bool_t fit_on)
{
	fIsFitOn = fit_on;
}

//__________________________________________________________
void BasePeak::Print(Option_t* /*opt*/) const
{
  printf("Height: %6.1f\n", fHeight);
  if (fIntensity > 0.)
    printf("Intensity: %7.1f\n", fIntensity);
}





