/***************************************************************************
 *   Copyright (C) 2004-2006 by Olivier Stezowski & Christian Finck        *
 *   stezow(AT)ipnl.in2p3.fr, cfinck(AT)ires.in2p3.fr                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/** \file CSFilter.cpp compiled in libGWPHYSICS.so */

#ifndef Gw_CSFilter
#include "CSFilter.h"
#endif

#ifndef Gw_BasePeak
#include "BasePeak.h"
#endif

using namespace Gw;

//
ClassImp(CSFilter)

TString CSFilter::fgFolderName = "Filter";

//__________________________________________________________
CSFilter::CSFilter() 
  : TNamed(),
	fCondition(1),
	fDimension(1),
    fLog("CSFilter")
{ 
  // default constructor
}

//__________________________________________________________
CSFilter::CSFilter(const char* name, const char* title) 
: TNamed(name, title),
  fCondition(1),
  fDimension(1),
  fPeakList(new TList()),
  IsBkg(false),
  fLog("CSFilter")
{ 
  // default constructor
  fPeakList->SetOwner(true);
 
}

//__________________________________________________________
CSFilter::~CSFilter()
{
  // default destructor
  delete fPeakList;
}

//__________________________________________________________
void CSFilter::Reset()
{
  fPeakList->Clear();
}
  
//__________________________________________________________
void CSFilter::Draw(Option_t* opt)
{
  TIter next(fPeakList);
  TObject* peak = 0x0;
   while ( (peak = next()) ) {
	  // clone to add peak to pad
	  BasePeak* cpeak = (BasePeak*)peak->Clone();
	  cpeak->SetBit(kCanDelete);
	  cpeak->SetBit(kMustCleanup);
	  cpeak->Draw(opt);
   }
}

//__________________________________________________________
void CSFilter::AddPeak(BasePeak* peak, Option_t* opt)
{
  TString option(opt);
  option.ToLower();
  if (option.Contains("reset")) {
	fPeakList->Clear();
  }
  
  fPeakList->Add(peak->Clone());
}



