/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   Copyright (C) 2010 by Christian Finck                                 * 
 *   stezow(AT)ipnl.in2p3.fr                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/** \file SymCorrelatedSpace.h header file for all type of SymCorrelatedSpace */  


#ifndef Gw_CSFilter
#define Gw_CSFilter

#include "GwLogMessage.h"

#ifndef Gw_BasePeak
#include "BasePeak.h"
#endif

#ifndef ROOT_TNamed
#include "TNamed.h"
#endif

#ifndef ROOT_TArrayD
#include "TArrayD.h"
#endif

#ifndef ROOT_TList
#include "TList.h"
#endif

#ifndef ROOT_TString
#include "TString.h"
#endif


namespace Gw {
  
/** CSFilter base class for filter
 
*/
class CSFilter : public TNamed
{  
  
protected:
  
  Short_t fCondition;         //! number of conditions for gating
  Short_t fDimension;         //! dimension for projection
  
  TList*  fPeakList;          //! list of peaks 
  Bool_t  IsBkg;     //! flag to know whether gate are made with bkg     

  mutable LogMessage fLog;    //! log message
    
  
private:
  
  static TString fgFolderName; //! name of the filter folder

public:
  
  //! Set Folder Name
  static void SetFolderName(const Char_t* name) { fgFolderName = name; }
  
  //! Get Folder name
  static const Char_t* GetFolderName() { return fgFolderName.Data(); }
  
public:
  CSFilter();
  CSFilter(const char* name, const char* title);
  virtual ~CSFilter();

  //! Get the pattern of energies inside the gates
  virtual Int_t IsInside(Double_t* /*energies*/, Int_t /*mult*/, Short_t* /*inGate*/, Int_t /*iGate*/ = 0) { return 0; }
  
  //! Flag for combined
  virtual Bool_t IsCombined() { return false; }
   
  //! Check if gate filled
  virtual Bool_t IsListFilled(Int_t /*idx*/) { return false; }

  //! Add Peak
  void AddPeak(BasePeak* peak, Option_t* opt = "update");

  //! Reset peak
  void Reset();
  
  //! Draw peaks
  void Draw(Option_t* opt);
  
  //! Get condition for gating
  Short_t GetCondition() const { return fCondition; }
  
   //! Set dimension for gating
  void SetCondition(Short_t cond) { fCondition = cond; }
  
  //! Get dimension for projected space
  Short_t GetDimension() const { return fDimension; }
  
  //! Set dimension for projected space
  void SetDimension(Short_t dim) { fDimension = dim; }
  
  //! Is for bkg gates
  Bool_t IsForBkg() { return IsBkg; }
   
  //! Get number of peaks
  Int_t GetNofPeaks() const { return fPeakList->GetEntries(); }
  
  //! Get peak
  BasePeak* GetPeak(Int_t idx) const { return (idx >= 0 && idx < fPeakList->GetEntries()) ? (BasePeak*)fPeakList->At(idx) : 0x0; }
   
  ClassDef(CSFilter,1)  // Correlated Space Filter
};
  
} // end namespace

#endif
