/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   Copyright (C) 2010 by Christian Finck                                 * 
 *   stezow(AT)ipnl.in2p3.fr                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/** \file CSManager.h header file for all type of CSManager */  


#ifndef Gw_CSManager
#define Gw_CSManager

#include "GwLogMessage.h"

#ifndef ROOT_TNamed
#include "TNamed.h"
#endif

#ifndef ROOT_TString
#include "TString.h"
#endif

class TCanvas;
class TH1D;
class TList;
class TVirtualPad;

namespace Gw {
class CorrelatedSpace;
class CSFilter;
class Peak1D;
   
/** CSManager correlated spaces manager
 
 @author Ch. Finck

*/
class CSManager : public TNamed
{  
  
private:
  
  static const TString fgkFolderName;        //! name of the CS Manager folder
  static const TString fgkFilterFolderName;  //! name of the filter folder
  static         Int_t fgNofInstance;        //! number of instances

protected:
    
  TList* fRegCS;            //! list of registred CS
  
  TString fFilterBaseName;  //! filter base name
  Int_t   fCInstance;       //! number of CS manager instanced
  Bool_t  fIsCanvasDiv;     //! flag for canvas division
  mutable LogMessage fLog;  //! log message

protected:
 
  //! Set folders
  void SetFolders();
   
  //! Get ratio for removing bgk spectrum from the gated one
  Float_t GetRatio(CSFilter* filter);
   
public:

  TString GetFolderName();
  TString GetFilterFolderName();
  
public:
  CSManager();
  CSManager(const char* name, const char* title);
  virtual ~CSManager();

  //!Start view
  TCanvas* GetCanvas();
   
  //! Get pad of total projection spectrum for a given CS
  TVirtualPad* GetProjTotPad(Int_t idx);
   
  //! Get pad of projected spectrum for a given CS
  TVirtualPad* GetProjPad(Int_t idx);
  
  //! Add correlated space
  void Register(CorrelatedSpace* cs);
  
  //! Get registred CS list
  const TList* GetRegCSList() { return fRegCS; } 

  //! Project
  void Project(Option_t* axis);
  
  //! Get Filter
  CSFilter* GetFilter(Option_t* gateName = "Gate");
  
  //! Collect gates on spectra
  Int_t CollectGate(Option_t* opt = "reset");
  
  //! Get histogramm
  TH1D* GetHisto(Int_t iCS) const;
  
  //! Get histogramm
  TH1D* GetHistoBkg(Int_t iCS) const;
  
  //! Get projection total
  TH1D* GetHistoTot(Option_t* axis = "x", Int_t iCS = 0) const;
  
  //! Draw projection
  void DrawProj(Option_t* opt);
  
  //! Draw total projection
  void DrawProjTot(Option_t* opt, Option_t* axis = "x");

  //! Convert correlated space to another correlated space
  void Convert(CorrelatedSpace* src, CorrelatedSpace* dst);
  
  //! Get Filter Base Name
  const Char_t* GetFilterBaseName() { return fFilterBaseName.Data(); }
  
  //! Set Filter Base Name
  void SetFilterBaseName(const Char_t* name) { fFilterBaseName = name; }
  
  //! Enable again canvas division
  void EnableCanvasDiv() { fIsCanvasDiv = false; }
  
  ClassDef(CSManager,0)  // Correlated Space Manager
};
  
} // end namespace

#endif
