/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   Copyright (C) 2010 by Christian Finck                                 * 
 *   stezow(AT)ipnl.in2p3.fr                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/** \file CorrelatedSpaceTH2.h header file for all type of CorrelatedSpaceTH2 */  


#ifndef Gw_CorrelatedSpaceTH2
#define Gw_CorrelatedSpaceTH2

#include "GwLogMessage.h"

#ifndef Gw_SymCorrelatedSpace
#include "ASymCorrelatedSpace.h"
#endif

#ifndef ROOT_TObject
#include "TObject.h"
#endif

class TBrowser;
class TFile;
class TH2;

namespace Gw {
	
class CSFilter;

/** CorrelatedSpaceTH2 class that manages a 2D symmetrical correlated space stored in a TH2.
    Suited for small space (typically a 4k matrix of float) and big number of events.
 
*/
class CorrelatedSpaceTH2 : public ASymCorrelatedSpace
{

protected:
  
  TH2*    fHisto;          // TH2 that contains the matrix
  UChar_t fType;           // Type of data: float, integer, etc
  Bool_t fAdopted;        // flag on when TH2 comes from outside
  
  mutable LogMessage fLog; //! log message

public:
  CorrelatedSpaceTH2(const char* name = CorrelatedSpace::GetDefaultName(), const char* title = "TH2", Option_t* type = "float");
  virtual ~CorrelatedSpaceTH2();
      
  //! fill correlated space
  void Fill(const Double_t* Xn, const Double_t weight = 1.);
  
  //! fill random
  void FillRandom(Int_t dimX, Int_t dimY);
  
  //! fill correlated space from H2
  void FillFromH2(TH2* h2);
  
  //! Adopt TH2, do not delete the matrix when destroying CS !
  void Adopt(TH2* h2);
   
  //! Project
  void Project(CSFilter* filter, Option_t* axis = "X");

  //! Project
  virtual void Project(Option_t* axis, Option_t* gateName = "Gate0");

  //! Browser method
   virtual void Browse(TBrowser* b);

  //! Get tree
  void AssertHisto();
  
  //! Get histo name
  const Char_t* GetHistoName() const;
  
  //! Get histo title
  const Char_t* GetHistoTitle() const;

  //! Get content
  Double_t GetContent(Int_t i, Int_t j);
  
  //! Get cell content
  Double_t GetCellContent(Int_t iCell);
  
  //! Set cell content
  void SetCellContent(Int_t iCell, Double_t weight);
    
  //! Get bin
  Int_t GetNbin(Int_t axis) const;
  
  ClassDef(CorrelatedSpaceTH2,1)  // Correlated Space Tree in 2 dim
};
	

  
} // end namespace

#endif
