


#ifndef Gw_CorrelatedSpaceTree2D
#define Gw_CorrelatedSpaceTree2D

#ifndef Gw_CorrelatedSpaceTree2
#include "CorrelatedSpaceTree2.h"
#endif

class TH2;
class TH1;

namespace Gw {

/** CorrelatedSpaceTree2D class that manages a 2D symmetrical correlated space of doubles store in a tree
 
*/
  
class CorrelatedSpaceTree2D : public CorrelatedSpaceTree2
{

protected:
  
  Double_t fCounts;              //! nb of counts for a given x,y
  
public:
  CorrelatedSpaceTree2D();
  virtual ~CorrelatedSpaceTree2D();
  
  //! fill counts
  void FillCounts(Int_t i, Int_t j, TH2* h2);

  //! fill counts from vector
  void FillCounts(Int_t i, Int_t j, Int_t dimX, const Double_t* Xn, const Double_t weight = 1.);

  //! fill counts random
  void FillCounts(Int_t j);
  
  //! Set counts
  void SetCounts(Double_t weight);
  
  //! Get counts
  Double_t GetCounts() {return fCounts; }
  
  //! Fill histo
  void FillHisto(Int_t j, TH1* h);
  
  //! Check if h2 already in tree
  void CheckH2InTree(TH2* h2);
    
  //! Set branch
  void SetBranch();
  
  //! Set address
  TBranch* SetAddress();
  
  ClassDef(CorrelatedSpaceTree2D,1)  // Correlated Space Tree in 2 dim for Integers
};
  
} // end namespace

#endif
