/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   Copyright (C) 2010 by Christian Finck                                 * 
 *   stezow(AT)ipnl.in2p3.fr                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/** \file CorrelatedSpaceTree2F.h header file for all type of CorrelatedSpaceTree2F */  


#ifndef Gw_CorrelatedSpaceTree2F
#define Gw_CorrelatedSpaceTree2F

#ifndef Gw_CorrelatedSpaceTree2
#include "CorrelatedSpaceTree2.h"
#endif

class TH2;
class TH1;

namespace Gw {

/** CorrelatedSpaceTree2F class that manages a 2D symmetrical correlated space of floats store in a tree
 
*/
class CorrelatedSpaceTree2F : public CorrelatedSpaceTree2
{

protected:
  
  Float_t fCounts;              //! nb of counts for a given x,y
  
public:
  CorrelatedSpaceTree2F();
  virtual ~CorrelatedSpaceTree2F();
  
  //! fill counts
  void FillCounts(Int_t i, Int_t j, TH2* h2);

  //! fill counts from vector
  void FillCounts(Int_t i, Int_t j, Int_t dimX, const Double_t* Xn, const Double_t weight = 1.);

  //! fill counts random
  void FillCounts(Int_t j);
  
  //! Set counts
  void SetCounts(Double_t weight);
  
  //! Get counts
  Double_t GetCounts() {return (Double_t)fCounts; }
  
  //! Fill histo
  void FillHisto(Int_t j, TH1* h);
  
  //! Check if h2 already in tree
  void CheckH2InTree(TH2* h2);
    
  //! Set branch
  void SetBranch();
  
  //! Set address
  TBranch* SetAddress();
  
  ClassDef(CorrelatedSpaceTree2F,1)  // Correlated Space Tree in 2 dim for Integers
};
  
} // end namespace

#endif
