/***************************************************************************
 *   Copyright (C) 2004-2006 by Olivier Stezowski & Christian Finck        *
 *   stezow(AT)ipnl.in2p3.fr, cfinck(AT)ires.in2p3.fr                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/** \file CorrelatedSpaceTree2I.cpp compiled in libGWPHYSICS.so */

#ifndef Gw_CorrelatedSpaceTree2I
#include "CorrelatedSpaceTree2I.h"
#endif

#ifndef ROOT_TFile
#include "TFile.h"
#endif

#ifndef ROOT_TH2
#include "TH2.h"
#endif

#ifndef ROOT_TRandom
#include "TRandom.h"
#endif

#ifndef ROOT_TTree
#include "TTree.h"
#endif

using namespace Gw;

//
ClassImp(CorrelatedSpaceTree2I)

//__________________________________________________________
CorrelatedSpaceTree2I::CorrelatedSpaceTree2I() 
  : CorrelatedSpaceTree2(),
	fCounts(0)
{ 
  // default constructor
  fLog.GetProcessName() = "CorrelatedSpaceTree2I";
}

//__________________________________________________________
CorrelatedSpaceTree2I::~CorrelatedSpaceTree2I()
{
  // default destructor
}

//__________________________________________________________
void CorrelatedSpaceTree2I::SetBranch()
{
  fTree->Branch(fBranchName.Data(), &fCounts, "fCounts/I");
}

//__________________________________________________________
TBranch* CorrelatedSpaceTree2I::SetAddress()
{
  TBranch* b = fTree->GetBranch(fBranchName.Data());
  if (b == 0x0) {
	fLog << warning << "Branch not find in tree" << dolog;
	return 0x0;
  }
  
  b->SetAddress(&fCounts);
  
  return b;
}

//__________________________________________________________
void CorrelatedSpaceTree2I::SetCounts(Double_t weight)
{
   fCounts = Int_t(weight+0.5);
}

//__________________________________________________________
void CorrelatedSpaceTree2I::FillCounts(Int_t i, Int_t j, TH2* h2)
{
  fCounts = (Int_t)(h2->GetBinContent(h2->GetBin(i,j)) + 0.5);
  GetHistoTot("x")->Fill(j, fCounts);
}

//__________________________________________________________
void CorrelatedSpaceTree2I::FillCounts(Int_t i, Int_t j, Int_t dimX, const Double_t* Xn, const Double_t weight) 
{
  fCounts = Int_t(Xn[i*dimX + j]*weight + 0.5);
  GetHistoTot("x")->Fill(j, fCounts);
}

//__________________________________________________________
void CorrelatedSpaceTree2I::FillCounts(Int_t j)
{
  fCounts = (Int_t)(gRandom->Uniform(0, 10)+0.5);
  GetHistoTot("x")->Fill(j, fCounts);
}

//__________________________________________________________
void CorrelatedSpaceTree2I::FillHisto(Int_t j, TH1* h1)
{
  h1->Fill(j, fCounts); 
}

//__________________________________________________________
void CorrelatedSpaceTree2I::CheckH2InTree(TH2* h2)
{
	
  if (fStatus == kDefined) {
	
	Int_t dimX = GetHistoTot("x")->GetNbinsX();
	Int_t dimY = GetHistoTot("y")->GetNbinsX();
	
	if (fTree->GetEntries() == dimX*dimY) {
	  
	  for (Int_t i = 0; i < dimX; ++i) {
		for (Int_t j = 0; j < dimY; ++j) {
		  fTree->GetEntry(i*dimX + j);
		  Double_t tmp = h2->GetBinContent(h2->GetBin(i,j));
		  h2->SetBinContent(h2->GetBin(i,j), tmp+(Double_t)fCounts);
		}	
	  }
	  
	  DeleteTree();
	  GetHistoTot("x")->Reset("ICE");
	}
  }
}


