


#ifndef Gw_CubicFilter
#define Gw_CubicFilter

#ifndef Gw_CSFilter
#include "CSFilter.h"
#endif

#ifndef ROOT_TObject
#include "TObject.h"
#endif

class TList;
namespace Gw {
class Gate;
  
/** CubicFilter class for cubic gates
 
*/
class CubicFilter : public CSFilter
{
  
private:
  
  TList*  fGateList;  //! list of gate
  Bool_t  IsPeak;     //! flag to know whether gate are made with peak     

private:
  
  //! fill gate for peak
  void FillGates();
  
  //! fill bkg gates
  void FillBkgGates();
  
  //! Reset gates
  void ResetGates();
    
public:
  CubicFilter();
  virtual ~CubicFilter();

  //! Get energies inside gates
  Int_t IsInside(Double_t* energies, Int_t mult, Short_t* inGate, Int_t iGate = 0);
  
  //! Make gates
  void MakeGates(Option_t* opt = "peak");
  
  //! Reset gates
  void Reset();
  
  //! Import gates from file
  void ImportGates(const Char_t* name,  Option_t* opt = "update");
  
  //! Add gate
  void AddGate(Double_t gateInf, Double_t gateSup, Int_t axis = 0, Option_t* opt = "update");
   
   //! Check if gate filled
   Bool_t IsListFilled(Int_t idx);
   
   //! Flag to combined
   Bool_t IsCombined();

  ClassDef(CubicFilter,1)  // Correlated Space Filter
};
 
  
class Gate : public TObject
{
  
private:
  
  TArrayD fGateInf;  //! inferior gate array
  TArrayD fGateSup;  //! superior gate array
    
public:
  
  Gate();
  virtual ~Gate();

   //! Get number of gates
  Int_t GetNofGates() const { return fGateSup.GetSize(); }
  
  //! Get gate inf
  Double_t GetGateInf(Int_t idx) { return fGateInf.At(idx); }
  
  //! Get Gate sup
  Double_t GetGateSup(Int_t idx) { return fGateSup.At(idx); }
  
  //! Add gate
  void AddGate(Double_t gateInf, Double_t gateSup, Option_t* opt = "update");
  
  //! Reset gates
  void Reset();
  
  ClassDef(Gate,1)  // Correlated Space Filter

};
	
} // end namespace

#endif
