/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/** \file ENSDF.h interface to ENSDF Evaluated Nuclear Structure Data File */  
 
#ifndef GW_ENSDF_H
#define GW_ENSDF_H

#ifndef GW_BASEENSDF_H
#include "BaseENSDF.h"
#endif

#ifndef ROOT_Rtypes
#include <Rtypes.h>
#endif
#ifndef ROOT_Gtypes
#include <Gtypes.h>
#endif
#ifndef ROOT_TString
#include <TString.h>
#endif

namespace Gw {

class Level;
class Link;

/**
ENSDF: Interface between ENSDF files and the GammaWare Nuclear data related objects. 

This class could be used directly and in a root interactive session once the correct library has been loaded. 
 
@author Olivier Stezowski
*/
class ENSDF : public BaseENSDF
{
private:
            
protected:
    

public:
    ENSDF();
    virtual ~ENSDF();

    void test();
    
    //! to get a link from a record 
    /*!
    	For the moment, only GammaLink or XGammaLink if the conversion coefficient is non-null.
	The string is not used, any possible offset in the gamma-ray energy is ignored.
	
	\todo The multipolarity part is not yet implemented !
    */
    virtual Link  *GetLink(const char *,  TString &);  
     
    //! to get a NuclearLevel from a Level record 
    /*!
	It returns NULL if this record is not a level record. If it is a level record, it returns
	a new NuclearLevel filled with the record values. \n
	If the level energy or/and the spin have an offset, it is written in the second the argument.
	
	The string length is 2 characters. The first character is for the energy and the second one is for the spin.
	Format "EJ" where E and J are the corresponding ENSDF reference to the energy and spin offset 
	as described in the ENSDF documentation (see also BaseENSDF::GetE).
		
	Ex: 
		-# "X " means: this level has an additionnal energy offset corresponding to the level
			labelled X in the dataset to which this level belongs to.
		-# " A" means: this level has an additionnal spin offset corresponding to the spin of
		the level labelled A in the dataset to which this belongs to.	
		-# "XA" means: this level has an additionnal energy and spin offset.
    */     
    virtual Level *GetLevel(const char *, TString &);
    
    virtual bool Open(const char *fname) { return BaseENSDF::Open(fname) ; }
    virtual void Close() { BaseENSDF::Close() ; }
    virtual bool IsOpen() { return BaseENSDF::IsOpen(); }

    virtual unsigned int IsDataSet(const char *nuclide, const char *dsid = "ADOPTED LEVELS, GAMMAS") const
      { return BaseENSDF::IsDataSet(nuclide,dsid); }   
    
   // virtual void ls(Option_t *) const ;    
    virtual void ls(std::ostream &) const ; 
    
    //! Root dictionary related
    ClassDef(ENSDF,0); // Interface to ENSDF
};

}
#endif
