/***************************************************************************
 *   Copyright (C) 2010 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "GCondition.h"

#include "PeakCreator.h"
#include "Env.h"
#include "Peak1D.h"

#include "TRegexp.h"
#include "TLeaf.h"
#include "TH2.h"
#include "TCanvas.h"
#include "TCut.h"
#include "TCutG.h"
#include "TFile.h"
#include "TROOT.h"
#include "TKey.h"

using namespace Gw;

ClassImp(GCondition);

GCondition::GCondition() :
	TNamed("AGCondition","x"),
	fListOfConditions(),
	fEnable(true),
	fExpectedResult(true),
	fHasPassed(false),
	fHasPassed_()
{	
	//! just add mime type so that Cond appears with specific icons in browser
	Gw::Env::AddRMimeType();
	//
	fListOfConditions.SetName("ListOfGConditions");
}

GCondition::GCondition(const char * name, const char * title) :
	TNamed(name,title),
	fListOfConditions(),
	fEnable(true),
	fExpectedResult(true),
	fHasPassed(false),
	fHasPassed_()
{
	//! just add mime type so that Cond appears with specific icons in browser
	Gw::Env::AddRMimeType();
	//
	fListOfConditions.SetName("ListOfGConditions");
}

GCondition::GCondition(const GCondition& p)
{
    TObject *obj; TIter next(&p.fListOfConditions);
	while ( (obj = next()) ) {
		fListOfConditions.Add(obj->Clone());
	}
    
    fEnable = p.fEnable; fExpectedResult = p.fExpectedResult; fHasPassed = p.fHasPassed; fHasPassed_ = p.fHasPassed_;
}


GCondition::~GCondition()
{
	fListOfConditions.Clear();
}

void GCondition::Delete(Option_t* option) 
{
	fListOfConditions.Clear(option);
}

void  GCondition::UseAsAnti(Bool_t r) 
{ 
	Style_t hash_style = 1; // 1 for coinc, 9 for anti
	
	if ( r == true ) 
		hash_style = 9;
	
	TObject *obj; TIter next(&fListOfConditions);
	while ( (obj = next()) ) {
		if ( obj->InheritsFrom("TAttLine") ) {
			dynamic_cast<TAttLine *>(obj)->SetLineColor(29);
			dynamic_cast<TAttLine *>(obj)->SetLineWidth(2);
			dynamic_cast<TAttLine *>(obj)->SetLineStyle(hash_style);
		}
	}

	fExpectedResult = !r; 
}

void GCondition::Enable()
{
    fEnable = !fEnable;
}

void GCondition::Print(Option_t * option) const
{
 	TObject *obj; TIter next(&fListOfConditions);
    std::cout << "Number of Condition in " << GetName() << " " << fListOfConditions.GetEntries() << std::endl;
	while ( (obj = next()) ) {
        obj->Print(option);
	}
}


ClassImp(GCond1D);

GCond1D::GCond1D() :
	GCondition(),
	fVarX("X")
{

}

GCond1D::GCond1D(const char * name, const char * title) :
	GCondition(name,title),
	fVarX("X")
{
	
}

GCond1D::GCond1D(const GCond1D &p) :
    GCondition(p),
    fVarX(p.fVarX)
{
    ;
}

GCond1D::~GCond1D()
{
	
}

Gw::PeakCreator *GCond1D::WindowGCreator(const char *c_name)
{
	Gw::PeakCreator *pc = 0x0; TVirtualPad *c_pad = 0x0; TString which_canvas(c_name);
	
	if ( which_canvas == "" )
		c_pad = TVirtualPad::Pad();
	else 
		c_pad = (TVirtualPad *)gROOT->GetListOfCanvases()->FindObject(c_name);
	
	if ( !c_pad ) 
		return pc;
	
	if ( !Gw::PeakCreator::IsPeakCreator(c_pad) ) {
		//
		Style_t hash_style = gCoincStyle; // 3004 for coinc, 3005 for anti
		if ( fExpectedResult == false ) 
			hash_style = gAnticStyle;
		//
		pc = Gw::PeakCreator::AddPeakCreator(c_pad);
		if ( pc ) {
			pc->SetGateMode(true);
			((Peak1D *)pc->DefaultPeak())->SetFillStylePeak(hash_style);
		}
	}	
	
	return pc;
}

void  GCond1D::UseAsAnti(Bool_t r) 
{ 
	Style_t hash_style = gCoincStyle; // 3004 for coinc, 3005 for anti
	
	if ( r == true ) 
		hash_style = gAnticStyle;
	
	TObject *obj; TIter next(&fListOfConditions);
	while ( (obj = next()) ) {
		Gw::Peak1D *p = dynamic_cast<Gw::Peak1D *> (obj);
		p->SetFillStylePeak(hash_style); 
	}
	
	fExpectedResult = !r; 
} 

void GCond1D::Draw(Option_t* /*option*/)
{
	TVirtualPad *c_pad = TVirtualPad::Pad();

	if ( ! c_pad ) 
		return;
	
	// it could be fListOfConditions.Draw. However some old version of root (5.24) uses option to select particular object of the collection to be drawn (based on expression)
	// so appaen current pas by myself
	TObject *obj; TIter next(&fListOfConditions);
	while ( (obj = next()) ) {
		if ( c_pad->GetListOfPrimitives()->FindObject(obj) == 0x0 ) 
			c_pad->GetListOfPrimitives()->Add(obj,"lg");
			c_pad->Modified();
	}
	if ( fListOfConditions.GetEntries() ) {
		WindowGCreator();
		c_pad->Update();
	}
}

Gw::Peak1D *GCond1D::AddWindow(Double_t min, Double_t max, Option_t *opt)
{
	TString option(opt); Double_t xmin = TMath::Min(min,max), xmax = TMath::Max(min,max);
	//
	if ( !option.Contains("+") ) 
		fListOfConditions.Clear();

	Gw::Peak1D *p = new Gw::Peak1D( Form("%s_%d",GetName(),fListOfConditions.GetEntries()), GetTitle() );
	p->SetPeak((xmin+xmax)/2., 1., (xmax-xmin) );

	if ( !fEnable )
		fEnable = true;
	
	p->SetBit(kCanDelete,false);
	p->SetBit(kMustCleanup,false);
	
	if ( fExpectedResult )
		p->SetFillStylePeak(gCoincStyle);
	else 
		p->SetFillStylePeak(gAnticStyle);
		
	gROOT->GetListOfSpecials()->Add(p);
	
	// add to condition
	fListOfConditions.Add(p); fHasPassed_.push_back(false);
	
	return p;
}

Bool_t  GCond1D::IsPassed(Double_t x, Double_t /*y*/) const
{
	if ( !fEnable || fListOfConditions.GetEntries() == 0 ) {
		fHasPassed = true; return fHasPassed;
	}
	
	Bool_t is_inside = false; 
	
	for (Int_t i = 0; i < fListOfConditions.GetEntries(); i++) {
		Gw::Peak1D *peak = (Gw::Peak1D *)fListOfConditions[i];
		Double_t p = peak->GetPosition();
		Double_t w = peak->GetFWHM()/2.;
		if ( x >= (p-w) && x <= (p+w) ) {
			is_inside = true;
			fHasPassed_[i] = fExpectedResult;
		}
		else fHasPassed_[i] = !fExpectedResult;
	}

	return (fHasPassed = (is_inside == fExpectedResult) );
	
}

Short_t GCond1D::Import(const char *what, const char *from, Option_t *opt)
{
	TString wh(what), origin(from), option(opt), tmp, obj_name;  Short_t result = 0; TDirectory *dir_to_search = gDirectory;
    //
    Bool_t wildcard = false;
    if ( wh.Contains("*") )
        wildcard = true;
    TRegexp rexp(Form("%s",what),wildcard);
    
	// start with empty container if option + is not there
	if ( !option.Contains("+") )
		fListOfConditions.Clear();

// Treat root file/dir cases
	if ( !origin.Contains(":") ) { // means local root
		if ( origin == "gdir" || origin == "gpad" ) {
			;
		}
		else origin.Prepend("file:");
	}
	if ( origin.BeginsWith("file:")  || 
		  origin.BeginsWith("root:")  ||  
		  origin.BeginsWith("roots:") ||  
		  origin.BeginsWith("rootk:") ||  
		  origin.BeginsWith("http:") ) 
	{
		TString rname = origin; 
		if ( origin.BeginsWith("file:") )
			rname.Replace(0,origin.First(":")+1,""); // only if local, remove file: protocol
		
		// TODO
		// if ( "" ) ==> open window 
		
		
		// look for similar already open if not open a new one
		TFile *f = 0x0;
		if ( !origin.EndsWith(".root") ) {
			rname.Append(".root"); origin.Append(".root");
		}
		
		if ( gROOT->GetListOfFiles()->FindObject(rname.Data()) ) {// already there 
			std::cout << "Import CUTS from OPEN FILE " << rname.Data() << std::endl ;
			f = (TFile *)gROOT->GetListOfFiles()->FindObject(rname.Data());
		}
		else {
			std::cout << "Import CUTS from NEW FILE " << origin.Data() << std::endl;
			f = TFile::Open(origin.Data(),"READ");
		}
		if ( f ) {
			dir_to_search = f; origin = "gdir"; // even if not current dir ! 
		}
		else origin = "FAIL";
	}
	
	if ( origin == "gdir" ) { // root file/directory
				
		// useful to browse the TDirectory
		TString class_name; TObject *obj_in_file = 0x0; TKey *key_in_file; TList new_cuts; TCut *cut; Gw::Peak1D *peak;
		new_cuts.SetOwner(true);
		
		// iterate over the key and keep only TCut and in case a TCut has been saved several time, only the highest cycle
		TIter it_keys(dir_to_search->GetListOfKeys());
		while ( (key_in_file = static_cast<TKey *>(it_keys())) ) { // next key		
			class_name  = key_in_file->GetClassName();
			obj_name    = key_in_file->GetName();
			// obj on disk is TCut and with a name not already in the list of Cond
			if ( class_name == "TCut" && obj_name.Contains(rexp) && fListOfConditions.FindObject(obj_name.Data()) == 0x0 ) { 
				if ( !new_cuts.FindObject(obj_name.Data()) ) { // list of key is ordered so that first for a given name correspond to the highest cycle
					obj_in_file = 
						key_in_file->ReadObj();
					new_cuts.Add(obj_in_file);
				}
			}
		}
		TIter it_cuts(&new_cuts);
		while ( (cut = static_cast<TCut *>(it_cuts())) ) { // next cut	
			peak =  CutToPeak(cut);
			if ( peak ) {
				
				obj_name = peak->GetName();
				if ( !obj_name.BeginsWith(GetName()) ) {
					peak->SetName(Form("%s_%s",GetName(),obj_name.Data()));
				}
				if ( fListOfConditions.FindObject(peak->GetName()) ) { // already a peak with same name in the list
					peak->SetName( Form("%s_%d",GetName(), fListOfConditions.GetEntries()) );
				}
				
				peak->SetDrawAs(Peak1D::kGate); 

				if ( !fEnable )
					fEnable = true;
				
				peak->SetBit(kCanDelete,false);
				peak->SetBit(kMustCleanup,false);
				
				// gROOT->GetListOfSpecials()->Add(peak);
				
				// add to condition
				fListOfConditions.Add(peak); fHasPassed_.push_back(false);
				
				result++;
			}
			else {
				std::cout << " Cannot convert " << cut->GetName() << " to peak " << std::endl; 
			}
		}			
	}
	if ( origin == "gpad" ) { // root current pad
		TSeqCollection *l_to_seach = 0x0; 
		if ( TVirtualPad::Pad() ) {
			l_to_seach = TVirtualPad::Pad()->GetListOfPrimitives();
		}
		TObject* obj = 0x0;
		TIter next(l_to_seach);
		while ( (obj = next()) ) { // iterator skips empty slots
			if ( obj->InheritsFrom("Gw::Peak1D") ) {
				tmp = obj->GetName();
				if ( tmp.Contains(rexp) ) {
					
					if ( fListOfConditions.FindObject(obj) ) { // already in the list
						printf("%s ALREADY IN THE LIST \n", obj->GetName());
						continue;
					}
					
					// change some attributes
					((Gw::Peak1D*)obj)->SetDrawAs(Peak1D::kGate); 
					if ( !tmp.BeginsWith(GetName()) ) {
						((Gw::Peak1D*)obj)->SetName(Form("%s_%s",GetName(),tmp.Data()));
					}
					if ( fListOfConditions.FindObject(obj->GetName()) ) { // already a peak with same name in the list
						((Gw::Peak1D*)obj)->SetName( Form("%s_%d",GetName(), fListOfConditions.GetEntries()) );
					}
					((Gw::Peak1D*)obj)->SetTitle(GetTitle());

					if ( !fEnable )
						fEnable = true;
					
					obj->SetBit(kCanDelete,false);
					obj->SetBit(kMustCleanup,false);
					
					// gROOT->GetListOfSpecials()->Add(obj);
					
					// add to condition
					fListOfConditions.Add(obj); fHasPassed_.push_back(false);
					result++;
				}
			}
		}
	}	

	if ( fListOfConditions.GetEntries() > 0 ) {
		printf("List of %d 1D Conditions \n",fListOfConditions.GetEntries() );
		fListOfConditions.ls();
	}
	
	return result;
}

Gw::Peak1D *GCond1D::TCutToPeak(TCut *cut)
{
	Gw::Peak1D *p = 0x0; Double_t min, max; std::string name_leaf1, name_leaf2; TString tmp(cut->GetTitle());
    
	// check good format
	if ( !tmp.Contains("(") || !tmp.Contains(")") || !tmp.Contains("&&") || tmp.CountChar('>') != 1 || tmp.CountChar('<') != 1 || (tmp.CountChar('<') > tmp.First('>')) ) {
		return p;
	}
	else {
		tmp.ReplaceAll("(","");
		tmp.ReplaceAll(")","");
		tmp.ReplaceAll(">","");
		tmp.ReplaceAll("<","");
		tmp.ReplaceAll("=","");
		tmp.ReplaceAll("&&","");
		
		tmp += " ";
	}
	std::istringstream decode(tmp.Data());
	decode.clear();
	decode >> name_leaf1 >> min >> name_leaf2 >> max ;
	if ( decode.good() ) {        
		if ( name_leaf1 == name_leaf2 ) {
            
			p = new Gw::Peak1D(cut->GetName(),name_leaf1.data());
            
            Double_t xmin = TMath::Min(min,max), xmax = TMath::Max(min,max);
			p->SetPeak((xmin+xmax)/2., 1., (xmax-xmin) );
            p->SetDrawAs(Peak1D::kGate);
		}
	}
	
	return p;
}

TCut *GCond1D::PeakToCut(const Gw::Peak1D *peak) const
{
	TCut *cut = 0x0;
	
	Double_t min = peak->GetPosition() - (peak->GetFWHM())/2. ;
	Double_t max = peak->GetPosition() + (peak->GetFWHM())/2. ;
	
	cut = new TCut( peak->GetName(),Form("(%s <= %f && %s >= %f)",fVarX.Data(),min,fVarX.Data(),max) );
	
	return cut;
}

Gw::Peak1D *GCond1D::CutToPeak(TCut *cut)
{
	Gw::Peak1D *p = GCond1D::TCutToPeak(cut);
    if ( p ) {
        if ( fVarX == p->GetTitle() ) {
            p->SetTitle(GetTitle());
        }
        else { delete p; p = 0x0; }
    }
	
	return p;
}


Short_t GCond1D::Export(const char *what, const char *to, Option_t *opt) const
{
	TString wh(what), destination(to), option(opt), tmp("ROOT"), rname;
    TDirectory *directory = gDirectory;
    Short_t result = 0; TObject* obj = 0x0;
   
	if ( !destination.Contains(":") ) { // means local root
		if ( destination != "gdir" ) {
			destination.Prepend("file:");
		}
	}
	if ( destination.BeginsWith("file:")  || 
		  destination.BeginsWith("root:")  ||  
		  destination.BeginsWith("roots:") ||  
		  destination.BeginsWith("rootk:") ||  
		  destination.BeginsWith("http:") ) 
	{
		TString rname = destination; 
		if ( destination.BeginsWith("file:") )
			rname.Replace(0,destination.First(":")+1,""); // only if local, remove file: protocol
		
		// TODO
		// if ( "" ) ==> open window 
			
		
		// look for similar already open if not open a new one
		TFile *f = 0x0;
		if ( !destination.EndsWith(".root") ) {
			rname.Append(".root"); destination.Append(".root");
		}

		if ( gROOT->GetListOfFiles()->FindObject(rname.Data()) ) {// already there 
			std::cout << "SAVE CUTS in OPEN FILE " << rname.Data() << std::endl ;
			f = (TFile *)gROOT->GetListOfFiles()->FindObject(rname.Data());
		}
		else {
			std::cout << "SAVE CUTS in OPEN FILE " << destination.Data() << std::endl;
			f = TFile::Open(destination.Data(),"UPDATE");
		}
		if ( f ) {
			directory = f;
		}
		else tmp = "FAIL";
	}

	if ( tmp == "ROOT" ) { 
		
        Bool_t wildcard = false;
        if ( wh.Contains("*") )
            wildcard = true;
        TRegexp rexp(Form("%s",what),wildcard);

		TIter next(&fListOfConditions);
		while ( (obj = next()) ) { // iterator skips empty slots
			tmp = obj->GetName();
			if ( tmp.Contains(rexp) ) {
				TCut *cut = PeakToCut((Gw::Peak1D*)obj); 
				if ( cut ) {
					tmp = cut->GetName(); // make sure the cut start with CONDNAME_
					if ( !tmp.BeginsWith(GetName()) ) {
						cut->SetName(Form("%s_%s",GetName(),cut->GetName()));
					}
					
					directory->WriteTObject(cut,"",option.Data());
					result++;
				}
			}
		}		
	}
	// other format
	/*
	if ( tmp == "GSI" ) {
		
	}
	 */
	
	return result;
}


ClassImp(GCond2D);

GCond2D::GCond2D() :
	GCondition()
{

}

GCond2D::GCond2D(const char * name, const char * title) :
	GCondition(name,title)
{
	
}

GCond2D::GCond2D(const GCond2D &p) :
    GCondition(p),
    fVarX(p.fVarX),
    fVarY(p.fVarY)
{
    ;
}

GCond2D::~GCond2D()
{
	
}

TCutG *GCond2D::AddWindow(Double_t xmin, Double_t ymin, Double_t xmax, Double_t ymax, Option_t *opt)
{
	TString option(opt); 
	
	Double_t xxmin = TMath::Min(xmin,xmax), xxmax = TMath::Max(xmin,xmax), yymin = TMath::Min(ymin,ymax), yymax = TMath::Max(ymin,ymax), xx[5], yy[5];
	
	xx[0] = xxmin; yy[0] = yymin;
	xx[1] = xxmax; yy[1] = yymin;
	xx[2] = xxmax; yy[2] = yymax;
	xx[3] = xxmin; yy[3] = yymax;
	xx[4] = xxmin; yy[4] = yymin;
	
	//
	if ( !option.Contains("+") ) 
		fListOfConditions.Clear();

	TCutG *p = 
		new TCutG(Form("%s_%d",GetName(),fListOfConditions.GetEntries()),5,xx,yy);
	p->SetTitle(GetTitle());
	p->SetVarX(fVarX.Data());
	p->SetVarY(fVarY.Data());
	
	if ( !fEnable )
		fEnable = true;
	
	if ( fExpectedResult ) {
		p->SetLineColor(29);
		p->SetLineWidth(2);
		p->SetLineStyle(1);		
	}
	else {
		p->SetLineColor(29);
		p->SetLineWidth(2);
		p->SetLineStyle(9);	
	}
	
	p->SetBit(kCanDelete,false);
	p->SetBit(kMustCleanup,false);
	
	gROOT->GetListOfSpecials()->Add(p);
	
	// add to condition
	fListOfConditions.Add(p); fHasPassed_.push_back(false);
	
	return p;
}

void GCond2D::Draw(Option_t* /*option*/)
{
	TVirtualPad *c_pad = TVirtualPad::Pad();
	
	if ( !c_pad ) 
		return;
	
	Style_t hash_style = 1; // 3004 for coinc, 3005 for anti
	if ( fExpectedResult == false ) 
		hash_style = 9;
	
	// it could be fListOfConditions.Draw. However some old version of root (5.24) uses option to select particular object of the collection to be drawn (based on expression)
	// so appaen current pas by myself
	TObject *obj; TIter next(&fListOfConditions);
	while ( (obj = next()) ) {
		if ( c_pad->GetListOfPrimitives()->FindObject(obj) == 0x0 ) {
			
			dynamic_cast<TAttLine *>(obj)->SetLineColor(29);
			dynamic_cast<TAttLine *>(obj)->SetLineWidth(2);
			dynamic_cast<TAttLine *>(obj)->SetLineStyle(hash_style);
			
			c_pad->GetListOfPrimitives()->Add(obj,"LP");
			c_pad->Modified();
		}
	}
	if ( fListOfConditions.GetEntries() ) {
		c_pad->Update();
	}
}


Bool_t GCond2D::IsPassed(Double_t x, Double_t y) const
{
	if ( !fEnable || fListOfConditions.GetEntries() == 0 ) {
		fHasPassed = true; return fHasPassed;
	}
	
	Bool_t is_inside = false; 
	
	for (Int_t i = 0; i < fListOfConditions.GetEntries(); i++) {
		TCutG *cut = (TCutG *)fListOfConditions[i];
		if ( cut->IsInside(x,y) ) {
			is_inside = true;
			fHasPassed_[i] = fExpectedResult;
		}
		else fHasPassed_[i] = !fExpectedResult;
	}
	
	return (fHasPassed = (is_inside == fExpectedResult) );
}

Short_t GCond2D::Import(const char *what, const char *from, Option_t *opt)
{
	TString wh(what), origin(from), option(opt), tmp, obj_name;TDirectory *dir_to_search = gDirectory; Short_t result = 0;
    //
    Bool_t wildcard = false;
    if ( wh.Contains("*") )
        wildcard = true;
    TRegexp rexp(Form("%s",what),wildcard);
    
	// start with empty container if option + is not there
	if ( !option.Contains("+") )
		fListOfConditions.Clear();

	// Treat root file/dir cases
	if ( !origin.Contains(":") ) { // means local root
		if ( origin == "gdir" || origin == "gpad" ) {
			;
		}
		else origin.Prepend("file:");
	}
	if ( origin.BeginsWith("file:")  || 
		 origin.BeginsWith("root:")  ||  
		 origin.BeginsWith("roots:") ||  
		 origin.BeginsWith("rootk:") ||  
		 origin.BeginsWith("http:") ) 
	{
		TString rname = origin; 
		if ( origin.BeginsWith("file:") )
			rname.Replace(0,origin.First(":")+1,""); // only if local, remove file: protocol
		
		// TODO
		// if ( "" ) ==> open window 
		
		
		// look for similar already open if not open a new one
		TFile *f = 0x0;
		if ( !origin.EndsWith(".root") ) {
			rname.Append(".root"); origin.Append(".root");
		}
		
		if ( gROOT->GetListOfFiles()->FindObject(rname.Data()) ) {// already there 
			std::cout << "Import CUTS from OPEN FILE " << rname.Data() << std::endl ;
			f = (TFile *)gROOT->GetListOfFiles()->FindObject(rname.Data());
		}
		else {
			std::cout << "Import CUTS from NEW FILE " << origin.Data() << std::endl;
			f = TFile::Open(origin.Data(),"READ");
		}
		if ( f ) {
			dir_to_search = f; origin = "gdir"; // even if not current dir ! 
		}
		else origin = "FAIL";
	}
	////	
	
	if ( origin == "gdir" ) { // root file/directory
		
		// useful to browse the TDirectory
		TString class_name; TObject *obj_in_file = 0x0; TKey *key_in_file; TList new_cuts; TCutG *cut; 
		
		// iterate over the key and keep only TCutG and in case a TCutG has been saved several time, only the highest cycle
		TIter it_keys(dir_to_search->GetListOfKeys());
		while ( (key_in_file = static_cast<TKey *>(it_keys())) ) { // next key		
			class_name  = key_in_file->GetClassName();
			obj_name    = key_in_file->GetName();
			// obj on disk is TCut and with a name not already in the list of Cond
			if ( class_name == "TCutG" && obj_name.Contains(rexp) && fListOfConditions.FindObject(obj_name.Data()) == 0x0 ) { 
				if ( !new_cuts.FindObject(obj_name.Data()) ) { // list of key is ordered so that first for a given name correspond to the highest cycle
					obj_in_file = 
					key_in_file->ReadObj();
					new_cuts.Add(obj_in_file);
				}
			}
		}
		TIter it_cuts(&new_cuts);
		while ( (cut = static_cast<TCutG *>(it_cuts())) ) { // next cut	
			if ( cut ) {
				
				obj_name = cut->GetName();
				if ( !obj_name.BeginsWith(GetName()) ) {
					cut->SetName(Form("%s_%s",GetName(),obj_name.Data()));
				}
				if ( fListOfConditions.FindObject(cut->GetName()) ) { // already a peak with same name in the list
					cut->SetName( Form("%s_%d",GetName(), fListOfConditions.GetEntries()) );
				}
								
				if ( !fEnable )
					fEnable = true;
				
				cut->SetBit(kCanDelete,false);
				cut->SetBit(kMustCleanup,false);
				
				gROOT->GetListOfSpecials()->Remove(cut); // to avoid keeping it in this folder and delete it
				
				// add to condition
				std::cout << "Add TCutG " << cut << std::endl;
				fListOfConditions.Add(cut); fHasPassed_.push_back(false);
				
				result++;
			}
			else {
				std::cout << " Cannot convert " << cut->GetName() << " to peak " << std::endl; 
			}
		}			
	}	
	
	if ( origin == "gpad" ) { // root current pad
		TSeqCollection *l_to_seach = 0x0; 
		if ( TVirtualPad::Pad() ) {
			l_to_seach = TVirtualPad::Pad()->GetListOfPrimitives();
		}
		TObject* obj = 0x0;
		TIter next(l_to_seach);
		while ( (obj = next()) ) { // iterator skips empty slots
			if ( obj->InheritsFrom("TCutG") ) {
				tmp = obj->GetName();
				if ( tmp.Contains(rexp) ) {
					
					if ( fListOfConditions.FindObject(obj) ) { // already in the list
						printf("%s ALREADY IN THE LIST \n", obj->GetName());
						continue;
					}

					// change some attributes
					if ( option.Contains("r") )
						((TCutG*)obj)->SetNameTitle( Form("%s_%d",GetName(),fListOfConditions.GetEntries()), GetTitle() );
					else 
						((TCutG*)obj)->SetNameTitle( Form("%s_%s",GetName(), obj->GetName()), GetTitle() );
					
					if ( !fEnable )
						fEnable = true;
					
					obj->SetBit(kCanDelete,false);
					obj->SetBit(kMustCleanup,false);
					
					gROOT->GetListOfSpecials()->Remove(obj);  // to avoid keeping it in this folder and delete it
					
					// add to condition
					fListOfConditions.Add(obj); fHasPassed_.push_back(false);
					result++;
				}
			}
		}
	}	
	
	
	if ( fListOfConditions.GetEntries() > 0 ) {
		printf("List of %d 2D Conditions \n",fListOfConditions.GetEntries());
		fListOfConditions.ls();
	}
	
	return result;
	
	/*
	 if ( origin == "gdir" ) { // root file/directory
	 l_to_seach = gROOT->GetListOfSpecials();
	 }
	 if ( origin == "gpad" ) { // root current pad
	 if ( TVirtualPad::Pad() ) {
	 l_to_seach = TVirtualPad::Pad()->GetListOfPrimitives();
	 }
	 }	
	 if ( l_to_seach ) {
	 TObject* obj = 0x0;
	 TIter next(l_to_seach);
	 while ( (obj = next()) ) { // iterator skips empty slots
	 if ( obj->InheritsFrom("TCutG") ) {
	 tmp = obj->GetName();
	 if ( tmp.Contains(rexp) ) {
	 
	 if ( fListOfConditions.FindObject(obj) ) { // already in the list
	 printf("%s ALREADY IN THE LIST \n", obj->GetName());
	 continue;
	 }
	 
	 // change some attributes
	 if ( option.Contains("r") )
	 ((TCutG*)obj)->SetNameTitle( Form("%s_%d",GetName(),fListOfConditions.GetEntries()), GetTitle() );
	 else 
	 ((TCutG*)obj)->SetNameTitle( Form("%s_%s",GetName(), obj->GetName()), GetTitle() );
	 
	 if ( !fEnable )
	 fEnable = true;
	 
	 ((TCutG*)obj)->SetVarX(fVarX.Data());
	 ((TCutG*)obj)->SetVarY(fVarY.Data());
	 
	 gROOT->GetListOfSpecials()->Add(obj);
	 
	 // add to condition
	 fListOfConditions.Add(obj); fHasPassed_.push_back(false);
	 result++;
	 }
	 }
	 }
	 }
	 */
	
}

Short_t GCond2D::Export(const char *what, const char *to, Option_t *opt) const
{
	TString wh(what), destination(to), option(opt), tmp("ROOT"), rname;
    TDirectory *directory = gDirectory;
    Short_t result = 0; TObject* obj = 0x0;
    //
    Bool_t wildcard = false;
    if ( wh.Contains("*") )
        wildcard = true;
    TRegexp rexp(Form("%s",what),wildcard);
    
	if ( !destination.Contains(":") ) { // means local root
		if ( destination != "gdir" ) {
			destination.Prepend("file:");
		}
	}
	if ( destination.BeginsWith("gdir") ) {
		// add path to subdirs of gdir ... something like gdir/sub helps 
	}
	if ( destination.BeginsWith("file:")  || 
		 destination.BeginsWith("root:")  ||  
		 destination.BeginsWith("roots:") ||  
		 destination.BeginsWith("rootk:") ||  
		 destination.BeginsWith("http:") ) 
	{
		TString rname = destination; 
		if ( destination.BeginsWith("file:") )
			rname.Replace(0,destination.First(":")+1,""); // only if local, remove file: protocol
		
		// TODO
		// if ( "" ) ==> open window 
		
		
		// look for similar already open if not open a new one
		TFile *f = 0x0;
		if ( !destination.EndsWith(".root") ) {
			rname.Append(".root"); destination.Append(".root");
		}
		
		if ( gROOT->GetListOfFiles()->FindObject(rname.Data()) ) {// already there 
			std::cout << "SAVE CUTS in OPEN FILE " << rname.Data() << std::endl ;
			f = (TFile *)gROOT->GetListOfFiles()->FindObject(rname.Data());
		}
		else {
			std::cout << "SAVE CUTS in NEW FILE " << destination.Data() << std::endl;
			f = TFile::Open(destination.Data(),"UPDATE");
		}
		if ( f ) {
			directory = f;
		}
		else tmp = "FAIL";
	}
	
	if ( tmp == "ROOT" ) { 
		
		TIter next(&fListOfConditions);
		while ( (obj = next()) ) { // iterator skips empty slots
			tmp = obj->GetName();
			if ( tmp.Contains(rexp) ) {
				
				tmp = obj->GetName(); // make sure the cut start with CONDNAME_
				if ( !tmp.BeginsWith(GetName()) ) {
					((TCutG*)obj)->SetName(Form("%s_%s",GetName(),obj->GetName()));
				}
				
				directory->WriteTObject(obj,"",option.Data());
				result++;			
			}
		}		
	}
	
	return result;
}

void GCond2D::WindowGCreator(const char * /*c_name*/)
{
	gROOT->SetEditorMode("CutG");
}









