


#ifndef GW_GSPlayer
#define GW_GSPlayer

#ifndef GW_BaseSpectrumPlayer
#include "BaseSpectrumPlayer.h"
#endif

class TPolyMarker;
class TSpline3;

namespace Gw {

/** Most of the methods relies on graphical approach
 
 Below an example of fitting in a 152Eu source spectrum:
 \code 
  TCanvas* c1 = new TCanvas("c1","titre",10,10,700,500);
  c1->Draw();
  Gw::HistoDB db;
  db.Open("toto", "rad:$PWD");
  TH1F* h = new TH1F();
  h->SetName("Eu152.spe");
  db >> *h;
  h->SetAxisRange(2100, 2400);
  h->Draw(); // draw the spectrum of 152Eu in the given range.
 
  GSPlayer* s = new GSPlayer();
  // place the peaks using 'a' key 
  // or use FindPeaks() method: press 's' whenever the line pointed a peak you want to add
  s->FitAll(); // fits the peaks with gaussian shape with a linear background
 \endcode
 
 The FitAll(..) method fits all the positionned peaks in the spectrum. Peaks separated less than 2*fwhm 
 (default wise fwhm = 6) are fitting with the CombinedFit(...) method.
 
 \code 
 // You can also defined background by pressing key 'b' to set each point of the background spline
 s->Background() // fit the background point with a TSpline
 s->DoBackground // fit the background point with a TSpline & removed background from current histogramm
 \endcode

*/
class GSPlayer : public BaseSpectrumPlayer 
{
public:
	GSPlayer();
	virtual ~GSPlayer();

	//! Find peak in current histo graphically  
	virtual Int_t FindPeaks(TH1 *histo, Option_t* opt = "");

	//! Find peak in current histo graphically 
	virtual Int_t FindPeaks(Option_t* opt = "") 
	  { return BaseSpectrumPlayer::FindPeaks(opt); }

	//! Compute the background for that histogram
	virtual TH1 *Background(const TH1 *histo, Option_t* /*opt = ""*/);

	//! Compute the background for the histogram in the current pad
	virtual TH1 *Background(Option_t* opt = "") 
		  { return BaseSpectrumPlayer::Background(opt); }

	//! To change the parameters for that algorithm
	virtual Bool_t SetParameter(const char* name, Double_t value);
	//! To change the parameters for that algorithm
	virtual Bool_t SetParameter(const char* name, Int_t value)
		{ return BaseSpectrumPlayer::SetParameter(name,value); }	
	virtual Bool_t SetParameter(const char* name, const TObject *value)
		{ return BaseSpectrumPlayer::SetParameter(name,value); }	

	ClassDef(GSPlayer, 0);  // some standard/graphical approach
};

} // end namespace
#endif
