/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file NuclearLevel.cpp compiled in libGWPHYSICS.so */

#ifndef Gw_NuclearLevel
#include "NuclearLevel.h"
#endif

#include <iostream>
#include <iomanip>

using namespace std;
using namespace Gw;

ClassImp(NuclearLevel);

//__________________________________________________________
NuclearLevel::NuclearLevel(): Level() 
{
}

//__________________________________________________________
NuclearLevel::~NuclearLevel() 
{

}

//__________________________________________________________
const char*  NuclearLevel::GetLabel0() 
{ 
  const char* p = fParity.IsParity(Parity::kPlus) ? "+" : "-"; 
  return Form("%s%s",fSpin.GetString(), p);
}

//__________________________________________________________
void NuclearLevel::SetLabels(const char *l0, const char *l1, const char *l2, const char *l3) 
{  
  TString spin(GetLabel0());
  
  if ( spin.Contains("+") ) {
    fParity.Set(Parity::kPlus);
    spin.Remove(spin.First("+"), 1);
  } else if ( spin.Contains("-") ) {
    fParity.Set(Parity::kMinus);
    spin.Remove(spin.First("-"), 1);
  } else
    fParity.Set(l1);
  
  fSpin.Set(l0);
  fT.Set(l2); 
  fEnergy.Set(l3);
  
  Int_t pos;
  TString tau(l2);
  pos = tau.Last(' ');
  tau[pos] = '\0';
  
  TString en(l3);
  pos = en.Last(' ');
  en[pos] = '\0';
  
  Level::SetLabels(l0, l1, tau.Data(), en.Data()); 
}



//__________________________________________________________
void NuclearLevel::ls(Option_t* o) const
{

  TString opt = o; string sep1, sep2;
  
  if ( opt.Contains("header") ) std::cout<< " Level energy (keV), \t spin (hbar), \t parity, \t half-life (s)" << std::endl;

  sep1 = sep2 = "";
  if ( fEnergy.IsData(InfoData::kTentative) ) { sep1 = '('; sep2 = ')'; }
  if ( fEnergy.IsData(InfoData::kTheo) )      { sep1 = '['; sep2 = ']'; }    
  if ( fEnergy.IsData(InfoData::kAbout) )     { sep1 = '~'; sep2 = ""; }    
  
  if ( fEnergy.GetValue() == 0 ) 
  	cout << sep1 << "GROUND STATE" << sep2 << "\t\t" << fSpin << "\t" << fParity ;  	
  else 
  	cout << sep1 << fEnergy.GetValue() << " <" << fEnergy.GetError() << ">" << sep2 << "\t\t" 
  	    << fSpin << "\t" << fParity ;
	    
  if ( fT.GetValue() != 0 ) {
  	sep1 = sep2 = "";
  	if ( fT.IsData(InfoData::kTentative) ) { sep1 = '('; sep2 = ')'; }
  	if ( fT.IsData(InfoData::kTheo) )      { sep1 = '['; sep2 = ']'; }    
  	if ( fT.IsData(InfoData::kAbout) )     { sep1 = '~'; sep2 = ""; }    
  	cout << "\t" << sep1 << fT.GetValue() << " <" << fT.GetError() << ">" << sep2 << endl;
  }
  else cout << endl;
      
  if ( gDebug > 0 ) TLine::ls(o);
}

//void NuclearLevel::Set(Double_t Energy, Double_t errEnergy){

	//printf("Energy = %f ; error = %f \n",Energy,errEnergy);
//	fE = Energy; fErrE = errEnergy;
//}

/*
void NuclearLevel::Print() const {
	printf("Energy = %f \n",fE);
	printf("Error = %f \n",fdE);
}

void NuclearLevel::SetEnergyBebert(Double_t E, Double_t dE) {
	fE = E;
	fdE = dE;
}

Double_t NuclearLevel::GetEnergyBebert() {
	return fE;
}
*/


