/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file Parity.cpp compiled in libGWPHYSICS.so */

#ifndef ROOT_TString
#include <TString.h>
#endif

#ifndef Gw_Parity
#include "Parity.h"
#endif

using namespace Gw;

ClassImp(Parity);

Parity::Parity()
 : QNumber(), InfoData()
{
   fParity = kPlus;
}


Parity::~Parity()
{
}

void Parity::Set(const char *s)
{
    	TString st = s;
	switch( WhatIs(s) ){
		case kKnown:
			if ( st.Contains('-') ) 
				fParity = kMinus;
			else 
				fParity = kPlus;
			SetInfo(kKnown);
			break;
		case kUnknown:
			SetInfo(kUnknown);
			break;	
		case kAbout:
			if ( st.Contains('-') ) 
				fParity = kMinus;
			else 
				fParity = kPlus;
			SetInfo(kAbout);	
			break;	
		case kTentative:
			if ( st.Contains('-') ) 
				fParity = kMinus;
			else 
				fParity = kPlus;	
			SetInfo(kTentative);	
			break;
		case kTheo:
			if ( st.Contains('-') ) 
				fParity = kMinus;
			else 
				fParity = kPlus;
			SetInfo(kTheo);	
			break;
		default:
			SetInfo(kUnknown);
			break;
	}
}

Parity& Parity::operator*= (Parity& p)
{
  fParity *= p.fParity;
  return *this;
}

std::ostream & operator << (std::ostream &out, const Parity &p)
{
    string sep1, sep2;
    
    if ( p.IsData(InfoData::kUnknown) ) {
    	out << "?" ; return out;
    }
    sep1 = sep2 = "";
    if ( p.IsData(InfoData::kTentative) ) 
		{ sep1 = '('; sep2 = ')'; }
    if ( p.IsData(InfoData::kTheo) )      
		{ sep1 = '['; sep2 = ']'; }    
    if ( p.IsData(InfoData::kAbout) )     
		{ sep1 = '~'; sep2 = ""; }  
         
    if ( p.IsParity(Parity::kPlus) ) { 
		out << sep1 << "+" << sep2; 
		return out; 
	} 
    else { 
		out  << sep1 << "-" << sep2; 
		return out; 
	}     
}


