/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file Parity.h header file for the Parity quantum number  */  

#ifndef Gw_Parity
#define Gw_Parity
#define GW_PARITY_H Gw_Parity

#ifndef GW_QNUMBER_H
#include "QNumber.h"
#endif

#ifndef GW_INFODATA_H
#include <InfoData.h>
#endif

// std includes
#include <iostream>
using namespace std;

namespace Gw {

/**
This class discribes a specific quantum number: the parity 

@author Olivier Stezowski
*/
class Parity : public QNumber, public InfoData
{
public:
    enum EParity { kMinus = -1, kPlus = 1 } ;
    
private:
    Short_t fParity;	// parity value 
    
public:
    Parity();
    Parity(const Parity &p) : QNumber(p), InfoData(p) { fParity = p.fParity; }
    explicit Parity(Parity::EParity p) { Set(p); }
    virtual ~Parity();
    
    //! to identify this quantum number
    virtual bool IsQNumber(QNumber::EQtype) const;
    
    void Set(Parity::EParity);
    bool IsParity(Parity::EParity) const;
    
    //! set the value, can be overloaded
    virtual void Set(const char *s) ;
  
    //! overload operator*
    Parity& operator*= (Parity& p);
  
    //! rootcint dictionary
    ClassDef(Parity,1); // the parity quantum number
};
// inline members
inline bool Parity::IsQNumber(QNumber::EQtype t) const { return kParity == t; }
inline void Parity::Set(Parity::EParity p) { fParity = p; }
inline bool Parity::IsParity(Parity::EParity p) const { return p == fParity; }
}

//! to write a parity in a stream
std::ostream & operator << (std::ostream &, const Gw::Parity &);
#endif
