/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file QNumber.h header file for a general quantum number (QNumber) */  

#ifndef GW_QNUMBER_H
#define GW_QNUMBER_H

#ifndef ROOT_Rtypes
#include "Rtypes.h"
#endif

namespace Gw {

/**
Base class for a quantum number.
A quantum number is a specific Data that inherits from InfoData since a quantum number could be known, unknown ... etc. 

To each quantum level is associated a set of quantum numbers. It is important to be able to know which 
quantum number is a good quantum number. This base class defines the basic types and some usefull members

\todo - A way to get a correspondance between the enum values and some equivalent strings

@author Olivier Stezowski - 07/10/2004
*/
class QNumber 
{
public:
    //! label type for any possible quantum number 
    enum EQtype{ kParity = 0, kSpin = 1, kK = 2, kSignature = 3, kSimplex = 4};
    
private:
    void SetQNumber() 
    	{ ; }
    void SetQNumber(QNumber::EQtype) 
    	{ ; }
    
public:
    QNumber();
    QNumber(const QNumber &/*q*/) 
    	{ ;}    
    virtual ~QNumber();
    
    virtual bool IsQNumber(QNumber::EQtype) const = 0;
    
    ClassDef(QNumber,0); // Base Class for a Quantum number
};
// inline members
}
#endif
