

#ifndef Gw_RootSpectrumPlayer
#define Gw_RootSpectrumPlayer

#ifndef Gw_BaseSpectrumPlayer
#include "BaseSpectrumPlayer.h"
#endif

class TSpectrum;
class TSpectrumFit;

namespace Gw {

/** RootSpectrumPlayer class to handle root spectra using the interface of BasePeak
 
 The spectrum has to be displayed before instanced the class.
 Use 'p' key to put peaks onto the histo. The position of the mouse gives
 the position of the peak.
 
 A method FindPeaks() based on TSpectrum Search method is also available.
 
 The background method is also based ond TSpectrum class.

*/
  
class RootSpectrumPlayer : public BaseSpectrumPlayer 
{
public:
  RootSpectrumPlayer();
  virtual ~RootSpectrumPlayer();

  //! Find peaks     
  virtual Int_t FindPeaks(TH1 *histo, Option_t* opt = "");
  
  //! Find peaks from the histo in the current pad and store them in the current collection
  virtual Int_t FindPeaks(Option_t* opt = "") { return BaseSpectrumPlayer::FindPeaks(opt); }

  //! Compute the background for that histogram
  virtual TH1 *Background(const TH1 *histo, Option_t* opt = "");
	  
  //! Compute the background for the histogram in the current pad
  virtual TH1 *Background(Option_t* opt = "") { return BaseSpectrumPlayer::Background(opt); }
  
  //! To change the parameters for that algorithm
  virtual Bool_t SetParameter(const char* name, Double_t value);
  virtual Bool_t SetParameter(const char* name, Int_t value); 
  virtual Bool_t SetParameter(const char* name, const TObject *value) { return BaseSpectrumPlayer::SetParameter(name, value); }

  //! print current peak list
  virtual void Print(Option_t* opt = "") const;  
  
private: 
  TParameter<Int_t>     fMaxPeak;    // max peak allowed
  TParameter<Double_t>  fSigma;      // sigma for search
  TParameter<Double_t>  fThreshold;  // threshold for search
  TParameter<Int_t>     fNbIter;     // nb of iteration for background
    
  TSpectrum    *fEngine1D;           // TSpectrum pointer

public: 
  ClassDef(RootSpectrumPlayer, 0); ///< Adaptator to TSpectrum like facilities
};

} // end namespace
#endif
