/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file Spin.h header file for a spin quantum number */  

#ifndef GW_SPIN_H
#define GW_Spin
#define GW_SPIN_H Gw_Spin

// GW includes
#ifndef GW_QNUMBER_H
#include "QNumber.h"
#endif

#ifndef GW_INFODATA_H
#include "InfoData.h"
#endif

// std includes
#include <iostream>

namespace Gw {

/**
A spin is defined by two integers: a numerator and a denominator

@author Olivier Stezowski
*/
class Spin : public QNumber, public InfoData
{
protected:
    Int_t fN; // numerator
    Int_t fD; // denominator - should be 1 or 2 for a spin
    
public:
    Spin();
    Spin(const Spin &spin): QNumber(spin), InfoData(spin) { fN = spin.fN; fD = spin.fD; }    
    virtual ~Spin();
    
    //! To set the spin
    void Set(Int_t n, Int_t d = 1);
    //! To get the spin as a float
    Float_t Get() const;
    //! To get the spin as a string
    const Char_t* GetString() const;
    //! To get the spin as a float
    Bool_t IsHalfInt() const { return fD == 2; }
		
    //! To get the spin (numerator)
    Int_t GetN() const	{ return fN; }
    //! To get the spin (denominator)
    Int_t GetD() const	{ return fD; }
			
    //! to set some information about this data   
    virtual void Set(const char *s) ;    
        
    //! QNumber members
    virtual Bool_t IsQNumber(QNumber::EQtype) const;
    
    //! operator +=
    Spin& operator+=(Int_t value);
    
    //! operator +=
    Spin& operator+=(Spin sp);

    //! rootcint dictionary
    ClassDef(Spin,1); // a Spin 
};
// inline members
inline Bool_t  Spin::IsQNumber(QNumber::EQtype t) const { return kSpin == t; }
inline Float_t Spin::Get() const { return Float_t(fN) / fD; }
}

//! to write a Spin in a stream
std::ostream & operator << (std::ostream &, const Gw::Spin &);

#endif
