/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   Copyright (C) 2010 by Christian Finck                                 * 
 *   stezow(AT)ipnl.in2p3.fr                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/** \file SymCorrelatedSpace.h header file for all type of SymCorrelatedSpace */  

#ifndef Gw_SymCorrelatedSpace
#define Gw_SymCorrelatedSpace

#ifndef Gw_CorrelatedSpace
#include "CorrelatedSpace.h"
#endif

class TBrowser;

namespace Gw {
	
class CSFilter;

/** SymCorrelatedSpace base class that contains correlated informations 
 
*/
class SymCorrelatedSpace : public CorrelatedSpace
{
   
protected:
  
  virtual Int_t GetAxisNumber(Option_t* name) const;
  
public:
  SymCorrelatedSpace();
  SymCorrelatedSpace(const char* name, const char* title);
  virtual ~SymCorrelatedSpace();

  //! fill correlated space
  virtual void Fill(const Double_t* /*Xn*/, const Double_t /*weight*/ = 1) = 0;
  
  //! project
  virtual void Project(CSFilter* /*filter*/, Option_t* /*axis*/) = 0;
  
  //! Get Bin
  virtual Int_t GetNbin(Int_t /*axis*/) const  = 0;
  
  //! Get cell content
  virtual Double_t GetCellContent(Int_t /*iCell*/)  = 0;
  
  //! Set cell content
  virtual void SetCellContent(Int_t /*iCell*/, Double_t /*weight*/)  = 0;
  
  //! Browser method
  virtual void Browse(TBrowser* b) = 0;

  //! project
  virtual void Project(Option_t* axis, Option_t* gateName = "Gate0");
  
  //! Set dimension
  void SetDimension(Int_t dimension);
  
  //! Get dimension
  Int_t GetDimension() const { return fDimension; }
  
  ClassDef(SymCorrelatedSpace,1)  // Correlated Space
};
  
} // end namespace

#endif
