/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file XGammaLink.cpp compiled in libGWPHYSICS.so */

#ifndef Gw_XGammaLink 
#include "XGammaLink.h"
#endif

using namespace Gw;

ClassImp(XGammaLink);

XGammaLink::XGammaLink()
 : GammaLink()
{
	fIsX = kFALSE; fX.SetOwner(kTRUE);
}

XGammaLink::~XGammaLink()
{
	fX.Clear();
}

void XGammaLink::SetX(Int_t nb, Float_t *energy, Float_t *weight)
{
	Int_t i; GammaLink *xray;
	
	if ( nb <= 0 ) return;
	if ( fConv.GetValue() <= 0 ) return;	
	
	// reset the previous definition
	fX.Clear(); fRandX.Clear("");
	
	// set new definition
	// slot 0 for this with an intensity = i_link / (1 + conv)
	fRandX.Add(this,fStrength.GetValue()/(1+fConv.GetValue()));
	
	//  compute sum weight for X-rays. 
	Float_t sumw, xw;  
	sumw = 0;
	for (i = 0; i < nb; i++ ) sumw += weight[i];	  
	
	for (i = 0; i < nb; i++ ) {
	
		// new x-rays with the same 
		xray = new GammaLink(); (*xray) = (*this);
		
		// to set each x-ray properties - to be completed !
		xw = fConv.GetValue()*fStrength.GetValue()/(1+fConv.GetValue());
		xw = xw * weight[i] / sumw;
		xray->GetStrength().Set(xw); xray->GetEnergy().Set(energy[i]);
				
		// add a new x-ray to the list of x-rays and the generator
		fX.Add(xray); fRandX.Add(xray,xw);
	} // i
	
	// now everything is all right to determine X-rays
	DoX(kTRUE);
}

/*
void XGammaLink::ls(Option_t* option) const 
{
	
//	  std::cout << "Gamma " << fEnergy.Get() << " " << fSpin << " " << fParity << std::endl;
}     
*/
    
void XGammaLink::DoCascade(TSeqCollection &cas, Option_t */*opt*/) 
{ 
	Link *link;
	
	if ( fX.GetSize() == 0 ) { cas.Add(this); return; } 

	if ( fIsX == kTRUE ) {
		link = (Link *)fRandX.Rand(); 
		if ( link == this ) cas.Add(this);	
		else link->DoCascade(cas);	
	}
	else { cas.Add(this) ; }
	/*	
	if ( fX.GetSize() == 0 ) cas.Add(this) ;
	else {
		link = (Link *)fRandX.Rand(); 
		if ( link == this ) cas.Add ( this) ;
		else link->DoCascade(cas);
	}
	printf("XGammaLink \n");
	GammaLink *l = (GammaLink *)fRandX.Rand();
	printf(" %f \n",l->GetEnergy().GetValue());
		printf("XGammaLink \n");	*/
		
//	if ( fX.GetSize() == 0 ) cas.Add(this) ;  
//	else cas.Add(fRandX.Rand());
	
}    


