#ifndef GW_ASCCONVERTER_H
#define GW_ASCCONVERTER_H

#ifndef GW_HISTODB_H
#include "HistoDB.h"
#endif

//#ifndef ROOT_TArrayI
//#include "TArrayI.h"
//#endif

/**
	Base class to read/write ascii (e.g.: asc for TNT board) spectra.
        
        The format of the file is given via the SetOption method in HistoDB.
        The parsing is as follow  h:x c:x s:a  p:x.y.z where 
        h:x corresponds to x lines of comment in the header
        c:x, s:a  corresponds to x columns separated by the char a
        p:x.y.z corresponds to the x parameter number, y is the position of the bin and z the position of the bin content
                in the parameter list. Value zero stands for empty.
        Default option set to "h:0 c:1 s: p:1.0.1" no header, one column for bin content.
       
\author Christian Finck & Olivier Stezowski
*/

using namespace Gw;

class ASCConverter: public HistoConverter 
{ 
private:
   std::ifstream fStreamIN;	  // stream used to read/write in a file
    
   Bool_t DecodeFormat(TArrayI& params);

public:	
    ASCConverter();
    ASCConverter(const char *name);
    virtual ~ASCConverter();
    
    //! Get histogram from number Id
    virtual TH1 *Get(int); 
    
    //! Get histogram from name
    virtual TH1 *Get(const char *); 
    
    //! Read/write 1-D histogram
    virtual bool Read(TH1 &);
    virtual bool Write(const TH1 &);
    
    //! Read/write 2-D histogram
    virtual bool Read(TH2 &);
    virtual bool Write(const TH2 &);
    
    //! Read/write from Stack histogram
    virtual bool Read(THStack &);
    virtual bool Write(const THStack &);
    
    //! instanciate new HistoConverter
    virtual HistoConverter *NewDB(const char *) const;
    
    // virtual void ls(Option_t *);
    virtual void ls(Option_t *) const; 
    
    //! get type
    virtual const char *GetType() const { return fType.Data(); }
    
    //! method to normalize string chains
    static TString Normalize(const char* line);
};

#endif
