


#ifndef Gw_AgataEventDisplay
#define Gw_AgataEventDisplay


#include <map>

#ifndef Gw_BaseEventDisplay
#include "BaseEventDisplay.h"
#endif

#ifndef ROOT_TArrayF
#include "TArrayF.h"
#endif

#ifndef ROOT_TString
#include "TString.h"
#endif

#ifndef ROOT_TGFrame
#include "TGFrame.h"
#endif

class TEveDigitSet;
class TGTextView;
class TGTextEntry;
class TGCheckButton;
class TList;
class TGeoVolume;

namespace Gw {
class AgataEventContainer;
/** AgataEventDisplay a class to work on a specific event display
 
*/
class AgataEventDisplay : public BaseEventDisplay
{
private:
  
   static AgataEventDisplay* fgInstance; // static instance of class
  
   static Bool_t fgAgataGeoDone;      // geometry done flag for Agata
   static Bool_t fgDanteGeoDone;      // geometry done flag for Dante
   static Bool_t fgPrismaGeoDone;     // geometry done flag for Prisma
   static Bool_t fgPassiveGeoDone;    // geometry done flag for target, beam pipe & floor
   static Bool_t fgFloorGeoDone;      // geometry done special flag for floor
   
   static const Char_t* fgDefaultAgataPath;  // Default path for Agata geometry files
   static const Char_t* fgDefaultDanteConf;  // Default configuration file for Dante geometry
   static const Char_t* fgDefaultPrismaConf; // Default configuration file for Prisma geometry
   
public:
  virtual ~AgataEventDisplay();
  
  static AgataEventDisplay* Instance();  
  
  void FirstEvent(); //*MENU*
  void NextEvent();  //*MENU*
  void PrevEvent();  //*MENU*
    
  void ShowDisplay();
    
  AgataEventContainer* GetEventContainer() const { return fAgataContainer; }

  // ! build default geometry dante or/and prima and Agata and Cie
  void BuildDefaultGeometry(Bool_t danteGeo = true, Bool_t prismaGeo = true, Bool_t agataGeo = true, Bool_t passiveGeo = true);

  //! Set width/height of box
  void SetBoxDefWidth(Float_t w)  { fBoxDefWidth = w; }
  void SetBoxDefHeight(Float_t h) { fBoxDefHeight = h; }
  
  //! Set width/height of quad
  void SetQuadDefWidth(Float_t w)  { fQuadDefWidth = w; }
  void SetQuadDefHeight(Float_t h) { fQuadDefHeight = h; }
  
  //! Get width/height of box
  Float_t GetBoxDefWidth()  { return fBoxDefWidth; }
  Float_t GetBoxDefHeight() { return fBoxDefHeight; }
  
  //! Get width/height of quad
  Float_t GetQuadDefWidth()  { return fQuadDefWidth; }
  Float_t GetQuadDefHeight() { return fQuadDefHeight; }
  
  //! Set track style (rec/cone)
  void SetTrackStyle(const Char_t* s, Option_t* type = "Agata");
  
  //! Update hits info
  void UpdateHitInfo(TEveDigitSet* qs, Int_t idx);
  
  //! Update tracks info
  void UpdateTrackInfo(TEveDigitSet* qs, Int_t idx);
  
  //! Clear info view
  void ClearInfoView();
  
  //! Disable GUI
  void DisableGUI() { fGUIFlag = false; }
  
  //!Reset
  void Reset();
  
  //! Register branch
  const Char_t* RegisterBranch(Option_t* type = "");
  
public:
   static void EnableDrawFloor()  { fgFloorGeoDone = true; }
   static void DisableDrawFloor() { fgFloorGeoDone = false; }
   
   static void SetDefaultAgataPath(const Char_t* name)  { fgDefaultAgataPath = name;  }
   static void SetDefaultDanteConf(const Char_t* name)  { fgDefaultDanteConf = name;  }
   static void SetDefaultPrismaConf(const Char_t* name) { fgDefaultPrismaConf = name; }
   
   static const Char_t* GetDefaultAgataPath()  { return fgDefaultAgataPath;  }
   static const Char_t* GetDefaultDanteConf()  { return fgDefaultDanteConf;  }
   static const Char_t* GetDefaultPrismaConf() { return fgDefaultPrismaConf; }
   
protected:
   
  //! Update elements to be displayed
  void     UpdateElements();
  
  //! Make GUI 
  void     MakeGUI();
  
private:
  
  AgataEventDisplay();
  
private:
  
  AgataEventContainer* fAgataContainer; // container of hits
    
  TList*   fQuadHitList;           // list of quad to display hits
  TList*   fLineTrackList;         // list of line to display tracks
    
  Float_t fBoxDefWidth;            // default width of track box
  Float_t fBoxDefHeight;           // default height of track box
  
  Float_t fQuadDefWidth;           // default width of hit quad
  Float_t fQuadDefHeight;          // default height of hit quad
    
  TGTextView*  fInfoView;          // text view for hit/track info
  TGTextEntry* fEventEntry;        // text entry for event number
  TGCheckButton* fRefreshButton;   // refresh display for each event otherwise superimpose events
  Bool_t fGUIFlag;                 // flag to disable or enable gui interface
  
  std::map<const Char_t*, const Char_t*> fMapStyle;// map style

  ClassDef(AgataEventDisplay, 0); // Base class for facilities to play with level scheme  
};

} // end namespace

#endif
