/***************************************************************************
 *   Copyright (C) 2004-2006 by Olivier Stezowski & Christian Finck        *
 *   stezow(AT)ipnl.in2p3.fr, cfinck(AT)ires.in2p3.fr                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   FreAgataHitDisplaye Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/** \file AgataHitDisplay.cpp compiled in libGWTOOLS.so */

#ifndef Gw_AgataHitDisplay
#include "AgataHitDisplay.h"
#endif


using namespace Gw;

//
ClassImp(AgataHitDisplay)

//__________________________________________________________
AgataHitDisplay::AgataHitDisplay(const Text_t* name) 
  : TEveQuadSet(name),
    fPalette(new TEveRGBAPalette()),
    fMaxEnergy(-1),
    fQuadWidth(0.4),
    fQuadHeight(0.4),
    fSelectedIdx(-1),
    fSelectedValue(-1),
    fLog("AgataHitDisplay")
{ 
  // default constructor
  SetOwnIds(kTRUE);
}

//__________________________________________________________
AgataHitDisplay::~AgataHitDisplay()
{
  // default destructor
  delete fPalette;
}

//__________________________________________________________
void AgataHitDisplay::DigitSelected(Int_t idx) 
{ 
  
  TEveDigitSet::DigitBase_t* selectedDigit = GetDigit(idx);

  if (selectedDigit)
    fSelectedValue = selectedDigit->fValue;
  
  fSelectedIdx = idx;
  
  SecSelected(this, idx);
}

//__________________________________________________________
void AgataHitDisplay::SetMaxEnergy(Float_t e) 
{ 
  fPalette->SetMax(Int_t(e+0.5));
  fMaxEnergy = Int_t(e+0.5); 
  SetPalette(fPalette);
} 

//__________________________________________________________
void AgataHitDisplay::AddHit(Float_t e, Float_t x, Float_t y, Float_t z)
{
  AddQuad(x-GetDefWidth()/2., y-GetDefHeight()/2., z);
  DigitValue(Int_t(e+0.5));
}

//__________________________________________________________
void AgataHitDisplay::ResetHits()
{
  Reset(TEveQuadSet::kQT_RectangleXY, kFALSE, 32);
}
