/***************************************************************************
 *   Copyright (C) 2004-2006 by Olivier Stezowski & Christian Finck        *
 *   stezow(AT)ipnl.in2p3.fr, cfinck(AT)ires.in2p3.fr                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   FreAgataTrackDisplayEditore Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/** \file AgataTrackDisplayEditor.cpp compiled in libGWTOOLS.so */

#ifndef Gw_AgataTrackDisplayEditor
#include "AgataTrackDisplayEditor.h"
#endif

#ifndef Gw_AgataTrackDisplay
#include "AgataTrackDisplay.h"
#endif

#ifndef ROOT_TG3DLine
#include "TG3DLine.h"
#endif

#ifndef ROOT_TGLabel
#include "TGLabel.h"
#endif

#ifndef ROOT_TGButton
#include "TGButton.h"
#endif

#ifndef ROOT_TGWindow
#include "TGWindow.h"
#endif

#ifndef ROOT_TH1F
#include "TH1F.h"
#endif

#ifndef ROOT_TPad
#include "TPad.h"
#endif

#ifndef ROOT_TStyle
#include "TStyle.h"
#endif

using namespace Gw;

//
ClassImp(AgataTrackDisplayEditor)

//__________________________________________________________
AgataTrackDisplayEditor::AgataTrackDisplayEditor(const TGWindow *p, Int_t width, Int_t height, UInt_t options, Pixel_t back) 
: TGedFrame(p, width, height, options| kVerticalFrame, back),
  fM(0x0),
  fLog("AgataTrackDisplayEditor")
{ 
  // default constructor
 
  TGVerticalFrame* infoFrame = CreateEditorTabSubFrame("Info");
  
  TGCompositeFrame *title = new TGCompositeFrame(infoFrame, 180, 10, kHorizontalFrame | kLHintsExpandX |
                                                                     kFixedWidth      | kOwnBackground);
  
  title->AddFrame(new TGLabel(title, "AgataTrackDisplay Info"),
                   new TGLayoutHints(kLHintsLeft, 1, 1, 0, 0));
  title->AddFrame(new TGHorizontal3DLine(title),
                   new TGLayoutHints(kLHintsExpandX, 5, 5, 7, 7));
  infoFrame->AddFrame(title, new TGLayoutHints(kLHintsTop, 0, 0, 2, 0));
  
  TGHorizontalFrame* histoButtFrame = new TGHorizontalFrame(infoFrame);
  TGTextButton* b = 0x0;
  b = new TGTextButton(histoButtFrame, "Histo");
  b->SetToolTipText("Show histogram of track total energy");
  histoButtFrame->AddFrame(b, new TGLayoutHints(kLHintsLeft|kLHintsExpandX, 1, 1, 0, 0));
  b->Connect("Clicked()", "Gw::AgataTrackDisplayEditor", this, "DoHisto()");
  
  b = new TGTextButton(histoButtFrame, "Range Histo");
  b->SetToolTipText("Show histogram of track total energy, given range");
  histoButtFrame->AddFrame(b, new TGLayoutHints(kLHintsLeft|kLHintsExpandX, 1, 1, 0, 0));
  b->Connect("Clicked()", "Gw::AgataTrackDisplayEditor", this, "DoRangeHisto()");
  
  infoFrame->AddFrame(histoButtFrame, new TGLayoutHints(kLHintsExpandX, 1, 1, 2, 5));
}

//__________________________________________________________
AgataTrackDisplayEditor::~AgataTrackDisplayEditor()
{
  // default destructor
}

//__________________________________________________________
void AgataTrackDisplayEditor::SetModel(TObject* obj)
{
  fM = dynamic_cast<Gw::AgataTrackDisplay*>(obj); 
}

//__________________________________________________________
void AgataTrackDisplayEditor::DoHisto()
{
  Int_t min, max;
  if (fM->GetPalette()) {
    min = fM->GetPalette()->GetLowLimit();
    max = fM->GetPalette()->GetHighLimit();
  } else {
    fM->ScanMinMaxValues(min, max);
  }
 
  PlotHisto(min, max);
}
  
//__________________________________________________________
void AgataTrackDisplayEditor::DoRangeHisto()
{
  Int_t min, max;
  if (fM->GetPalette()) {
    min = fM->GetPalette()->GetMinVal();
    max = fM->GetPalette()->GetMaxVal();
  } else {
    fM->ScanMinMaxValues(min, max);
  }
  
  PlotHisto(min, max);
}

//__________________________________________________________
void AgataTrackDisplayEditor::PlotHisto(Int_t min, Int_t max)
{
  Int_t nbins = max - min + 1;

  TH1F* h = new TH1F(Form("%s%s",fM->GetName(), " Energy"), fM->GetTitle(), nbins, min-0.5, max+0.5);
  h->SetDirectory(0);
  h->SetBit(kCanDelete);
  
  Int_t energy;
  fM->FirstEnergy(); // first create iterator !
  while ( (energy = fM->NextEnergy()) != -1) {
    h->Fill(energy);
  }
  
  gStyle->SetOptStat(1111111);
  h->Draw();
  gPad->Modified();
  gPad->Update();
}
