/***************************************************************************
 *   Copyright (C) 2004-2006 by Olivier Stezowski & Christian Finck        *
 *   stezow(AT)ipnl.in2p3.fr, cfinck(AT)ires.in2p3.fr                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/** \file BaseEventDisplay.cpp compiled in libGWTOOLS.so */


#ifndef Gw_BaseEventDisplay
#include "BaseEventDisplay.h"
#endif

#ifndef ROOT_TGeoManager
#include "TGeoManager.h"
#endif

#ifndef ROOT_TGeoVolume
#include "TGeoVolume.h"
#endif

#ifndef ROOT_TMath
#include "TMath.h"
#endif

using namespace Gw;

//
ClassImp(BaseEventDisplay)

Bool_t BaseEventDisplay::fgIsGeoLoaded    = false;
Bool_t BaseEventDisplay::fgIsDisplayed    = false;

//__________________________________________________________
BaseEventDisplay::BaseEventDisplay() 
  : TEveEventManager(),
    fCurrentEventId(0),
    fFirstEventDone(false),
    fMaxEnergy(4000),
    fLog("BaseEventDisplay")
{ 
  // default constructor

}

//__________________________________________________________
BaseEventDisplay::~BaseEventDisplay()
{
  // default destructor
}

//__________________________________________________________
void BaseEventDisplay::LoadGeometry(const Char_t* fileName)
{  
  if (fgIsGeoLoaded) 
	  return;
  
  if ( !TGeoManager::Import(fileName) ) {
	fLog.SetProcessMethod("LoadGeometry(const Char_t* )");
	fLog << error << "Failed to load geometry" << nline; 
  } 
  fgIsGeoLoaded = true;
}

//__________________________________________________________
void BaseEventDisplay::AddGeometry(TGeoVolume* volume, TGeoMatrix* transf)
{
   fLog.SetProcessMethod("AddGeometry(TGeoVolume*,  const Char_t* )");
   
   static Int_t nb = 0;
   TObjArray* list =  fTopVolume->GetNodes();
   if (list) {
	  for (Int_t i = 0; i < list->GetEntries(); ++i) {
		 TGeoNode* vol = (TGeoNode*)list->At(i);
		 TString volName(vol->GetName());
		 if (volName.Contains(volume->GetName())) {
			fLog << warning << Form("Geometry of %s already loaded",volume->GetName()) << dolog;
			return;
		 }
	  }
   }
   
   fTopVolume->AddNode(volume, nb++, transf);
   fLog << dolog; 
}

//__________________________________________________________
void BaseEventDisplay::SetTransparency(Char_t  transparency)
{
  // refresh must be done by user
  if (!gGeoManager) 
	  return;
  TObjArray* list =  gGeoManager->GetListOfVolumes();
  if (!list) return;
  for (Int_t i = 0; i < list->GetEntries(); ++i) {
	TGeoVolume* vol = (TGeoVolume*)list->At(i);
	vol->SetTransparency(transparency);
  }
}