


#ifndef Gw_BaseEventDisplay
#define Gw_BaseEventDisplay

#include "GwLogMessage.h"

#ifndef ROOT_TEveEventManager
#include "TEveEventManager.h"
#endif

#ifndef ROOT_TGeoMatrix
#include "TGeoMatrix.h"
#endif

class TGeoVolume;

namespace Gw {

class BaseEventContainer;

/** BaseEventDisplay base class to work on event display
 
*/
class BaseEventDisplay : public TEveEventManager
{
protected:
  //! default constructor
  BaseEventDisplay();
  
protected:
   static Bool_t fgIsGeoLoaded;       // flag if geometry loaded
   static Bool_t fgIsDisplayed;       // flag if a display already opened  
  
public:
  virtual ~BaseEventDisplay();
  
  virtual void FirstEvent() { return; } //*MENU*
  virtual void NextEvent()  { return; } //*MENU*
  virtual void PrevEvent()  { return; } //*MENU*
  
  virtual void SetTransparency(Char_t transparency = 50); //*MENU*
   
  virtual void ShowDisplay()  { return; }
  
  virtual void LoadGeometry(const Char_t* fileName);
  
  virtual void AddGeometry(TGeoVolume* volume, TGeoMatrix* matrix = gGeoIdentity);
     
  Int_t GetCurrentEventId() const { return fCurrentEventId; }
  
  void    SetMaxEnergy(Float_t e) { fMaxEnergy = e; } 
  Float_t GetMaxEnergy()          { return fMaxEnergy; } 
   
   
protected:

  Int_t fCurrentEventId;    // Current event id
  
  Bool_t fFirstEventDone;   // flag for first event processed
  
  Float_t fMaxEnergy;       // maximum energy fo palette (in keV)
  
  TGeoVolume* fTopVolume;   // top volume of geometry
   
  Bool_t fDrawFloor;        // flag for drawing floor

  mutable LogMessage fLog;  //! log message

  
  ClassDef(BaseEventDisplay, 1); // Base class for facilities to play with level scheme  
};

} // end namespace

#endif
