/***************************************************************************
 *   Copyright (C) 2005-2006 by Christophe Theisen & Olivier Stezowski     *
 *   christophe.theisen(AT)cea.fr, stezow(AT)ipnl.in2p3.fr                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file H1Calibrator.h header file for the calibration facility */
 
#ifndef GW_CALIBCO60_H
#define GW_CALIBCO60_H

#ifndef GW_BASECALIB_H
#include "BaseCalib.h"
#endif

// root's includes
#ifndef ROOT_TH1_H
#include <TH1.h>
#endif
#ifndef ROOT_TAxis_H
#include <TAxis.h>
#endif
#ifndef ROOT_TF1_H
#include <TF1.h>
#endif
#ifndef ROOT_TSpectrum_H
#include <TSpectrum.h>
#endif


namespace Gw {

/**
CalibCo60 is a service class in order to find calibration coefficients for a TH1 espacially with Co60 source
In constructors, the raw histogram must be given and will not be changed

\author Bertrand Rossé
*/
class CalibCo60 : public BaseCalib
{
public:
	CalibCo60(); //constructor
	virtual ~CalibCo60(); // destructor
  
	//! Perform the calibration "by hand"
	void Calibrate(const TH1 *histo, TF1 *function, Double_t channel1, Double_t channel2, 
				   Option_t* AxisOption = "x"); 
  
	//! Perform the calibration "auto"
	void Calibrate(const TH1 *histo, TF1 *function, Option_t* AxisOption="x", Double_t sigma = 2, 
				   Option_t* SearOption = "", Double_t threshold = 0.05); 
	
protected:

		
//! rootcint dictionary
	ClassDef(CalibCo60,0); // a CalibCo60
};

// inline members

}

#endif



