/***************************************************************************
 *   Copyright (C) 2004-2006 by Olivier Stezowski & Christophe Theisen     *
 *   stezow(AT)ipnl.in2p3.fr, christophe.theisen(AT)cea.fr                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/** \file EGConverter.cpp compiled in libGWTOOLS.so */

#ifndef GW_EGCONVERTER_H
#include "EGConverter.h"
#endif
 
#ifndef ROOT_TRegexp
#include <TRegexp.h>
#endif

#ifndef ROOT_TSystem
#include <TSystem.h>
#endif

#ifndef ROOT_TSystemDirectory
#include <TSystemDirectory.h>
#endif

#if ROOT_VERSION_CODE >= ROOT_VERSION(5,15,0)
#include "TList.h"
#endif

#include <iostream>
using namespace std;

namespace { EGConverter gEGConverter; } // prototype ONLY use to create a new DB system

//_________________________________________________________________
EGConverter::EGConverter(const char *name) : HistoConverter(name)
{ 
  // default type for gpsi  
  fType = "eg";
  CheckDirectory(name);
  fStreamIN.clear(); fCycle = '_'; // because ; is not supported by all file systems 
  fLog.GetProcessName() = "EGConverter";
} 

//_________________________________________________________________
EGConverter::EGConverter() : HistoConverter()
{ 
   // default type for gpsi  
   fType = "eg";
   fFileExt = ".eg";
   fOptions = "";
   // add this interface to the DB service 
   AddPrototype(GetType(),this);
} 

//_________________________________________________________________
EGConverter::~EGConverter()
{ 
	CloseEG();
} 

//_________________________________________________________________
HistoConverter *EGConverter::NewDB(const char *name) const 
{ 
  return new EGConverter(name);
}

//_________________________________________________________________
void EGConverter::PrintHeader() const
{
	cout << "magic number  : " << fHeader.magic_number << endl;
	cout << "spectrum name : " << fHeader.spectrum_name << endl;
	cout << "creation date : " << fHeader.creation_date << endl;
	cout << "modifi. date  : " << fHeader.modification_date << endl;
	cout << "fold          : " << fHeader.fold << endl;
	cout << "size          : " << fHeader.count_free_space << endl;

	unsigned long i;
	for(i=0; i < fHeader.fold; i++){
		printf("dim %d : init = %4d range = %4d \n",
			(int)i,(int)fHeader.base_info[i],(int)fHeader.range[i]);
	} 

	printf("layout = %1d type = %1d pointer %d \n",
			(int)fHeader.data_array_descriptor1[0],
			(int)fHeader.data_array_descriptor1[1],
			(int)fHeader.data_array_descriptor1[4]);

        /*
	printf("FIP : ");
	for(i=0; i<32; i++)
		printf("%2d : %d \n",i,fHeader.fip[i]);

	printf("Anotation pointer : \n");
	for(i=0; i<8; i++)
		printf("%d : %d \n",i,fHeader.anotation_pointer[i]);

	printf("Calibration pointer : \n");
	for(i=0; i<8; i++)
		printf("%d : %d \n",i,fHeader.calibration_pointer[i]);

	printf("Efficiency pointer : \n");
	for(i=0; i<8; i++)
		printf("%d : %d \n",i,fHeader.efficiency_pointer[i]);

	printf("Data array desciptor 1: \n");
	for(i=0; i<5 ; i++)
		printf("%1d : %d \n",i,fHeader.data_array_descriptor1[i]);
	printf("Data array desciptor 2: \n");
	for(i=0; i<5 ; i++)
		printf("%1d : %d \n",i,fHeader.data_array_descriptor2[i]);



	printf("base_address_string_space = %d \n",
		fHeader.base_address_string_space);
	printf("string_free_space = %d \n",
		fHeader.string_free_space);
	printf("top_string_space = %d \n",
		fHeader.top_string_space);
	printf("base_address_count_space = %d \n",
		fHeader.base_address_count_space);
	printf("count_free_space = %d \n",
		fHeader.count_free_space);
	printf("top_count_space = %d \n",
		fHeader.top_count_space);
	

	printf("Unused : \n");
	for(i=0; i<20; i++)
		printf("%2d : %d \n",i,fHeader.unused[i]);
	*/	
}

//_________________________________________________________________
void  EGConverter::InitHeader(int dimx, int dimy, HeaderEG &header, const char *name, int type)
{
  time_t cl;
  char str[27];   
  int i;

	header.magic_number = 0x189c5e39;
	header.magic_number = ULong((char*)&header.magic_number);
	header.version = 1;
	header.version = ULong((char*)&header.version);
	strncpy(header.spectrum_name,name,31);
	header.spectrum_name[31] = 0;

	if (dimy == 1)
		header.fold = 1;
	else
		header.fold = 2;

	header.fold = ULong((char*)&header.fold);

        time(&cl);
        strcpy(str, ctime(&cl));
        if (str[8] == ' ')
                header.creation_date[0] = header.modification_date[0] = '0';
        else
                header.creation_date[0] = header.modification_date[0] = str[8];
        header.creation_date[1] = header.modification_date[1] = str[9];
        header.creation_date[2] = header.modification_date[2] = '-';
        for (i = 3; i < 6; ++i)
                header.creation_date[i] = header.modification_date[i] = str[i+1];
        header.creation_date[6] = header.modification_date[6] = '-';
        for (i = 7; i < 11; ++i)
                header.creation_date[i] = header.modification_date[i] = str[i+13];
        header.creation_date[11] = header.modification_date[11] = ' ';
        for (i = 12; i < 20; ++i)
                header.creation_date[i] = header.modification_date[i] = str[i-1];

	for(i=0; i<8; i++){
		header.base_info[i] = 0xFFFFFFFF;
		header.base_info[i] = ULong((char*)&header.base_info[i]);
		header.range[i] = 0xFFFFFFFF;
		header.range[i] = ULong((char*)&header.range[i]);
	}

	switch(dimy){
		case 1:
			header.base_info[0] = 0;
			header.base_info[0] = ULong((char*)&header.base_info[0]);
			header.range[0] = dimx;
			header.range[0] = ULong((char*)&header.range[0]);
			break;
		default:
			header.base_info[0] = 0;
			header.base_info[0] = ULong((char*)&header.base_info[0]);
			header.range[0] = dimx;
			header.range[0] = ULong((char*)&header.range[0]);
			header.base_info[1] = 0;
			header.base_info[1] = ULong((char*)&header.base_info[1]);
			header.range[1] = dimy;
			header.range[1] = ULong((char*)&header.range[1]);
	}

	for(i=0; i<32; i++)
		header.fip[i] = 0xFFFFFFFF;

	header.fip[0] = 0;
	for(i=0; i<32; i++)
		header.fip[i] = ULong((char*)&header.fip[i]);
	
	for(i=0; i<8; i++){
		header.anotation_pointer[i] = -1;
		header.anotation_pointer[i] = Long((char*)&header.anotation_pointer[i]);
		header.calibration_pointer[i] = -1;
		header.calibration_pointer[i] = Long((char*)&header.calibration_pointer[i]);
		header.efficiency_pointer[i] = -1;
		header.efficiency_pointer[i] = Long((char*)&header.efficiency_pointer[i]);
	 }
	
	header.data_array_descriptor1[0] = 0;
	header.data_array_descriptor1[0] = ULong((char*)&header.data_array_descriptor1[0]);
	header.data_array_descriptor1[1] = type;
	header.data_array_descriptor1[1] = ULong((char*)&header.data_array_descriptor1[1]);
	header.data_array_descriptor2[1] = type;
	header.data_array_descriptor2[1] = ULong((char*)&header.data_array_descriptor2[1]);
	header.data_array_descriptor2[4] = 0xFFFFFFFF;
	header.data_array_descriptor2[4] = ULong((char*)&header.data_array_descriptor2[5]);

	switch(type){
		case 0: // unsigned char
		case 1: // char
			header.count_free_space = dimx*dimy;
			header.top_count_space = dimx*dimy -1;
			header.base_address_string_space = dimx*dimy + 512;
			break;
		case 2: // unsigned short
		case 3: // short
			header.count_free_space = dimx*dimy*2;
			header.top_count_space = dimx*dimy*2 -1;
			header.base_address_string_space = dimx*dimy*2 + 512;
			break;
		case 4: // unsigned int
		case 5: // int
		case 6: // float
			header.count_free_space = dimx*dimy*4;
			header.top_count_space = dimx*dimy*4 -1;
			header.base_address_string_space = dimx*dimy*4 + 512;
			break;
	}

	header.count_free_space = ULong((char*)&header.count_free_space);
	header.top_count_space = ULong((char*)&header.top_count_space);
	header.base_address_string_space = Long((char*)&header.base_address_string_space);

	header.base_address_count_space = 512;
	header.base_address_count_space = ULong((char*)&header.base_address_count_space);
	header.top_string_space = 1023;
	header.top_string_space = ULong((char*)&header.top_string_space);
	header.string_free_space = 256;
	header.string_free_space = Long((char*)&header.string_free_space);

}

//_________________________________________________________________
void  EGConverter::ReadHeader(char *buffer, HeaderEG &header)
{

	int offset, i;
	
	offset = 0;	
	header.magic_number = ULong(buffer+offset);
	
	offset = 4;	
	header.version	= ULong(buffer+offset);
	
	offset = 8;
	for ( i = 0; i < 32; i++ )
	  header.spectrum_name[i] = buffer[offset+i];
	  
	offset = 40;
	header.fold = ULong(buffer+offset);
	
	offset = 44;	
	for ( i = 0; i < 20; i++ )	
	  header.creation_date[i] = buffer[offset+i];
	  
	offset = 64;		  
	for ( i = 0; i < 20; i++ )	
	  header.modification_date[i] = buffer[offset+i];	

	offset = 84;	  
	for ( i = 0; i < 8; i++ )
	  header.base_info[i] = ULong(buffer+offset+sizeof(unsigned long)*i);
	  
	offset = 116;	 
	for ( i = 0; i < 8; i++ )	 
	  header.range[i] = ULong(buffer+offset+sizeof(unsigned long)*i);

	offset = 148;	
	for ( i = 0; i < 32; i++ )	  
	  header.fip[32]  = ULong(buffer+offset+sizeof(unsigned long)*i);
	  
	offset = 276;	
	for ( i = 0; i < 8; i++ )
	  header.anotation_pointer[i] = Int(buffer+offset+sizeof(int)*i);
	
	offset = 308;
	for ( i = 0; i < 8; i++ )
	  header.calibration_pointer[i] = Int(buffer+offset+sizeof(int)*i);
	  
	offset = 340;	
	for ( i = 0; i < 8; i++ )
	  header.efficiency_pointer[i] = Int(buffer+offset+sizeof(int)*i);
	
	offset = 372;
	for ( i = 0; i < 5; i++ )  
	  header.data_array_descriptor1[i] = ULong(buffer+offset+sizeof(unsigned long)*i);

	offset = 392;
	for ( i = 0; i < 5; i++ )
	  header.data_array_descriptor2[5] = ULong(buffer+offset+sizeof(unsigned long)*i);

	offset = 412;	
	header.base_address_string_space = Int(buffer+offset);

	offset = 416;	
	header.string_free_space = Int(buffer+offset);

	offset = 420;	
	header.top_string_space = ULong(buffer+offset);

	offset = 424;	
	header.base_address_count_space = ULong(buffer+offset);

	offset = 428;	
	header.count_free_space = ULong(buffer+offset);

	offset = 432;	
	header.top_count_space = ULong(buffer+offset);

	offset = 436;
	for ( i = 0; i < 20; i++ )  
	  header.unused[i] = ULong(buffer+offset+i*sizeof(unsigned long));

   
}

//_________________________________________________________________
bool  EGConverter::OpenEG(const char *basename)
{	 
	TString filename = fName.Data();  filename += basename;
	
        if (!filename.EndsWith(fFileExt)) filename += fFileExt;

	CloseEG(); // to be sure it is not already open
	
	// open the file
	fStreamIN.open(filename.Data()); fStreamIN.clear();
	if ( ! fStreamIN.is_open() ){ 
          fLog.GetProcessMethod() = "OpenEG(const char* )";
		SetError(HistoConverter::kFail,
		         Form("Cannot open file %s",filename.Data()));	
		return false; 
	}
	
	// to read the header part
	char buffer[512]; 
	fStreamIN.read(buffer,512);
	if ( ! fStreamIN.good()  ) { 
          fLog.GetProcessMethod() = "OpenEG(const char* )";
		SetError(HistoConverter::kFail,
		         Form("Cannot read EG header in %s",filename.Data()));	
		fStreamIN.close(); return false; 
	}
	
	// check out the magic number
	unsigned long magic = ULong(buffer);
	if ( magic != 0x189c5e39 && magic != 0x395e9c18 ) { 
		SetError(HistoConverter::kFail,
		         Form("[EGConverter::OpenEG] %s is not an eg file",filename.Data()));	
		fStreamIN.close(); return false; 
	}
	if ( magic == 0x395e9c18 ) { fIsSwapping = !fIsSwapping; } 
	
	// now decode the header part
	ReadHeader(buffer,fHeader); return true;
}

//_________________________________________________________________
bool EGConverter::IsEG(const char *basename, Option_t *o) const
{
	TString filename = fName.Data(), option = o;  filename += basename;
	
	if (!filename.EndsWith(fFileExt)) 
		filename += fFileExt;
        
	ifstream file;
	// open the file
	file.open(filename.Data()); 
	if ( ! file.is_open() ){ 
          if ( gDebug > 0 ) {	
            fLog.SetProcessMethod("IsEG(const char*, Option_t* )");
            fLog << debug << "Cannot open file " << basename << dolog;	
          }
          return false; 
	}
	
	// to read the header part
	char buffer[512]; file.read(buffer,512);
	if ( !file.good() ) { 
          if ( gDebug > 0 ) {
            fLog.SetProcessMethod("IsEG(const char*, Option_t* )");
            fLog<< debug << "Cannot read EG header in " << basename << dolog;
          }
          file.close(); return false; 
	}
	
	// check out the magic number
	unsigned long *magic = (unsigned long*)buffer, *fold = (unsigned long*)(buffer+40);
	if ( magic[0] != 0x189c5e39 && magic[0] != 0x395e9c18 ) { 
          if ( gDebug > 0 ) {
            fLog.SetProcessMethod("IsEG(const char*, Option_t* )");
            fLog << debug << basename << " is not an eg file" << dolog;
          }
          file.close(); return false; 
	}
	bool ok = true;
	if ( magic[0] == 0x189c5e39 ) { // no byteswap
		if ( option.Contains("1D") && (*fold) != 1 ) ok = false;
		if ( option.Contains("2D") && (*fold) != 2 ) ok = false;	
	}
	else { // byteswap
		if ( option.Contains("1D") && (*fold) != 0x01000000 ) ok = false;
		if ( option.Contains("2D") && (*fold) != 0x02000000 ) ok = false;
	}	
        
	file.close(); return ok;
}

//_________________________________________________________________
void EGConverter::ls(Option_t *o) const
{
	HistoConverter::ls(o);
	TSystemDirectory dir; dir.SetDirectory(fName.Data()); TList *list = dir.GetListOfFiles();
		
	TIter iter(list); TObject *entry; int count = 0;
	while ( (entry = iter()) ) {

		if ( IsEG(entry->GetName()) ) { // this is a true eg file
			cout << " [" << count++ <<  "] " << entry->GetName() ;
			
			if ( (count+1) % 5 == 0 ) cout << endl;
			else cout << "\t";	
		}
			
	}
	cout << endl; if ( list ) delete list;
}

//_________________________________________________________________
TH1 *EGConverter::Get(int which)
{ 
	// list all the file to find the entry name correponding to which
	TSystemDirectory dir; dir.SetDirectory(fName.Data()); TList *list = dir.GetListOfFiles();
		
	TIter iter(list); TObject *entry; int count = 0; TString result = "_EMPTY_";
	while ( (entry = iter()) ) {
		if ( IsEG(entry->GetName()) ) {
			if ( which == count ) result = entry->GetName(); count++;
		}
	}
	if ( list ) delete list;

	if ( result != "_EMPTY_" ) return Get(result.Data()); // an entry has been found
	
        fLog.GetProcessMethod() = "Get(Int_t  )";
	SetError(HistoConverter::kFail,
		Form("Cannot find entry # %d",which));
			
	return NULL;
} 

//_________________________________________________________________
TH1 *EGConverter::Get(const char *name)
{ 
        if (fName.IsNull())
          fName = "./";
          
	if ( IsEG(name,"1D") ) {
		TH1F *h = new TH1F(); 
		if ( h ) { 
		  h->SetName(name);
		  if ( !Read((*h)) ) { delete h; h = NULL; } 
		}
		return h;
	}
	if ( IsEG(name,"2D") ) {
		TH2F *h = new TH2F();	
		if ( h ) { 		
		  h->SetName(name);
		  if ( !Read((*h)) ) { delete h; h = NULL; } 
		}
		return h;  
	}	
        
	return NULL;
} 

//_________________________________________________________________
bool EGConverter::Read(TH1 &h)
{ 
	if ( IsEG(h.GetName(),"1D") == false ) {
                fLog.GetProcessMethod() = "Read(TH1& )";
		SetError(HistoConverter::kFail,
			Form("%s is not an EG 1D file",h.GetName()));
		return false;
	}
	OpenEG(h.GetName()); // to open the file and load the header part
/*	if ( fHeader.fold != 1 ) {
		SetError(HistoConverter::kFail,
			Form("[EGConverter::Read] %s is not a 1D spectrum",h.GetName()));
		return false;		
	}*/	
	if ( gDebug > 0 ) PrintHeader();

	char *buffer = new char[fHeader.count_free_space];
	
	// read the array with data
	fStreamIN.seekg(fHeader.base_address_count_space,ios::beg); fStreamIN.read(buffer,fHeader.count_free_space);  
	if ( fStreamIN.fail() ) {
                fLog.GetProcessMethod() = "Read(TH1&  )";
		SetError(HistoConverter::kFail,
			Form("Cannot read array in the EG file %s",h.GetName()));
		CloseEG(); 
		delete buffer; 
		return false;		
	}

	// now decode the spectrum
	unsigned long i;
       fLog << error << nline;
       fLog.SetProcessMethod("Read(TH1& )");
	switch(fHeader.data_array_descriptor1[1]){ // switch depending on the array (histo) type
		case 0:
			fLog << nline;
			fLog << " Warning in [EGConverter::Read]: ROOT histogram type "  << h.ClassName()
			     << " does not match EG array type 8 bit unsigned integer " << nline;
			fLog << "  ---> there might be some problems during the convertion  " << nline;  
			     
			h.SetBins(fHeader.count_free_space,fHeader.base_info[0],fHeader.count_free_space); 
			for (i = 0; i < fHeader.count_free_space; i++)
				h.Fill(i,buffer[i]);
			break;
		case 1:
			if ( ! h.InheritsFrom("TH1C") ) {
			  fLog << nline;
			  fLog << " Warning in [EGConverter::Read]: ROOT histogram type "  << h.ClassName()
			       << " does not match EG array type 8 bit signed integer "   << nline;
			  fLog << "  ---> there might be some problems during the convertion  " << nline;  
			}     
			h.SetBins(fHeader.count_free_space,fHeader.base_info[0],fHeader.count_free_space); 
			for (i = 0; i < fHeader.count_free_space; i++)
				h.Fill(i,buffer[i]);
			break;
		case 2:
			fLog << nline;
			fLog << " Warning in [EGConverter::Read]: ROOT histogram type "  << h.ClassName()
			     << " does not match EG array type 16 bit unsigned integer "   << nline;
			fLog << "  ---> there might be some problems during the convertion  " << nline;  
			h.SetBins(fHeader.count_free_space/2,fHeader.base_info[0],fHeader.count_free_space/2); 
			for (i = 0; i < fHeader.count_free_space/2; i++)
				h.Fill(i,UShort(buffer+2*i));
			break;
		case 3:
			if ( ! h.InheritsFrom("TH1S") ) {
			  fLog << nline;
			  fLog << " Warning in [EGConverter::Read]: ROOT histogram type "  << h.ClassName()
			       << " does not match EG array type 16 bit signed integer "   << nline;
			  fLog << "  ---> there might be some problems during the convertion  " << nline;  
			}	
			h.SetBins(fHeader.count_free_space/2,fHeader.base_info[0],fHeader.count_free_space/2); 
			for (i = 0; i < fHeader.count_free_space/2; i++)
				h.Fill(i,Short(buffer+2*i));
			break;
		case 4:
			fLog << nline ;
			fLog << " Warning in [EGConverter::Read]: ROOT histogram type "  << h.ClassName()
			     << " does not match EG array type 32 bit unsigned integer "   << nline;
			fLog << "  ---> there might be some problems during the convertion  " << nline;  
			h.SetBins(fHeader.count_free_space/4,fHeader.base_info[0],fHeader.count_free_space/4); 
			for (i = 0; i < fHeader.count_free_space/4; i++)
				h.Fill(i,UInt(buffer+4*i));
			break;
		case 5:
			if ( ! h.InheritsFrom("TH1I") ) {
			  fLog << nline ;
			  fLog << " Warning in [EGConverter::Read]: ROOT histogram type "  << h.ClassName()
			       << " does not match EG array type 32 bit signed integer "   << nline;
			  fLog << "  ---> there might be some problems during the convertion  " << nline;  
			}
			h.SetBins(fHeader.count_free_space/4,fHeader.base_info[0],fHeader.count_free_space/4); 
			for (i = 0; i < fHeader.count_free_space/4; i++)
				h.Fill(i,Int(buffer+4*i));
			break;
		case 6:
			if ( ! h.InheritsFrom("TH1F") ) {
			  fLog << nline;
			  fLog << " Warning in [EGConverter::Read]: ROOT histogram type "  << h.ClassName()
			       << " does not match EG array type 32 bit floating point "   << nline ;
			  fLog << "  ---> there might be some problems during the convertion  " << nline;  
			}
			h.SetBins(fHeader.count_free_space/4,fHeader.base_info[0],fHeader.count_free_space/4); 
			for (i = 0; i < fHeader.count_free_space/4; i++)
				h.Fill(i,Float(buffer+4*i));
			break;			
		default:
			fLog << error << "Unknown type "
			     << fHeader.data_array_descriptor1[1] << " ! " << nline;
	} 
	delete buffer; 
	
	fLog << dolog; 
	return true;
} 

//_________________________________________________________________
bool EGConverter::Write(const TH1 &h)
{ 
  int type=0, i;
  TString hname;
  int file_id;
  char *specc;
  short *specs;
  int * speci;
  float *specf;
  int error;
  char string_end[1024];
  
#if defined(R__BYTESWAP)
	fIsSwapping = true;	// writing always as big endian
#endif

	// write to char EG spectrum : type 1
	if ( h.InheritsFrom("TH1C") ) // char
		type = 1;

	// write to short EG spectrum : type 3
	if ( h.InheritsFrom("TH1S") )
		type = 3; 

	// write to int EG spectrum : type 5
	if ( h.InheritsFrom("TH1I") )
		type = 5; 

	// write to float EG spectrum : type 6
	if ( h.InheritsFrom("TH1F") )
		type = 6; 

	if ( h.InheritsFrom("TH1D") ){
                fLog.GetProcessMethod() = "Write(const TH1& )";
		fLog << warning  << "ROOT histogram type "  << h.ClassName()
                     << " writen as float EG spectrum " << nline;
		type = 6;
	}

	hname = h.GetName(); hname.ReplaceAll(" ",""); 

	InitHeader(h.GetNbinsX(), 1, fHeader, hname, type);

	file_id = open(hname,O_WRONLY | O_CREAT, 0644);
	if (file_id < 0) {
                fLog.GetProcessMethod() = "Write(const TH1& )";
		SetError(HistoConverter::kFail,
			Form("Error writing %s file",h.GetName()));
		return false;
	}
	lseek(file_id, 0, SEEK_SET);
	if (write(file_id, (void *)&(fHeader), 512) != 512) {
                fLog.GetProcessMethod() = "Write(const TH1& )";
		SetError(HistoConverter::kFail,
			Form("Error writing %s file",h.GetName()));
		return false;
	}
		
        error = 0;
	Double_t tmp=0.0;
	switch(type){
		case 1: // char
			specc = new char[h.GetNbinsX()];
			for (i=0; i<h.GetNbinsX(); i++){
				tmp = h.GetBinContent(i+1);
				specc[i] = (char)tmp;
			}
			if (write(file_id, (void *) specc, h.GetNbinsX()) != h.GetNbinsX())
				error = 1;
			delete [] specc;
			break;
		case 3: // short
			specs = new short[h.GetNbinsX()];
			for (i=0; i<h.GetNbinsX(); i++){
				tmp = h.GetBinContent(i+1);
				specs[i] = Short((char*)&tmp);
			}
			if (write(file_id, (void *) specs, h.GetNbinsX()*2) != h.GetNbinsX()*2)
				error = 1;
			delete [] specs;
			break;
		case 5: // int
			speci = new int[h.GetNbinsX()];
			for (i=0; i<h.GetNbinsX(); i++){
				tmp = h.GetBinContent(i+1);
				speci[i] = Int((char*)&tmp);
			}
			if (write(file_id, (void *) speci, h.GetNbinsX()*4) != h.GetNbinsX()*4)
				error = 1;
			delete [] speci;
			break;
		case 6: // float
			specf = new float[h.GetNbinsX()];
			for (i=0; i<h.GetNbinsX(); i++){
				tmp = h.GetBinContent(i+1);
				specf[i] = Float((char*)&tmp);
			}
			if (write(file_id, (void *) specf, h.GetNbinsX()*4) != h.GetNbinsX()*4)
				error = 1;
			delete [] specf;
			break;
	}

	if (error) {
                fLog.GetProcessMethod() = "Write(const TH1& )";
		SetError(HistoConverter::kFail,
			Form("Error writing %s file",h.GetName()));
		return false;
	}

	for(i=0; i<1024; i++)
		string_end[i] = 0;

	strcpy(&string_end[4],"Gammaware Spectrum");
	string_end[3] = strlen("Gammaware Spectrum");

	if (write(file_id, (void *) string_end, 1024) != 1024) {
                fLog.GetProcessMethod() = "Write(const TH1& )";
		SetError(HistoConverter::kFail,
			Form("Error writing %s file",h.GetName()));
		return false;
	}

	close(file_id);
	return true;

} 

//_________________________________________________________________
bool EGConverter::Read(TH2 &h)
{ 
        fLog.GetProcessMethod() = "Read(TH2& )";

	if ( IsEG(h.GetName(),"2D") == false ) {
                fLog.GetProcessMethod() = "Read(TH2& )";
		SetError(HistoConverter::kFail,
			Form("%s is not an EG 2D file",h.GetName()));
		return false;
	}
	OpenEG(h.GetName()); // to open the file and load the header part
/*	if ( fHeader.fold != 2 ) {
		SetError(HistoConverter::kFail,
			Form("[EGConverter::Read] %s is not a 2D spectrum",h.GetName()));
		return false;		
	} */	
	if ( gDebug > 0 ) PrintHeader();

	char *buffer = new char[fHeader.count_free_space];
	
	// read the array with data
	fStreamIN.seekg(fHeader.base_address_count_space,ios::beg); fStreamIN.read(buffer,fHeader.count_free_space);  
	if ( fStreamIN.fail() ) {
		SetError(HistoConverter::kFail,
			Form("Cannot read array in the EG file %s",h.GetName()));
		CloseEG(); delete buffer; return false;		
	}

	// now decode the spectrum
	unsigned long i,j; Int_t cursor = 0;
        fLog << error;

	switch(fHeader.data_array_descriptor1[1]) { // switch depending on the array (histo) type
		case 0:
			fLog << nline;
			fLog << " Warning in [EGConverter::Read]: ROOT histogram type "  << h.ClassName()
			     << " does not match EG array type 8 bit unsigned integer " << nline;
			fLog << "  ---> there might be some problems during the convertion  " << nline;  
			     
			h.SetBins(fHeader.range[0],fHeader.base_info[0],fHeader.range[0],
				  fHeader.range[1],fHeader.base_info[1],fHeader.range[1]);

			if ( fHeader.data_array_descriptor1[0] == 0 ) { // symetric  
			  for (i = 0; i < fHeader.range[0]; i++)
			    for (j = 0; j < fHeader.range[1]; j++) 
			    		h.Fill(i,j,buffer[cursor++]);
			}
			else { // asymetric
			  for (i = 0; i < fHeader.range[0]; i++)
			    for (j = i; j < fHeader.range[1]; j++) {
			    	if ( i == j ) 
					h.Fill(i,j,buffer[cursor++]);
				else 
					h.Fill(i,j,buffer[cursor]); h.Fill(j,i,buffer[cursor++]);
			    } // j
			}
			break;
		case 1:
			if ( ! h.InheritsFrom("TH2C") ) {
			  fLog << nline;
			  fLog << " Warning in [EGConverter::Read]: ROOT histogram type "  << h.ClassName()
			       << " does not match EG array type 8 bit signed integer "   << nline ;
			  fLog << "  ---> there might be some problems during the convertion  " << nline ;  
			}     
			h.SetBins(fHeader.range[0],fHeader.base_info[0],fHeader.range[0],
				  fHeader.range[1],fHeader.base_info[1],fHeader.range[1]);

			if ( fHeader.data_array_descriptor1[0] == 0 ) { // symetric  
			  for (i = 0; i < fHeader.range[0]; i++)
			    for (j = 0; j < fHeader.range[1]; j++) 
			    		h.Fill(i,j,buffer[cursor++]);
			}
			else { // asymetric
			  for (i = 0; i < fHeader.range[0]; i++)
			    for (j = i; j < fHeader.range[1]; j++) {
			    	if ( i == j ) 
					h.Fill(i,j,buffer[cursor++]);
				else 
					h.Fill(i,j,buffer[cursor]); h.Fill(j,i,buffer[cursor++]);
			    } // j
			}
			break;
		case 2:
			fLog << nline ;
			fLog << " Warning in [EGConverter::Read]: ROOT histogram type "  << h.ClassName()
			     << " does not match EG array type 16 bit unsigned integer "   << nline ;
			fLog << "  ---> there might be some problems during the convertion  " << nline ;  
			
			h.SetBins(fHeader.range[0],fHeader.base_info[0],fHeader.range[0],
				  fHeader.range[1],fHeader.base_info[1],fHeader.range[1]);

			if ( fHeader.data_array_descriptor1[0] == 0 ) { // symetric  
			  for (i = 0; i < fHeader.range[0]; i++)
			    for (j = 0; j < fHeader.range[1]; j++) 
			    		{ h.Fill(i,j,UShort(buffer+cursor)); cursor += 2; }
			}
			else { // asymetric
			  for (i = 0; i < fHeader.range[0]; i++)
			    for (j = i; j < fHeader.range[1]; j++) {
			    	if ( i == j ) 
					h.Fill(i,j,UShort(buffer+cursor)); 
				else 
				{ 
					UShort_t v = UShort(buffer+cursor);
					h.Fill(i,j,v); h.Fill(j,i,v); 
				}
				cursor += 2; 
			    } // j
			}	
			break;
		case 3:
			if ( ! h.InheritsFrom("TH2S") ) {
			  fLog << nline ;
			  fLog << " Warning in [EGConverter::Read]: ROOT histogram type "  << h.ClassName()
			       << " does not match EG array type 16 bit signed integer "   << nline ;
			  fLog << "  ---> there might be some problems during the convertion  " << nline ;  
			}

			h.SetBins(fHeader.range[0],fHeader.base_info[0],fHeader.range[0],
				  fHeader.range[1],fHeader.base_info[1],fHeader.range[1]);

			if ( fHeader.data_array_descriptor1[0] == 0 ) { // symetric  
			  for (i = 0; i < fHeader.range[0]; i++)
			    for (j = 0; j < fHeader.range[1]; j++) 
			    		{ h.Fill(i,j,Short(buffer+cursor)); cursor += 2; }
			}
			else { // asymetric
			  for (i = 0; i < fHeader.range[0]; i++)
			    for (j = i; j < fHeader.range[1]; j++) {
			    	if ( i == j ) 
					h.Fill(i,j,Short(buffer+cursor)); 
				else 
				{ 
					Short_t v = Short(buffer+cursor);
					h.Fill(i,j,v); h.Fill(j,i,v); 
				}
				cursor += 2; 
			    } // j
			}		  
			break;
		case 4:
			fLog << nline ;
			fLog << " Warning in [EGConverter::Read]: ROOT histogram type "  << h.ClassName()
			     << " does not match EG array type 32 bit unsigned integer "   << nline ;
			fLog << "  ---> there might be some problems during the convertion  " << nline ;

			h.SetBins(fHeader.range[0],fHeader.base_info[0],fHeader.range[0],
				  fHeader.range[1],fHeader.base_info[1],fHeader.range[1]);

			if ( fHeader.data_array_descriptor1[0] == 0 ) { // symetric  
			  for (i = 0; i < fHeader.range[0]; i++)
			    for (j = 0; j < fHeader.range[1]; j++) 
			    		{ h.Fill(i,j,UInt(buffer+cursor)); cursor += 4; }
			}
			else { // asymetric
			  for (i = 0; i < fHeader.range[0]; i++)
			    for (j = i; j < fHeader.range[1]; j++) {
			    	if ( i == j ) 
					h.Fill(i,j,UInt(buffer+cursor)); 
				else 
				{ 
					UInt_t v = UInt(buffer+cursor);
					h.Fill(i,j,v); h.Fill(j,i,v); 
				}
				cursor += 4; 
			    } // j
			}	
			break;
		case 5:
			if ( ! h.InheritsFrom("TH2I") ) {
			  fLog << nline ;
			  fLog << " Warning in [EGConverter::Read]: ROOT histogram type "  << h.ClassName()
			       << " does not match EG array type 32 bit signed integer "   << nline;
			  fLog << "  ---> there might be some problems during the convertion  " << nline;  
			}

			h.SetBins(fHeader.range[0],fHeader.base_info[0],fHeader.range[0],
				  fHeader.range[1],fHeader.base_info[1],fHeader.range[1]);

			if ( fHeader.data_array_descriptor1[0] == 0 ) { // symetric  
			  for (i = 0; i < fHeader.range[0]; i++)
			    for (j = 0; j < fHeader.range[1]; j++) 
			    		{ h.Fill(i,j,Int(buffer+cursor)); cursor += 4; }
			}
			else { // asymetric
			  for (i = 0; i < fHeader.range[0]; i++)
			    for (j = i; j < fHeader.range[1]; j++) {
			    	if ( i == j ) 
					h.Fill(i,j,Int(buffer+cursor)); 
				else 
				{ 
					Int_t v = Int(buffer+cursor);
					h.Fill(i,j,v); h.Fill(j,i,v); 
				}
				cursor += 4; 
			    } // j
			}	
			break;
		case 6:
			if ( ! h.InheritsFrom("TH2F") ) {
			  fLog << nline ;
			  fLog << " Warning in [EGConverter::Read]: ROOT histogram type "  << h.ClassName()
			       << " does not match EG array type 32 bit floating point "   << nline;
			  fLog << "  ---> there might be some problems during the convertion  " << nline;  
			}
			h.SetBins(fHeader.range[0],fHeader.base_info[0],fHeader.range[0],
				  fHeader.range[1],fHeader.base_info[1],fHeader.range[1]);

			if ( fHeader.data_array_descriptor1[0] == 0 ) { // symetric  
			  for (i = 0; i < fHeader.range[0]; i++)
			    for (j = 0; j < fHeader.range[1]; j++) 
			    		{ h.Fill(i,j,Float(buffer+cursor)); cursor += 4; }
			}
			else { // asymetric
			  for (i = 0; i < fHeader.range[0]; i++)
			    for (j = i; j < fHeader.range[1]; j++) {
			    	if ( i == j ) 
					h.Fill(i,j,Float(buffer+cursor)); 
				else 
				{ 
					Float_t v = Float(buffer+cursor);
					h.Fill(i,j,v); h.Fill(j,i,v); 
				}
				cursor += 4; 
			    } // j
			}	
			break;	
		default:
			fLog << "Unknown type "
			     << fHeader.data_array_descriptor1[1] << " ! " << nline ;
	} 
	delete buffer; return true;
} 

//_________________________________________________________________
bool EGConverter::Write(const TH2 &/*h*/)
{ 
        fLog.GetProcessMethod() = "Write(const TH2& )";
	SetError(HistoConverter::kFail,"NO IMPLEMENTATION !!");	
	return false;
} 

//_________________________________________________________________
bool EGConverter::Read(THStack &hs)
{ 
	TString hsname = hs.GetName();
	
	// to avoid blanck in name 
	hsname.ReplaceAll(" ",""); 
	
	// pattern expression
    Bool_t wildcard = false;
    if ( hsname.Contains("*") )
        wildcard = true;    
	TRegexp all("*",kTRUE), pattern(hsname.Data(),wildcard);
    
	if ( pattern.Status() != TRegexp::kOK ) { // to be sure the user gave a correct pattern
		if ( gDebug > 0 ) cerr << "pattern [*] intead of " << hsname << endl ;
		pattern = all; 
	}
	else { if ( gDebug > 0 ) cerr << "pattern " << hsname << endl ; }
	
	// loop on files to load the good ones.
	TSystemDirectory dir; dir.SetDirectory(fName.Data()); TList *list = dir.GetListOfFiles();
		
	TIter iter(list); TObject *entry; 
	while ( (entry = iter()) ) {
		if ( IsEG(entry->GetName()) ) { // should be added to the collection
			TH1 *h = Get(entry->GetName()); if ( h ) { hs.Add(h); }
		}
	}
	if ( list ) delete list;
	
	return true;
} 

//_________________________________________________________________
bool EGConverter::Write(const THStack &/*hs*/)
{ 
        fLog.GetProcessMethod() = "Write(const THStack& )";
	SetError(HistoConverter::kFail,"NO IMPLEMENTATION !!");	
	return false;
} 


