#ifndef GW_EGCONVERTER_H
#define GW_EGCONVERTER_H

#ifndef GW_HISTODB_H
#include "HistoDB.h"
#endif

/**
	Base class to read/write eurogam (eg) spectra.
	see <a href="http://nnsa.dl.ac.uk/Euroball/documents/edoc061/edoc061.html"> 
		http://nnsa.dl.ac.uk/Euroball/documents/edoc061/edoc061.html</a> 
	for an explanation of the file format.

\author Christophe Theisen & Olivier Stezowski
*/

using namespace Gw;

class EGConverter: public HistoConverter 
{ 
private:
   std::ifstream fStreamIN;	  // stream used to read/write in a file
    
   //! header part of an eurogam spectrum
   struct HeaderEG
   {
	unsigned long  magic_number;
	unsigned long  version;		// expect 1 !!!
	char spectrum_name[32];		// spectrum name
	unsigned long  fold;
	char creation_date[20];		//creation date
	char modification_date[20];	// modif date	
	unsigned long  base_info[8];		// base info for dim 1-8
	unsigned long  range[8];		// range info for dim 1-8
	unsigned long  fip[32];		//  information pointer	
	int anotation_pointer[8];
	int calibration_pointer[8];
	int efficiency_pointer[8];
	unsigned long data_array_descriptor1[5];
		// 32 bits : array layout (0 for histo/matrix)
		// 32 bits : array type (5 for 32 fits signed )
		// 32 bits : reserved for future
		// 32 bits : reserved for future
		// 32 bits : pointer
	unsigned long  data_array_descriptor2[5];
	int base_address_string_space;
	int string_free_space;
	unsigned long  top_string_space;
	unsigned long  base_address_count_space;
	unsigned long  count_free_space;
	unsigned long  top_count_space;
	unsigned long  unused[20];	// not used ...
    };
    HeaderEG fHeader;   
    
    //! to check out if a file is an EG type.
    bool IsEG(const char *filename, Option_t *o = "") const;
    
    void PrintHeader() const;    
    //! to decode the header part
    void ReadHeader(char *, HeaderEG &);
    //! fill the header with histogram parameters
    void InitHeader(int, int, HeaderEG &, const char *, int);

    //! to open a eg file
    /**
    	try to open the corresponding file. If it fails or this file is not
	a eg file, it returns NaEG (Not an EG). Otherwise, the file is open
	and the header is read). In case of success, it returns the array type 
	as defined in edoc061 (http://npg.dl.ac.uk/Exogam/documents/edoc061/edoc061.html).
    */
    bool OpenEG(const char *);
    void CloseEG() { if ( fStreamIN.is_open() ) fStreamIN.close(); fStreamIN.clear(); }
        
public:	
    EGConverter();
    EGConverter(const char *name);
    virtual ~EGConverter();

    virtual TH1 *Get(int);    
    virtual TH1 *Get(const char *); 
    
    virtual bool Read(TH1 &);
    virtual bool Write(const TH1 &);

    virtual bool Read(TH2 &);
    virtual bool Write(const TH2 &);

    virtual bool Read(THStack &);
    virtual bool Write(const THStack &);

    virtual HistoConverter *NewDB(const char *) const;

   // virtual void ls(Option_t *);
    virtual void ls(Option_t *) const; 
     
    virtual const char *GetType() const { return fType.Data(); }
};

#endif
