/***************************************************************************
 *   Copyright (C) 2004-2006 by Olivier Stezowski & Christophe Theisen     *
 *   stezow(AT)ipnl.in2p3.fr, christophe.theisen(AT)cea.fr                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file HistoDB.cpp compiled in libGWTOOLS.so */

#ifndef GW_HISTODB_H
#include "HistoDB.h"
#endif

#ifndef ROOT_TSystem
#include <TSystem.h>
#endif

#include <iostream>

using namespace std;
using namespace Gw;

 
// ROOT dictionnary
ClassImp(HistoDB);
  
//__________________________________________________________
HistoDB::HistoDB()
{
	fCurrentDB = NULL; fStatus = HistoDB::kGood;
}

//__________________________________________________________
HistoDB::~HistoDB()
{
	fCurrentDB = NULL; fStatus = HistoDB::kGood;

	map<const char *, HistoConverter *>::iterator im; // iterator on the map prototype
	for (im = fConnections.begin() ; im !=  fConnections.end() ; im++ ) delete im->second;		
}

//__________________________________________________________
void HistoDB::AddFileExt(const char* type, const char* ext)
{
      TString dbtype(type);
  
      // find out the corresponding DB service in the prototype list
      map<const char *, HistoConverter *>::iterator im; // iterator on the map 
      
      for (im =   fCurrentDB->GetMapPrototype()->begin() ; im !=   fCurrentDB->GetMapPrototype()->end() ; im++ ) { 
        if ( strncmp(im->first, dbtype.Data(), sizeof(im->first)) == 0 ) 
          im->second->SetFileExt(ext); 
      }
}

//_________________________________________________________________
void HistoDB::AddOption(const char* type, Option_t* opt, const char* mode) 
{ 
    TString dbtype(type);
    
    // find out the corresponding DB service in the prototype list
    map<const char *, HistoConverter *>::iterator im; // iterator on the map 
      
      for (im =   fCurrentDB->GetMapPrototype()->begin() ; im !=   fCurrentDB->GetMapPrototype()->end() ; im++ ) { 
        if ( strncmp(im->first, dbtype.Data(), sizeof(im->first)) == 0 ) 
          im->second->SetOption(opt,mode); 
      }
} 


//__________________________________________________________
void HistoDB::SetCurrent(const char *alias)
{
	if ( fConnections.find(alias) != fConnections.end() ) { fCurrentDB = fConnections[alias]; }
}

//________________________________________________________________
HistoConverter *HistoDB::Open(const char *alias, const char *name)
{
	TString dbtype, dbname; HistoConverter *newdb; newdb = NULL;
	
	// to get the db service type and its name
	dbtype = name; dbname = name;
	if ( dbtype.Contains(':') ) 
		{ dbtype.Remove(dbtype.First(':')); dbname.Remove(0,dbname.First(':')+1);	}
	else { cout << name << " is not a valid name " << endl; return NULL; } 
	
	// find out the corresponding DB service in the prototype list
	map<const char *, HistoConverter *>::iterator im; // iterator on the map 
	
	for (im =   fCurrentDB->GetMapPrototype()->begin() ; im !=   fCurrentDB->GetMapPrototype()->end() ; im++ ) { 
		if ( strncmp(im->first,dbtype.Data(),sizeof(im->first)) == 0 ) { 
                  newdb = im->second->NewDB(dbname.Data());
                  newdb->SetFileExt( im->second->GetFileExt() );
                  newdb->SetOption( im->second->GetOption() ); 
                  newdb->CheckMineTypes();                
                } 
	}
	// allocation failed, don't know how to allocate this service. 
	if ( newdb == NULL ) { // opening fails
		cout << " Cannot allocate HistoDB service " << name << endl; return NULL;
	}	
	// check out if this is not a zombie (zombie = not well open)
	if ( newdb->GetStatus() ==  fCurrentDB->kZombie ) {
		cout << " Cannot open a connection to " 
		     << name << endl << "  --> " << newdb->GetError() << endl;
		     
		delete newdb; return NULL;	
	}

	// look if there is already an open connection corresponding to alias
	bool isnew = false;	
	if ( fConnections.find(alias) == fConnections.end() ) { isnew = true; }
		
	if (isnew) fConnections[alias] = newdb;
	else {
		cout << " An HistoDB service already exists with the alias " << name << endl;
		delete newdb; newdb = NULL;
	}
	if ( newdb ) fCurrentDB = newdb; // newdb becomes the current one
	
	return newdb;
}

//_____________________________________________________
void HistoDB::SetMode(const char *alias, Option_t *mode)
{	
	TString opt = mode;
	
	if ( fConnections.find(alias) == fConnections.end() ) {
		if ( opt.Contains("overwrite") ) 
			fConnections[alias]->SetMode(HistoConverter::kOverwrite);
		else 
			fConnections[alias]->SetMode(HistoConverter::kVersion);	
	}	
}

//__________________________________________________________
void HistoDB::SetMode(Option_t *mode)
{	
	TString opt = mode;
	
	if ( fCurrentDB ) {
		if ( opt.Contains("overwrite") ) 
			fCurrentDB->SetMode(HistoConverter::kOverwrite);
		else 
			fCurrentDB->SetMode(HistoConverter::kVersion);
	}
}

//__________________________________________________________
void HistoDB::ls(Option_t *o) 
{
	TString opt = o;
	map<const char *, HistoConverter *>::iterator im; // iterator on the map prototype

	if ( opt.Contains("proto") ) { // list of prototypes
		cout << "Known DB system to import/export ROOT histograms" << endl;
		for (im =   fCurrentDB->GetMapPrototype()->begin() ; im !=   fCurrentDB->GetMapPrototype()->end() ; im++ ) { 
			cout << " - " << im->first << endl ;
		}	
	}
	if ( opt.Contains("cdb") ) { // current db system
		if ( fCurrentDB == NULL ) {
			cout << "No current DB service defined !" << endl;
		}
		else { 
			cout << "Current DB service is:" << endl; fCurrentDB->ls(o);
		}
	}
	if ( opt.Contains("all") ) { // all db system defined
		cout << "List of all DB services open:" << endl;
		for (im = fConnections.begin() ; im !=  fConnections.end() ; im++ ) { 
			cout << "\t" << im->first << " "; im->second->ls(o); }	
	}		
}

//__________________________________________________________
void HistoDB::SetVerboseLevel(const int level){

	HistoConverter::SetVerboseLevel(level);

}

//__________________________________________________________
int HistoDB::GetVerboseLevel(){

	return HistoConverter::GetVerboseLevel();

}

//__________________________________________________________
TH1 *HistoDB::Get(int i)
{
	TH1 *histo = NULL;
	
	if ( fCurrentDB && fStatus == HistoDB::kGood ) {
		if ( (histo = fCurrentDB->Get(i)) == NULL ) {
			fStatus = HistoDB::kFail; cerr << fCurrentDB->GetError() << endl;
		}	
	}
	return histo;
}

//__________________________________________________________
TH1 *HistoDB::Get(const char *name)
{
	TH1 *histo = NULL;
	
	if ( fCurrentDB && fStatus == HistoDB::kGood ) {
		if ( (histo = fCurrentDB->Get(name)) == NULL ) {
			fStatus = HistoDB::kFail; cerr << fCurrentDB->GetError() << endl;
		}	
	}
	return histo;
}

//__________________________________________________________
bool HistoDB::Write(const TH1 &h)
{
bool result = false;	
	if ( fCurrentDB && fStatus == HistoDB::kGood ) {
	        result = fCurrentDB->Write(h);
		if ( result == false ) {
			fStatus = HistoDB::kFail; cerr << fCurrentDB->GetError() << endl;
		}	
	}
	return result;
}
//__________________________________________________________
bool HistoDB::Write(const TH2 &h)
{
bool result = false;	
	if ( fCurrentDB && fStatus == HistoDB::kGood ) {
	        result = fCurrentDB->Write(h);
		if ( result == false ) {
			fStatus = HistoDB::kFail; cerr << fCurrentDB->GetError() << endl;
		}	
	}
	return result;
}

//__________________________________________________________
HistoDB & HistoDB::operator << (const TH1 &h)
{
	if ( fCurrentDB && fStatus == HistoDB::kGood ) {
		if ( fCurrentDB->Write(h) == false ) {
			fStatus = HistoDB::kFail; cerr << fCurrentDB->GetError() << endl;
		}
	}	
	return (*this); 
}

//__________________________________________________________
HistoDB & HistoDB::operator >> (TH1 &h)
{
	
	if ( fCurrentDB && fStatus == HistoDB::kGood ) {
		if ( fCurrentDB->Read(h) == false ) {
			fStatus = HistoDB::kFail; cerr << fCurrentDB->GetError() << endl;		
		}
	}	
	return (*this); 
}

//__________________________________________________________
HistoDB & HistoDB::operator << (const TH2 &h)
{
	if ( fCurrentDB && fStatus == HistoDB::kGood ) {
		if ( fCurrentDB->Write(h) == false ) {
			fStatus = HistoDB::kFail; cerr << fCurrentDB->GetError() << endl;
		}
	}	
	return (*this); 
}

//__________________________________________________________
HistoDB & HistoDB::operator >> (TH2 &h)
{
	if ( fCurrentDB && fStatus == HistoDB::kGood ) {
		if ( fCurrentDB->Read(h) == false ) {
			fStatus = HistoDB::kFail; cerr << fCurrentDB->GetError() << endl;		
		}
	}	
	return (*this); 
}

//__________________________________________________________
HistoDB & HistoDB::operator << (const THStack &hs)
{
	if ( fCurrentDB && fStatus == HistoDB::kGood ) {
		if ( fCurrentDB->Write(hs) == false ) {
			fStatus = HistoDB::kFail; cerr << fCurrentDB->GetError() << endl;
		}
	}	
	return (*this); 
}

//__________________________________________________________
HistoDB & HistoDB::operator >> (THStack &hs)
{
	if ( fCurrentDB && fStatus == HistoDB::kGood ) {
		if ( fCurrentDB->Read(hs) == false ) {
			fStatus = HistoDB::kFail; cerr << fCurrentDB->GetError() << endl;
		}
	}	
	return (*this); 	
}

//__________________________________________________________
void HistoDB::MakeConverter(const char *classname) const 
{
	TString tmp;
	
	// open the file for the header part
	tmp = classname; tmp += ".h"; ofstream header(tmp.Data());
	if ( !header ) { cout << " Cannot open file " << tmp.Data() << endl; return; }
	
	// open the file for the implementation part
	tmp = classname; tmp += ".C"; ofstream imp(tmp.Data());
	if ( !imp ) { cout << " Cannot open file " << tmp.Data() << endl; header.close(); return; }
	
	// set the header part	
	header << "#ifndef " << classname << "_H" << endl;
	header << "#define " << classname << "_H 1" << endl;		
	header << endl;		
	header << "#include <HistoDB.h> " << endl;
	header << endl;	
	header << "class "<< classname <<": public HistoConverter " << endl;
	header << "{ " << endl;	
	header << "public:" << endl;	
	header << "    "<< classname <<"();" << endl;	
	header << "    "<< classname <<"(const char *name);" << endl;
	header << "    virtual ~"<< classname <<"();" << endl;	
	header << endl;		
	header << "    virtual TH1 *Get(int); " << endl;
	header << "    virtual TH1 *Get(const char *); " << endl;
	header << endl;			
	header << "    virtual bool Read(TH1 &);" << endl;	
	header << "    virtual bool Write(const TH1 &);" << endl;
	header << endl;		
	header << "    virtual bool Read(TH2 &);" << endl;	
	header << "    virtual bool Write(const TH2 &);" << endl;
	header << endl;	
	header << "    virtual bool Read(THStack &);" << endl;	
	header << "    virtual bool Write(const THStack &);" << endl;
	header << endl;					
	header << "    virtual HistoConverter *NewDB(const char *) const;" << endl;	
	header << endl;				
	header << "    virtual const char *GetType() const { return \"" 
	       << classname << "\" ; }" << endl;	
	header << "};" << endl;	
	header << "#endif " << endl;
	
	imp << endl;	
	imp << "#include \"" << classname << ".h\"" << endl;	
	imp << endl;	
	imp << "namespace { " <<  classname << " g" << classname 
	    << "; } // prototype ONLY use to creat a new DB system" << endl;
	imp << endl;	
	imp << classname << "::" << classname << "(const char *name) : HistoConverter(name)" << endl; 
	imp << "{ " << endl;	
	imp << endl;
	imp << "} " << endl;
	imp << endl;	
	imp << classname << "::" << classname << "() : HistoConverter()" << endl; 
	imp << "{ " << endl;	
	imp << endl;
	imp << "   // add this interface to the DB service " << endl;
	imp << "   AddPrototype(GetType(),this);" << endl;
	imp << "} " << endl;		
	imp << classname << "::~" << classname << "()" << endl; 
	imp << "{ " << endl;	
	imp << endl;
	imp << "} " << endl;	
	imp << endl;	 
	imp << "HistoConverter *" << classname 
	    << "::NewDB(const char *name) const { return new "<< classname << "(name) ;}" << endl; 
	imp << endl;	    
	imp << "TH1 *" << classname << "::Get(int which)" << endl;
	imp << "{ " << endl;
	imp << "  SetError(HistoConverter::kFail,\"[" << classname << "::Get(int which)"
	    << "] NO IMPLEMENTATION !!\") ; return NULL;" << endl;
	imp << "} " << endl;	
	imp << endl;	    
	imp << "TH1 *" << classname << "::Get(const char *name)" << endl;
	imp << "{ " << endl;	
	imp << "  SetError(HistoConverter::kFail,\"[" << classname << "::Get(const char *name)"
	    << "] NO IMPLEMENTATION !!\") ; return NULL;" << endl;	
	imp << "} " << endl;	
	imp << endl;				
	imp << "bool " << classname << "::Read(TH1 &h)" << endl;
	imp << "{ " << endl;	
	imp << "  SetError(HistoConverter::kFail,\"[" << classname << "::Read(TH1 &h)"
	    << "] NO IMPLEMENTATION !!\") ; return false;" << endl;	
	imp << "} " << endl;	
	imp << endl;	    
	imp << "bool " << classname << "::Write(const TH1 &h)" << endl;
	imp << "{ " << endl;	
	imp << "  SetError(HistoConverter::kFail,\"[" << classname << "::Write(const TH1 &h)"
	    << "] NO IMPLEMENTATION !!\") ; return false;" << endl;	
	imp << "} " << endl;
	imp << endl;	    
	imp << "bool " << classname << "::Read(TH2 &h)" << endl;
	imp << "{ " << endl;	
	imp << "  SetError(HistoConverter::kFail,\"[" << classname << "::Read(TH2 &h)"
	    << "] NO IMPLEMENTATION !!\") ; return false;" << endl;	
	imp << "} " << endl;	
	imp << endl;	    
	imp << "bool " << classname << "::Write(const TH2 &h)" << endl;
	imp << "{ " << endl;	
	imp << "  SetError(HistoConverter::kFail,\"[" << classname << "::Write(const TH2 &h)"
	    << "] NO IMPLEMENTATION !!\") ; return false;" << endl;	
	imp << "} " << endl;
	imp << endl;	    
	imp << "bool " << classname << "::Read(THStack &hs)" << endl;
	imp << "{ " << endl;	
	imp << "  SetError(HistoConverter::kFail,\"[" << classname << "::Read(THStack &hs)"
	    << "] NO IMPLEMENTATION !!\") ; return false;" << endl;	
	imp << "} " << endl;	
	imp << endl;	    
	imp << "bool " << classname << "::Write(const THStack &hs)" << endl;
	imp << "{ " << endl;	
	imp << "  SetError(HistoConverter::kFail,\"[" << classname << "::Write(const THStack &hs)"
	    << "] NO IMPLEMENTATION !!\") ; return false;" << endl;	
	imp << "} " << endl;		
	
	header.close(); imp.close();
}

