/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file LevelEditor.cpp compiled in libGWPHYSICS.so */
 
// root include's 
#ifndef ROOT_TGColorSelect
#include <TGColorSelect.h>
#endif
#ifndef ROOT_TGColorDialog
#include <TGColorDialog.h>
#endif
#ifndef ROOT_TGClient
#include <TGClient.h>
#endif
#ifndef ROOT_TGComboBox
#include <TGComboBox.h>
#endif
#ifndef ROOT_TColor
#include <TColor.h>
#endif
#ifndef ROOT_TVirtualPad
#include <TVirtualPad.h>
#endif
#ifndef ROOT_TGTextEntry
#include <TGTextEntry.h>
#endif
#ifndef ROOT_TGNumberEntry
#include <TGNumberEntry.h>
#endif
#ifndef ROOT_TGLabel
#include <TGLabel.h>
#endif

// gw include's 
#ifndef GW_LEVELEDITOR_H
#include "LevelEditor.h"
#endif

using namespace Gw;

ClassImp(LevelEditor)

enum ELineWid {
	kCOLOR,
	kLINE_WIDTH,
	kLINE_STYLE
};


LevelEditor::LevelEditor(const TGWindow *p, Int_t /*id*/)
#if ROOT_VERSION_CODE >= ROOT_VERSION(5,14,0)	       
	: TGedFrame(p)
#else
	: TGedFrame(p, id)
#endif
{
	fLevel = 0;
	
	MakeTitle("Label0");
	
	TGCompositeFrame *f2 = new TGCompositeFrame(this, 150, 20, kHorizontalFrame);
	AddFrame(f2, new TGLayoutHints(kLHintsTop, 1, 1, 0, 0));
	
	fColorSelect0 = new TGColorSelect(f2, 0, kCOLOR);
	f2->AddFrame(fColorSelect0, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	//fColorSelect0->Associate(this);
	fTextEntry0 = new TGTextEntry(f2); fTextEntry0->Resize(70,20);
	f2->AddFrame(fTextEntry0, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	//fTextEntry0->Associate(this);
	
	TGCompositeFrame *f2bis = new TGCompositeFrame(this, 80, 20, kHorizontalFrame);
	AddFrame(f2bis, new TGLayoutHints(kLHintsTop, 1, 1, 0, 0));
	TGLabel *fOffsetLblX0 = new TGLabel(f2bis, "DX ");
	f2bis->AddFrame(fOffsetLblX0, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	fOffsetX0 = new TGNumberEntry(f2bis,0.00,6,-1,TGNumberFormat::kNESRealTwo); fOffsetX0->Resize(50,20);
	f2bis->AddFrame(fOffsetX0, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	TGLabel *fOffsetLblY0 = new TGLabel(f2bis, " DY ");
	f2bis->AddFrame(fOffsetLblY0, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	fOffsetY0 = new TGNumberEntry(f2bis,0.00,6,-1,TGNumberFormat::kNESRealTwo); fOffsetY0->Resize(50,20);
	f2bis->AddFrame(fOffsetY0, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	
	MakeTitle("Label1");
	
	TGCompositeFrame *f3 = new TGCompositeFrame(this, 80, 20, kHorizontalFrame);
	AddFrame(f3, new TGLayoutHints(kLHintsTop, 1, 1, 0, 0));
	
	fColorSelect1 = new TGColorSelect(f3, 0, kCOLOR);
	f3->AddFrame(fColorSelect1, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	//fColorSelect1->Associate(this);
	fTextEntry1 = new TGTextEntry(f3); fTextEntry1->Resize(70,20);
	f3->AddFrame(fTextEntry1, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	//fTextEntry1->Associate(this);
	
	TGCompositeFrame *f3bis = new TGCompositeFrame(this, 80, 20, kHorizontalFrame);
	AddFrame(f3bis, new TGLayoutHints(kLHintsTop, 1, 1, 0, 0));
	TGLabel *fOffsetLblX1 = new TGLabel(f3bis, "DX ");
	f3bis->AddFrame(fOffsetLblX1, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	fOffsetX1 = new TGNumberEntry(f3bis,0.00,6,-1,TGNumberFormat::kNESRealTwo); fOffsetX1->Resize(50,20);
	f3bis->AddFrame(fOffsetX1, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	TGLabel *fOffsetLblY1 = new TGLabel(f3bis, " DY ");
	f3bis->AddFrame(fOffsetLblY1, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	fOffsetY1 = new TGNumberEntry(f3bis,0.00,6,-1,TGNumberFormat::kNESRealTwo); fOffsetY1->Resize(50,20);
	f3bis->AddFrame(fOffsetY1, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	
	MakeTitle("Label2");
	
	TGCompositeFrame *f4 = new TGCompositeFrame(this, 80, 20, kHorizontalFrame);
	AddFrame(f4, new TGLayoutHints(kLHintsTop, 1, 1, 0, 0));
	
	fColorSelect2 = new TGColorSelect(f4, 0, kCOLOR);
	f4->AddFrame(fColorSelect2, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	//fColorSelect2->Associate(this);
	fTextEntry2 = new TGTextEntry(f4); fTextEntry2->Resize(70,20);
	f4->AddFrame(fTextEntry2, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	//fTextEntry2->Associate(this);
	
	TGCompositeFrame *f4bis = new TGCompositeFrame(this, 80, 20, kHorizontalFrame);
	AddFrame(f4bis, new TGLayoutHints(kLHintsTop, 1, 1, 0, 0));
	TGLabel *fOffsetLblX2 = new TGLabel(f4bis, "DX ");
	f4bis->AddFrame(fOffsetLblX2, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	fOffsetX2 = new TGNumberEntry(f4bis,0.00,6,-1,TGNumberFormat::kNESRealTwo); fOffsetX2->Resize(50,20);
	f4bis->AddFrame(fOffsetX2, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	TGLabel *fOffsetLblY2 = new TGLabel(f4bis, " DY ");
	f4bis->AddFrame(fOffsetLblY2, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	fOffsetY2 = new TGNumberEntry(f4bis,0.00,6,-1,TGNumberFormat::kNESRealTwo); fOffsetY2->Resize(50,20);
	f4bis->AddFrame(fOffsetY2, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	
	MakeTitle("Label3");
	
	TGCompositeFrame *f5 = new TGCompositeFrame(this, 80, 20, kHorizontalFrame);
	AddFrame(f5, new TGLayoutHints(kLHintsTop, 1, 1, 0, 0));
	
	fColorSelect3 = new TGColorSelect(f5, 0, kCOLOR);
	f5->AddFrame(fColorSelect3, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	//ColorSelect3->Associate(this);
	fTextEntry3 = new TGTextEntry(f5); fTextEntry3->Resize(70,20);
	f5->AddFrame(fTextEntry3, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	//fTextEntry3->Associate(this);
	
	TGCompositeFrame *f5bis = new TGCompositeFrame(this, 80, 20, kHorizontalFrame);
	AddFrame(f5bis, new TGLayoutHints(kLHintsTop, 1, 1, 0, 0));
	TGLabel *fOffsetLblX3 = new TGLabel(f5bis, "DX ");
	f5bis->AddFrame(fOffsetLblX3, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	fOffsetX3 = new TGNumberEntry(f5bis,0.00,6,-1,TGNumberFormat::kNESRealTwo); fOffsetX3->Resize(50,20);
	f5bis->AddFrame(fOffsetX3, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	TGLabel *fOffsetLblY3 = new TGLabel(f5bis, " DY ");
	f5bis->AddFrame(fOffsetLblY3, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	fOffsetY3 = new TGNumberEntry(f5bis,0.00,6,-1,TGNumberFormat::kNESRealTwo); fOffsetY3->Resize(50,20);
	f5bis->AddFrame(fOffsetY3, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	
#if ROOT_VERSION_CODE >= ROOT_VERSION(5,14,0)
#else 
	TClass *cl = Level::Class();
	TGedElement *ge = new TGedElement;
	ge->fGedFrame = this;
	ge->fCanvas = 0;
	cl->GetEditorList()->Add(ge);
#endif
}


LevelEditor::LevelEditor(const TGWindow *p)
#if ROOT_VERSION_CODE >= ROOT_VERSION(5,14,0)	       
	: TGedFrame(p)
#else
	: TGedFrame(p,0)
#endif
{
	fLevel = 0;
	
	MakeTitle("Label0");
	
	TGCompositeFrame *f2 = new TGCompositeFrame(this, 150, 20, kHorizontalFrame);
	AddFrame(f2, new TGLayoutHints(kLHintsTop, 1, 1, 0, 0));
	
	fColorSelect0 = new TGColorSelect(f2, 0, kCOLOR);
	f2->AddFrame(fColorSelect0, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	//fColorSelect0->Associate(this);
	fTextEntry0 = new TGTextEntry(f2); fTextEntry0->Resize(70,20);
	f2->AddFrame(fTextEntry0, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	//fTextEntry0->Associate(this);
	
	TGCompositeFrame *f2bis = new TGCompositeFrame(this, 80, 20, kHorizontalFrame);
	AddFrame(f2bis, new TGLayoutHints(kLHintsTop, 1, 1, 0, 0));
	TGLabel *fOffsetLblX0 = new TGLabel(f2bis, "DX ");
	f2bis->AddFrame(fOffsetLblX0, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	fOffsetX0 = new TGNumberEntry(f2bis,0.00,6,-1,TGNumberFormat::kNESRealTwo); fOffsetX0->Resize(50,20);
	f2bis->AddFrame(fOffsetX0, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	TGLabel *fOffsetLblY0 = new TGLabel(f2bis, " DY ");
	f2bis->AddFrame(fOffsetLblY0, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	fOffsetY0 = new TGNumberEntry(f2bis,0.00,6,-1,TGNumberFormat::kNESRealTwo); fOffsetY0->Resize(50,20);
	f2bis->AddFrame(fOffsetY0, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	
	MakeTitle("Label1");
	
	TGCompositeFrame *f3 = new TGCompositeFrame(this, 80, 20, kHorizontalFrame);
	AddFrame(f3, new TGLayoutHints(kLHintsTop, 1, 1, 0, 0));
	
	fColorSelect1 = new TGColorSelect(f3, 0, kCOLOR);
	f3->AddFrame(fColorSelect1, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	//fColorSelect1->Associate(this);
	fTextEntry1 = new TGTextEntry(f3); fTextEntry1->Resize(70,20);
	f3->AddFrame(fTextEntry1, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	//fTextEntry1->Associate(this);
	
	TGCompositeFrame *f3bis = new TGCompositeFrame(this, 80, 20, kHorizontalFrame);
	AddFrame(f3bis, new TGLayoutHints(kLHintsTop, 1, 1, 0, 0));
	TGLabel *fOffsetLblX1 = new TGLabel(f3bis, "DX ");
	f3bis->AddFrame(fOffsetLblX1, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	fOffsetX1 = new TGNumberEntry(f3bis,0.00,6,-1,TGNumberFormat::kNESRealTwo); fOffsetX1->Resize(50,20);
	f3bis->AddFrame(fOffsetX1, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	TGLabel *fOffsetLblY1 = new TGLabel(f3bis, " DY ");
	f3bis->AddFrame(fOffsetLblY1, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	fOffsetY1 = new TGNumberEntry(f3bis,0.00,6,-1,TGNumberFormat::kNESRealTwo); fOffsetY1->Resize(50,20);
	f3bis->AddFrame(fOffsetY1, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	
	MakeTitle("Label2");
	
	TGCompositeFrame *f4 = new TGCompositeFrame(this, 80, 20, kHorizontalFrame);
	AddFrame(f4, new TGLayoutHints(kLHintsTop, 1, 1, 0, 0));
	
	fColorSelect2 = new TGColorSelect(f4, 0, kCOLOR);
	f4->AddFrame(fColorSelect2, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	//fColorSelect2->Associate(this);
	fTextEntry2 = new TGTextEntry(f4); fTextEntry2->Resize(70,20);
	f4->AddFrame(fTextEntry2, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	//fTextEntry2->Associate(this);
	
	TGCompositeFrame *f4bis = new TGCompositeFrame(this, 80, 20, kHorizontalFrame);
	AddFrame(f4bis, new TGLayoutHints(kLHintsTop, 1, 1, 0, 0));
	TGLabel *fOffsetLblX2 = new TGLabel(f4bis, "DX ");
	f4bis->AddFrame(fOffsetLblX2, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	fOffsetX2 = new TGNumberEntry(f4bis,0.00,6,-1,TGNumberFormat::kNESRealTwo); fOffsetX2->Resize(50,20);
	f4bis->AddFrame(fOffsetX2, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	TGLabel *fOffsetLblY2 = new TGLabel(f4bis, " DY ");
	f4bis->AddFrame(fOffsetLblY2, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	fOffsetY2 = new TGNumberEntry(f4bis,0.00,6,-1,TGNumberFormat::kNESRealTwo); fOffsetY2->Resize(50,20);
	f4bis->AddFrame(fOffsetY2, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	
	MakeTitle("Label3");
	
	TGCompositeFrame *f5 = new TGCompositeFrame(this, 80, 20, kHorizontalFrame);
	AddFrame(f5, new TGLayoutHints(kLHintsTop, 1, 1, 0, 0));
	
	fColorSelect3 = new TGColorSelect(f5, 0, kCOLOR);
	f5->AddFrame(fColorSelect3, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	//ColorSelect3->Associate(this);
	fTextEntry3 = new TGTextEntry(f5); fTextEntry3->Resize(70,20);
	f5->AddFrame(fTextEntry3, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	//fTextEntry3->Associate(this);
	
	TGCompositeFrame *f5bis = new TGCompositeFrame(this, 80, 20, kHorizontalFrame);
	AddFrame(f5bis, new TGLayoutHints(kLHintsTop, 1, 1, 0, 0));
	TGLabel *fOffsetLblX3 = new TGLabel(f5bis, "DX ");
	f5bis->AddFrame(fOffsetLblX3, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	fOffsetX3 = new TGNumberEntry(f5bis,0.00,6,-1,TGNumberFormat::kNESRealTwo); fOffsetX3->Resize(50,20);
	f5bis->AddFrame(fOffsetX3, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	TGLabel *fOffsetLblY3 = new TGLabel(f5bis, " DY ");
	f5bis->AddFrame(fOffsetLblY3, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	fOffsetY3 = new TGNumberEntry(f5bis,0.00,6,-1,TGNumberFormat::kNESRealTwo); fOffsetY3->Resize(50,20);
	f5bis->AddFrame(fOffsetY3, new TGLayoutHints(kLHintsLeft, 1, 1, 1, 1));
	
#if ROOT_VERSION_CODE >= ROOT_VERSION(5,14,0)
#else 
	TClass *cl = Level::Class();
	TGedElement *ge = new TGedElement;
	ge->fGedFrame = this;
	ge->fCanvas = 0;
	cl->GetEditorList()->Add(ge);
#endif
}

LevelEditor::~LevelEditor()
{
#if ROOT_VERSION_CODE >= ROOT_VERSION(5,14,0)
#else 	
	TGFrameElement *el;
	TIter next(GetList());
	
	while ((el = (TGFrameElement *)next())) {
	if (!strcmp(el->fFrame->ClassName(), "TGCompositeFrame"))
		((TGCompositeFrame *)el->fFrame)->Cleanup();
	}
	Cleanup();
#endif	
}

void LevelEditor::ConnectSignals2Slots()
{
	fColorSelect0->Connect("ColorSelected(Pixel_t)", "Gw::LevelEditor", this, "DoTextColor0(Pixel_t)");
	fColorSelect1->Connect("ColorSelected(Pixel_t)", "Gw::LevelEditor", this, "DoTextColor1(Pixel_t)");
	fColorSelect2->Connect("ColorSelected(Pixel_t)", "Gw::LevelEditor", this, "DoTextColor2(Pixel_t)");
	fColorSelect3->Connect("ColorSelected(Pixel_t)", "Gw::LevelEditor", this, "DoTextColor3(Pixel_t)");

	fTextEntry0->Connect("TextChanged(Char_t*)", "Gw::LevelEditor", this, "DoTextEntry0(Char_t*)");
	fTextEntry1->Connect("TextChanged(Char_t*)", "Gw::LevelEditor", this, "DoTextEntry1(Char_t*)");
	fTextEntry2->Connect("TextChanged(Char_t*)", "Gw::LevelEditor", this, "DoTextEntry2(Char_t*)");
	fTextEntry3->Connect("TextChanged(Char_t*)", "Gw::LevelEditor", this, "DoTextEntry3(Char_t*)");

	fOffsetX0->Connect("ValueSet(Long_t)","Gw::LevelEditor", this, "DoOffsetX0()");
	fOffsetY0->Connect("ValueSet(Long_t)","Gw::LevelEditor", this, "DoOffsetY0()");
	fOffsetX1->Connect("ValueSet(Long_t)","Gw::LevelEditor", this, "DoOffsetX1()");
	fOffsetY1->Connect("ValueSet(Long_t)","Gw::LevelEditor", this, "DoOffsetY1()");
	fOffsetX2->Connect("ValueSet(Long_t)","Gw::LevelEditor", this, "DoOffsetX2()");
	fOffsetY2->Connect("ValueSet(Long_t)","Gw::LevelEditor", this, "DoOffsetY2()");
	fOffsetX3->Connect("ValueSet(Long_t)","Gw::LevelEditor", this, "DoOffsetX3()");
	fOffsetY3->Connect("ValueSet(Long_t)","Gw::LevelEditor", this, "DoOffsetY3()");
  
	fInit = kFALSE;
}


// NEW
void LevelEditor::SetModel(TObject* obj)
{
#if ROOT_VERSION_CODE >= ROOT_VERSION(5,14,0)
#else 	
	fModel = 0; fPad = 0;
	
	if ( !obj || !obj->InheritsFrom(Level::Class()) || obj->InheritsFrom(TVirtualPad::Class())) {
		SetActive(kFALSE);
		return;
	}
	fModel = obj; fPad = TVirtualPad::Pad();
#endif
#if ROOT_VERSION_CODE >= ROOT_VERSION(5,14,0)	
	fLevel = dynamic_cast<Level *>(obj); fAvoidSignal = kTRUE;
#else 	
	fLevel = dynamic_cast<Level *>(fModel);
#endif	
	
	Color_t c; Pixel_t p;
	c = fLevel->GetLabel(0).GetTextColor();
	p = TColor::Number2Pixel(c);
	fColorSelect0->SetColor(p);
	c = fLevel->GetLabel(1).GetTextColor();
	p = TColor::Number2Pixel(c);
	fColorSelect1->SetColor(p);
	c = fLevel->GetLabel(2).GetTextColor();
	p = TColor::Number2Pixel(c);
	fColorSelect2->SetColor(p);
	c = fLevel->GetLabel(3).GetTextColor();
	p = TColor::Number2Pixel(c);
	fColorSelect3->SetColor(p);
	
	fTextEntry0->SetText(fLevel->GetLabel(0).GetTitle());
	fTextEntry1->SetText(fLevel->GetLabel(1).GetTitle());
	fTextEntry2->SetText(fLevel->GetLabel(2).GetTitle());
	fTextEntry3->SetText(fLevel->GetLabel(3).GetTitle());
	
	
	if (TVirtualPad::Pad()) {
		Float_t echX,echY;
		echX = TVirtualPad::Pad()->GetUxmax() - TVirtualPad::Pad()->GetUxmin();
		echY = TVirtualPad::Pad()->GetUymax() - TVirtualPad::Pad()->GetUymin();
		fOffsetX0->SetNumber((fLevel->GetLabel(0).GetX() - fLevel->GetX1()) / echX);
		fOffsetY0->SetNumber((fLevel->GetLabel(0).GetY() - fLevel->GetY1()) / echY);
		fOffsetX1->SetNumber((fLevel->GetLabel(1).GetX() - fLevel->GetX2()) / echX);
		fOffsetY1->SetNumber((fLevel->GetLabel(1).GetY() - fLevel->GetY1()) / echY);
		fOffsetX2->SetNumber((fLevel->GetLabel(2).GetX() - fLevel->GetX2()) / echX);
		fOffsetY2->SetNumber((fLevel->GetLabel(2).GetY() - fLevel->GetY1()) / echY);
		fOffsetX3->SetNumber((fLevel->GetLabel(3).GetX() - fLevel->GetX1()) / echX);
		fOffsetY3->SetNumber((fLevel->GetLabel(3).GetY() - fLevel->GetY1()) / echY);
	}
	
	if (fInit) ConnectSignals2Slots();
	
#if ROOT_VERSION_CODE >= ROOT_VERSION(5,14,0)	
	fAvoidSignal = kFALSE;
#else	
	SetActive();
#endif
}


void LevelEditor::SetModel(TVirtualPad* /*pad*/, TObject* obj, Int_t)
{
#if ROOT_VERSION_CODE >= ROOT_VERSION(5,14,0)
#else 	
	fModel = 0; fPad = 0;
	
	if ( !obj || !obj->InheritsFrom(Level::Class()) || obj->InheritsFrom(TVirtualPad::Class())) {
		SetActive(kFALSE);
		return;
	}
	fModel = obj; fPad = pad;
#endif

#if ROOT_VERSION_CODE >= ROOT_VERSION(5,14,0)	
	fLevel = dynamic_cast<Level *>(obj); fAvoidSignal = kTRUE;
#else 	
	fLevel = dynamic_cast<Level *>(fModel);
#endif	
	
	Color_t c; Pixel_t p;
	c = fLevel->GetLabel(0).GetTextColor();
	p = TColor::Number2Pixel(c);
	fColorSelect0->SetColor(p);
	c = fLevel->GetLabel(1).GetTextColor();
	p = TColor::Number2Pixel(c);
	fColorSelect1->SetColor(p);
	c = fLevel->GetLabel(2).GetTextColor();
	p = TColor::Number2Pixel(c);
	fColorSelect2->SetColor(p);
	c = fLevel->GetLabel(3).GetTextColor();
	p = TColor::Number2Pixel(c);
	fColorSelect3->SetColor(p);
	
	fTextEntry0->SetText(fLevel->GetLabel(0).GetTitle());
	fTextEntry1->SetText(fLevel->GetLabel(1).GetTitle());
	fTextEntry2->SetText(fLevel->GetLabel(2).GetTitle());
	fTextEntry3->SetText(fLevel->GetLabel(3).GetTitle());
	
	
	if (TVirtualPad::Pad()) {
		Float_t echX,echY;
		echX = TVirtualPad::Pad()->GetUxmax() - TVirtualPad::Pad()->GetUxmin();
		echY = TVirtualPad::Pad()->GetUymax() - TVirtualPad::Pad()->GetUymin();
		fOffsetX0->SetNumber((fLevel->GetLabel(0).GetX() - fLevel->GetX1()) / echX);
		fOffsetY0->SetNumber((fLevel->GetLabel(0).GetY() - fLevel->GetY1()) / echY);
		fOffsetX1->SetNumber((fLevel->GetLabel(1).GetX() - fLevel->GetX2()) / echX);
		fOffsetY1->SetNumber((fLevel->GetLabel(1).GetY() - fLevel->GetY1()) / echY);
		fOffsetX2->SetNumber((fLevel->GetLabel(2).GetX() - fLevel->GetX2()) / echX);
		fOffsetY2->SetNumber((fLevel->GetLabel(2).GetY() - fLevel->GetY1()) / echY);
		fOffsetX3->SetNumber((fLevel->GetLabel(3).GetX() - fLevel->GetX1()) / echX);
		fOffsetY3->SetNumber((fLevel->GetLabel(3).GetY() - fLevel->GetY1()) / echY);
	}
	
	if (fInit) ConnectSignals2Slots();
	
#if ROOT_VERSION_CODE >= ROOT_VERSION(5,14,0)	
	fAvoidSignal = kFALSE;
#else	
	SetActive();
#endif
}


void LevelEditor::DoTextColor0(Pixel_t color)
{
	// Slot connected to the label0 text color.
	fLevel->GetLabel(0).SetTextColor(TColor::GetColor(color));
	Update();
}


void LevelEditor::DoTextColor1(Pixel_t color)
{
	// Slot connected to the label1 text color.
	fLevel->GetLabel(1).SetTextColor(TColor::GetColor(color));
	Update();
}


void LevelEditor::DoTextColor2(Pixel_t color)
{
	// Slot connected to the label2 text color.
	fLevel->GetLabel(2).SetTextColor(TColor::GetColor(color));
	Update();
}


void LevelEditor::DoTextColor3(Pixel_t color)
{
	// Slot connected to the label3 text color.
	fLevel->GetLabel(3).SetTextColor(TColor::GetColor(color));
	Update();
}


void LevelEditor::DoTextEntry0(const Char_t *name)
{
	// Slot connected to the label0 text field.
	fLevel->GetLabel(0).SetTitle(name);
	Update();
}


void LevelEditor::DoTextEntry1(const Char_t *name)
{
	// Slot connected to the label1 text field.
	fLevel->GetLabel(1).SetTitle(name);
	Update();
}


void LevelEditor::DoTextEntry2(const Char_t *name)
{
 	// Slot connected to the label2 text field.
	fLevel->GetLabel(2).SetTitle(name);
	Update();
}


void LevelEditor::DoTextEntry3(const Char_t *name)
{
	// Slot connected to the label3 text field.
	fLevel->GetLabel(3).SetTitle(name);
	Update();
}


void LevelEditor::DoOffsetX0()
{
	// Slot connected to the Label0 offset in X axis.
	if (TVirtualPad::Pad()) {
		Float_t offset = fOffsetX0->GetNumber() * (TVirtualPad::Pad()->GetUxmax() - TVirtualPad::Pad()->GetUxmin());
		fLevel->GetLabel(0).SetX(fLevel->GetX1() + offset);
		Update();
	}
}


void LevelEditor::DoOffsetY0()
{
	// Slot connected to the Label0 offset in Y axis.
	if (TVirtualPad::Pad()) {
		Float_t offset = fOffsetY0->GetNumber() * (TVirtualPad::Pad()->GetUymax() - TVirtualPad::Pad()->GetUymin());
		fLevel->GetLabel(0).SetY(fLevel->GetY1() + offset);
		Update();
	}
}


void LevelEditor::DoOffsetX1()
{
	// Slot connected to the Label1 offset in X axis.
	if (TVirtualPad::Pad()) {
		Float_t offset = fOffsetX1->GetNumber() * (TVirtualPad::Pad()->GetUxmax() - TVirtualPad::Pad()->GetUxmin());
		fLevel->GetLabel(1).SetX(fLevel->GetX2() + offset);
		Update();
	}
}


void LevelEditor::DoOffsetY1()
{
	// Slot connected to the Label1 offset in Y axis.
	if (TVirtualPad::Pad()) {
		Float_t offset = fOffsetY1->GetNumber() * (TVirtualPad::Pad()->GetUymax() - TVirtualPad::Pad()->GetUymin());
		fLevel->GetLabel(1).SetY(fLevel->GetY1() + offset);
		Update();
	}
}


void LevelEditor::DoOffsetX2()
{
	// Slot connected to the Label2 offset in X axis.
	if (TVirtualPad::Pad()) {
		Float_t offset = fOffsetX2->GetNumber() * (TVirtualPad::Pad()->GetUxmax() - TVirtualPad::Pad()->GetUxmin());
		fLevel->GetLabel(2).SetX(fLevel->GetX2() + offset);
		Update();
	}
}


void LevelEditor::DoOffsetY2()
{
	// Slot connected to the Label2 offset in Y axis.
	if (TVirtualPad::Pad()) {
		Float_t offset = fOffsetY2->GetNumber() * (TVirtualPad::Pad()->GetUymax() - TVirtualPad::Pad()->GetUymin());
		fLevel->GetLabel(2).SetY(fLevel->GetY1() + offset);
		Update();
	}
}


void LevelEditor::DoOffsetX3()
{
	// Slot connected to the Label3 offset in X axis.
	if (TVirtualPad::Pad()) {
		Float_t offset = fOffsetX3->GetNumber() * (TVirtualPad::Pad()->GetUxmax() - TVirtualPad::Pad()->GetUxmin());
		fLevel->GetLabel(3).SetX(fLevel->GetX1() + offset);
		Update();
	}
}


void LevelEditor::DoOffsetY3()
{
	// Slot connected to the Label3 offset in Y axis.
	if (TVirtualPad::Pad()) {
		Float_t offset = fOffsetY3->GetNumber() * (TVirtualPad::Pad()->GetUymax() - TVirtualPad::Pad()->GetUymin());
		fLevel->GetLabel(3).SetY(fLevel->GetY1() + offset);
		Update();
	}
}
