/***************************************************************************
 *   Copyright (C) 2004 by Olivier Stezowski                               *
 *   stezow(AT)ipnl.in2p3.fr                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/** \file LevelEditor.h header file for a Level GUI Editor */

#ifndef GW_LEVELEDITOR_H
#define GW_LEVELEDITOR_H

#ifndef ROOT_TGButton
#include "TGWidget.h"
#endif
#ifndef ROOT_TGedFrame
#include "TGedFrame.h"
#endif

#ifndef GW_LEVEL_H
#include "Level.h"
#endif

class TGColorSelect;
class TGTextEntry;
class TGNumberEntry;

namespace Gw {

/**
	Level GUI Editor : \n 
	The goal of the level editor is to change graphical attributes for a level.
	As a Level derives from a TLine, the TLine editor is still open and the user can change graphical attributes of the level line (color, style, width).

	All labels can be modified (color, text, offset in X direction, offset in Y direction).

	\todo perhaps remove text fields for the labels because in NuclearLevel, labels are related to nuclear data as spin, parity, ...

	@author Bertrand ROSSE
*/
class LevelEditor : public TGedFrame 
{
protected:
	Level                *fLevel;             // level object
   
	TGColorSelect        *fColorSelect0;      // text color widget for label 0
	TGColorSelect        *fColorSelect1;      // text color widget for label 1
	TGColorSelect        *fColorSelect2;      // text color widget for label 2
	TGColorSelect        *fColorSelect3;      // text color widget for label 3

	TGTextEntry          *fTextEntry0;        // text entry for label 0
	TGTextEntry          *fTextEntry1;        // text entry for label 1
	TGTextEntry          *fTextEntry2;        // text entry for label 2
	TGTextEntry          *fTextEntry3;        // text entry for label 3

	TGNumberEntry        *fOffsetX0;    //
	TGNumberEntry        *fOffsetY0;    //
	TGNumberEntry        *fOffsetX1;    //
	TGNumberEntry        *fOffsetY1;    //
	TGNumberEntry        *fOffsetX2;    //
	TGNumberEntry        *fOffsetY2;    //
	TGNumberEntry        *fOffsetX3;    //
	TGNumberEntry        *fOffsetY3;    //

	virtual void   ConnectSignals2Slots();

public:
	LevelEditor(const TGWindow *p = 0);
	LevelEditor(const TGWindow *p, Int_t id);
	virtual ~LevelEditor();
	
	//! to connect the current Level to the Editor (for ROOTVERSION > 5.12)
	virtual void   SetModel(TObject *obj);
	
	//! to connect the current Level to the Editor (for ROOTVERSION <= 5.12)
	/*!
		This method 
	*/	
	virtual void   SetModel(TVirtualPad *pad, TObject *obj, Int_t event);

	
	virtual void   DoTextColor0(Pixel_t color);
	virtual void   DoTextColor1(Pixel_t color);
	virtual void   DoTextColor2(Pixel_t color);
	virtual void   DoTextColor3(Pixel_t color);

	virtual void   DoTextEntry0(const Char_t *name = "");
	virtual void   DoTextEntry1(const Char_t *name = "");
	virtual void   DoTextEntry2(const Char_t *name = "");
	virtual void   DoTextEntry3(const Char_t *name = "");

	virtual void   DoOffsetX0();
	virtual void   DoOffsetY0();
	virtual void   DoOffsetX1();
	virtual void   DoOffsetY1();
	virtual void   DoOffsetX2();
	virtual void   DoOffsetY2();
	virtual void   DoOffsetX3();
	virtual void   DoOffsetY3();

	ClassDef(LevelEditor,0)  // GUI for editing line attributes
};

}

#endif
