#ifndef GW_RADCONVERTER_H
#define GW_RADCONVERTER_H

#ifndef GW_HISTODB_H
#include "HistoDB.h"
#endif

/**
Base class to read/write Radware spectra.

Radware spectra can be imported and exported. An directory is considered as a DB system.
So, to open a Radware DB system, a directory is required: \n
	- HistoDB hdb; hdb.Open("myspe","rad:/where/are/my/spectra");
	
The directory path must be an absolute path (must start with /). However, environment variables
could be used: \n
	- HistoDB hdb; hdb.Open("myspe","rad:$(PWD)");
	
If the directory doesn't exist, the DB system is not open (set as a zombie).
the convertion for writting a path is the unix one (with /). However, it should 
also work on windows systems (NOT YET TESTED).
	
As for any HistoConverter, the content of the directory could be listed with hdb.ls().

The swapping is guessed (in reading mode), so no need to worry about the origin of the file.

Pattern are supported to read/write 1D spectra via THStack. For instance, to read all the spectra
from the directory that starts with P1:
	- THstack hs; hs.SetName("P1*"); hdb >> hs;  

Only TH2S and TH2I could be exported to Radware format.

\author Christophe Theisen & Olivier Stezowski
*/

using namespace Gw;

class RADConverter: public HistoConverter 
{ 
private:
    bool DecodeMAT(TH2 &, std::ifstream &);
    bool DecodeSPN(TH2 &, std::ifstream &);
        
public:
    RADConverter();
    RADConverter(const char *name);
    virtual ~RADConverter();

    virtual TH1 *Get(int);     
    virtual TH1 *Get(const char *); 
    virtual bool Read(TH1 &);
    virtual bool Write(const TH1 &);

    virtual bool Read(TH2 &);
    virtual bool Write(const TH2 &);

    virtual bool Read(THStack &);
    virtual bool Write(const THStack &);

    virtual HistoConverter *NewDB(const char *) const;
    
    virtual void ls(Option_t *) const ;
    virtual const char *GetType() const { return fType.Data(); }
};

#endif

