#include<fstream>
#include<iostream>
//#include<stdio>
using namespace std;

#include<FuzzyCalib.h>
#include<TMath.h>
#include<TSpectrum.h>
#include<TFile.h>
#include<TCanvas.h>
#include<TROOT.h>
#include<TText.h>

#define MAXPEAK 50

FuzzyCalib::FuzzyCalib() : BaseCalib(), fListPeak() ,fListRef(), fAllPeak(MAXPEAK) {
	fListRef.SetOwner();
	fAllPeak.SetOwner();

	for (Int_t i=0 ; i<MAXPEAK ; i++){
		fAllPeak.Add(new Peak());
	}

	fMatScore = NULL;
	fPos = NULL;
	cout <<"Warning - List of FuzzySets not initialysed ... "<<endl;
}

FuzzyCalib::FuzzyCalib(const Char_t *filename): BaseCalib(), fListPeak() ,fListRef(), fAllPeak(MAXPEAK) {
	//Constructor --> build the FuzzySet by extracting data in a .sou file (Radford source file)
	//The file must have the following format : 
	//     peak_position(in keV)   peak_position_error(in keV)    peak_intensity    peak_intensity_error 
	//Exemple of Eu152.sou :
        /* 	121.7830       .0020      13620.        160.
        	244.6920       .0020       3590.         60.
        	295.9390       .0080        211.          5.
        	344.2760       .0040      12750.         90.
        	367.7890       .0050        405.          8.
        	411.1150       .0050       1070.         10.
        	443.9760       .0050       1480.         20.
        	488.6610       .0390        195.          2.
        	564.0210       .0080        236.          5.
        	586.2940       .0060        220.          5.
        	678.5780       .0030        221.          4.
        	688.6780       .0060        400.          8.
        	778.9030       .0060       6190.         80.
        	867.3880       .0080       1990.         40.
        	964.1310       .0090       6920.         90.
        	1005.2790       .0170        310.          7.
        	1089.7000       .0150        820.         10.
        	1109.1800       .0120         88.          2.
        	1112.1160       .0170       6490.         90.
        	1212.9500       .0120        670.          8.
        	1299.1240       .0120        780.         10.
        	1408.0110       .0140      10000.         30. */
		
		
	fListRef.SetOwner();
	fAllPeak.SetOwner();

	for (Int_t i=0 ; i<MAXPEAK ; i++){
		fAllPeak.Add(new Peak());
	}

	fMatScore = NULL;
	fPos = NULL;

	Char_t dumc[1000];
	Float_t position[100], positionerr[100], intensity[100], intensityerr[100];
	Int_t nbpeak = 0;
	Char_t name[100];

	ifstream infile(filename);
	if (infile.fail()) {
		if (fErr) fErrFile << "Cannot open file "<<filename<<endl;
	}

	while(1){  //read the file
		infile.getline(dumc,1000);
		if(!infile.good()) break;
		sscanf(dumc, "%f %f %f %f", &position[nbpeak], &positionerr[nbpeak], &intensity[nbpeak], &intensityerr[nbpeak]);
		nbpeak++;
	}

	Float_t param[30];

	for (Int_t i=0 ; i < nbpeak ; i++){
 		if (i==0) {  //first peak
			param[0] = position[i];
			param[1] = 7;
			param[2] = 7;
			param[3] = 100;
			param[4] = 100;
			//-------------------------------------------
			param[5] = intensity[i];
			param[6] = 0;
			param[7] = 0;
			param[8] = 0.5*intensity[i];
			param[9] = 0.5*intensity[i];
			//-------------------------------------------
			param[10] = position[i+1]-position[i];
			param[11] = 7;
			param[12] = 7;
			param[13] = 0.75*(position[i+1]-position[i]);
			param[14] = 0.75*(position[i+1]-position[i]);
			//-------------------------------------------
			param[15] = intensity[i+1]/intensity[i];
			param[16] = 0;
			param[17] = 0;
			param[18] = 0.5*(intensity[i+1]/intensity[i]);
			param[19] = 0.5*(intensity[i+1]/intensity[i]);
			//-------------------------------------------
			param[20] = position[i+2]-position[i];
			param[21] = 7;
			param[22] = 7;
			param[23] = 0.75*(position[i+2]-position[i]);
			param[24] = 0.75*(position[i+2]-position[i]);
			//-------------------------------------------
			param[25] = intensity[i+2]/intensity[i];
			param[26] = 0;
			param[27] = 0;
			param[28] = 0.5*(intensity[i+2]/intensity[i]);
			param[29] = 0.5*(intensity[i+2]/intensity[i]);
		}
		else if (i==nbpeak-1){  //last peak
			param[0] = position[i];
			param[1] = 7;
			param[2] = 7;
			param[3] = 100;
			param[4] = 100;
			//-------------------------------------------
			param[5] = intensity[i];
			param[6] = 0;
			param[7] = 0;
			param[8] = 0.5*intensity[i];
			param[9] = 0.5*intensity[i];
			//-------------------------------------------
			param[10] = position[i-1]-position[i];
			param[11] = 7;
			param[12] = 7;
			param[13] = 0.75*(position[i-1]-position[i]);
			param[14] = 0.75*(position[i-1]-position[i]);
			//-------------------------------------------
			param[15] = intensity[i-1]/intensity[i];
			param[16] = 0;
			param[17] = 0;
			param[18] = 0.5*(intensity[i-1]/intensity[i]);
			param[19] = 0.5*(intensity[i-1]/intensity[i]);
			//-------------------------------------------
			param[20] = position[i-2]-position[i];
			param[21] = 7;
			param[22] = 7;
			param[23] = 0.75*(position[i-2]-position[i]);
			param[24] = 0.75*(position[i-2]-position[i]);
			//-------------------------------------------
			param[25] = intensity[i-2]/intensity[i];
			param[26] = 0;
			param[27] = 0;
			param[28] = 0.5*(intensity[i-2]/intensity[i]);
			param[29] = 0.5*(intensity[i-2]/intensity[i]);
		} else {
			param[0] = position[i];
			param[1] = 7;
			param[2] = 7;
			param[3] = 100;
			param[4] = 100;
			//-------------------------------------------
			param[5] = intensity[i];
			param[6] = 0;
			param[7] = 0;
			param[8] = 0.5*intensity[i];
			param[9] = 0.5*intensity[i];
			//-------------------------------------------
			param[10] = position[i-1]-position[i];
			param[11] = 7;
			param[12] = 7;
			param[13] = 0.75*(position[i-1]-position[i]);
			param[14] = 0.75*(position[i-1]-position[i]);
			//-------------------------------------------
			param[15] = intensity[i-1]/intensity[i];
			param[16] = 0;
			param[17] = 0;
			param[18] = 0.5*(intensity[i-1]/intensity[i]);
			param[19] = 0.5*(intensity[i-1]/intensity[i]);
			//-------------------------------------------
			param[20] = position[i+1]-position[i];
			param[21] = 7;
			param[22] = 7;
			param[23] = 0.75*(position[i+1]-position[i]);
			param[24] = 0.75*(position[i+1]-position[i]);
			//-------------------------------------------
			param[25] = intensity[i+1]/intensity[i];
			param[26] = 0;
			param[27] = 0;
			param[28] = 0.5*(intensity[i+1]/intensity[i]);
			param[29] = 0.5*(intensity[i+1]/intensity[i]);

		}
		sprintf(name,"Ref_%d",i);
		FuzzySet *f = new FuzzySet(name,param);
		fListRef.Add(f);

	}  //i

}

FuzzyCalib::~FuzzyCalib(){
	//delete all objects in fRefList and in fAllPeak
	fListRef.Clear();
	fAllPeak.Clear();
	if (fMatScore != NULL) delete[] fMatScore;
	if (fPos != NULL) delete [] fPos;

}

Int_t FuzzyCalib::InitFuzzySets(const TObjArray &ref){
	//return the number of entries in fListRef
	for (Int_t i=0 ; i<ref.GetEntries() ; i++){
		fListRef.Add(ref.At(i));
	}
	fListRef.Compress();
	return fListRef.GetEntries();
}

Int_t FuzzyCalib::InitFuzzySets(const Char_t *filename, const Char_t *name){
	//Function to initialyse the private member fListRef with a TObjArray named name in the file named filename
	//The TObjArray in the file
	//return the number of entries in fListRef
	TFile *file = new TFile(filename,"READ");
	if (file == NULL) {fErrFile <<"Dynamic Allocation failed in InitFuzzySets method"<<endl; return -1;}
	TObjArray *local = (TObjArray *)file->Get(name);
	if (local == NULL) {fErrFile <<"No TObjArray named "<<name<<" in the file ..."<<endl; return -1;}
	InitFuzzySets(*local);

	file->Close();
	if (file != NULL) delete file;
	if (local != NULL) delete local;

	return fListRef.GetEntries() ;
}

Int_t FuzzyCalib::Init(TH1D *histo, TF1 *formula){
	//To init the private members fListPeak
	//Use the TSpectrum member Search (See Root WebSite Version 4.00)
	//   parameters for Search are :
	//	sigma = 3 ; threshold = 0.005
	//Return the number of Peaks stored in fListPeak

	//Check histo
	if (histo == NULL) {
		if (fErr) fErrFile <<"histogram -> Pointer NULL, Abort ... "<<endl;
		cout <<"histogram -> Pointer NULL, Abort ... "<<endl;
		return -1;
	}

	//Check formula
	if(formula == NULL) {
		if (fErr) fErrFile <<"formula -> Pointer NULL, Abort ... "<<endl;
		cout <<"formula -> Pointer NULL, Abort ... "<<endl;
		return -1;
	}

	//Clear and compress the List of found Peaks
	fListPeak.Clear();
	fListPeak.Compress();

	TSpectrum spectrum(100);  //max 100 peaks to be found (see root website for this number)
	if (gDebug > 0) {
		spectrum.Search(histo,3,"",0.005);
	} else {
		spectrum.Search(histo,3,"goff",0.005);
	}

	Float_t *position = spectrum.GetPositionX();
	Float_t *intensity = spectrum.GetPositionY();

	//Allocate fPos
	AllocPos(spectrum.GetNPeaks());

	if (spectrum.GetNPeaks() > fAllPeak.GetSize()){
		Int_t sizebeforeexpand = fAllPeak.GetSize();

		fAllPeak.Expand(spectrum.GetNPeaks());
		for (Int_t i=0 ; i<spectrum.GetNPeaks()-sizebeforeexpand ; i++){
			fAllPeak.AddAt(new Peak(),i+sizebeforeexpand);
		}

	}

	if (fLog) fLogFile <<"	Number of Peak found : "<<spectrum.GetNPeaks()<<endl;
        cout <<"Number of Peak found : "<<spectrum.GetNPeaks()<<endl;
	for (Int_t i=0 ; i < spectrum.GetNPeaks() ; i++){
		fPos[i] = position[i]; //on garde la position touvee des pics
		//Perform the first calibration on found peaks
		position[i] = formula->Eval(position[i]);
		if (fLog) fLogFile <<"	Peak number "<<i<<" - position : "<<position[i]<<" keV - Intensity : "<<intensity[i]<<endl;
		//Change the peak position and intensity in fAllPeak
		((Peak *)fAllPeak.At(i))->SetPos(position[i]);
		((Peak *)fAllPeak.At(i))->SetInt(intensity[i]);
		fListPeak.Add(fAllPeak.At(i));
	}

	fListPeak.Compress();

	//Allocate the matrice of scores
	AllocMat(fListPeak.GetEntries(),fListRef.GetEntries());



	return (fListPeak.GetEntries());
}

const Char_t* FuzzyCalib::DoCalib(TH1D *histo, TF1 *formula, TGraphErrors *graph, const Char_t *option){

	//Check the histogram
	if (histo == NULL) {
		if (fErr) {fErrFile <<"no histogram to calib - pointer null in DoCalib method ..."<<endl;}
		return ("no histogram to calib - pointer null in DoCalib method ...");
	}

	//Check the formula
	if (formula == NULL) {
		if (fErr) {fErrFile <<"no formula to calib - pointer null in DoCalib method ..."<<endl;}
		return ("no formula to calib - pointer null in DoCalib method ...");
	}

	//Check the graph
	if (graph == NULL) {
		if (fErr) {fErrFile <<"no graph to calib - pointer null in DoCalib method ..."<<endl;}
		return ("no graph to calib - pointer null in DoCalib method ...");
	}

	if (fLog) fLogFile << "Analyse Histogram : "<<histo->GetName()<<endl<<endl;
	cout << "Analysing Histogram : "<<histo->GetName()<<" ..."<<endl;

	if (fLog){
		fLogFile << "	Formula Parameters before calibration : "<<endl;
		for (Int_t i=0 ; i<formula->GetNpar() ; i++){
			fLogFile<<"		Parameter "<<formula->GetParName(i)<<" = "<<formula->GetParameter(i)<<endl;
		}
	}

	//Create the Canvas to display spectrum wiyh found peaks
	TCanvas *cPrincipal;
	if ( ( cPrincipal = (TCanvas *)gROOT->GetListOfFunctions()->FindObject("cPrincipal") ) ) { delete cPrincipal; }
	if (gDebug > 0) {
		cPrincipal = new TCanvas("cPrincipal","Fuzzy Calibration",0,0,800,500);
	} else {
		cPrincipal = new TCanvas(kFALSE);
	}

	//Store number of Reference peaks in the Log File
	if (fLog) fLogFile<<"Number of reference peaks : "<<fListRef.GetEntries()<<endl;

	//Perform Init (TH1 *histo, TF1 *formula)
	cout <<"Initialysing the List of Peaks ..."<<endl;
	Init(histo,formula);

	//Check fListRef
	if (fListRef.GetEntries() == 0){
		if (fErr) {fErrFile <<"no entries in fListRef ..."<<endl;}
		return ("no entries in fListRef ...");
	}

	//Check fListPeak
	if (fListPeak.GetEntries() == 0){
		if (fErr) {fErrFile <<"no entries in fListPeak ..."<<endl;}
		return("no entries in fListPeak ...");
	}

	//Perform the Algorithme to identify peaks
	DoScore1();

	//Defuzzycation
	DeFuzzycation1(graph);

	//Fit the graph with the formula
	FitGraph(graph,formula);


	if (fLog){
		fLogFile << "	Formula Parameters after calibration : "<<endl;
		for (Int_t i=0 ; i<formula->GetNpar() ; i++){
			fLogFile<<"		Parameter "<<formula->GetParName(i)<<" = "<<formula->GetParameter(i)<<endl;
		}
		fLogFile<<endl;
	}


	return ("End of Calibration for this histogram");

}


void FuzzyCalib::DoCalib(TObjArray *collhisto, TObjArray *collformula, TObjArray *collgraph, const Char_t *option){

	//Check  collhisto
	if (collhisto == NULL){
		if (fErr) fErrFile << "Pointer Null collhisto in DoCalib method ..."<<endl;
		cout <<"Pointer Null collhisto in DoCalib method ..."<<endl;
		return;
	}

	//Check collformula
	if (collformula == NULL){
		if (fErr) fErrFile << "Pointer Null collformula in DoCalib method ..."<<endl;
		cout <<"Pointer Null collformula in DoCalib method ..."<<endl;
		return;
	}

	//Check collgraph
	if (collgraph == NULL){
		if (fErr) fErrFile << "Pointer Null collgraph in DoCalib method ..."<<endl;
		cout <<"Pointer Null collgraph in DoCalib method ..."<<endl;
		return;
	}

	//Check the dimension of the three collections
	if ((collhisto->GetEntries() > collformula->GetEntries()) && (collhisto->GetEntries() > collgraph->GetEntries())){
		if (fErr) fErrFile << "Dimension collhisto > Dimension collformula or Dimension collhisto > Dimension collgraph..."<<endl;
		cout <<"Dimension collhisto > Dimension collformula  or Dimension collhisto > Dimension collgraph ..."<<endl;
		return;
	}

	TString string(option);

	//Loop on all histogram
	for (Int_t i=0 ; i<collhisto->GetEntries() ; i++){
		cout << DoCalib((TH1D *)collhisto->At(i), (TF1 *)collformula->At(i),(TGraphErrors *)collgraph->At(i), string.Data() )<<endl;
		//gPad->Update();
		//gPad->WaitPrimitive();
	}//i
	
	collformula->Write("Formula_Calibrated",TObject::kSingleKey);
	collgraph->Write("Graph_Calibrated",TObject::kSingleKey);

}

Float_t FuzzyCalib::Algorithme1(const FuzzySet &ref, const Peak &central, const Peak &neighbour1, const Peak &neighbour2){
	//return the score for the following algorithme
	//--------------------------------------
	//--       position of the central peak
	//-- AND   position of neighbour1 peak
	//-- AND   intensity of neighbour1 peak
	//-- AND   position of neighbour2 peak
	//-- AND   intensity of neighbour2 peak
	//--------------------------------------
	//Operator AND is the function MIN
	//
	Float_t score[5]= {0.0,0.0,0.0,0.0,0.0};
	Int_t index[5];
	score[0] = ref.ScorePos(central); //position of central peak
	score[1] = ref.ScoreNeighbour1Pos(central,neighbour1); //position of neighbour1 peak
	score[2] = ref.ScoreNeighbour1Int(central,neighbour1); //intensity of neighbour1 peak
	score[3] = ref.ScoreNeighbour2Pos(central,neighbour2); //position of neighbour2 peak
	score[4] = ref.ScoreNeighbour2Int(central,neighbour2); //intensity of neighbour2 peak

	TMath::Sort(5,score,index,kFALSE);
	return(score[index[0]]);

}

void FuzzyCalib::DoScore1(){
	FuzzySet *ref;
	Peak *central;
	Peak *neib1;
	Peak *neib2;
	Float_t score;
	Float_t scorepeak;

	if (fLog) fLogFile << "	Candidats pour identification : "<<endl;

	for (Int_t i=0 ; i<fListPeak.GetEntries(); i++){ //loop on the found peaks
	//for (Int_t i=0 ; i<1; i++){ //loop on the found peaks
		scorepeak = 0; //score min pour un pic donne
		central = (Peak *)fListPeak.At(i);
		for (Int_t j=0 ; j<fListRef.GetEntries() ; j++){  //loop on reference peaks
		//for (Int_t j=0 ; j<1 ; j++){  //loop on reference peaks
			ref =(FuzzySet *)fListRef.At(j);
			score =0;  //score min pour un pic central et un pic de reference donnes
			for (Int_t k=0 ; k<fListPeak.GetEntries() ; k++){ //loop on the neighbour1
				neib1 = (Peak *)fListPeak.At(k);
				if (k==i) continue;
				for (Int_t l=0 ; l<fListPeak.GetEntries() ; l++){ //loop on the neighbour2
					neib2 = (Peak *)fListPeak.At(l);
					if (l==i) continue;
					if (l==k) continue;
					score = TMath::Max(Algorithme1(*ref,*central,*neib1,*neib2),score);
				} //l --> neighbour2
			} //k -->neighbour1
			// on stock le score dans la matrice
			fMatScore[i*fListRef.GetEntries()+j] = score;
			if (score > 0) {  //Ce pic de reference est un candidat
				if (fLog) fLogFile <<"		Pic numero : "<<i<< "- Candidat :"<<((FuzzySet *)fListRef.At(j))->GetName()<<"- score = "<<score<<endl;
			}
		} //j --> reference peaks
	} //i -->found peaks
}


void FuzzyCalib::DeFuzzycation1(TGraphErrors *graph){
	//--
	//Gestion de la matrice des scores
	//Pour chaque pic de reference (colonne de la matrice) , on regarde le meilleur pic (ligne de la matrice)
	//Pour un pic identifie, on ajoute le point dans le graph
	//
	//Pour l'instant, on ne regarde pas les ambigus
	//--
	
	
	//Check fMatScore
	if (fMatScore == NULL) {
		if (fErr) fErrFile << "No Score Matrice - pointer null, abort defuzzycation "<<endl;
		cout <<"No Score Matrice - pointer null, abort defuzzycation "<<endl;
		return;
	}

	Float_t score;
	Int_t nbgood = 0;
	Int_t index;  //indice du meilleur pic
	Float_t pospeak; //position for a found peak
	Float_t posref; //position for a reference peak
	Float_t intpeak; //intensity of a found peak

	Char_t text[50];
	TText rtext;
	rtext.SetTextAlign(21); rtext.SetTextSize(0.05);

	if (fLog) fLogFile<<"	Defuzzycation version 1"<<endl;
	for(Int_t j=0 ; j<fListRef.GetEntries(); j++){  //loop on reference peaks
		score = 0;
		index = 0;
		for (Int_t i=0 ; i<fListPeak.GetEntries(); i++){ //loop on found peaks
			if (fMatScore[i*fListRef.GetEntries()+j] > score){
				score = fMatScore[i*fListRef.GetEntries()+j];
				index = i;
			}
		}//i
		//Check the score
		if (score < 0.05) {
			if (fLog) fLogFile<<"		No peak identify for reference peak "<<((FuzzySet *)fListRef.At(j))->GetName()<<endl<<endl;
		} else { //score >0.05

			pospeak = fPos[index];
			posref  = ((TF1 *)((FuzzySet *)fListRef.At(j))->GetFuzzyPos())->GetParameter(0);
			intpeak  = ((Peak *)fListPeak.At(index))->GetInt();

			if (fLog) {
				fLogFile<<"		Reference Peak "<<((FuzzySet *)fListRef.At(j))->GetName()<<" identifie au pic "<<index<<endl;
				fLogFile<<"			Position Ref  = "<<posref<<" keV"<<endl;
				fLogFile<<"			Position Peak = "<<pospeak<<" keV"<<endl;
				fLogFile<<"			Score : "<<score<<endl<<endl;
			}
			nbgood++;
			graph->Set(graph->GetN()+1); //on rajoute un point de coordonnees (0,0), les valeurs precedentes sont gardees
			graph->SetPoint(graph->GetN()-1,pospeak,posref);

			sprintf(text,"%6.2f",posref);

			if (gDebug > 0) rtext.DrawText(pospeak,intpeak,text);


		}

	}// j
	if (gDebug>0){
		gPad->Update();
		gPad->WaitPrimitive();
	}

	if (fLog) fLogFile << "	Number of good identified peaks : "<<nbgood<<endl;
	cout <<"Number of good identified peaks : "<<nbgood<<endl;
}

void FuzzyCalib::Test(){
	FuzzySet *ref;
	Peak *central;
	Peak *neib1;
	Peak *neib2;
	ref = (FuzzySet *)fListRef.At(0);
	central = (Peak *)fListPeak.At(0);
	neib1 =(Peak *)fListPeak.At(1);
	neib2 =(Peak *)fListPeak.At(2);

	Float_t score[5];
	Int_t index[5];
	score[0] = ref->ScorePos(*central); //position of central peak
	score[1] = ref->ScoreNeighbour1Pos(*central,*neib1); //position of neighbour1 peak
	score[2] = ref->ScoreNeighbour1Int(*central,*neib1); //intensity of neighbour1 peak
	score[3] = ref->ScoreNeighbour2Pos(*central,*neib2); //position of neighbour2 peak
	score[4] = ref->ScoreNeighbour2Int(*central,*neib2); //intensity of neighbour2 peak

	for (Int_t i=0 ; i<5 ; i++){
	cout <<score[i]<<endl;
	}

	TMath::Sort(5,score,index,kFALSE);
	for (Int_t i=0 ; i<5 ; i++){
	cout <<index[i]<<endl;
	}

	cout << "Min = "<< score[index[0]]<<endl;;



}


TObjArray& FuzzyCalib::GetListPeak(){
	return(fListPeak);
}

TObjArray& FuzzyCalib::GetAllPeak(){
	return(fAllPeak);
}

TObjArray& FuzzyCalib::GetListRef(){
	return(fListRef);
}

void FuzzyCalib::AllocMat(const Int_t ligne, const Int_t colonne){
	if (fMatScore == NULL){
		fMatScore = new Float_t[ligne*colonne];
	} else {  //fMatScore != NULL
		delete[] fMatScore;
		fMatScore = new Float_t[ligne*colonne];
	}

	for (Int_t i = 0 ; i<ligne*colonne ; i++){
		fMatScore[i] = 0.0;
	}

}

void FuzzyCalib::AllocPos(const Int_t nb){
	if (fPos == NULL){
		fPos = new Float_t[nb];
	} else {  //fPos != NULL
		delete[] fPos;
		fPos = new Float_t[nb];
	}

	for (Int_t i = 0 ; i<nb ; i++){
		fPos[i] = 0.0;
	}

}

