#ifndef FUZZYCALIB_H
#define FUZZYCALIB_H

#include<BaseCalib.h>
#include<FuzzySet.h>
#include<Peak.h>
#include<TGraphErrors.h>

/* Class FuzzyCalib
	FuzzyCalib is defined by a List of Reference peaks (fListRef) (List of fuzzysets) fListRef is the owner of the fuzzysets
			      by a list of Found Peaks (fListPeak) (List of Peak) --> not the owner of the peaks
			      by a list of all peaks (fAllPeaks) (List of Peak) --> Owner of the peaks

*/

class FuzzyCalib : public BaseCalib {
public:
	FuzzyCalib();
	FuzzyCalib(const Char_t*);
virtual ~FuzzyCalib();

	Int_t InitFuzzySets(const TObjArray &); //To initialyse the FuzzySets with a TObjArray of FuzzySets
	Int_t InitFuzzySets(const Char_t*, const Char_t*);//To initialyse the FuzzySets with a ROOT file which contains a TObjArray of FuzzySets
	Int_t Init(TH1D *histo, TF1 *formula);  //Initialyse fListPeak by searching peaks with TSpectrum::Search
	Float_t Algorithme1(const FuzzySet &, const Peak &, const Peak &, const Peak &);  //return the score for a ref peak with algorithme1

	void DoScore1();  //calcule tous les scores avec l'algorithme1

virtual const Char_t* DoCalib(TH1D *, TF1 *, TGraphErrors *, const Char_t *);
	void DoCalib(TObjArray *, TObjArray *, TObjArray *, const Char_t *);

	void DeFuzzycation1(TGraphErrors *); //gestion de la matrice version 1, remplissage du Graph

	TObjArray& GetListPeak(); //Get the fListPeak
	TObjArray& GetAllPeak(); //Get fAllPeak
	TObjArray& GetListRef(); //Get fListRef

	void Test();

private:
	TObjArray fListPeak;  //current list of peaks, not the owner of peaks, we work on this List of Peaks
	TObjArray fListRef;   //List of Reference peaks (List of FuzzySets), is the owner of the fuzzysets
	TObjArray fAllPeak;   //List of peaks, is the owner of the peaks

	Float_t* fMatScore;  //Matrice to store all the score -- ligne = peak -- colonne = reference

	void AllocMat(const Int_t,const Int_t); //to allocate the matrice fMatScore for a spectrum

	Float_t *fPos; //Tableau pour les positions des pics trouves sans changement avec la formula

	void AllocPos(const Int_t);

};

#endif
