#include<iostream>
using namespace std;

#include<FuzzySet.h>
#include<TMath.h>
#include<Peak.h>
#include<TROOT.h>

#define SCALEENERGY 6000 // 6 MeV

Double_t fuzzyfunction(Double_t *x, Double_t *par){ //Fuzzy Function
/*     <-par[1]-><-   par[2]   ->
1       -------------------------
|      /        |               \
|     /         |                 \
|    /          |                   \
|   /           |                     \
0   ------------|-----------------------
   <-  par[3] -><-       par[4]       ->
                |
	      par[0]
5 parametres : par[0] --> position centrale du pic de reference
               par[1] --> largeur au sommet gauche
	       par[2] --> largeur au sommet droit
	       par[3] --> largeur a la base gauche
	       par[4] --> largeur a la base droit

Controle des parametres : si par[i>0] < 0 --> on prend la valeur absolue
                          si par[3] < par[1] --> alors par[3] = par[1]
			  si par[4] < par[2] --> alors par[4] = par[2]
			  si tous les parametres =0, return 1, on ne veut pas de cette condition floue
*/
	//Check the parameters
	for (Int_t i = 1 ; i<5 ; i++){
		par[i] = TMath::Abs(par[i]);
	}

	if ((par[1]==0.0)&&(par[2]==0.0)&&(par[3]==0.0)&&(par[4]==0.0)) return 1;

	if (par[3] < par[1]) par[3]=par[1];
	if (par[4] < par[2]) par[4]=par[2];

	if ((x[0] >= par[0]-par[1]) && (x[0] <= par[0]+par[2])) return 1;
	if ((x[0] >= par[0]+par[4]) || (x[0] <= par[0]-par[3])) return 0;
	if ((x[0] > par[0]-par[3]) && (x[0] < par[0]-par[1])) return(x[0]/(par[3]-par[1])-(par[0]-par[3])/(par[3]-par[1]));
	if ((x[0] > par[0]+par[2]) && (x[0] < par[0]+par[4])) return(x[0]/(par[2]-par[4])-(par[0]+par[4])/(par[2]-par[4]));
	
	return 1;
}

ClassImp(FuzzySet);


FuzzySet::FuzzySet() : TNamed(),fFuzzySet(6){
	// default constructor.
	//Don't forget to process the Init(Float_t par*) method to initialize the fuzzyset
	fFuzzySet.SetOwner();
	Initialisation();
}

FuzzySet::FuzzySet(const Char_t* name) : TNamed(name,"FuzzySet"),fFuzzySet(6){
	// default constructor.
	//Don't forget to process the Init(Float_t par*) method to initialize the fuzzyset
	fFuzzySet.SetOwner();
	Initialisation();
}

FuzzySet::FuzzySet(const Char_t *name,const Float_t *par) : TNamed(name,"FuzzySet"),fFuzzySet(6) {
	fFuzzySet.SetOwner();
	Initialisation();
	Init(par);
}

void FuzzySet::Initialisation(){
	//Position
	TF1 *position = new TF1("fuzzyPos",fuzzyfunction,0,SCALEENERGY,5);
	fFuzzySet.Add(position);
	gROOT->GetListOfFunctions()->Remove(position);
	//Intensity
	TF1 *intensity = new TF1("fuzzyInt",fuzzyfunction,0,SCALEENERGY,5);
	fFuzzySet.Add(intensity);
	gROOT->GetListOfFunctions()->Remove(intensity);
	//Position of neighbour1
	TF1 *neighbour1pos = new TF1("fuzzyNeighbour1Pos",fuzzyfunction,0,SCALEENERGY,5);
	fFuzzySet.Add(neighbour1pos);
	gROOT->GetListOfFunctions()->Remove(neighbour1pos);
	//Intensity of neighbour1
	TF1 *neighbour1int = new TF1("fuzzyNeighbour1Int",fuzzyfunction,0,SCALEENERGY,5);
	fFuzzySet.Add(neighbour1int);
	gROOT->GetListOfFunctions()->Remove(neighbour1int);
	//Position of neighbour2
	TF1 *neighbour2pos = new TF1("fuzzyNeighbour2Pos",fuzzyfunction,0,SCALEENERGY,5);
	fFuzzySet.Add(neighbour2pos);
	gROOT->GetListOfFunctions()->Remove(neighbour2pos);
	//Intensity of neighbour2
	TF1 *neighbour2int = new TF1("fuzzyNeighbour2Int",fuzzyfunction,0,SCALEENERGY,5);
	fFuzzySet.Add(neighbour2int);
	gROOT->GetListOfFunctions()->Remove(neighbour2int);
}

FuzzySet::~FuzzySet(){
	fFuzzySet.Clear();
}



void FuzzySet::Init(const Float_t *par){
	TF1 *locfunction;

	//position
	locfunction = (TF1 *)fFuzzySet.FindObject("fuzzyPos");
	if (locfunction != NULL){
		locfunction->SetParameters(par[0],par[1],par[2],par[3],par[4]);
	} else {
		cerr << "Cannot initialize fuzzyset position ..."<<endl;
	}

	//Intensity
	locfunction = (TF1 *)fFuzzySet.FindObject("fuzzyInt");
	if (locfunction != NULL){
		locfunction->SetParameters(par[5],par[6],par[7],par[8],par[9]);
	} else {
		cerr << "Cannot initialize fuzzyset intensity ..."<<endl;
	}
	//Position of neighbour1
	locfunction =(TF1 *)fFuzzySet.FindObject("fuzzyNeighbour1Pos");
	if (locfunction != NULL){
		locfunction->SetParameters(par[10],par[11],par[12],par[13],par[14]);
	} else {
		cerr << "Cannot initialize fuzzyset neighbour1 position ..."<<endl;
	}
	//Intensity of neighbour1
	locfunction = (TF1 *)fFuzzySet.FindObject("fuzzyNeighbour1Int");
	if ( locfunction != NULL){
		locfunction->SetParameters(par[15],par[16],par[17],par[18],par[19]);
	} else {
		cerr << "Cannot initialize fuzzyset neighbour1 intensity ..."<<endl;
	}
	//Position of neighbour2
	locfunction = (TF1 *)fFuzzySet.FindObject("fuzzyNeighbour2Pos");
	if ( locfunction!= NULL){
		locfunction->SetParameters(par[20],par[21],par[22],par[23],par[24]);
	} else {
		cerr << "Cannot initialize fuzzyset neighbour2 position ..."<<endl;
	}
	//Intensity of neighbour2
	locfunction = (TF1 *)fFuzzySet.FindObject("fuzzyNeighbour2Int");
	if ( locfunction != NULL){
		locfunction ->SetParameters(par[25],par[26],par[27],par[28],par[29]);
	} else {
		cerr << "Cannot initialize fuzzyset neighbour2 intensity ..."<<endl;
	}

	locfunction = NULL;
}


void FuzzySet::AddFunction(TF1 *function){
	fFuzzySet.Add(function);
}

void FuzzySet::RemoveFunction(Int_t index){
	if (index >= fFuzzySet.GetSize()){
		 cerr << "Error, index > Size of the fuzzyset ..."<<endl;
		 return;
	}
	if (fFuzzySet.At(index) != NULL) {
		fFuzzySet.RemoveAt(index);
	}
	fFuzzySet.Compress();
}

TObjArray& FuzzySet::GetFuzzySet() {
	return fFuzzySet;
}

TF1* FuzzySet::GetFuzzyFunction(const Char_t *name) const {
	if ((TF1 *)fFuzzySet.FindObject(name) != NULL){
		return ((TF1 *)fFuzzySet.FindObject(name));
	} else {
		cerr << "NULL pointer ..."<<endl;
		return NULL;
	}
}

TF1* FuzzySet::GetFuzzyPos() const {
	return(GetFuzzyFunction("fuzzyPos"));
}

TF1* FuzzySet::GetFuzzyInt() const {
	return(GetFuzzyFunction("fuzzyInt"));
}

TF1* FuzzySet::GetFuzzyNeighbour1Pos() const {
	return(GetFuzzyFunction("fuzzyNeighbour1Pos"));
}

TF1* FuzzySet::GetFuzzyNeighbour1Int() const {
	return(GetFuzzyFunction("fuzzyNeighbour1Int"));
}

TF1* FuzzySet::GetFuzzyNeighbour2Pos() const {
	return(GetFuzzyFunction("fuzzyNeighbour2Pos"));
}

TF1* FuzzySet::GetFuzzyNeighbour2Int() const {
	return(GetFuzzyFunction("fuzzyNeighbour2Int"));
}


Float_t FuzzySet::Score(const Float_t value, const Char_t *name) const{
	if ((TF1 *)fFuzzySet.FindObject(name) != NULL){
		return (((TF1 *)fFuzzySet.FindObject(name))->Eval(value));
	} else {
		cerr << "No TF1 named "<<name<<" found in the fuzzyset ..."<<endl;
		return -1;
	}
}

Float_t FuzzySet::ScorePos(const Float_t value) const{
	//Return value
	return(Score(value,"fuzzyPos"));
}

Float_t FuzzySet::ScorePos(const Peak &peak) const{
	return(ScorePos(peak.GetPos()));
}

Float_t FuzzySet::ScorePos(const Peak &centralpeak , const Peak &neighbour1peak, const Peak &neighbour2peak) const{
	Float_t score[2]={0.0,0.0};
	score[0] = ScoreNeighbour1Pos(neighbour1peak.GetPos()-centralpeak.GetPos());
	score[1] = ScoreNeighbour2Pos(neighbour2peak.GetPos()-centralpeak.GetPos());
	return(TMath::Min(score[0],score[1]));
}

Float_t FuzzySet::ScoreInt(const Float_t value) const{
	return(Score(value,"fuzzyInt"));
}

Float_t FuzzySet::ScoreInt(const Peak &peak) const{
	return(ScoreInt(peak.GetInt()));
}

Float_t FuzzySet::ScoreInt(const Peak &centralpeak, const Peak &neighbour1peak, const Peak &neighbour2peak) const{
	Float_t score[2] = {0.0,0.0};
	//score[0] = ScoreInt(centralpeak);
	score[0] = ScoreNeighbour1Int(neighbour1peak.GetInt()/centralpeak.GetInt());
	score[1] = ScoreNeighbour2Int(neighbour2peak.GetInt()/centralpeak.GetInt());
	return(TMath::Min(score[0],score[1]));

}

Float_t FuzzySet::ScoreNeighbour1Pos(const Float_t value) const{
	return(Score(value,"fuzzyNeighbour1Pos"));
}

Float_t FuzzySet::ScoreNeighbour1Pos(const Peak &centralpeak, const Peak &neighbour1peak) const{
	return(ScoreNeighbour1Pos(neighbour1peak.GetPos()-centralpeak.GetPos()));
}

Float_t FuzzySet::ScoreNeighbour1Int(const Float_t value) const{
	return(Score(value,"fuzzyNeighbour1Int"));
}

Float_t FuzzySet::ScoreNeighbour1Int(const Peak &centralpeak, const Peak &neighbour1peak) const{
	return(ScoreNeighbour1Int(neighbour1peak.GetInt()/centralpeak.GetInt()));
}

Float_t FuzzySet::ScoreNeighbour2Pos(const Float_t value) const{
	return(Score(value,"fuzzyNeighbour2Pos"));
}

Float_t FuzzySet::ScoreNeighbour2Pos(const Peak &centralpeak, const Peak &neighbour2peak) const{
	return(ScoreNeighbour2Pos(neighbour2peak.GetPos()-centralpeak.GetPos()));
}

Float_t FuzzySet::ScoreNeighbour2Int(const Float_t value) const{
	return(Score(value,"fuzzyNeighbour2Int"));
}

Float_t FuzzySet::ScoreNeighbour2Int(const Peak &centralpeak, const Peak &neighbour2peak) const{
	return(ScoreNeighbour2Int(neighbour2peak.GetInt()/centralpeak.GetInt()));
}


