#ifndef FUZZYSET_H
#define FUZZYSET_H

#include<TObjArray.h>
#include<TF1.h>
#include<Peak.h>

class FuzzySet : public TNamed{
public:
	FuzzySet(); //default constructor
	FuzzySet(const Char_t*);  //constructor 
	FuzzySet(const Char_t*,const Float_t*); //Constructor -> the fuzzyset is initalized with par
       ~FuzzySet();
        void Init(const Float_t *);  //Initialistation of the FuzzySet (pos,int,posvoisin1,intvoisin1,posvoisin2,intvoisin2) with a list of parameters
	//void Init(Char_t *); //Initialistation of the FuzzySet with a file named "name" where parameters are stored
	void AddFunction(TF1 *); //Add a function TF1 in the fuzzyset at the end of the TObjArray
	void RemoveFunction(Int_t); //Remove the TF1 placed at the position index in the fuzzyset
	TObjArray& GetFuzzySet(); //return the fuzzyset
	TF1 *GetFuzzyFunction(const Char_t *)const; //return the TF1 fuzzy function called name
	TF1 *GetFuzzyPos() const; //return the TF1 fuzzy function position of the fuzzyset
	TF1 *GetFuzzyInt() const; //return the TF1 fuzzy function intensity of the fuzzyset
	TF1 *GetFuzzyNeighbour1Pos() const; //return the TF1 neighbour1 fuzzy function position of the fuzzyset
	TF1 *GetFuzzyNeighbour1Int() const; //return the TF1 neighbour1 fuzzy function intensity of the fuzzyset
	TF1 *GetFuzzyNeighbour2Pos() const; //return the TF1 neighbour2 fuzzy function position of the fuzzyset
	TF1 *GetFuzzyNeighbour2Int() const; //return the TF1 neighbour2 fuzzy function intensity of the fuzzyset
	Float_t Score(const Float_t, const Char_t*) const;  //return the score of a value with the fuzzy function called "name"
	Float_t ScorePos(const Float_t) const; // return the score of a value with the fuzzy function position
	Float_t ScorePos(const Peak&) const; //return the score in position for a peak
	Float_t ScorePos(const Peak&, const Peak&, const Peak&) const; //return the score in position for a central peak AND a neighbour1 peak AND a neighbour2 peak (function Min)
	Float_t ScoreInt(const Float_t) const; // return the score of a value with the fuzzy function intensity
	Float_t ScoreInt(const Peak&) const; //return the score in intensity for a peak
	Float_t ScoreInt(const Peak&, const Peak&, const Peak&) const; //return the score in intensity for a central peak AND a neighbour1 peak AND a neighbour2 peak (function Min)
	Float_t ScoreNeighbour1Pos(const Float_t) const; // return the score of a value with the neighbour1 fuzzy function position
	Float_t ScoreNeighbour1Pos(const Peak&, const Peak&) const; //return the score in relative position between neighbour1peak and centralpeak
	Float_t ScoreNeighbour1Int(const Float_t) const; // return the score of a value with the neighbour1 fuzzy function intensity
	Float_t ScoreNeighbour1Int(const Peak&, const Peak&) const; //return the score in relative intensity between neighbour1peak and centralpeak
	Float_t ScoreNeighbour2Pos(const Float_t) const; // return the score of a value with the neighbour2 fuzzy function position
	Float_t ScoreNeighbour2Pos(const Peak&, const Peak&) const; //return the score in relative position between neighbour2peak and centralpeak
	Float_t ScoreNeighbour2Int(const Float_t) const; // return the score of a value with the neighbour2 fuzzy function intensity
	Float_t ScoreNeighbour2Int(const Peak&, const Peak&) const; //return the score in relative intensity between neighbour2peak and centralpeak



private:
	TObjArray fFuzzySet;     //Table which contains all the fuzzy functions (position, intensity, ...)

	void Initialisation();

public:
	ClassDef(FuzzySet,1);

};

#endif
